# NAME: General Migration Functions
#
#
# VERSION: 1.2
#
#
# COPYRIGHT:                                                        
# 
#   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
#   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
#   All Rights Reserved
#   Licensed Materials - Property of IBM
#  
#   US Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# 
#                                                                   
#
# SYNOPSIS:
#
#
#
# DESCRIPTION:
#
#
# This file contains functions which perform the actions necessary for
# the migration of CICS systems.
#
# It contains DCE migration functions necessary for the
# migration of CICS systems to the new DCE naming convention.  The changes
# required on an individual machine can be made by a single call to this
# file, or by a series of calls specifying named regions or server types.
# The functions are called as part of the wider migration process, which is
# controlled by 'cicsmigrate'.
#
# There are 3 levels of migration needed for the DCE names changes.  These
# are :
# 1.  at the DCE cell (change CICS root CDS dir from /.:/cics6000 to /.:/cics,
# 2.  at the machine (change hidden client principal from cics6000/client/<num>
#						    to cics/client/<num>,
# 3.  at the individual server and region level (change principals, CDS
#       directories and objects and stanza entries).
#
# As the third level cannot be performed without the first level changes
# being in place, if this file is invoked with a request to migrate an
# individual server type or region, the general (first and second level)
# changes will be performed first.  This may result in multiple attempts
# to create the new root CDS directories - the function is designed not to
# fail if the new directories are already in place.
#
# The individual functions are as follows:
#
# GeneralDCEMigrate - create new CDS dirs for DCE cell
#		     create cics_regions group if one does not exist
#
# MigrateDCERegions - generate list of regions & call MigrateRegion for
#		     each one
#
# MigrateDCESfs - generate list of sfsservers & call MigrateSfsServer for
#		 each one.
#
# MigrateDCESfsSRC - generate list of sfsservers & migrate the SRC notify 
#                 entry for each one 
#
# MigrateSfsPathSRC - migrate the SRC database entry for the sfsservers to
#                     have the correct path for sfs
#
# MigrateRegionSRC - migrate the SRC database entry for individual region
#
# GeneralDCEClean - delete old CDS dirs for DCE cell
#
# MigrateRegion - perform migration for individual region
#
# MigrateSfsServer - perform migration for individual sfs server
#
# CreateTempMigrationDir - create a temporary migration directory, called
#			  mig#temp in /var/cics_regions
#
# RemoveTempMigrationDir - remove the temporary migration directory mig#temp
#
# CreateClassDir - this creates the directory structure for a new class in the 
#       database, and copies the default stanzas for both cold and warm starts.
#
# MigrateEnUSMaps - this creates an equivalent maps/en_US directory structure 
#       within a region that is linked to the maps/En_US structure.
#
# AddppcgwyACLs - add the cics_ppcgwy DCE security group, and updates the
#       Access Control Lists for each cds directory for this new group.
#
# AddppcgwyACLregion - add ACLs for the cics_ppcgwy DCE  group to the specified
#       region cds directory.  This is so that a PPC gateway has the correct
#       permissions to communicate with a region.
#
# SetCICSUnauthenticatedAccess - add ACLs to /.:/cics and /.:/cics/profile
#	to allow unauthenticated users to read the CDS entries and so to
#	locate the regions.  This allows users to connect freely to regions
#	with RuntimeProtection=none.


#################################################################################
#
#
#
# FUNCTION: GeneralDCEMigrate:
###############################
# The following cds directories are created by calls to CreateSingleCdsDir:
#
#	  /.:/cics
#	  /.:/cics/log
#	  /.:/cics/sfs
#	  /.:/cics/ppc
#	  /.:/cics/ppc/gateway
#	  /.:/cics/trpc
#	  /.:/cics/tmxa
#	  /.:/cics/profile
#
#
# ACLs on these directories are set for the 4 groups created by cicssetupdce,
# for the principal that is excuting the command and the CDS groups
# subsys/dce/cds-admin and subsys/dce/cds-server. Initial container
# and initial object ACLs are also set.
#
# The current principal, and members of cics_admin, subsys/dce/cds-admin
# and subsys/dce/cds-server are granted full permissions to all directories
# and objects.
#
# Members of the other groups are granted more restrictive access on
# the basis of their needs.
#
# The group cics_regions is checked for existence, and if not present,
# then it is created.
#
# FUNCTION: MigrateDCERegions:
##############################
#
# This uses 'cicsname' to generate a list of CICS regions for the machine.
# Each region is examined to see if it has already been migrated (*** how? ***)
# if not, MigrateRegion is called to perform the necessary actions.
#
# FUNCTION: MigrateDCESfs:
##########################
#
# FUNCTION: MigrateDCEClient:
#############################
#
# This calls the script 'cicsmigrateclients' which creates the new client
# principal and deletes the old one.
#
# FUNCTION: GeneralDCEClean:
#############################
#
# This function deletes the old versions of the DCE entities listed above,
# and all directories and objects below them. These contain the root directory
# name "/.:/cics6000".   It does not treat the
# non-existence of any of these entities as an error, and hence can be
# run more than once without error.
#
#
# FUNCTION GeneralDCEClean
###########################
#
#     IF CICS_CDS_ROOT exists
#
#	 descend CDS tree obtaining list of all directories
#
#	 # for efficiency, implement following loop as single
#	 # cdscp command, user will then get full list of
#	 # all entities that cannot be deleted
#
#	 FOR each directory in list, starting with the lowest
#	     delete each object in directory
#	     delete directory
#	 ENDFOR
#
#	 IF any errors
#	     terminate unless ignoring errors
#	 ENDIF
#
#     ENDIF
#
# ENDFUNCTION
#
# FUNCTION CheckLoginAndDce
###########################
#
#    use klist to check that dce context exists, remember principal name
#
#    use RgyEdit to check that it is possible to view
#    the registry. The intention here is to be able to assume
#    for the rest of the script that a failure to view a
#    registry entry is due to its non-existence rather than
#    an ACL problem.
#
# ENDFUNCTION
#
#
# FUNCTION CreateTempMigrationDir
#################################
#
#    if we cannot find or access the region directory itself
#       exit with error message
#    fi
#
#    if the region mig#temp already exists we cannot continue
#       exit with error message
#    fi
#
#    Copy the archive of the default region.
#    if could not locate the default region
#       exit with error message
#    fi
#
#    create the temporary directory mig#temp
#    cd to the new directory
#    cpio copy the default region to the newly-created temporary directory
#    return to the callers directory
#
# ENDFUNCTION
#
# FUNCTION RemoveTempMigrationDir
#################################
#
#    if mig#temp exists
#    then
#      rm -rf mig#temp
#    fi
#
# ENDFUNCTION
#
# FUNCTION CreateClassDir
#################################
#
# if we cannot find the database directory
#   exit with error message
# fi
# if the class subdirectory does not exist
#   create it
#   change ownership/access to cics:cics 770
# fi
# if the cold start image does not exist
#   copy it from the mig#temp region
#   change ownership/access to cics:cics 660
# fi
# if the warm start subdirectory does not exist
#   create it
#   change ownership/access to cics:cics 770
# fi
# if the warm start image does not exist
#   write the integrity warning to the warm start file
#   concatenate the cold start image
#   change ownership/access to cics:cics 660
# fi
#
# ENDFUNCTION
#
# FUNCTION MigrateEnUSMaps
#################################
#
# if we cannot find the database directory
#   exit with error message
# fi
# if the maps subdirectory does not exist
#   create it
#   change ownership/access to cics:cics 770
# fi
# if the En_US maps directory exists
#   if the en_US maps directory does not exist
#     create soft link between en_US and En_US maps directory
#   else
#     create soft links between files in En_US maps directory and
#     en_US maps directory
# fi
#
# ENDFUNCTION
#
# FUNCTION AddppcgwyACLs
########################
#
# The group cics_ppcgwy is checked for existence, and if not present,
# then it is created.
#
# Set ACLs on the cds directories for the group cics_ppcgwy.
#
# ENDFUNCTION
#
# FUNCTION AddppcgwyACLregion
#############################
#
# Set ACLs on the cds %REGION% directory for the group cics_ppcgwy.
#
# ENDFUNCTION
#
# FUNCTION SetCICSUnauthenticatedAccess
#######################################
#
# Uses CICSEntryUnauthenticatedAccess to set read and traverse ACLs to the
# /.:/cics/<Region>, /.:/cics/<Region>/ts, /.:/cics/profile and 
# /.:/cics/profile/cics_profile CDS entries.
#
#*/

# =============================================================================
# Please do not remove this comment block, it is required to delimit
# the data which is extracted for automatic document generation
# =============================================================================
# CANNA PRIVATE END



ERZ4629E=29
ERZ4629E_TEXT='"ERZ046029E: Unable to create directory '%1\$s'"'
ERZ4631E=31
ERZ4631E_TEXT='"ERZ046031E: Unable to create symbolic link '%1\$s' for '%2\$s'"'
ERZ4644I=44
ERZ4644I_TEXT='"ERZ046044I: '%1\$s' is a symbolic link to '%2\$s'"'
ERZ4646E=46
ERZ4646E_TEXT='"ERZ046046E: Could not locate archive of default region"'
ERZ4647E=47
ERZ4647E_TEXT='"ERZ046047E: Region '%1\$s' is locked - command not allowed"'
ERZ4648E=48
ERZ4648E_TEXT='"ERZ046048E: Region '%1\$s' already exists in this DCE cell"'
ERZ4649E=49
ERZ4649E_TEXT='"ERZ046049E: Region '%1\$s' already exists"'
ERZ4660E=59
ERZ4660E_TEXT='"ERZ046060E: Command line option '%1\$s' is invalid"'
ERZ4663E=62
ERZ4663E_TEXT='"ERZ046063E: You have insufficient privilege to run the '%1\$s' program"'
ERZ4664E=63
ERZ4664E_TEXT='"ERZ046064E: You must be logged into the DCE as a cell administrator to run this command"'
ERZ4670E=69
ERZ4670E_TEXT='"ERZ046070E: Unable to create CDS directory for region '%1\$s'"'
ERZ4671E=70
ERZ4671E_TEXT='"ERZ046071E: Unable to create DCE principal for region '%1\$s'"'
ERZ4672E=71
ERZ4672E_TEXT='"ERZ046072E: Unable to create DCE account for region, '%1\$s'"'
ERZ4679W=73
ERZ4679W_TEXT='"ERZ046079W: Unable to add principal '%1\$s' to CICS user group '%2\$s'"'
ERZ4681E=75
ERZ4681E_TEXT='"ERZ046081E: Region name '%1\$s' is greater than 8 characters long"'
ERZ46105E=96
ERZ46105E_TEXT='"ERZ046105E: Cannot find region directory for region '%1\$s'"'
ERZ46154I=143
ERZ46154I_TEXT='"ERZ046154I: Enter your DCE password: "'
ERZ46155E=144
ERZ46155E_TEXT='"ERZ046155E: The DCE password in environment variable '%1\$s' is incorrect"'
ERZ46156E=145
ERZ46156E_TEXT='"ERZ046156E: Cannot execute '%1\$s'"'
ERZ46157E=146
ERZ46157E_TEXT='"ERZ046157E: Cannot destroy server '%1\$s' - server running"'
ERZ46158W=147
ERZ46158W_TEXT='"ERZ046158W: Subsystem definition '%1\$s' could not be found"'
ERZ46159I=148
ERZ46159I_TEXT='"ERZ046159I: The subsystem definition for server '%1\$s' has been removed"'
ERZ46161W=149
ERZ46161W_TEXT='"ERZ046161W: The restart file '%1\$s' could not be found"'
ERZ46162E=150
ERZ46162E_TEXT='"ERZ046162E: Attempt to list the contents of the CDS directory '%1\$s' was not successful"'
ERZ46163E=151
ERZ46163E_TEXT='"ERZ046163E: Could not create list of regions to migrate"'
ERZ46164E=152
ERZ46164E_TEXT='"ERZ046164E: Migration of region '%1\$s' was unsuccessful"'
ERZ46168E=153
ERZ46168E_TEXT='"ERZ046168E: Could not create list of SFS servers to migrate"'
ERZ46169E=154
ERZ46169E_TEXT='"ERZ046169E: Migration of SFS server '%1\$s' was unsuccessful"'
ERZ46171E=155
ERZ46171E_TEXT='"ERZ046171E: No subsystem definition for region '%1\$s' on this machine"'
ERZ46173W=156
ERZ46173W_TEXT='"ERZ046173W: Could not list old CDS directories for deletion"'
ERZ46174W=157
ERZ46174W_TEXT='"ERZ046174W: Old CDS directories not deleted"'
ERZ46176E=158
ERZ46176E_TEXT='"ERZ046176E: DCE login was unsuccessful"'
ERZ46177E=159
ERZ46177E_TEXT='"ERZ046177E: DCE registry cannot be interrogated"'
ERZ46178W=160
ERZ46178W_TEXT='"ERZ046178W: An attempt to find CDS entity '%1\$s' was not successful"'
ERZ46179E=161
ERZ46179E_TEXT='"ERZ046179E: An attempt to create CDS directory '%1\$s' was not successful"'
ERZ46180E=162
ERZ46180E_TEXT='"ERZ046180E: Attempt to modify ACL for directory '%1\$s' was not successful"'
ERZ46181W=163
ERZ46181W_TEXT='"ERZ046181W: %1\$s has successfully deleted the CDS entities"'
ERZ46182I=164
ERZ46182I_TEXT='"ERZ046182I: Usage: cicsmigratelevel {-?| [-r regionName] -c className}"'
ERZ46183E=165
ERZ46183E_TEXT='"ERZ046183E: Cannot create lock file for server '%1\$s'"'
ERZ46184E=166
ERZ46184E_TEXT='"ERZ046184E: Cannot perform migration - server '%1\$s' running"'
ERZ46185E=167
ERZ46185E_TEXT='"ERZ046185E: Cannot read entry from server database for server '%1\$s'"'
ERZ46186E=168
ERZ46186E_TEXT='"ERZ046186E: Unable to access details on user '%1\$s'"'
ERZ46187E=169
ERZ46187E_TEXT='"ERZ046187E: Unsuccessful attempt at changing owner of files in server directory '%1\$s' for server '%2\$s'"'
ERZ46188E=170
ERZ46188E_TEXT='"ERZ046188E: Cannot perform migration"'
ERZ46189E=171
ERZ46189E_TEXT='"ERZ046189E: Cannot create subsystem definition '%1\$s' for server '%2\$s'"'
ERZ46190W=172
ERZ46190W_TEXT='"ERZ046190W: Unable to delete CDS entity '%1\$s', return code is '%2\$s'"'
ERZ46191W=173
ERZ46191W_TEXT='"ERZ046191W: Unable to delete entity from the registry, return code is %1\$s"'
ERZ46192W=174
ERZ46192W_TEXT='"ERZ046192W: Cannot perform migration"'
ERZ46193E=175
ERZ46193E_TEXT='"ERZ046193E: Migration of database for server '%1\$s' was unsuccessful"'
ERZ46196E=178
ERZ46196E_TEXT='"ERZ046196E: Region '%1\$s' already exists"'
ERZ46199W=181
ERZ46199W_TEXT='"ERZ046199W: CDS entity '%1\$s' was not found"'
ERZ46200W=182
ERZ46200W_TEXT='"ERZ046200W: An attempt to create group '%1\$s' was not successful"'
ERZ46201E=183
ERZ46201E_TEXT='"ERZ046201E: Insufficient space to complete the copy.  The file system is more than 94 percent full."'
ERZ46202E=184
ERZ46202E_TEXT='"ERZ046202E: The copy of the default stanzas to the temporary directory failed"'
ERZ46204E=186
ERZ46204E_TEXT='"ERZ046204E: Cannot find database subdirectory for region '%1\$s'"'
ERZ46255I=237
ERZ46255I_TEXT='"ERZ046255I: The keytab file for region '%1\$s' may be corrupted. Check before running the region."'
ERZ46275I=257
ERZ46275I_TEXT='"ERZ046275I: Changing ACLs for CICS CDS entries.  This may take a few minutes"'
ERZ46276I=258
ERZ46276I_TEXT='"ERZ046276I: Changes to ACLs for CICS CDS entries now complete"'

#
# Module what string
#
#@(#)regrm,22:08:59,Apr 08 1998,s211-L980223
#


MSN_START=99


typeset -r REGRM_MSN_1=${MSN_START}01
typeset -r REGRM_MSN_2=${MSN_START}02
typeset -r REGRM_MSN_3=${MSN_START}03
typeset -r REGRM_MSN_4=${MSN_START}04
typeset -r REGRM_MSN_5=${MSN_START}05
typeset -r REGRM_MSN_6=${MSN_START}06
typeset -r REGRM_MSN_7=${MSN_START}07
typeset -r REGRM_MSN_8=${MSN_START}08
typeset -r REGRM_MSN_9=${MSN_START}09
typeset -r REGRM_MSN_10=${MSN_START}10
typeset -r REGRM_MSN_11=${MSN_START}11
typeset -r REGRM_MSN_12=${MSN_START}12
typeset -r REGRM_MSN_13=${MSN_START}13
typeset -r REGRM_MSN_14=${MSN_START}14
typeset -r REGRM_MSN_15=${MSN_START}15
typeset -r REGRM_MSN_16=${MSN_START}16
typeset -r REGRM_MSN_17=${MSN_START}17
typeset -r REGRM_MSN_18=${MSN_START}18
typeset -r REGRM_MSN_19=${MSN_START}19
typeset -r REGRM_MSN_20=${MSN_START}20
typeset -r REGRM_MSN_21=${MSN_START}21
typeset -r REGRM_MSN_22=${MSN_START}22
typeset -r REGRM_MSN_23=${MSN_START}23
typeset -r REGRM_MSN_24=${MSN_START}24
typeset -r REGRM_MSN_25=${MSN_START}25
typeset -r REGRM_MSN_26=${MSN_START}26
typeset -r REGRM_MSN_27=${MSN_START}27
typeset -r REGRM_MSN_28=${MSN_START}28
typeset -r REGRM_MSN_29=${MSN_START}29
typeset -r REGRM_MSN_30=${MSN_START}30
typeset -r REGRM_MSN_31=${MSN_START}31
typeset -r REGRM_MSN_32=${MSN_START}32
typeset -r REGRM_MSN_33=${MSN_START}33
typeset -r REGRM_MSN_34=${MSN_START}34
typeset -r REGRM_MSN_35=${MSN_START}35
typeset -r REGRM_MSN_36=${MSN_START}36
typeset -r REGRM_MSN_37=${MSN_START}37
typeset -r REGRM_MSN_38=${MSN_START}38
typeset -r REGRM_MSN_39=${MSN_START}39
typeset -r REGRM_MSN_40=${MSN_START}40
typeset -r REGRM_MSN_41=${MSN_START}41
typeset -r REGRM_MSN_42=${MSN_START}42
typeset -r REGRM_MSN_43=${MSN_START}43
typeset -r REGRM_MSN_44=${MSN_START}44
typeset -r REGRM_MSN_45=${MSN_START}45
typeset -r REGRM_MSN_46=${MSN_START}46
typeset -r REGRM_MSN_47=${MSN_START}47
typeset -r REGRM_MSN_48=${MSN_START}48
typeset -r REGRM_MSN_49=${MSN_START}49
typeset -r REGRM_MSN_50=${MSN_START}50
typeset -r REGRM_MSN_51=${MSN_START}51
typeset -r REGRM_MSN_52=${MSN_START}52
typeset -r REGRM_MSN_53=${MSN_START}53
typeset -r REGRM_MSN_54=${MSN_START}54
typeset -r REGRM_MSN_55=${MSN_START}55
typeset -r REGRM_MSN_56=${MSN_START}56
typeset -r REGRM_MSN_57=${MSN_START}57
typeset -r REGRM_MSN_58=${MSN_START}58
typeset -r REGRM_MSN_59=${MSN_START}59
typeset -r REGRM_MSN_60=${MSN_START}60
typeset -r REGRM_MSN_61=${MSN_START}61
typeset -r REGRM_MSN_62=${MSN_START}62
typeset -r REGRM_MSN_63=${MSN_START}63
typeset -r REGRM_MSN_64=${MSN_START}64
typeset -r REGRM_MSN_65=${MSN_START}65
typeset -r REGRM_MSN_66=${MSN_START}66
typeset -r REGRM_MSN_67=${MSN_START}67
typeset -r REGRM_MSN_68=${MSN_START}68
typeset -r REGRM_MSN_69=${MSN_START}69
typeset -r REGRM_MSN_70=${MSN_START}70
typeset -r REGRM_MSN_71=${MSN_START}71
typeset -r REGRM_MSN_72=${MSN_START}72
typeset -r REGRM_MSN_73=${MSN_START}73
typeset -r REGRM_MSN_74=${MSN_START}74
typeset -r REGRM_MSN_75=${MSN_START}75
typeset -r REGRM_MSN_76=${MSN_START}76
typeset -r REGRM_MSN_77=${MSN_START}77
typeset -r REGRM_MSN_78=${MSN_START}78
typeset -r REGRM_MSN_79=${MSN_START}79
typeset -r REGRM_MSN_80=${MSN_START}80
typeset -r REGRM_MSN_81=${MSN_START}81
typeset -r REGRM_MSN_82=${MSN_START}82
typeset -r REGRM_MSN_83=${MSN_START}83
typeset -r REGRM_MSN_84=${MSN_START}84
typeset -r REGRM_MSN_85=${MSN_START}85
typeset -r REGRM_MSN_86=${MSN_START}86
typeset -r REGRM_MSN_87=${MSN_START}87
typeset -r REGRM_MSN_88=${MSN_START}88
typeset -r REGRM_MSN_89=${MSN_START}89
typeset -r REGRM_MSN_90=${MSN_START}90
typeset -r REGRM_MSN_91=${MSN_START}91
typeset -r REGRM_MSN_92=${MSN_START}92
typeset -r REGRM_MSN_93=${MSN_START}93
typeset -r REGRM_MSN_94=${MSN_START}94
typeset -r REGRM_MSN_95=${MSN_START}95
typeset -r REGRM_MSN_96=${MSN_START}96
typeset -r REGRM_MSN_97=${MSN_START}97
typeset -r REGRM_MSN_98=${MSN_START}98
typeset -r REGRM_MSN_99=${MSN_START}99
typeset -r REGRM_MSN_100=${MSN_START}100
typeset -r REGRM_MSN_101=${MSN_START}101
typeset -r REGRM_MSN_102=${MSN_START}102
typeset -r REGRM_MSN_103=${MSN_START}103
typeset -r REGRM_MSN_104=${MSN_START}104
typeset -r REGRM_MSN_105=${MSN_START}105
typeset -r REGRM_MSN_106=${MSN_START}106
typeset -r REGRM_MSN_107=${MSN_START}107
typeset -r REGRM_MSN_108=${MSN_START}108
typeset -r REGRM_MSN_109=${MSN_START}109
typeset -r REGRM_MSN_110=${MSN_START}110
typeset -r REGRM_MSN_111=${MSN_START}111
typeset -r REGRM_MSN_112=${MSN_START}112
typeset -r REGRM_MSN_113=${MSN_START}113
typeset -r REGRM_MSN_114=${MSN_START}114
typeset -r REGRM_MSN_115=${MSN_START}115
typeset -r REGRM_MSN_116=${MSN_START}116
typeset -r REGRM_MSN_117=${MSN_START}117
typeset -r REGRM_MSN_118=${MSN_START}118
typeset -r REGRM_MSN_119=${MSN_START}119
typeset -r REGRM_MSN_120=${MSN_START}120
typeset -r REGRM_MSN_121=${MSN_START}121
typeset -r REGRM_MSN_122=${MSN_START}122
typeset -r REGRM_MSN_123=${MSN_START}123




# =============================================================================
#
# Declarations and constants
#
# =============================================================================


typeset -r CICS_CDS_ROOT="/.:/cics"
typeset -r CICS_OLD_CDS_ROOT="/.:/cics6000"

					  # standard CDS server groups, any cell
					  # where this does not exist needs
					  # careful configuration, we
					  # don't allow over-riding of
					  # ACLs in such cells.
typeset -r CDS_SERVER_GROUP="subsys/dce/cds-server"

					  # standard cds admin group
typeset -r CDS_ADMIN_GROUP="subsys/dce/cds-admin"

					  # Bool values - for (( )) contructs
typeset -ir TRUE=1
typeset -ir FALSE=0
typeset -i MSPECIFIED		     # Migrate all machine resources
typeset -i RSPECIFIED		     # Migrate specified region only
typeset -i LSPECIFIED		     # Migrate logservers only
typeset -i SSPECIFIED		     # Migrate sfsservers only

					  # flags, controlling script operation
typeset -i CDS_SERVER_GROUP_OK	    # true if group cds-servers exists
typeset -i CDS_ADMIN_GROUP_OK	     # true if group cds-admin exists
typeset -r MCATNAME="cics.cat"	    # message catalogue
typeset -r NULLFILE="/dev/null"	   # Target for unwanted output

typeset -r CICS_UGROUP="cics_users"       # group for cics processes
typeset -r CICS_AGROUP="cics_admin"       # group for administrators
typeset -r CICS_SGROUP="cics_sfs"	 # group for sfs servers
typeset -r CICS_LGROUP="cics_log"	 # group for log servers
typeset -r CICS_RGROUP="cics_regions"     # group for log servers
typeset -r CICS_PGROUP="cics_ppcgwy"      # group for the PPC gateway server


					  # the set of groups we need
typeset -r CICS_GROUPS="$CICS_UGROUP $CICS_AGROUP $CICS_SGROUP $CICS_LGROUP $CICS_RGROUP"

typeset CELLNAME			  # DCE cell name for this machine
typeset -x ADMLG_DCE_PASSWORD	     # DCE passwd needed by server
typeset PW_ENV_VAR # Name of passwd environment varible
typeset -r RM_SYS_NAME="rmsys"	    # Executable to remove system
					  #  admin functions
typeset ADMINPRIN			 # DCE principal of user

typeset TEMPREGION=""		     # name of the temporary migration dir
typeset ARCHIVEFILE=""		    # the cics default region dir
typeset -r CURRDIR=$PWD		   # the calling directory

# =============================================================================
# FUNCTION: ParseCommandLine
#
# Set flags on basis of command line options
# =============================================================================
ParseCommandLine()
{

    while getopts :r:mlsc OPT
    do
	case $OPT in
	m)
	    # migrate all resources on machine
	    MSPECIFIED=TRUE
	    ;;
	r)
	    # migrate specified region
	    RSPECIFIED=TRUE
	    REGIONSPEC="$OPTARG"
	    ;;
	l)
	    # migrate all logservers on this machine
	    LSPECIFIED=TRUE
	    ;;
	s)
	    # migrate all sfsservers on this machine
	    SSPECIFIED=TRUE
	    ;;
	\?)
	    # unrecognised option - or ? requesting Usage
	    DispMessage $REGRM_MSN_1 ERZ4660E $OPTARG"
	    return 1
	    ;;
	:)
	    # option needing a parameter -
	    DispMessage $REGRM_MSN_2 ERZ4660E $OPTARG"
	    return 1
	    ;;
	*)
	    # no case
	    DispMessage $REGRM_MSN_3 ERZ4660E $OPTARG
	    return 1
	    ;;
	esac
    done
    return 0
}

# =============================================================================
# FUNCTION: CheckLoginAndDce
#
# Check that we're dce_logged in and can interrogate registry
# =============================================================================
CheckLoginAndDce()
{
    typeset RGY_OUT



    # check that the script is being run by "root"  and also dce_logged in
    # Root access is required to perform many of the functions.  Also,
    # "root" should be able to run AIX SRC commands used to create the region.
    #
    [ "$(GetEUid)" != "0" ] &&  DispMessage $REGRM_MSN_4 ERZ4663E $PROGNAME && return 1

    #
    # We only enforce the DCE login if the DCE security services have been
    # configured
    #
    $BINDIR/cicscheckdceconf > /dev/null 2>&1
    if [ $? -ne 0 ]
    then
    
	#
        # Must be dce_logged in
        #
    
        OWNRINFO=$(LANG=C </dev/null klist)
        [ $? -ne 0 ] && DispMessage $REGRM_MSN_5 ERZ4664E && return 1
    
        #
        # Get this administrator's principal
    
        ADMINPRIN=$(echo "$OWNRINFO" | awk '$1 ~ /Principal:/ {print $3}')
    
    
        CELLNAME=$(echo "$OWNRINFO" | awk '$1 ~ /Cell:/ {print $3}')
    
        #
        # Get the current DCE password, Also allow the password to be passed
        # via cell_admin_pw or ERZ_SvJsYmPw.
        #
    
        if [ -n "$cell_admin_pw" ]
        then
            ADMLG_DCE_PASSWORD=$cell_admin_pw
            PW_ENV_VAR="cell_admin_pw"
        else
            if [ -n "$ERZ_SvJsYmPw" ]
            then
                ADMLG_DCE_PASSWORD=$ERZ_SvJsYmPw
                PW_ENV_VAR="ERZ_SvJsYmPw"
            else
	        PROMPT=$(DispMessage $REGRM_MSN_6 ERZ46154I 2>&1)
	        {
	        stty -echo
	        read ADMLG_DCE_PASSWORD?"${PROMPT}"
	        stty echo
	        echo
	        }  </dev/tty >/dev/tty
            fi
        fi
    
        # Validate the DCE password given
        OUTPUT=$(dce_login_noexec "$ADMINPRIN" "$ADMLG_DCE_PASSWORD" 2>&1)
        if [ $? -ne 0 ]
        then
            echo "$OUTPUT"
            if [ -n "$PW_ENV_VAR" ]
            then
                DispMessage $REGRM_MSN_7 ERZ46155E $PW_ENV_VAR
            else
                DispMessage $REGRM_MSN_86 ERZ46176E
            fi
            return 1
        fi
    
        # rgy_errors go to stdout so save it
        # and display in case of error
    
        RGY_OUT=$(RgyEdit -g -v ) ||
           (print "$RGY_OUT" >&2; DispMessage $REGRM_MSN_8 ERZ46177E && return 1)
    
        # We need to ensure that CDS servers can get to our
        # CDS directories. Assume that if a cds-servers group
        # exists then we can use it, otherwise we must prohibit
        # the over-riding of existing ACLs, relying on the initial
        # container ACLs to propogate the permissions for the cds-servers.
    
        print "$RGY_OUT" | grep "^$CDS_SERVER_GROUP " >$NULLFILE
    
        (( CDS_SERVER_GROUP_OK = ($? == 0) ))
    
        # Also remember whether the cds-admin group exist
    
        print "$RGY_OUT" | grep "^$CDS_ADMIN_GROUP " >$NULLFILE
    
        (( CDS_ADMIN_GROUP_OK = ($? == 0) ))
    
    
        # Check that we can run rmssys
    fi
    
    CheckCanRun rmssys
    if [ $? -ne 0 ]
    then
	DispMessage $REGRM_MSN_9 ERZ46156 $RM_SYS_NAME
	return 1
    fi

    return 0
}

# =============================================================================
# FUNCTION: DispMessage
#
# Display a message on the screen with a MSN (low level utility)
# The first parameter is the MSN number,
# subsequent parametrs are the dspmsg parameters
#
# the "echo" command is used since sed does not seem to recognise EOF as EOL
# =============================================================================

DispMessage()
{

    typeset _MSN
    typeset _MSNO
    typeset _DEFMSG

    _MSN=$1; shift			      # the MSN is the first param
    eval _MSNO="\${$1}"			 # MESSAGE ID
    eval _DEFMSG="\${$1_TEXT}" ; shift	  # DEFAULT MESSAGE TEXT

    ( dspmsg ${MCATNAME} -s ${MSET} ${_MSNO} "${_DEFMSG}" "$@" ; echo "" ) |
    sed "s:ERZ[0-9]*[EWI]:&/$_MSN:" >&2

}

# =============================================================================
# FUNCTION: GeneralDCEMigrate
#
# This function creates each of the required CDS directories
# =============================================================================
GeneralDCEMigrate()
{

			   # if the server and admin groups exist
			   # then set up ACLs for them to be applied
			   # to all directories and objects. Note:
			   # if we over-ride the ACLs (ACL_ACTION=-s)
			   # then we MUST let the CDS servers at
			   # the directories, so the CDS_SERVER_GROUP
			   # is vital.
    typeset  CELL_DIR_ACLS
    typeset  CELL_OBJ_ACLS
    typeset  GROUP_LIST

    CELL_DIR_ACLS=""

    (( CDS_ADMIN_GROUP_OK )) &&
	  CELL_DIR_ACLS="$CELL_DIR_ACLS group:${CDS_ADMIN_GROUP}:rwdtcia"

    (( CDS_SERVER_GROUP_OK )) &&
	  CELL_DIR_ACLS="$CELL_DIR_ACLS group:${CDS_SERVER_GROUP}:rwdtcia"

    CELL_OBJ_ACLS=""

    (( CDS_ADMIN_GROUP_OK )) &&
	  CELL_OBJ_ACLS="$CELL_OBJ_ACLS group:${CDS_ADMIN_GROUP}:rwdtc"

    (( CDS_SERVER_GROUP_OK )) &&
	  CELL_OBJ_ACLS="$CELL_OBJ_ACLS group:${CDS_SERVER_GROUP}:rwdtc"


    # CICS root directory
    # only administrators need access here
    CreateSingleCdsDir $CICS_CDS_ROOT     \
		       "$CELL_DIR_ACLS
			user:$ADMINPRIN:rwdtcia
			group:$CICS_AGROUP:rwdtcia
			group:$CICS_UGROUP:rt
			group:$CICS_SGROUP:rt
			group:$CICS_LGROUP:rt" \
			\
		       "$CELL_DIR_ACLS
			user:$ADMINPRIN:rwdtcia
			group:$CICS_AGROUP:rwdtcia
			group:$CICS_UGROUP:rt
			any_other:r--t---" \
			\
		       "$CELL_OBJ_ACLS
			user:$ADMINPRIN:rwdtc
			group:$CICS_AGROUP:rwdtc
			group:$CICS_UGROUP:rt
			any_other:r--t---"


    # PPC
    CreateSingleCdsDir "$CICS_CDS_ROOT/ppc" \
		       "$CELL_DIR_ACLS
			user:$ADMINPRIN:rwdtcia
			group:$CICS_AGROUP:rwdtcia
			group:$CICS_UGROUP:rt
			group:$CICS_SGROUP:-
			group:$CICS_LGROUP:-" \
			\
		       "$CELL_DIR_ACLS
			user:$ADMINPRIN:rwdtcia
			group:$CICS_AGROUP:rwdtcia
			group:$CICS_UGROUP:rt
			group:$CICS_SGROUP:-
			group:$CICS_LGROUP:-" \
			\
		       "$CELL_OBJ_ACLS
			user:$ADMINPRIN:rwdtc
			group:$CICS_AGROUP:rwdtc
			group:$CICS_UGROUP:rt
			group:$CICS_SGROUP:-
			group:$CICS_LGROUP:-"

    # PPC/GATEWAY
    CreateSingleCdsDir "$CICS_CDS_ROOT/ppc/gateway" \
		       "$CELL_DIR_ACLS
			user:$ADMINPRIN:rwdtcia
			group:$CICS_AGROUP:rwdtcia
			group:$CICS_UGROUP:rt
			group:$CICS_SGROUP:-
			group:$CICS_LGROUP:-" \
			\
		       "$CELL_DIR_ACLS
			user:$ADMINPRIN:rwdtcia
			group:$CICS_AGROUP:rwdtcia
			group:$CICS_UGROUP:rt
			group:$CICS_SGROUP:-
			group:$CICS_LGROUP:-" \
			\
		       "$CELL_OBJ_ACLS
			user:$ADMINPRIN:rwdtc
			group:$CICS_AGROUP:rwdtc
			group:$CICS_UGROUP:rt
			group:$CICS_SGROUP:-
			group:$CICS_LGROUP:-"


    # TRPC directory
    # cics and sfs needs full access because they use trpc
    # log needs none
    CreateSingleCdsDir "$CICS_CDS_ROOT/trpc" \
		       "$CELL_DIR_ACLS
			user:$ADMINPRIN:rwdtcia
			group:$CICS_AGROUP:rwdtcia
			group:$CICS_UGROUP:rwdtcia
			group:$CICS_SGROUP:rwdtcia
			group:$CICS_LGROUP:-" \
			\
		       "$CELL_DIR_ACLS
			user:$ADMINPRIN:rwdtcia
			group:$CICS_AGROUP:rwdtcia
			group:$CICS_UGROUP:rt
			group:$CICS_SGROUP:rt
			group:$CICS_LGROUP:-" \
			\
		       "$CELL_OBJ_ACLS
			user:$ADMINPRIN:rwdtc
			group:$CICS_AGROUP:rwdtc
			group:$CICS_UGROUP:rwdtc
			group:$CICS_SGROUP:rwdtc
			group:$CICS_LGROUP:-"


    # TMXA directory
    # cics needs full access, log and sfs do not need any
    CreateSingleCdsDir "$CICS_CDS_ROOT/tmxa" \
			"$CELL_DIR_ACLS
			user:$ADMINPRIN:rwdtcia
			group:$CICS_AGROUP:rwdtcia
			group:$CICS_UGROUP:rwdtcia
			group:$CICS_SGROUP:-
			group:$CICS_LGROUP:-" \
			\
		       "$CELL_DIR_ACLS
			user:$ADMINPRIN:rwdtcia
			group:$CICS_AGROUP:rwdtcia
			group:$CICS_UGROUP:rt
			group:$CICS_SGROUP:-
			group:$CICS_LGROUP:-" \
			\
		       "$CELL_OBJ_ACLS
			user:$ADMINPRIN:rwdtc
			group:$CICS_AGROUP:rwdtc
			group:$CICS_UGROUP:rwdtc
			group:$CICS_SGROUP:-
			group:$CICS_LGROUP:-"


    # PROFILE directory
    # all regions need full permissions, log and sfs have no interest
    CreateSingleCdsDir "$CICS_CDS_ROOT/profile" \
		       "$CELL_DIR_ACLS
			user:$ADMINPRIN:rwdtcia
			group:$CICS_AGROUP:rwdtcia
			group:$CICS_UGROUP:rwdtcia
			group:$CICS_SGROUP:-
			group:$CICS_LGROUP:-" \
			\
		       "$CELL_DIR_ACLS
			user:$ADMINPRIN:rwdtcia
			group:$CICS_AGROUP:rwdtcia
			group:$CICS_UGROUP:rt
			group:$CICS_SGROUP:-
			group:$CICS_LGROUP:-" \
			\
		       "$CELL_OBJ_ACLS
			user:$ADMINPRIN:rwdtc
			group:$CICS_AGROUP:rwdtc
			group:$CICS_UGROUP:rwdtc
			group:$CICS_SGROUP:-
			group:$CICS_LGROUP:-"


    # SFS directory
    # sfsservers need full permissions, cics needs to find sfsservers
    # log has no need to access
    CreateSingleCdsDir "$CICS_CDS_ROOT/sfs" \
		       "$CELL_DIR_ACLS
			user:$ADMINPRIN:rwdtcia
			group:$CICS_AGROUP:rwdtcia
			group:$CICS_UGROUP:rt
			group:$CICS_SGROUP:rwtdcia
			group:$CICS_LGROUP:-" \
			\
		       "$CELL_DIR_ACLS
			user:$ADMINPRIN:rwdtcia
			group:$CICS_AGROUP:rwdtcia
			group:$CICS_UGROUP:rt
			group:$CICS_SGROUP:rwdtcia
			group:$CICS_LGROUP:-" \
			\
		       "$CELL_OBJ_ACLS
			user:$ADMINPRIN:rwdtc
			group:$CICS_AGROUP:rwdtc
			group:$CICS_UGROUP:rt
			group:$CICS_SGROUP:rwdtc
			group:$CICS_LGROUP:-"


    # LOG directory
    # logservers need full permissions, sfs and cics
    # need to find logservers.
    CreateSingleCdsDir "$CICS_CDS_ROOT/log" \
		       "$CELL_DIR_ACLS
			user:$ADMINPRIN:rwdtcia
			group:$CICS_AGROUP:rwdtcia
			group:$CICS_UGROUP:rt
			group:$CICS_SGROUP:rt
			group:$CICS_LGROUP:rwdtcia" \
			\
		       "$CELL_DIR_ACLS
			user:$ADMINPRIN:rwdtcia
			group:$CICS_AGROUP:rwdtcia
			group:$CICS_UGROUP:rt
			group:$CICS_SGROUP:rt
			group:$CICS_LGROUP:rwdtcia" \
			\
		       "$CELL_OBJ_ACLS
			user:$ADMINPRIN:rwdtc
			group:$CICS_AGROUP:rwdtc
			group:$CICS_UGROUP:rt
			group:$CICS_SGROUP:rt
			group:$CICS_LGROUP:rwdtc"


     # check to see if the group cics_regions exists, so get a list from the
     # registry
     GROUP_LIST=$( RgyEdit -g -v )
     if [ $? -ne 0 ]
     then
	# cannot even list registry - something badly wrong
	# display RgyEdit errors from stdout
	print "$GROUP_LIST"
	DispMessage $REGRM_MSN_094 ERZ46177E
	return 1
     fi

     # check for our cics_regions in list produced
     print "$GROUP_LIST" | grep "^$CICS_RGROUP " >$NULLFILE
     if [ $? -ne 0 ]
     then
	# need to create it
	GROUP_LIST=$( RgyEdit -g -update << ENDFILE
	add $CICS_RGROUP
	quit
ENDFILE
	)
	if [ $? -ne 0 ]
	then
	   # failed to create it - display warning and continue
	   print "$GROUP_LIST"
	   DispMessage $REGRM_MSN_095 ERZ46200W $CICS_RGROUP
	fi
     fi

}

# =============================================================================
# FUNCTION: CreateSingleCdsDir
#
# This function creates a single CDS directory entry.
#
# Parameters:
#
#    directory to create, fully specified
#    acl for the directory
#    initial container acl for the directory
#    initial object acl for the directory
#
#	 if any of the acl parameters are null, then that acl will
#	 not be adjusted from the value it is assigned initially by
#	 DCE. ie. the initial object and container acls of the parent
#	 directory will determine the values that are assigned.
#
# =============================================================================
CreateSingleCdsDir()
{
    typeset -r DIR=$1	     # directory to create
    typeset -r PARENT_DIR=$(dirname $DIR)
				  # parent of that directory
    typeset -r DIR_ACL=$2	 # acl for directory
    typeset -r DIR_IC_ACL=$3      # initial container acl
    typeset -r DIR_IO_ACL=$4      # initial object acl
    typeset -r ACL_ACTION="-s"    # substitute existing acls

    typeset CDS_OUT	     # result of cdscp
    typeset ACL_OUT	       # result of acl_edit


    # Check whether directory already exists. This requires
    # that we have read permissions to the parent of the directory
    # being created.
    #
    # First check that we can list the contents of parent
    #
    CDS_OUT=$(cdscp list dir "$PARENT_DIR/*" 2>$NULLFILE)
    if [ $? -ne 0 ]
    then
	if [ "$PARENT_DIR" = "/.:" ]
	then
	    # if we cannot read the root dir, then we are
	    # probably not using the right principal, give up
	    DispMessage $REGRM_MSN_10 ERZ46162E $PARENT_DIR
	    return 1
	else
	    # cannot list it, the permissions are not as we expect
	    # so do not try to go any further for this directory
	    DispMessage $REGRM_MSN_11 ERZ46178W $DIR
	    return 1
	fi
    fi

    # now see if the directory is amongst the entries
    print "$CDS_OUT" | grep "^$(basename ${DIR})$" >$NULLFILE 2>&1
    if [ $? -eq 0 ]
    then
	# already there
	return 0
    else
	CDS_OUT=$( cdscp create dir $DIR 2>$NULLFILE)
	if [ $? -ne 0 ]
	then
	    print "$CDS_OUT"
	    DispMessage $REGRM_MSN_12 ERZ46179E $DIR
	    return 1
	fi
    fi

    if [ -n "$DIR_ACL" ]
    then

	# errors go to stderr, so just let them come out

	# replace existing acls
	ACL_OUT=$(acl_edit $DIR $ACL_ACTION $DIR_ACL >$NULLFILE)

	if [ $? -ne 0 ]
	then
	    # error
	    print "$ACL_OUT"
	    DispMessage $REGRM_MSN_13 ERZ46180E $DIR
	    return 1
	fi
    fi

    if [ -n "$DIR_IC_ACL" ]
    then

	# errors go to stderr, so just let them come out

	# replace existing acls
	ACL_OUT=$(acl_edit $DIR -ic $ACL_ACTION $DIR_IC_ACL >$NULLFILE)

	if [ $? -ne 0 ]
	then
	    # error
	    print "$ACL_OUT"
	    DispMessage $REGRM_MSN_14 ERZ46180E $DIR
	    return 1
	fi
    fi

    if [ -n "$DIR_IO_ACL" ]
    then

	# errors go to stderr, so just let them come out

	# replace existing acls
	ACL_OUT=$(acl_edit $DIR -io $ACL_ACTION $DIR_IO_ACL >$NULLFILE)

	if [ $? -ne 0 ]
	then
	    # error
	    print "$ACL_OUT"
	    DispMessage $REGRM_MSN_15 ERZ46180E $DIR
	    return 1
	fi
    fi
    return 0

}

# =============================================================================
# FUNCTION: SetCICSUnauthenticatedAccess
#
# This function modifies the ACLs for the /.:/cics/profile and for all CICS
# region directories and ts entries to allow anyone to read them.  This will
# enable even unauthenticated users to connect to regions if their protection
# level is none.
# =============================================================================
SetCICSUnauthenticatedAccess()
{
    typeset TSENTRY

    #
    # Tell user that we are starting a potentially long operation
    #
    DispMessage $REGRM_MSN_117 ERZ46275I 

    #
    # Set the initial object and object acls for the /.:/cics/profile container
    # and the object acl for the /.:/cics/profile/cics_profile profile.
    # This will mean that even if the profile is completely recreated it
    # will still have the correct attributes.
    #
    SetEntryUnauthenticatedAccess "" "$CICS_CDS_ROOT/profile" "-io"
    SetEntryUnauthenticatedAccess "-e" "$CICS_CDS_ROOT/profile" ""
    SetEntryUnauthenticatedAccess "-e" "$CICS_CDS_ROOT/profile/cics_profile" ""

    #
    # Set the initial object, initial container and object acls for the 
    # /.:/cics container.  This will cause subsequently created region 
    # containers to have the correct acls.
    #
    SetEntryUnauthenticatedAccess "" "$CICS_CDS_ROOT" "-io"
    SetEntryUnauthenticatedAccess "" "$CICS_CDS_ROOT" "-ic"
    SetEntryUnauthenticatedAccess "-e" "$CICS_CDS_ROOT" ""

    #
    # For all containers under /.:/cics, if they have a ts object within them,
    # set the initial object acl and object acl for the container and the
    # object acl for the ts object.  This will allow any user to read the 
    # entries for regions.
    #
    for RGNENTRY in $(cdscp list dir "$CICS_CDS_ROOT/*" \
		      | grep "^[^ 	]" 2>$NULLFILE)
    do
        TSENTRY=$(cdscp list object "$CICS_CDS_ROOT/$RGNENTRY/*" \
		  | grep "^ts$" 2>$NULLFILE)
	if [ "$TSENTRY" != "" ]
	then
            SetEntryUnauthenticatedAccess "" "$CICS_CDS_ROOT/$RGNENTRY" "-io"
            SetEntryUnauthenticatedAccess "-e" "$CICS_CDS_ROOT/$RGNENTRY" ""
            SetEntryUnauthenticatedAccess \
				"-e" "$CICS_CDS_ROOT/$RGNENTRY/$TSENTRY" ""
	fi
    done

    #
    # Tell user that we have finished
    #
    DispMessage $REGRM_MSN_118 ERZ46276I 
}

# =============================================================================
# FUNCTION: SetEntryUnauthenticatedAccess
#
# This function modifies the ACLs for a single CDS entry
# =============================================================================
SetEntryUnauthenticatedAccess()
{
    typeset -r ACLACTION=$1	     # whether attribute is for CDS item
    typeset -r ENTRYNAME=$2	     # entry to allow unauthenticated access
    typeset -r ACLCLASS=$3	     # class of acl to modify (io, ic)
    typeset ACL_OUT	       	     # result of acl_edit

    ACL_OUT=$(acl_edit $ACLACTION $ENTRYNAME $ACLCLASS << ENDFILE
	      m unauthenticated:rt
	      m any_other:rt
	      commit
	      exit
ENDFILE )

    if [ $? -ne 0 ]
    then
	# error
	DispMessage $REGRM_MSN_116 ERZ46180E $ENTRYNAME
	return 1
    fi
}

# =============================================================================
# FUNCTION: MigrateDCERegions
#
# Create list of CICS Regions on this machine
# For each region, call MigrateRegion
# =============================================================================
MigrateDCERegions()
{
    typeset REGIONLIST
    # Create list of regions on this machine


    REGIONLIST=$($BINDIR/cicslist | cut -f1)
    [ $? -ne 0 ] && DispMessage $REGRM_MSN_74 ERZ46163E && return 0


    for NEXTINLIST in $REGIONLIST
    do

	MigrateRegion $NEXTINLIST
	[ $? -ne 0 ] && DispMessage $REGRM_MSN_75 ERZ46164E $NEXTINLIST

	# How about alternative msg "Migration of reg $RN succeeded " ?
    done

    return 0

}

# =============================================================================
# FUNCTION: MigrateDCESfs
#
# Create list of SFS Servers on this machine
# For each sfs server, call MigrateSfsServer
# =============================================================================
MigrateDCESfs()
{
    # Create list of sfsservers on this machine

    SFSLIST=$($BINDIR/cicsget -c ssd -l | cut -f1 -d " ")
    [ $? -ne 0 ] && DispMessage $REGRM_MSN_79 ERZ46168E && return 0


    # For each sfsserver,
    #   if not already done,
    #   call MigrateSfsServer(Servername)
    #   if bad return
    #       log that migration of this server failed

    for OLDSFSNAME in $SFSLIST
    do
	# create new name by changing "cics6000" to "cics" in old name
	NEWSFSNAME=$(echo $OLDSFSNAME | sed "s/\/.:\/cics6000/\/.:\/cics/")


	MigrateSfsServer $OLDSFSNAME $NEWSFSNAME
	[ $? -ne 0 ] && DispMessage $REGRM_MSN_80 ERZ46169E $OLDSFSNAME
    done

    return 0
}

# =============================================================================
# FUNCTION: MigrateDCESfsSRC
#
# Create list of SFS Servers on this machine
# Change the SRC notify entry for each sfs server to remove the v1.1 directory
# dependency.
# =============================================================================
MigrateDCESfsSRC()
{
    # Create list of sfsservers on this machine

    SFSLIST=$($BINDIR/cicsget -c ssd -l | cut -f1 -d " ")
    [ $? -ne 0 ] && DispMessage $REGRM_MSN_103 ERZ46168E && return 0


    # For each sfsserver,
    #   use cicsget to get the short name
    #   change the SRC database entry using the short name
    #   if bad return
    #	   issue msg that migration of named sfs server SRC failed

    for SFSNAME in $SFSLIST
    do
        echo ""
        echo "Migrating SRC database for sfs server $SFSNAME to have new notify path of $BINDIR/cicsnotify"
        SHORTNAME=$($BINDIR/cicsget -c ssd $SFSNAME | grep ShortName | cut -f2 -d '"')  
        rmnotify -n cicssfs.$SHORTNAME
        mknotify -n cicssfs.$SHORTNAME -m $BINDIR/cicssfsnotify

	[ $? -ne 0 ] && DispMessage $REGRM_MSN_104 ERZ46169E $SFSNAME   \
           && return 1
    done

    return 0
}


# =============================================================================
# FUNCTION: MigrateSfsPathSRC
#
# Create list of SFS Servers on this machine
# Change the SRC entry to change the program path for the sfs program from
# the old location to the current location. 
# =============================================================================
MigrateSfsPathSRC()
{
    #  Create list of sfsservers on this machine

    SFSLIST=$($BINDIR/cicsget -c ssd -l | cut -f1 -d " ")
    [ $? -ne 0 ] && DispMessage $REGRM_MSN_79 ERZ46168E && return 0


    # For each sfsserver,
    #   if not already done,
    #   change the SRC database to use the correct path for sfs 
    #   if bad return
    #       log that migration of this server failed

    for SFSNAME in $SFSLIST
    do
        echo ""
        echo "Migrating SRC database for sfs server $SFSNAME to have new path for sfs"
        SHORTNAME=$($BINDIR/cicsget -c ssd $SFSNAME | grep ShortName | cut -f2 -d '"')
        chssys -s cicssfs.$SHORTNAME -p $(whence sfs)

        [ $? -ne 0 ] && DispMessage $REGRM_MSN_104 ERZ46169E $SFSNAME \
	&& return 1
    done

    return 0

}


# =============================================================================
# FUNCTION: MigrateRegion
#
# Create new DCE and CDS entities
# Delete old DCE and CDS entities
# Change stanza entries
#
# =============================================================================
MigrateRegion()
{
    typeset -r REGIONNAME=$1
	typeset DCE_ORG=${CICS_DCE_ORGANISATION:-"none"}

    typeset REGION=""		   # dir of the imported region
    typeset REGION_KEYTAB_FILE=""	       # region keytab file
    typeset TMP_KTLIST=""
    typeset -i RET=0

    # CDS/Security Variables
    typeset     NEW_REGION_PRINCIPAL
    typeset     OLD_REGION_PRINCIPAL
    typeset     OWNRINFO
    typeset     CDSINFO
    typeset     ACCTINFO
    typeset     PRININFO
    typeset     SRCINFO

    typeset _CREATEOUTPUT1=""
    typeset _CREATEOUTPUT2=""
    typeset _CREATEOUTPUT3=""

    typeset OLD_CDS_DIR
    typeset OLD_TRPC_DIR

    #
    #  Setup & Checks for Region Migration
    #


    REGION=`$BINDIR/cicsname -r$REGIONNAME regiondir`   # Dir of the region
    RET=$?

    [ $RET -eq 5 ] && DispMessage $REGRM_MSN_16 ERZ4681E $REGIONNAME && return 1
    [ -z "$REGION" ] && DispMessage $REGRM_MSN_17 ERZ46105E $REGIONNAME && return 1


    # if the region exists check that it is not still running.
    if [ -d "$REGION" ]
    then
	$BINDIR/cicsgpid -r "$REGIONNAME" > /dev/null 2>&1
	case $? in
	[02346]) DispMessage $REGRM_MSN_18 ERZ4647E $REGIONNAME
		 return 1
	;;
	esac
    fi

    #  initialise CDS variables

    NEW_REGION_PRINCIPAL="cics/$REGIONNAME"

    OLD_REGION_PRINCIPAL="cics6000/$REGIONNAME"

    OLD_CDS_DIR=$OLD_REGION_PRINCIPAL

    OLD_TRPC_DIR="$OLD_CDS_DIR/trpc"

    REGION_KEYTAB_FILE="$REGION/keytab"

    CDSINFO=$(</dev/null cdscp show dir /.:/cics/$REGIONNAME | grep "MyName =")

    ACCTINFO=$(</dev/null RgyEdit -a -v | grep "${NEW_REGION_PRINCIPAL}[   ]")

    PRININFO=$(</dev/null RgyEdit -p -v | grep "${NEW_REGION_PRINCIPAL}[ 	]")

    SRCINFO=$(lssrc -s "cics.$REGIONNAME")

    [ $? -ne 0 ] && DispMessage $REGRM_MSN_82 ERZ46171E $REGIONNAME

    #  Check for existence of new principal.  If it exists, migration of this
    #  region cannot continue.  It may indicate that the region has already
    #  been migrated.  If it hasn't been migrated, the user must delete
    #  this principal and try again.
    #

    if [ ! -z "$PRININFO" ]
    then
	DispMessage $REGRM_MSN_19 ERZ4648E $REGIONNAME
	return 1
    fi

    #
    #  Configure new DCE and CDS entries for region
    #
    #
    # Create the region cds tree if it doesn't exist.
    # (If the actual creation of the region node fails then
    # output all the messages resulting from the tree creation to the
    # user.)
    #
    if [ -z "$CDSINFO" ]
    then
	# /.:/cics should already exist, attempt a create just in case
	# it does not. Any failure other than already exists will be caught
	# by the real create request.
	_CREATEOUTPUT1=$(cdscp create dir /.:/cics 2>&1 )

	_CREATEOUTPUT2=$(cdscp create dir /.:/cics/$REGIONNAME 2>&1 )
	[ $? -eq 0 ] && \
	  _CREATEOUTPUT3=$(cdscp create dir /.:/cics/$REGIONNAME/trpc 2>&1 )
	if [ $? -ne 0 ]    # CDS dir creates failed
	then
	    echo "$_CREATEOUTPUT2"
	    [ -n "$_CREATEOUTPUT3" ] && echo "$_CREATEOUTPUT3"
	    DispMessage $REGRM_MSN_20 ERZ4670E $REGIONNAME
	    return 1
	fi
    fi

    #
    # Add the New Region Principal entry into the DCE Registry
    #
    RgyAddPrincipal -p $NEW_REGION_PRINCIPAL -u cics

    # Add the Region Account entry into the registry

    RgyEdit -update << ENDFILE
    domain account
    add $NEW_REGION_PRINCIPAL -g none -o $DCE_ORG -m "security info for $REGIONNAME" -h $REGION -s /bin/ksh -x none -pw $REGIONNAME -mp $ADMLG_DCE_PASSWORD
    quit
ENDFILE

    RgyEdit -update << ENDFILE
    domain account
    ktadd -p $NEW_REGION_PRINCIPAL -pw $REGIONNAME -f $REGION_KEYTAB_FILE
    ktadd -p $NEW_REGION_PRINCIPAL -r -a -f $REGION_KEYTAB_FILE
    quit
ENDFILE

    #Make keytab owned by cics (region's effective uid) with setuid set
    chown cics $REGION_KEYTAB_FILE && chmod u+s $REGION_KEYTAB_FILE
# Following message is wrong - a defect is being raised.
    [ $? -ne 0 ] && DispMessage $REGRM_MSN_21 ERZ46104W $REGION_KEYTAB_FILE

    (RgyEdit -p -v "${NEW_REGION_PRINCIPAL}" | egrep -s "^${NEW_REGION_PRINCIPAL}[ 	]") || DispMessage $REGRM_MSN_22 ERZ4671E $REGIONNAME
    (RgyEdit -a -v "${NEW_REGION_PRINCIPAL}" | egrep -s "^${NEW_REGION_PRINCIPAL}[ 	]") || DispMessage $REGRM_MSN_23 ERZ4672E $REGIONNAME

    # Check that Keytab entry exists, ERZ46255I if not
    #
    TMP_KTLIST=$(RgyEdit << ENDFILE
	     domain account
	     ktlist -f $REGION_KEYTAB_FILE
	     quit
ENDFILE )

    echo "$TMP_KTLIST" | grep "${CELLNAME}/${NEW_REGION_PRINCIPAL}[ 	]" 1>/dev/null 2>&1 || DispMessage $REGRM_MSN_24 ERZ46255I $REGIONNAME


    # Add the new region principal to the cics group and cics_regions group
    # regardless of whether we created it here

    RgyEdit -g -update << ENDFILE
    member ${CICS_UGROUP} -a ${NEW_REGION_PRINCIPAL}
    member ${CICS_RGROUP} -a ${NEW_REGION_PRINCIPAL}
    exit
ENDFILE
    #
    # check that the principal has been added to cicsgroup
    #
    TMP_KTLIST=$(RgyEdit -g << ENDFILE
	     domain group
	     view ${CICS_UGROUP} -m
	     quit
ENDFILE )

    echo "$TMP_KTLIST" | egrep " ${NEW_REGION_PRINCIPAL}((,)|(\$))"  1>/dev/null 2>&1 || DispMessage $REGRM_MSN_25 ERZ4679W ${NEW_REGION_PRINCIPAL} ${CICS_UGROUP}

    #
    # check that the principal has been added to cics_regions group
    #
    TMP_KTLIST=$(RgyEdit -g << ENDFILE
	     domain group
	     view ${CICS_RGROUP} -m
	     quit
ENDFILE )

    echo "$TMP_KTLIST" | egrep " ${NEW_REGION_PRINCIPAL}((,)|(\$))"  1>/dev/null 2>&1 || DispMessage $REGRM_MSN_26 ERZ4679W ${NEW_REGION_PRINCIPAL} ${CICS_RGROUP}

    #
    # Change the acls for the cds region tree
    #
    acl_edit /.:/cics/$REGIONNAME << ENDFILE
m user:$NEW_REGION_PRINCIPAL:rwtdcia
commit
exit
ENDFILE

    acl_edit /.:/cics/$REGIONNAME/trpc << ENDFILE
m user:$NEW_REGION_PRINCIPAL:rwtdcia
commit
exit
ENDFILE

#
# Delete old DCE data associated with the region
#

    # Call cdscp to delete the old region/trpc CDS directory, including any
    # objects in the directory

    # Call cdscp to delete the old region CDS directory, and any objects in the
    # directory, providing that the trpc directory was successfully deleted

    for DIR in $CELLNAME/$OLD_TRPC_DIR $CELLNAME/$OLD_CDS_DIR
    do
	cdscp show directory $DIR >>$T 2>>$T
	ERRCODE=$?
	if [ $ERRCODE -eq 0 ]
	then
	    for OBJECT in `cdscp list object $DIR/* 2>>$T | \
		grep -v "^ "`
	    do
		if [ -n "$OBJECT" ]
		then
		    cdscp delete object $DIR/$OBJECT >>$T 2>>$T
		    ERRCODE=$?
		    if [ $ERRCODE -ne 0 ]
		    then
			DispMessage $REGRM_MSN_27 ERZ46190W "$DIR/$OBJECT" $ERRCODE
		    fi
		fi
	    done
	    cdscp delete directory $DIR >>$T 2>>$T
	    ERRCODE=$?
	    if [ $ERRCODE -ne 0 ]
	    then
		DispMessage $REGRM_MSN_28 ERZ46190W $OLD_TRPC_DIR $ERRCODE
	    fi
	else
	    DispMessage $REGRM_MSN_30 ERZ46199W $OLD_TRPC_DIR $ERRCODE
	fi
    done

    # Call RgyEdit to delete the old region principal


    RgyEdit 1>>$T 2>>$T << VIEWPRIN
	domain principal
	view $OLD_REGION_PRINCIPAL
	quit
VIEWPRIN
    ERRCODE=$?
    if [ $ERRCODE -eq 0 ]
    then
	RgyEdit -update 1>>$T 2>>$T << DELPRIN
	    domain principal
	    delete $OLD_REGION_PRINCIPAL
	    quit
DELPRIN
	ERRCODE=$?
	if [ ERRCODE -ne 0 ]
	then
	    DispMessage $REGRM_MSN_29 ERZ46191W $ERRCODE
	fi
    fi

    #
    # Update region stanza information with cicsupdateclass
    #

    cicsupdateclass -r $REGIONNAME -c rd -w -a LocalQServer -s /\\/.:\\/cics6000/\\/.:\\/cics/
    cicsupdateclass -r $REGIONNAME -c cd -w -a DCECell -s /\\/.:\\/cics6000/\\/.:\\/cics/
    cicsupdateclass -r $REGIONNAME -c fd -w -a FileServer -s /\\/.:\\/cics6000/\\/.:\\/cics/

    return 0

}


# =============================================================================
# FUNCTION: MigrateSfsServer
#
# Create new DCE and CDS entities
# Delete old DCE and CDS entities
# Change stanza entries
#
# (stuff pinched from 'cicssfscreate', cicssfsdestroy'
# =============================================================================
MigrateSfsServer()
{


    unset OLD_SFS_LOCKED
    unset NEW_SFS_LOCKED
    unset VERBOSE
    unset SFS_SHORTNAME
    unset SFS_USERID
    unset NEW_SFS_SERVER
    unset OLD_SFS_SERVER
    unset SRC_ENTRY				   # SRC entry

    typeset OLD_SFS_LOCKED=0	    # Was cicssfslock used ?
    typeset NEW_SFS_LOCKED=0	    # Was cicssfslock used ?
    typeset VERBOSE=1               # Verbose Mode Enable
    typeset SFS_SHORTNAME	       # Shortname for server
    typeset SFS_USERID		  # Userid from server stanza entry
    typeset NEW_SFS_SERVER	      # New name of server
    typeset OLD_SFS_SERVER	      # Old Name of server
    typeset SRC_ENTRY				   # SRC entry


    OLD_SFS_SERVER=$1
    NEW_SFS_SERVER=$2

    # Check Server names were given
    #

#    [ -z "$OLD_SFS_SERVER" -o $# -ne 1 ] && DispMessage $REGRM_MSN_57 $REGRM_MSN_58
#    [ -z "$OLD_SFS_SERVER" ] && DispMessage $REGRM_MSN_59 $REGRM_MSN_60

#    [ -z "$NEW_SFS_SERVER" -o $# -ne 1 ] && DispMessage $REGRM_MSN_61 $REGRM_MSN_62
#    [ -z "$NEW_SFS_SERVER" ] && DispMessage $REGRM_MSN_63 $REGRM_MSN_64

    typeset OLD_SFS_SERVER_DIR=$(WhatServerDir SSD $OLD_SFS_SERVER)
    typeset NEW_SFS_SERVER_DIR=$(WhatServerDir SSD $NEW_SFS_SERVER)

    #
    # Lock the servers.
    #
    $BINDIR/cicssfslock -t $OLD_SFS_SERVER
    [ $? -ne 0 ] && DispMessage $REGRM_MSN_65 ERZ46183E $OLD_SFS_SERVER && return 1
    OLD_SFS_LOCKED=1

    #
    # Make the new sfs servers directory.
    #

    GeneralMkDir $NEW_SFS_SERVER_DIR $VERBOSE || return 1

    # call cicssfslock -t for new server name

    $BINDIR/cicssfslock -t $NEW_SFS_SERVER
    [ $? -ne 0 ] && DispMessage $REGRM_MSN_66 ERZ46183E $NEW_SFS_SERVER && return 1
    NEW_SFS_LOCKED=1

    # IF server is runnin

    OUTPUT=$($BINDIR/cicspingserver $OLD_SFS_SERVER 2>&1)

    [ $? -eq 0 ] && DispMessage $REGRM_MSN_35 ERZ46157E $OLD_SFS_SERVER && return 1

    # Get UserId and ShortName for this server from the server database.
    # Use this information to set up new DCE and SRC entries for the
    # server using the changed server name, and delete the entries for
    # the old name.

    # call cicssfsdbadmin -P to get stanza info

    OUTPUT=$($BINDIR/cicssfsdbadmin -s $OLD_SFS_SERVER -P)

    [ $? -ne 0 ] && echo "$OUTPUT" && DispMessage $REGRM_MSN_67 ERZ46185E && return 1
    #
    # use getopts to parse args
    #
    while getopts :l:s:T:i:f:o:r:V:c:d:u:N:b:R:M:e: Opt $OUTPUT
    do
	# strip quotes from args
	VALUE=$OPTARG
	[ ! -z $VALUE ] && VALUE=`eval echo $VALUE`

	case $Opt in
	s) SFS_SHORTNAME=$VALUE
	   ;;
	u) SFS_USERID=$VALUE
	   ;;
	*) ;;   # Ignore
	esac
    done

    #
    # Check UserId
    #
    GeneralCheckUser E $SFS_USERID
    [ $? -ne 0 ] && DispMessage $REGRM_MSN_68 ERZ46186E $SFS_USERID && return 1

    #
    # Change owner of files in new server dir to servers user id
    #
    GeneralChangeOwner $SFS_USERID $(WhatServerDir SSD $NEW_SFS_SERVER)
    [ $? -ne 0 ] && DispMessage $REGRM_MSN_69 ERZ46187E $SFS_USERID $NEW_SFS_SERVER && return 1


    #
    # Add new DCE entities (principal, account and keytab)
    #
    OUTPUT=$($BINDIR/cicssfsdceadmin -s $NEW_SFS_SERVER -S $SFS_SHORTNAME -u $SFS_USERID -A -I)

    [ $? -ne 0 ] && DispMessage $REGRM_MSN_70 $ERZ46188E $NEW_SFS_SERVER && return 1

    #
    # Destroy old SRC entries
    #
    #  note: Both the old and new SRC entries need to use the same ShortName,
    #  so cannot coexist.  The old SRC entry had to be destroyed before the
    #  new one can be created.
    #

    SRC_ENTRY=cicssfs.$SFS_SHORTNAME

    #
    # Does an SRC entry exist ?
    #
    OUTPUT=$(lssrc -s $SRC_ENTRY 2>&1)

    if [ $? -ne 0 ]
    then
	DispMessage $REGRM_MSN_36 ERZ46158W $SRC_ENTRY
    else
	#
	#     call cicssrcdestroy -s to delete SRC entry
	#

	OUTPUT=$($BINDIR/cicssrcdestroy -s $SFS_SHORTNAME 2>&1)
	[ $? -eq 0 ] &&
	   DispMessage $REGRM_MSN_37 ERZ46159I $OLD_SFS_SERVER

    fi

    # Create SRC entry for new name
    #  (try this even if destroy of old one failed)

    OUTPUT=$($BINDIR/cicssrccreate -s $SFS_SHORTNAME -n $NEW_SFS_SERVER -u $SFS_USERID -I)

    # But give up on this server if SRC create fails for other reason

    [ $? -ne 0 ] && echo "$OUTPUT" && DispMessage $REGRM_MSN_71 ERZ46189E $SFS_SHORTNAME $NEW_SFS_SERVER && return 1

    # Migrate stanzas using cicsupdateclass
    #

    SFS_KEY=$(echo $OLD_SFS_SERVER | sed "s/\/.:\/cics6000/\/.\;\/cics6000/")

    # change "cics6000" to "cics" in entry for this server in sfs stanzas

    cicsupdateclass -c ssd -w -x -k $SFS_KEY -s "/\\/.;\\/cics6000/\\/.;\\/cics/"

    # Check return - if bad, migration of sfsserver failed.
    [ $? -ne 0 ] && DispMessage REGRM_MSN_89 ERZ46193E $OLD_SFS_SERVER && return 1

    #
    # Remove old DCE entities (principal, account and keytab entries)
    #
    OUTPUT=$($BINDIR/cicssfsdceadmin -I -s $OLD_SFS_SERVER -S $SFS_SHORTNAME -D)

    # Don't exit if this fails
    [ $? -ne 0 ] && echo "$OUTPUT" && DispMessage $REGRM_MSN_72 ERZ46192W $OLD_SFS_SERVER

    #
    # call cicssfslock -u to unlock both server names
    #

    $BINDIR/cicssfslock -u $OLD_SFS_SERVER
    $BINDIR/cicssfslock -u $NEW_SFS_SERVER


    # Now cleanup the old server directory.
    #
    # There may be all sorts of other files in this directory - for example
    # EncinaBacktrace files appear to be written to the UserId home dir.
    # Don't try to clear the other stuff out?

    if [ -d $OLD_SFS_SERVER_DIR ]
    then
	cd $OLD_SFS_SERVER_DIR
	[ -f key ] && rm key >/dev/null 2>&1
	[ -f msg ] && rm msg >/dev/null 2>&1
    fi

}

# =============================================================================
# FUNCTION: MigrateRegionSRC
#
# Change the SRC database entry for the region to remove the v1.1 directory
# dependency.
# =============================================================================
MigrateRegionSRC()
{
    typeset -r REGIONNAME=$1
    typeset -i RET=0

    echo ""
    echo "Migrating SRC database for region $REGIONNAME to have new path"
    echo "    of $BINDIR/cics"
    chssys -s cics.$REGIONNAME -p $BINDIR/cics 
    RET=$?

    [ $RET -ne 0 ] && DispMessage $REGRM_MSN_99 ERZ46171E $REGIONNAME \
       && return 1

    echo ""
    echo "Migrating SRC database for region $REGIONNAME to have new notify"
    echo "    path of $BINDIR/cicsnotify"
    rmnotify -n cics.$REGIONNAME
    mknotify -n cics.$REGIONNAME -m $BINDIR/cicsnotify
    RET=$?

    [ $RET -ne 0 ] && DispMessage $REGRM_MSN_100 ERZ46171E $REGIONNAME \
       && return 1
    echo ""

    return 0
}

# =============================================================================
# FUNCTION: GeneralDCEClean
#
# Cleanup all CDS entitities created by the script
# =============================================================================
GeneralDCEClean()
{
    typeset COMPLETELIST
    typeset SEARCHLIST
    typeset COMPLETELIST
    typeset NEWLIST
    typeset DELDIR
    typeset DIR
    typeset OBJ
    typeset CDSOUTPUT



    # Check that we have something to delete.
    #
    # First check that we can list the contents of /.:/
    #
    CDSOUTPUT=$(cdscp list dir "/.:/*" 2>$NULLFILE)
    if [ $? -ne 0 ]
    then
	DispMessage $REGRM_MSN_84 ERZ46173W
	return 0
    fi

    # now see if the CICS root directory is amongst the entries
    print "$CDSOUTPUT" | grep "^$(basename ${CICS_OLD_CDS_ROOT})$" >$NULLFILE 2>&1
    if [ $? -ne 0 ]
    then
	# no entry there, nothing to delete
	return 0
    fi

    # list of directories to be deleted built top down
    # always includes root
    COMPLETELIST="$CICS_OLD_CDS_ROOT"

    # list of directories at current level - start at root
    SEARCHLIST="$CICS_OLD_CDS_ROOT"

    while [ -n "$SEARCHLIST" ]
    do
	# get list of directories from cds, ignore cdscp headings
	# and prepend current directory to get fully qualified
	# directory names. Errors from cdscp are ignored, if the
	# ACLs do not permit the search then the delete directory
	# command will still be applied to to CICS_OLD_CDS_ROOT, and
	# will fail if that is not empty. Any cdscp errors go to
	# stderr and so will be seen by the user.
	NEWLIST=$(for DIR in $SEARCHLIST
	    do
		for NEWDIR in $(cdscp list dir $DIR/* | grep -v "^[	 ]")
		do
		    if [ -n "NEWDIR" ]
		    then
			print $DIR/$NEWDIR
		    fi
		done
	    done)

	# get ready for next level
	SEARCHLIST=$NEWLIST

	# append to list we're building
	if [ -n "$NEWLIST" ]
	then
	    COMPLETELIST="$COMPLETELIST $NEWLIST"
	fi
    done

    # need to delete from bottom up, so reverse order
    # do it all in one cdscp invocation, but suppress
    # the output, because it includes the print of our
    # commands. Save the output because it includes
    # error text - cdscp uses stdout for errors.
    ( print $COMPLETELIST |
	awk '{for ( i = NF; i > 0; i--) print $i;}'  |
	    while read DELDIR
	    do
		for OBJ in $(cdscp list obj $DELDIR/* | grep -v "^[	 ]")
		do
		    print delete object $DELDIR/$OBJ
		done

		print delete directory $DELDIR

	    done && print exit ) | CDSOUTPUT=$(cdscp 2>&1)

    if [ $? -eq 0 ]
    then
	DispMessage $REGRM_MSN_39 ERZ46181I $PROGNAME
    else
	# if we had an error, display the full
	# set of information that cdscp displayed
	print "$CDSOUTPUT"
	DispMessage $REGRM_MSN_85 ERZ46174W
    fi
    return 0

}

# =============================================================================
# FUNCTION: CreateTempMigrationDir
#
# Create a temporary region directory for cicsmigrate
# =============================================================================
CreateTempMigrationDir()
{
TEMPREGION=$($BINDIR/cicsname -r mig#temp regiondir)
if [ -z "$TEMPREGION" ]
then
  # we cannot find or access the region directory itself
  GeneralDispMsg $REGRM_MSN_90 ERZ46105E "mig#temp"
  exit 1
fi
if [ -d "$TEMPREGION" ]
then
  # the region mig#temp already exists, so we cannot continue
  GeneralDispMsg $REGRM_MSN_91 ERZ46196E "mig#temp"
  exit 1
fi

# Copy from archive of default region.
ARCHIVEFILE=$($BINDIR/cicsname defaultregion)
if [ -z "$ARCHIVEFILE" ]
then
  # could not locate the default region, so we cannot continue
  GeneralDispMsg $REGRM_MSN_92 ERZ4646E
  exit 1
fi

#create temporary directory
mkdir $TEMPREGION 2>/dev/null
[ $? -ne 0 ] && \
    GeneralDispMsg $REGRM_MSN_93 ERZ4629E "$TEMPREGION" && exit 1
cd $TEMPREGION

# copy the default region to the newly-created temporary directory
# ignore symbolic link messages from cpio
eval "cpio -id  < $ARCHIVEFILE" 2>&1 | grep -v -- "0511-010"

# the following code traps a non zero return code from the
# CreateTempMigrationDir function.
# There has been an instance where the /var/cics_regions file
# system has been full, so creating a temp dir has completed
# sucessfully, but then moving the stanzas into this dir fails
# with a non-friendly error message. So, if a non zero return
# code occurs, this code checks how full the file system is
# and outputs a message saying if it is too full and should be
# purged, or that some other errors is causing the failure.

if [ $? -ne 0 ]
then
df . >temp
exec 3< temp
read -u3 first_line
read -u3 a b c d the_rest
FS1=$d
FS2=${FS1%?}
exec 3<&-
rm temp
if [ "$FS2" -gt 95 ]
then
GeneralDispMsg $REGRM_MSN_96 ERZ46201E
exit 1
else
GeneralDispMsg $REGRM_MSN_97 ERZ46202E
exit 1
fi
fi

chown -R cics:cics $TEMPREGION
cd $CURRDIR
return 0
}

# =============================================================================
# FUNCTION: RemoveTempMigrationDir
#
# Removes the temporary migration region directory
# =============================================================================
RemoveTempMigrationDir()
{
if [ -d "$TEMPREGION" ]
then
    rm -rf $TEMPREGION
fi
return 0
}

# =============================================================================
# FUNCTION: CreateClassDir <regionname> <class>
#
# creates, if necessary, the default cold and warm start files for a new class
# =============================================================================
CreateClassDir ()
{
  REGNAME=$1
  CLASSNAME=$2

  DBASEDIR=$($BINDIR/cicsname -r $REGNAME regiondbase)
  if [ -z "$DBASEDIR" ]
  then
   # can't find the database directory - we've no hope
    GeneralDispMsg $REGRM_MSN_98 ERZ46204E $REGNAME
    exit 1
  fi

  CLASSDIR=$DBASEDIR/$CLASSNAME
  if [ ! -d "$CLASSDIR" ]
  then
    mkdir $CLASSDIR
    chmod 770 $CLASSDIR
    chown cics:cics $CLASSDIR
  fi

  FROMDIR=$($BINDIR/cicsname -r mig#temp regiondbase)/$CLASSNAME

  PERMSTANZA=$CLASSDIR/$CLASSNAME.stanza
  if [ ! -a "$PERMSTANZA" ]
  then
    cp $FROMDIR/$CLASSNAME.stanza $PERMSTANZA
    chmod 660 $PERMSTANZA
    chown cics:cics $PERMSTANZA
  fi

  AUTODIR=$CLASSDIR/$CLASSNAME.$REGNAME
  if [ ! -d "$AUTODIR" ]
  then
    mkdir $AUTODIR
    chmod 770 $AUTODIR
    chown cics:cics $AUTODIR
  fi
  AUTOSTANZA=$AUTODIR/$CLASSNAME.auto
  if [ ! -a "$AUTOSTANZA" ]
  then
    echo "#                  DO NOT EDIT THIS FILE!"                                  >$AUTOSTANZA
    echo "# This file contains CICS runtime recovery data."                          >>$AUTOSTANZA
    echo "# Alteration of this file will result in the loss of CICS data integrity." >>$AUTOSTANZA
    cat $PERMSTANZA >>$AUTOSTANZA
    chmod 660 $AUTOSTANZA
    chown cics:cics $AUTOSTANZA
  fi
}

# =============================================================================
# FUNCTION: MigrateEnUSMaps <regionname>
#
# creates, if necessary, a migration of En_US maps to en_US maps
# =============================================================================
MigrateEnUSMaps ()
{
  REGNAME=$1

  BASEDIR=$($BINDIR/cicsname -r $REGNAME regiondir)
  if [ -z "$BASEDIR" ]
  then
   # can't find the region directory - we've no hope
    GeneralDispMsg $REGRM_MSN_119 ERZ46105E $REGNAME
    exit 1
  fi

  MAPSDIR=$BASEDIR/maps
  if [ ! -d "$MAPSDIR" ]
  then
    # create a maps directory if there isnt one
    mkdir $MAPSDIR
    chmod 770 $MAPSDIR
    chown cics:cics $MAPSDIR
  fi

  FROMDIR=$MAPSDIR/En_US
  TODIR=$MAPSDIR/en_US

  if [ -d "$FROMDIR" ]
  then
    if [ ! -d "$TODIR" ]
    # no en_US directory exists so link one to En_US
    then
      ln -s $FROMDIR $TODIR
      if [ $? -ne 0 ]
      then
	DispMessage $REGRM_MSN_120 ERZ4631E $TODIR $FROMDIR
      fi
    else
    # a en_US directory exists - link any files in En_US to en_US
      for INFILE in `ls $FROMDIR`
      do
        FILE=${INFILE##/*/}              # Get just the filename
        OUTFILE=$TODIR/$FILE
        if [ ! -a "$OUTFILE" ]
        then
          ln -s $FROMDIR/$FILE $OUTFILE
          if [ $? -ne 0 ]
          then
    	    DispMessage $REGRM_MSN_122 ERZ4631E $OUTFILE $FROMDIR/$FILE
          fi
        else
    	  DispMessage $REGRM_MSN_123 ERZ4631E $OUTFILE $FROMDIR/$FILE
        fi
      done
    fi
  fi
}

# =============================================================================
# FUNCTION: AddppcgwyACLs 
#
# creates, if necessary, the DCE security group cics_ppcgwy, then adds this 
# group to each of the CDS directories.
# =============================================================================
AddppcgwyACLs ()
{
     typeset GROUP_LIST                # output from RgyEdit list
     typeset ACL_OUT                   # output from acl_edit
     typeset -r ROOTDIR="/.:/cics"     # CDS root directory
     typeset -r TRPCDIR="/.:/cics/trpc" # trpc cds directory
     typeset -r PPCDIR="/.:/cics/ppc"  # ppc cds directory
     typeset -r PPCGWYDIR="/.:/cics/ppc/gateway" # ppc/gateway cds directory

     # check to see if the group cics_ppcgwy exists, so get a list from the
     # registry
     GROUP_LIST=$( RgyEdit -g -v )
     if [ $? -ne 0 ]
     then
        # cannot even list registry - something badly wrong
        # display RgyEdit errors from stdout
        print "$GROUP_LIST"
        DispMessage $REGRM_MSN_105 ERZ46177E
        return 1
     fi

     # check for our cics_ppcgwy in list produced
     print "$GROUP_LIST" | grep "^$CICS_PGROUP " >$NULLFILE
     if [ $? -ne 0 ]
     then
	# need to create it
	GROUP_LIST=$( RgyEdit -g -update << ENDFILE
	add $CICS_PGROUP
	quit
ENDFILE
	)
        if [ $? -ne 0 ]
        then
           # failed to create it - display warning and continue
           print "$GROUP_LIST"
           DispMessage $REGRM_MSN_106 ERZ46200W $CICS_PGROUP
        fi
     fi

    #
    # Change the acls on the cds directories to add the new group cics_ppcgwy
    #
    ACL_OUT=$(acl_edit $ROOTDIR -m group:$CICS_PGROUP:rt >$NULLFILE)
    if [ $? -ne 0 ]
    then
       # failed to add cics_ppcgwy
       print "$ACL_OUT"
       DispMessage $REGRM_MSN_107 ERZ46180E $ROOTDIR
       return 1
    fi

    ACL_OUT=$(acl_edit $TRPCDIR -m group:$CICS_PGROUP:rwdtcia >$NULLFILE)
    if [ $? -ne 0 ]
    then
       # failed to add cics_ppcgwy
       print "$ACL_OUT"
       DispMessage $REGRM_MSN_108 ERZ46180E $TRPCDIR
       return 1
    fi

    ACL_OUT=$(acl_edit $TRPCDIR -io -m group:$CICS_PGROUP:rwdtc >$NULLFILE)
    if [ $? -ne 0 ]
    then
       # failed to add cics_ppcgwy
       print "$ACL_OUT"
       DispMessage $REGRM_MSN_109 ERZ46180E $TRPCDIR
       return 1
    fi

    ACL_OUT=$(acl_edit $PPCDIR -m group:$CICS_PGROUP:rwdtcia >$NULLFILE)
    if [ $? -ne 0 ]
    then
       # failed to add cics_ppcgwy
       print "$ACL_OUT"
       DispMessage $REGRM_MSN_110 ERZ46180E $PPCDIR
       return 1
    fi

    ACL_OUT=$(acl_edit $PPCDIR -io -m group:$CICS_PGROUP:rwdtc >$NULLFILE)
    if [ $? -ne 0 ]
    then
       # failed to add cics_ppcgwy
       print "$ACL_OUT"
       DispMessage $REGRM_MSN_111 ERZ46180E $PPCDIR
       return 1
    fi

    ACL_OUT=$(acl_edit $PPCGWYDIR -m group:$CICS_PGROUP:rwdtcia >$NULLFILE)
    if [ $? -ne 0 ]
    then
       # failed to add cics_ppcgwy
       print "$ACL_OUT"
       DispMessage $REGRM_MSN_112 ERZ46180E $PPCGWYDIR
       return 1
    fi

    ACL_OUT=$(acl_edit $PPCGWYDIR -io -m group:$CICS_PGROUP:rwdtc >$NULLFILE)
    if [ $? -ne 0 ]
    then
       # failed to add cics_ppcgwy
       print "$ACL_OUT"
       DispMessage $REGRM_MSN_113 ERZ46180E $PPCGWYDIR
       return 1
    fi

return 0
}
 
# =============================================================================
# FUNCTION: AddppcgwyACLregion <region>
#
# adds DCE group cics_ppcgwy to the cds directory for the cics region.
# =============================================================================
AddppcgwyACLregion ()
{
     REG=$1

     typeset ACL_OUT                   # output from acl_edit
     typeset -r ROOTDIR="/.:/cics"     # CDS root directory

    ACL_OUT=$(acl_edit $ROOTDIR/$REG -m group:$CICS_PGROUP:rwdtcia >$NULLFILE)
    if [ $? -ne 0 ]
    then
       # failed to add cics_ppcgwy
       print "$ACL_OUT"
       DispMessage $REGRM_MSN_114 ERZ46180E $ROOTDIR/$REG
       return 1
    fi

    ACL_OUT=$(acl_edit $ROOTDIR/$REG -io -m group:$CICS_PGROUP:rwdtc >$NULLFILE)
    if [ $? -ne 0 ]
    then
       # failed to add cics_ppcgwy
       print "$ACL_OUT"
       DispMessage $REGRM_MSN_115 ERZ46180E $ROOTDIR/$REG
       return 1
    fi

return 0
}
