#!/bin/ksh
#
# NAME:		cics_portable.fnc - Provide portability layer for shell scripts
#
# VERSION:	1.5
#
# COPYRIGHT:                                                        
# 
#   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
#   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
#   All Rights Reserved
#   Licensed Materials - Property of IBM
#  
#   US Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# 
#                                                                   
#
# SYNOPSIS:	
#
# CheckCanRun File ...
#
# $CICS_LOG_LV_RAW_PATH
# $CICS_LOG_LV_BLOCK_PATH
# $CICS_SFS_LV_RAW_PATH
# $CICS_SFS_LV_BLOCK_PATH
# $CICS_PPCGWY_LV_RAW_PATH
# $CICS_PPCGWY_LV_BLOCK_PATH
# $CICS_SIGINT
# $CICS_SIGTERM
# $CICS_SIGUSR1
# $CICS_SIGHUP
#
# GetEUid
#
# CompileAndLink [<args>]
#
# lsuser -a {id | home | groups} <username>
#
# ListLV Pathname
#
# startsrc [<args>]
# stopsrc [<args>]
# lssrc [<args>]
# mkssys [<args>]
# rmssys [<args>]
# chssys [<args>]
# mknotify [<args>]
# rmnotify [<args>]
#
# dspmsg [<args>]
#
# dce_login_noexec <principal> <password>
#
# GetServerName <sub-system_name>
#
# GetSubSysStdOut <sub-system_name>
#
# GetRegionName <sub-system_name>
#
# MapLVol <server> <path_to_raw_device>
#
# RgyEdit [<args>]
#
# RgyAddPrincipal [<args>]
#
# CicsDispMsg MSN MSID [<args>]
#
# $GETENCINAPATH
#
# DESCRIPTION:	
# This file contains all shell parameters and functions needed to emulate
# any functionality used by the base CICS shell script code where
# a platform doesn't have native support for that function.
#
# Where a native function is wrapped into a function in this file, the
# native function is aliased to a PortabilityTrap function. This prevents
# future coders from introducing calls to this non-portable native function.
#
# On the HP 9000/800 under HP-UX v9.0 support is required for:
#
#	- Checking for execute permission on programs
#       - Paths to filesystem nodes where logical volumes are kept
#	- Emulating AIX lsuser command
#       - A portable interface to listing logical volume entries
#	- Emulating the AIX Systems Resources Controller (SRC)
#	- Emulating the AIX dspmsg command
#	- Emulating dce_login_noexec which HP-UX DCE doesn't support
#	- A portable means to get the log/sfs server name for an SRC sub-system
#	- A consistent interface for mapping logical volumes to Encina
#	- Obtaining consistent DCE rgy_edit behaviour
#
# CAVEATS/WARNINGS:
# The lsuser emulation only supports a small subsets of the full AIX lsuser
# command options.
#
# ListLV only supports listing a single LV per call.
#		
# RETURNED VALUES:
# CheckCanRun returns 0 if one or more programs cannot be run, otherwise 1.
#
# lsuser returns 0 if the user is valid, otherwise 1.
#
# ListLV returns 0 if the LV exists, otherwise 1.
#
# startsrc, stopsrc, lssrc, mkssys, rmssys, mknotify, rmnotify, chssys: 
# return 0 for success, otherwise 1.
#
# dspmsg returns 1 for a usage error, otherwise 0.
#
# dce_login_noexec returns 0 if logged in ok, otherwise > 0.
#
# GetServerName returns 0 if the query is satisfied or the subsystem is
# undefined, or 1 if a processing error occurs.  This simulates the behaviour
# of AIX's odmget, and is crucial for come callers.
#
# GetSubSysStdOut returns 0 if the query is satisfied or the subsystem is
# undefined, or 1 if a processing error occurs.  This simulates the behaviour
# of AIX's odmget, and is crucial for come callers.
#
# GetRegionName returns 0 if the query is satisfied or the subsystem is
# undefined, or 1 if a processing error occurs.  This simulates the behaviour
# of AIX's odmget, and is crucial for come callers.
#
# MapLVol returns 0 if all is well, >0 otherwise.
#
# RgyEdit returns 0 if all is well, >0 otherwise.
#
# RELATED INFORMATION:
# None.
#
#
#*/

# This must be exported on HP-UX for SFS initialisation to work properly.
export ENCINA_TRPC_AUTHN=1

#------------------------------------------------------------------------
#
# SYNOPSIS: PortabilityTrap
# DESCRIPTION:
# All identified non-portable programs end up in here to trap coders
# injecting porting errors which have already been addressed. They should
# use one of the facilities provided by this file instead.
#

alias rgy_edit=PortabilityTrap
alias lvdisplay=PortabilityTrap

PortabilityTrap()
{
    print -u2 "ASSERT: non-portable command called - see cicsportable.fnc"
    exit 1
}

#------------------------------------------------------------------------
#
# SYNOPSIS: GetEUid
# DESCRIPTION:
# Return the effective userid of the current user
#
alias GetEUid="id -u"

#------------------------------------------------------------------------
#
# SYNOPSIS: CompileAndLink
# DESCRIPTION:
# Perform system specific processing for compiling and linking a program
# for a specified language. Currently C and COBOL are supported. The 1st
# argument passed indicates whether the function should actually execute
# or simply echo the commands required for processing.
#

CompileAndLink()
{

	eval typeset CLType=$1	# type of processing required
	eval typeset CLLang=$2	# language being processed
	
	case ${CLLang} in
	C)	eval typeset 	CLCICS=$3	# CICS library
	 	eval typeset	CLUser=$4	# User library
		eval typeset	CLCC=$5		# CC flags
		eval typeset	CLFile=$6	# file name
		eval typeset	CLLD=$7		# link flags
		eval typeset 	CLIncl=$8	# include libraries

	# -Aa puts the HP compiler in ANSI mode
	# +z  position independence flag
	# -c  only perform a compile - a separate link is required

		CLCC=" -I${CLIncl} ${CLCC} -Aa +z -c"
		COMPILE="cc ${CLCC} ${CLFile}.c"

	# -b tells the linker to create a shared library, rather
	#    than a normal executable.

		LINK="ld -b +e main -o ${CLFile} ${CLFile}.o -L ${CLCICS}"
		LINK="${LINK} ${CLLD} ${CLUser}"

		;;
	COBOL)	eval typeset CLaOPT=$3	# animator option
		eval typeset CLCOBO=$4	# cobol flags
		eval typeset CLFile=$5	# file name
		eval typeset CLuOPT=$6  # snt option
		COMPILE="/opt/cobol/bin/cob ${CLaOPT} ${CLuOPT} ${CLCOBO} ${CLFile}.cbl"
		;;
	esac

	case $CLType in
	e)	case $CLLang in
		C)   echo "${COMPILE} && ${LINK}"
		     ;;
		COBOL) echo "${COMPILE}"
		     ;;
		esac
		return 0
		;;
	x)	case $CLLang in
		C)   ${COMPILE} && ${LINK}
		     ;;
		COBOL) ${COMPILE}
		     ;;
		esac
		return $?
		;;
	esac
}

#------------------------------------------------------------------------
#
# SYNOPSIS: $CICS_LOG_LV_RAW_PATH $CICS_LOG_LV_BLOCK_PATH
#	    $CICS_SFS_LV_RAW_PATH $CICS_SFS_LV_BLOCK_PATH
#	    $CICS_PPCGWY_LV_RAW_PATH $CICS_PPCGWY_LV_BLOCK_PATH
#	    $CICS_SIGINT $CICS_SIGTERM $CICS_USR1 $CICS_HUP
# DESCRIPTION:
# This pair of shell parameters provide a platform independent location for
# the filesystem node where logical volumes are kept. All code wishing to
# check access etc. to an LV should use these variables as the path to the
# LV. Note that checking if an LV exists should be via cicslslv which
# performs sanity checks on the LV.
#
# On HP-UX logical volumes are located within volume groups. CICS supports
# configuring it's Log Server LVs and SFS Server LVs into (optionally)
# seperate VGs. The default is that LVs are in VG "vg00". The CICS_LOG_VG,
# CICS_SFS_VG and CICS_PPCGWY_VG parameters can be specified by the user to 
# override the default VG.
#
# The numeric values associated with signals recognised by CICS changes
# from system to system. Mnemonics do not. These variables map an expected
# signal type to its mnemonic, if supported, or its numeric value.

CICS_LOG_VG=${CICS_LOG_VG:-vg00}
CICS_SFS_VG=${CICS_SFS_VG:-vg00}
CICS_PPCGWY_VG=${CICS_PPCGWY_VG:-vg00}

CICS_LOG_LV_RAW_PATH=/dev/${CICS_LOG_VG}/r
CICS_LOG_LV_BLOCK_PATH=/dev/${CICS_LOG_VG}/
CICS_SFS_LV_RAW_PATH=/dev/${CICS_SFS_VG}/r
CICS_SFS_LV_BLOCK_PATH=/dev/${CICS_SFS_VG}/
CICS_PPCGWY_LV_RAW_PATH=/dev/${CICS_PPCGWY_VG}/r
CICS_PPCGWY_LV_BLOCK_PATH=/dev/${CICS_PPCGWY_VG}/

CICS_SIGINT=INT		# value for SIGINT on this system.
CICS_SIGTERM=TERM	# value for SIGTERM on this system.
CICS_SIGUSR1=USR1	# value for SIGUSR1 on this system
CICS_SIGHUP=HUP		# value for SIGHUP on this system

#------------------------------------------------------------------------
#
# SYNOPSIS: CheckCanRun File ...
# DESCRIPTION:
# Checks caller can run all supplied program names
#
# This function allows the portability layer to map a check for execute
# permission on an emulation function into a check on the underlying program.
# i.e. a check for execution of lssrc is a direct check of the lssrc program
# on AIX, but on HP-UX lssrc is a function which calls cicslssrc so we
# perform the execute check on cicslssrc.
#
# The addition of any emulation functions that are args to CheckCanRun
# calls will require an addition to the Location switch below.
#
CheckCanRun()
{
    typeset File
    typeset Location

    for File in $*
    do
	# whence returns the location of $File in different formats
	# depending on the type of object $File is:
	#
	#     program => path to $File including at least a leading ./
	#     alias => the alias expansion
	#     function => $File itself
	#     undefined, not on PATH, or not executable => nothing
	#
	Location=$(whence $File)	# find the object
	#
	# If we can exec it, check for lack of path element i.e. a function
	#
	if [ -n "$Location" -a "${Location##*/}" = "$Location" ]
	then				# we have an emulation function name
	    case $Location in		# so map to underlying program
	    startsrc|stopsrc|mkssys|rmssys|lssrc|mknotify|rmnotify|chssys)
		File=$BINDIR/cics${0##*/}
		;;
	    *)  print "ASSERT in $0: no lookup for $Location"
		exit 1 ;;
	    esac
	    Location=$(whence $File)	# lookup underlying prog
	fi

	if [ -z "$Location" ]		# can I exec $File or underlying prog?
	then
	    return 1
	fi
    done
    return 0
}

#------------------------------------------------------------------------
#
# SYNOPSIS: lsuser -a {id|home|groups} <username>
# DESCIPTION:
# Emulate -a {id|home|groups} options of AIX lsuser command.
#
lsuser()
{
    if [ $# -ne 3 -o "$1" != "-a" ]
    then
	print "ASSERT in $0: unsupported emulation requested '$*'"
	exit 1
    fi

    typeset Value
    typeset Type=$2
    typeset User=$3

    case $Type in
    id) 
    	Value=$(pwget -n $User | cut -d':' -f3)
	;;
    home)
    	Value=$(pwget -n $User | cut -d':' -f6)
	;;
    groups)
	Value=$(groups $User 2>/dev/null | sed 's/ /,/g')
	;;
    *) print "ASSERT in $0: unsupported emulation requested '$*'"
       exit 1 ;;
    esac

    if [ -z "$Value" ]		# no data indicates the user is undefined
    then
	return 1
    fi

    print "$User $Type=$Value"
    return 0
}

#------------------------------------------------------------------------
#
# SYNOPSIS: ListLV Pathname
# DESCRIPTION:
# List the volume manager definition for a logical volume.
#
# This should be a simple call to lvdisplay, but it doesn't (currently)
# return a non-zero exit status if the LV is undefined. The whole point of
# calling ListLV is to check the LV is defined and a valid LV - we must
# return 0 and 1 for success and failure. So we jump through a few hoops..
#
ListLV()
{
    typeset OneLine
    typeset -i Ret=1			# assume the worst

    \lvdisplay $1 | while read OneLine	# let stderr go out, but capture stdout
    do
	print -- "$OneLine"		# echo output to the caller
	Ret=0				# and set OK as stdout=LV is defined
    done
    return $Ret
}

#------------------------------------------------------------------------
#
# SYNOPSIS: startsrc [<args>]
#	    stopsrc [<args>]
#	    mkssys [<args>]
#	    rmssys [<args>]
#           chssys [<args>]
#	    lssrc [<args>]
#	    mknotify [<args>]
#	    rmnotify [<args>]
# DESCRIPTION:
# Simply map the AIX SRC command name into a cics provided command to
# emulate the AIX function under HP_UX. Allow for alias expansion of
# following command words (hence alias value ends in space).
#
alias startsrc='$BINDIR/cicsstartsrc '
alias stopsrc='$BINDIR/cicsstopsrc '
alias mkssys='$BINDIR/cicsmkssys '
alias rmssys='$BINDIR/cicsrmssys '
alias chssys='$BINDIR/cicschssys '
alias lssrc='$BINDIR/cicslssrc '
alias mknotify='$BINDIR/cicsmknotify '
alias rmnotify='$BINDIR/cicsrmnotify '

#------------------------------------------------------------------------
#
# SYNOPSIS: dspmsg [<args>]
# DESCRIPTION:
# Simply map the AIX dspmsg command name into a cics provided command to
# emulate the AIX function under HP_UX. Allow for alias expansion of
# following command words (hence alias value ends in space).
#
alias dspmsg='$BINDIR/cicsdspmsg '

#------------------------------------------------------------------------
#
# SYNOPSIS: dce_login_noexec <principal> <password>
# DESCRIPTION:
# HP-UX DCE doesn't support dce_login_noexec so emulate it via dce_login.
#
dce_login_noexec()
{
    dce_login "$@" <<EOF
	echo \$KRB5CCNAME
	exit
EOF
    return $?
}

#------------------------------------------------------------------------
#
# SYNOPSIS: GetServerName <sub-system_name>
# DESCRIPTION:
# Obtain the log/sfs server name for a given SRC sub-system.
#
GetServerName()
{
    typeset    DBEntry
    typeset    ArgsField
    typeset -i ExitCode

    . $LIBDIR/cicssrc.fnc		# rarely needed so dot here

    trap "" $CICS_TRAP_SIGS		# Avoid signal termination w/o unlock

    CICS_SRCDBLock			# Lock the cicssrc database
    DBEntry="$(CICS_SRCDBGet $1)"	# Query by subsys name
    ExitCode=$?				# Record exit code
    if [ $ExitCode -ne 0 ]		# DBGet error?
    then
	CICS_SRCDBUnlock		# Unlock the cicssrc database
	[ $ExitCode -eq 1 ] && return 0	# If non-existent subsys, return 0
	return 1			# No, query processing error; return 1
    fi
    CICS_SRCDBUnlock			# Unlock the cicssrc database

					# get server name from ARGS field
    ArgsField="$( echo "$DBEntry" | CICS_SRCDBProject $SSYS_ARGS_FLD )"
    print -- $(expr "${ArgsField}" : '-n[ ]*\(.*\)')
    return 0
}

#------------------------------------------------------------------------
#
# SYNOPSIS: GetSubSysStdOut <sub-system_name>
# DESCRIPTION:
# Obtain the path for standard output associated with a given SRC 
# sub-system.
#
GetSubSysStdOut()
{
    typeset DBEntry
    typeset StdOutField
    typeset -i ExitCode

    . $LIBDIR/cicssrc.fnc		# rarely needed so dot here

    trap "" $CICS_TRAP_SIGS		# Avoid signal termination w/o unlock

    CICS_SRCDBLock			# Lock the cicssrc database
    DBEntry="$(CICS_SRCDBGet $1)"	# Query by subsys name
    ExitCode=$?				# Record exit code
    if [ $ExitCode -ne 0 ]		# DBGet error?
    then
	CICS_SRCDBUnlock		# Unlock the cicssrc database
	[ $ExitCode -eq 1 ] && return 0	# If non-existent subsys, return 0
	return 1			# No, query processing error; return 1
    fi
    CICS_SRCDBUnlock			# Unlock the cicssrc database

					# get path info from STDOUT field
    StdOutField="$( echo "$DBEntry" | CICS_SRCDBProject $SSYS_STDOUT_FLD )"
    print -- ${StdOutField}
    return 0
}

#------------------------------------------------------------------------
#
# SYNOPSIS: GetRegionName <sub-system_name>
# DESCRIPTION:
# Obtain the region name associated with a given SRC sub-system.
#
GetRegionName()
{
    typeset DBEntry
    typeset ArgsField
    typeset -i ExitCode

    . $LIBDIR/cicssrc.fnc		# rarely needed so dot here

    trap "" $CICS_TRAP_SIGS		# Avoid signal termination w/o unlock

    CICS_SRCDBLock			# Lock the cicssrc database
    DBEntry="$(CICS_SRCDBGet $1)"	# Query by subsys name
    ExitCode=$?				# Record exit code
    if [ $ExitCode -ne 0 ]		# DBGet error?
    then
	CICS_SRCDBUnlock		# Unlock the cicssrc database
	[ $ExitCode -eq 1 ] && return 0	# If non-existent subsys, return 0
	return 1			# No, query processing error; return 1
    fi
    CICS_SRCDBUnlock			# Unlock the cicssrc database

					# get region name from ARGS field
    ArgsField="$( echo "$DBEntry" | CICS_SRCDBProject $SSYS_ARGS_FLD )"
    print -- $(expr "${ArgsField}" : '-r[ ]*\(.*\)')
    return 0
}

#------------------------------------------------------------------------
#
# SYNOPSIS: MapLVol <server> <path_to_raw_device>
# DESCRIPTION:
# Performs processing required to map Encina logical volumes to the
# underlying operating system volumes.
#
MapLVol()
{
    [ -n "$ENCINA_TK_SERVER" ] && typeset SAVE_TK_SERVER=$ENCINA_TK_SERVER
    export ENCINA_TK_SERVER=$1
    typeset RawDisk=$2
    typeset EncinaPVol=${RawDisk##*/r}_P
    typeset EncinaLVol=${RawDisk##*/r}
    typeset ChunkSize=64
    typeset Retcode

    tkadmin init disk $RawDisk &&
    tkadmin create pvol $EncinaPVol $ChunkSize 1 $RawDisk 0 &&
    tkadmin create lvol $EncinaLVol $EncinaPVol

    Retcode=$?

    [ -n "$SAVE_TK_SERVER" ] && export ENCINA_TK_SERVER=$SAVE_TK_SERVER
    return $Retcode
}

#-------------------------------------------------------------------------
#
# SYNOPSIS: RgyEdit [<args>]
# DESCRIPTION:
# On HP-UX rgy_edit returns non-zero exit status for most failures, but it
# returns 0 when a "-v <object>" request fails to find the object. This
# means we can't enquire on an object and report an error if it's missing.
# RgyEdit detects "not found" errors and returns non-zero.
#
RgyEdit()
{
    typeset OUTPUT
    typeset Ret

    OUTPUT=$(\rgy_edit "$@")		# run the command and keep o/p
    Ret=$?
    print -- "$OUTPUT"			# always need to send o/p back to user

    [ $Ret -ne 0 ] && return $Ret	# return if command failed
					# double check for errors
    if print -- "$OUTPUT" | grep -q "^?(rgy_edit)"
    then
	return 1
    else
	return 0
    fi
}

#-------------------------------------------------------------------------
#
# SYNOPSIS: RgyAddPrincipal [<args>]
# DESCRIPTION:
# On HP-UX it is necessary to alias the DCE principal to the CICS userid
# to be used whenever UNIX managed resources are created. If this isn't 
# done the resources are created using the assigned DCE UID.
# Note that the way in which id is used requires the script to be executing
# as root or be similarly authorised.
#
RgyAddPrincipal()
{

    typeset OUTPUT
    typeset RGYCMD
    typeset Ret
    typeset Rgy_flag

    while getopts :p:f:u: Rgy_flag
    do
        case $Rgy_flag in
	    p) RgyPrincipal="$OPTARG";;
	    f) RgyText="$OPTARG";;
	    u) RgyUser="$OPTARG";;
	    ?) print "Unrecognised option: $OPTARG";;
	    *) ;;
	esac
    done
    shift OPTIND-1

    if [ -z "$RgyPrincipal" ]
    then
	print "ASSERT in RgyAddPrincipal: principal name is required"
    else
	RGYCMD="add $RgyPrincipal"
	if [ -n "$RgyUser" ]
	then
	    RGYCMD="$RGYCMD $(id -u $RgyUser)"
	fi
        if [ -n "$RgyText" ]
	then 
	    RGYCMD="$RGYCMD -f \"$RgyText\""
	fi
	if [ -n "$RgyUser" ]
	then
	    RGYCMD="$RGYCMD"
	fi
        OUTPUT=$(RgyEdit -p -update <<ENDFILE
	$RGYCMD
	quit
ENDFILE
	)				# run the command and keep o/p
    fi
    Ret=$?
    print -- "$OUTPUT"			# always need to send o/p back to user
    return $Ret				# return result to user
}

#------------------------------------------------------------------------
#
# SYNOPSIS: CicsDispMsg MSN MSG_NO [parameters]
# DESCIPTION:
# Logs message to standard out.
#
CicsDispMsg()
{

    typeset -r _MCATNAME="cics.cat" # message catalogue
    typeset _MSN                    # MSN is the first param
    typeset _MSNO                   # Message Number of the message
    typeset _DEFMSG                 # Default message text corresponding to
                                    # msg id

    _MSN=$1; shift                              # MSN
    _MSNO="`eval echo \\$${1}`"                 # MESSAGE ID
    _DEFMSG="`eval echo \\$${1}_TEXT`" ; shift  # DEFAULT MESSAGE TEXT

    (${BINDIR}/cicsdspmsg ${_MCATNAME} -s ${MSET} ${_MSNO} "${_DEFMSG}" "$@"; echo "") |
	    sed "s!ERZ[0-9]*[EWI]!&/$_MSN!"

}

#------------------------------------------------------------------------
#
# SYNOPSIS: CicsDispErr MSN MSG_NO [parameters]
# DESCIPTION:
# Logs message to standard error.
#
CicsDispErr()
{

    typeset -r _MCATNAME="cics.cat" # message catalogue
    typeset _MSN                    # MSN is the first param
    typeset _MSNO                   # Message Number of the message
    typeset _DEFMSG                 # Default message text corresponding to
                                    # msg id

    _MSN=$1; shift                              # MSN
    _MSNO="`eval echo \\$${1}`"                 # MESSAGE ID
    _DEFMSG="`eval echo \\$${1}_TEXT`" ; shift  # DEFAULT MESSAGE TEXT

    (${BINDIR}/cicsdspmsg ${_MCATNAME} -s ${MSET} ${_MSNO} "${_DEFMSG}" "$@"; echo "") |
	    sed "s!ERZ[0-9]*[EWI]!&/$_MSN!" >&2

}


#-------------------------------------------------------------------------
#
# SYNOPSIS: ConfigureKernelExtensions
# DESCRIPTION:
# For HP-UX this call does nothing but return 0.
  
ConfigureKernelExtensions()
{
return 0
}

#-------------------------------------------------------------------------
#
# SYNOPSIS: UpdateInittab
# DESCRIPTION:
# For HP-UX this call does nothing but return 0.
  
UpdateInittab()
{
return 0
}

#-------------------------------------------------------------------------
#
# SYNOPSIS: $GETENCINAPATH
# DESCRIPTION:
# For HP this sets up the encina path as /opt/encina/bin.

GETENCINAPATH=/opt/encina/bin


#-------------------------------------------------------------------------
#
# SYNOPSIS: $EGREPFLAGS
# DESCRIPTION:
# For HP-UX this sets up EGREPFLAGS as -s

EGREPFLAGS=-s

#------------------------------------------------------------------------

#------------------------------------------------------------------------      
#                                                                             
#  CicsSafeLn
#                                                                             
# Perform ln safely without overwriting existing target files.
#                                                                             
#                                                                             
alias CicsSafeLn="ln"

#------------------------------------------------------------------------

#
# Module what string
#
#@(#)supos,21:43:56,Apr 08 1998,s211-L980223
#
