/*
 * NAME:        cics_epi.h - CICS External Presentation Interface header
 *
 * VERSION:     1.1
 *
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * DESCRIPTION:
 *  This file contains all the External Presentation Interface (EPI)
 *  public definitions.
 */

#ifndef _CICS_EPI_H            /* Prevent multiple #includes of this file */

#define _CICS_EPI_H


#include <cicstype.h>          /* External datatypes                      */

/*
 * First, single-terminal version. Incomplete implementation of the EPI
 */

#define CICS_EPI_VERSION_001       0x00010001

/*
 * This, multi-terminal version. Complete implementation of the EPI
 */

#define CICS_EPI_VERSION_100      0x00010000
#define CICS_EPI_VERSION_101      0x00010010

/*
 * Constants.
 */

#define CICS_EPI_ABEND_MAX            4
#define CICS_EPI_DESCRIPTION_MAX     60
#define CICS_EPI_NETNAME_MAX          8
#define CICS_EPI_SYSTEM_MAX           8
#define CICS_EPI_TRANSID_MAX          4
#define CICS_EPI_DEVTYPE_MAX         16
#define CICS_EPI_ERROR_MAX           60

#define CICS_EPI_TERM_INDEX_NONE   0xFFFF

/*
 * Data Types.
 */

typedef enum            /* EPI Event types */
{
   CICS_EPI_EVENT_SEND,
   CICS_EPI_EVENT_CONVERSE,
   CICS_EPI_EVENT_END_TRAN,
   CICS_EPI_EVENT_START_ATI,
   CICS_EPI_EVENT_END_TERM,
   CICS_EPI_EVENT_HELP
} CICS_EpiEvent_t;

typedef enum            /* EPI Terminal end reasons */
{
   CICS_EPI_END_SIGNOFF,
   CICS_EPI_END_SHUTDOWN,
   CICS_EPI_END_OUTSERVICE,
   CICS_EPI_END_UNKNOWN,
   CICS_EPI_END_FAILED
} CICS_EpiEnd_t;

typedef enum            /* EPI ATI status */
{
   CICS_EPI_ATI_ON,
   CICS_EPI_ATI_HOLD,
   CICS_EPI_ATI_QUERY
} CICS_EpiATIState_t;

typedef enum            /* EPI Sensecode errors */
{
   CICS_EPI_SENSE_OPCHECK,
   CICS_EPI_SENSE_REJECT
} CICS_EpiSenseCode_t;

typedef enum            /* EPI EpiGetEvent() wait codes */
{
   CICS_EPI_WAIT,
   CICS_EPI_NOWAIT
} CICS_EpiWait_t;


typedef struct          /* EPI CICS System Identifier */
{
   cics_char_t SystemName[CICS_EPI_SYSTEM_MAX+1];
   cics_char_t Description[CICS_EPI_DESCRIPTION_MAX+1];
} CICS_EpiSystem_t;

typedef struct          /* EPI Terminal Details */
{
   cics_char_t       NetName[CICS_EPI_NETNAME_MAX+1];
   cics_sshort_t     NumLines;
   cics_sshort_t     NumColumns;
   cics_ushort_t     MaxData;
   cics_sshort_t     ErrLastLine;
   cics_sshort_t     ErrIntensify;
   cics_sshort_t     ErrColor;
   cics_sshort_t     ErrHilight;
   cics_sshort_t     Hilight;
   cics_sshort_t     Color;
   cics_sshort_t     Printer;
} CICS_EpiDetails_t;

typedef struct          /* EPI Event Data */
{
   cics_ushort_t       TermIndex;
   CICS_EpiEvent_t     Event;
   CICS_EpiEnd_t       EndReason;
   cics_char_t         TransId[CICS_EPI_TRANSID_MAX+1];
   cics_char_t         AbendCode[CICS_EPI_ABEND_MAX+1];
   cics_ubyte_t        *Data;
   cics_ushort_t       Size;
} CICS_EpiEventData_t;

typedef struct          /* EPI System Error Information */
{
   cics_ulong_t    Cause;
   cics_ulong_t    Value;
   cics_char_t     Msg[CICS_EPI_ERROR_MAX+1];
} CICS_EpiSysError_t;

                        /* EPI Event Notification Callback */
typedef void (*CICS_EpiNotify_t)(cics_ushort_t TermIndex);


/*
 * Functions.
 */

cics_sshort_t CICS_EpiInitialize
		( cics_ulong_t		 Version );

cics_sshort_t CICS_EpiTerminate
		( void );

cics_sshort_t CICS_EpiListSystems
		( cics_char_t		*Namespace
		, cics_ushort_t		*Systems
		, CICS_EpiSystem_t	 List[]
		);

cics_sshort_t CICS_EpiAddTerminal
		( cics_char_t		*Namespace
		, cics_char_t		 System[CICS_EPI_SYSTEM_MAX+1]
		, cics_char_t		 NetName[CICS_EPI_NETNAME_MAX+1]
		, cics_char_t		 DevType[CICS_EPI_DEVTYPE_MAX+1]
		, CICS_EpiNotify_t	 NotifyFn
		, CICS_EpiDetails_t	*TermDetails
		, cics_ushort_t		*TermIndex
		);

cics_sshort_t CICS_EpiDelTerminal
		( cics_ushort_t		 TermIndex );

cics_sshort_t CICS_EpiStartTran
		( cics_ushort_t		 TermIndex
		, cics_char_t		 TransId[CICS_EPI_TRANSID_MAX+1]
		, cics_ubyte_t		*Data
		, cics_ushort_t		 Size
		);

cics_sshort_t CICS_EpiReply
		( cics_ushort_t		 TermIndex
		, cics_ubyte_t		*Data
		, cics_ushort_t		 Size
		);

cics_sshort_t CICS_EpiATIState
		( cics_ushort_t		 TermIndex
		, CICS_EpiATIState_t	*AtiState
		);

cics_sshort_t CICS_EpiSenseCode
		( cics_ushort_t		 TermIndex
		, CICS_EpiSenseCode_t	 SenseCode
		);

cics_sshort_t CICS_EpiGetEvent
		( cics_ushort_t		 TermIndex
		, CICS_EpiWait_t	 Wait
		, CICS_EpiEventData_t	*Event
		);

cics_sshort_t CICS_EpiGetSysError
		( cics_ushort_t		 TermIndex
		, CICS_EpiSysError_t	*SysErr
		);

cics_sshort_t CICS_EpiInquireSystem
		( cics_ushort_t		 TermIndex
		, cics_char_t		 System[CICS_EPI_SYSTEM_MAX+1]
		);


/*
 * Error Codes.
 */

#define CICS_EPI_NORMAL                0
#define CICS_EPI_ERR_FAILED           -1
#define CICS_EPI_ERR_VERSION          -2
#define CICS_EPI_ERR_IS_INIT          -3
#define CICS_EPI_ERR_NOT_INIT         -4
#define CICS_EPI_ERR_NO_SYSTEMS       -5
#define CICS_EPI_ERR_MORE_SYSTEMS     -6
#define CICS_EPI_ERR_SYSTEM           -7
#define CICS_EPI_ERR_MAX_TERMS        -8
#define CICS_EPI_ERR_BAD_INDEX        -9
#define CICS_EPI_ERR_TTI_ACTIVE      -10
#define CICS_EPI_ERR_ATI_ACTIVE      -11
#define CICS_EPI_ERR_TRAN_ACTIVE     -12
#define CICS_EPI_ERR_NO_CONVERSE     -13
#define CICS_EPI_ERR_ATI_STATE       -14
#define CICS_EPI_ERR_SENSE_CODE      -15
#define CICS_EPI_ERR_WAIT            -16
#define CICS_EPI_ERR_NO_EVENT        -17
#define CICS_EPI_ERR_MORE_DATA       -18
#define CICS_EPI_ERR_MORE_EVENTS     -19
#define CICS_EPI_ERR_NO_DATA         -20

/*
 * The following error codes are only returned for CICS_EPI_VERSION_101
 */
#define CICS_EPI_ERR_NULL_PARM       -21
#define CICS_EPI_ERR_IN_CALLBACK     -22
#define CICS_EPI_ERR_SECURITY        -23
#define CICS_EPI_ERR_SERVER_DOWN     -24

#endif /* _CICS_EPI_H */


/* EOF */
