/*
 *  NAME: epimenu.c
 *
 *
 *  VERSION: 1.1
 *
 *
 *  COPYRIGHT:                                                        
 *
 *  
 *    (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *    (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *    All Rights Reserved
 *    Licensed Materials - Property of IBM
 *   
 *    US Government Users Restricted Rights - Use, duplication or
 *    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *  
 *                                                                    
 *
 *
 *  Base level
 *
 *  SYNOPSIS:
 *
 *
 *
 *  DESCRIPTION:
 *
 * EPI-based workload driver for CICS/6000 ... 'menu' transactions
 *
 * There are several component parts to a workload driver
 *
 * (1) Set-up, for things like the number of terminals, the name(s) of
 *     region(s) to connect to, the number of statistics.
 *
 * (2) The sequencing of each terminal, with keying and thinking delay times,
 *     transaction patterns, and observation of response times.
 *
 * (3) The generation of inbound datastreams for a transaction on a terminal
 *
 * (4) The checking of returned datastreams and the handling of abends. For a
 *     simple workload driver, dummy checkers are sufficient.
 *
 * (5) The monitoring and reporting of statistics across all terminals
 *
 * This file is structured according to the above pattern of components. They
 * must be present in all workload drivers; the contents of each section
 * define what work is driven, where and when it is driven, and how it is
 * reported on.
 *
 * To run this workload driver :
 * (a) build the toolkit, in the "../src" directory
 * (b) build the workload driver with "make -f epimenu.mk"
 * (c) configure the region for the EPIMENU map and MENU transaction program
 * (d) dce_login
 * (e) using 'cicsterm', check that the work can be driven successfully
 *     by hand.
 * (f) set the following environment variables
 *     TERMS : the number of terminals to simulate
 *     REGION : the region you wish to drive
 *     DEVICE_TYPE : if you require a special CICS device type for the terminals
 *     SLOTTIME : how many seconds to run between reports
 *     SLOTS : how many reporting intervals to run
 *     All of the above variables appear only in this file (i.e. not in the
 *     toolkit itself); so you can change the behaviour by understanding this
 *     file and making the changes as required.
 * (g) run with "./epimenu"
 *
 * The workload driver provides a C language 'main' function, which performs
 * all necessary setup, connection to CICS region, terminal creation and
 * autoinstall, synchronisation management, and so on. You concentrate on
 * coding the workload; the functions you define in this file will be called
 * at the appropriate times.
 *
 * Functions such as 'env_int', 'delaySeconds', etc. are provided as part of
 * the toolkit. The complete list of functions that you may need is in
 * "../include/epiload.h"; you need to read this file to understand what they
 * do, but it is not necessary for the purposes of writing a workload
 * driver to understand how they work.
 *
 *  CAVEATS/WARNINGS:
 *
 *  RETURNED VALUES:
 *
 *
 *  RELATED INFORMATION:
 *
 *
 *
 *
 */

#include <stdio.h>
#include <epiload.h>

/*
 * (1) Set-up
 */
/*
 * Give names to the transaction(s), for convenience
 */
typedef enum
{
  Menu_Transaction,
  Number_Of_Transaction_Types
} TransactionType_t ;

/*
 * Indicate how many statistics will be collected. Usually, there will be
 * 1 statistic (the response time) per transaction type.
 */
int StatisticCount = Number_Of_Transaction_Types ;

/*
 * Select the number of terminals
 */
int TerminalCount(void) {
   return env_int("TERMS",5) ;
}

/*
 * Select a region and a device type. It is possible to attach different
 * terminals to different regions; but in the simple case they are all
 * attached to the same region.
 *
 * These functions must return pointers to storage which will continue to
 * exist for the length of the run. A value returned from 'getenv'
 * (hence 'env_string') is OK, but if you want to build the region or device
 * name with 'sprintf', you must 'malloc' some store to build it in.
 */

const char * RegionName(int Terminal) {
   return env_string("REGION","DEFAULT") ;
}

const char * DeviceType(int Terminal) {
   return env_string("DEVICE_TYPE",NULL) ;
}


/*
 * (2) Sequencing the terminals
 */

/*
 * This structure defines the data which needs to be passed from the
 * transaction sequencer to the datastream generator. In this case, we only
 * need to pass the transaction type; and even this is constant, because
 * every transaction is the same.
 */
typedef struct
{
  TransactionType_t TransactionType ;
} TranKey_t ;

/*
 * This function is used only within this file.
 * The monitoring/statistics section makes recommendations about the
 * 'thinking time' for the typist at each simulated terminal; this
 * function maps the recommended time onto the actual thinking time, and
 * would typically be used to implement randomisation of times.
 *
 * For the simple workload driver, we accept the monitor's recommendation
 * directly.
 *
 */
static double ThinkTime(double recommended_time)
{
  return recommended_time ;
}

/*
 * This function operates a simulated terminal. It decides on transactions to
 * run, delay time between transactions, and values to be used in datastream
 * generation; but it does not generate 3270 datastreams here.
 *
 * When a transaction completes, the underlying system reports the response
 * time to this function; and this function is expected to collect statistics
 * as may be required.
 *
 * Statistics assist is provided for calculating mean and standard deviation,
 * and for performing up to 32 tests on the acceptability of each
 * response time.
 *
 * This function will be called once for each terminal, concurrently. This
 * is done by allocating a separate program stack for each call. To find
 * out more about this, you should read the 'pthreads' (POSIX threads)
 * documentation; but it is not necessary to find out more at this stage if
 * you do not want to.
 */
void TerminalThread(Terminal_t * Terminal)
{
  int Transaction ;
  double Resp ;
  TranKey_t TranKey ;


  TranKey.TransactionType = Menu_Transaction ;
  /*
   * Synchronise (wait) until all terminals are added
   */
  syncTerminal(Terminal) ;

  /*
   * Fire in transactions, exit if the monitor asks for it
   * Keep statistics for later summarisation
   */
  for (Transaction = 1 ; Transaction <= 1000000; Transaction += 1 )
  {
    Resp = runTransaction(Terminal,&TranKey) ;
    delaySeconds(ThinkTime(askInterval(Terminal))) ;
    if (finishRequest(Terminal))
    {
       break ;
    } /* endif */
    setCount(Terminal, Transaction) ;
    collectStatistic(Terminal, Menu_Transaction, Resp,
                       ATTRIBUTE(0, Resp < 0.005)
                     + ATTRIBUTE(1, Resp < 0.01)
                     + ATTRIBUTE(2, Resp < 0.02)
                     + ATTRIBUTE(3, Resp < 0.05)
                     + ATTRIBUTE(4, Resp < 0.1)
                     + ATTRIBUTE(5, Resp < 0.2)
                     + ATTRIBUTE(6, Resp < 0.5)
                    ) ;
  } /* endfor */

  /*
   * That's all
   */
}

/*
 * (3) Generation of inbound data
 */
/*
 * Build a datastream for a transaction
 */
void BuildData(Datastream_t * Data, void * TranKey )
{
   setTransaction(Data,"MENU") ;
}

/*
 * Provide a dummy datastream builder for any EXEC CICS CONVERSE requests
 * which arrive. This should never be driven, because the MENU transaction
 * does not perform EXEC CICS CONVERSE
 */
void BuildDataConverse(Datastream_t * Data,
		       void * TranKey,
		       unsigned int ConverseCount)
{
}

/*
 * (4) Checking of responses.
 */
/*
 * For this simple workload driver, we assume that the responses are
 * always correct. Here are 'dummy' checkers.
 */
void HandleReplyFromCICS(Terminal_t * Terminal, void *TranKey, Reply_t *Reply)
{
}

void HandleTransIdFromCICS(Terminal_t * Terminal, void * TranKey, const char * TransID)
{
}

void HandleAbendFromCICS(Terminal_t * Terminal, void * TranKey, const char * AbendCode)
{
  fprintf(stderr, "Warning : Terminal %i has received abend code %s\n",
                  terminalNumber(Terminal), AbendCode) ;
}

/*
 * (5) Monitoring
 */

/*
 * Monitor the terminals. For the 'menu' test case, this is straightforward.
 */
void TerminalMonitor (Monitor_t * Monitor, int Terminals )
{
  int slot ;
  int prev_transactions = 0 ;
  int total_transactions ;
  int interval_transactions ;
  const double SlotTime = env_int("SLOTTIME", 5) ;
  const int Slots = env_int("SLOTS", 1) ;
  double ThinkTime = 0.0 ; /* Initial typist think time */
  /*
   * Request think time per terminal
   */
  setInterval(Monitor, ThinkTime) ;
  /*
   * Synchronise (wait) until all terminals are added
   */
  syncMonitor(Monitor) ;
  /*
   * Report on progress periodically
   */
  for (slot = 0 ; slot < Slots ; slot += 1)
  {
     delaySeconds(SlotTime) ;
     total_transactions = totalTransactions(Monitor) ;
     interval_transactions = total_transactions - prev_transactions ;
     prev_transactions = total_transactions ;
     fprintf(stderr,"%i transactions run in %f seconds.\n",
              interval_transactions, SlotTime) ;
     summariseStatistic(Monitor,
                        Menu_Transaction,
                        "MENU response time (seconds)") ;
  } /* endfor */
  /*
   * Ask for the run to end
   */
  requestFinish(Monitor) ;
}
