/*
 *  NAME: epimenu2.c
 *
 *
 *  VERSION: 1.1
 *
 *
 *  COPYRIGHT:                                                        
 *
 *  
 *    (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *    (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *    All Rights Reserved
 *    Licensed Materials - Property of IBM
 *   
 *    US Government Users Restricted Rights - Use, duplication or
 *    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *  
 *                                                                    
 *
 *
 *  Base level
 *
 *  SYNOPSIS:
 *
 *
 *
 *  DESCRIPTION:
 *
 * This is a more complete workload driver for CICS/6000. These instructions
 * assume that you have read and understood the simple workload driver
 * 'epimenu.c', and explain the additional steps.
 *
 * To understand this program, it will help if you first install and run the
 * transaction programs 'by hand', from a 'cicsterm'. You should try the
 * effect of the PF keys, and of filling in data on the panels that are
 * presented.
 *
 * To run this workload driver :
 * (a) build the toolkit, in the "../src" directory
 * (b) build the workload driver with "make -f epimenu.mk"
 * (c) configure the region for the EPIMENU map and
 *     EPIMENA/B/C transaction programs. The region should be set up so that
 *     transaction MENA runs program EPIMENA, transaction MENB runs EPIMENB,
 *     and transaction MENC runs program EPIMENC.
 * (d) dce_login
 * (e) using 'cicsterm', check that the work can be driven successfully
 *     by hand. You start the top-level menu by keying "MENA" and 
 *     pressing enter. The transaction programs trace their activity in the
 *     CSMT TD queue.
 * (f) set the following environment variables
 *     TERMS : the number of terminals to simulate
 *     REGION : the region you wish to drive
 *     DEVICE_TYPE : if you require a special CICS device type for the terminals
 *     SLOTTIME : how many seconds to run between reports
 *     SLOTS : how many reporting intervals to run
 *     THINK_TIME : the mean number of seconds between transactions submitted
 *                  from a terminal
 *     All of the above variables appear only in this file (i.e. not in the
 *     toolkit itself); so you can change the behaviour by understanding this
 *     file and making the changes as required.
 * (g) run with "./epimenu2"
 * (h) look at the CSMT TD queue to confirm the sequence of operations.    
 *
 * The workload driver provides a C language 'main' function, which performs
 * all necessary setup, connection to CICS region, terminal creation and
 * autoinstall, synchronisation management, and so on. You concentrate on
 * coding the workload; the functions you define in this file will be called
 * at the appropriate times.
 *
 * Functions such as 'env_int', 'delaySeconds', etc. are provided as part of
 * the toolkit. The complete list of functions that you may need is in
 * "../include/epiload.h"; you need to read this file to understand what they
 * do, but it is not necessary for the purposes of writing a workload
 * driver to understand how they work.
 *
 * The EPIMENA application consists of a main menu, and 2 sub-menus called
 * 'signon' and 'lookup'. You access the sub-menus by pressing the appropriate
 * PF key on the main menu; on a sub-menu, you fill in the required fields and
 * press 'enter'; then the main menu is redisplayed, with a status field to
 * confirm the results of the action performed on the sub-menu. The cycle
 * repeats until you press 'exit' on any menu.
 *
 * The workload driver leads the simulated terminals through a pseudorandom
 * sequence of menus in this application.
 *
 *  CAVEATS/WARNINGS:
 *
 *  RETURNED VALUES:
 *
 *
 *  RELATED INFORMATION:
 *
 *
 *
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <epiload.h>
#include <dfhaid.h>

/*
 * (1) Set-up
 */
/*
 * Give names to the transaction(s), for convenience
 */
typedef enum
{
  Menu_Transaction,
  Signon_Transaction,
  Lookup_Transaction,
  Number_Of_Transaction_Types
} TransactionType_t ;

/*
 * Indicate how many statistics will be collected. Usually, there will be
 * 1 statistic (the response time) per transaction type.
 */
int StatisticCount = Number_Of_Transaction_Types ;

/*
 * Select the number of terminals
 */
int TerminalCount(void) {
   return env_int("TERMS",5) ;
}

/*
 * Select a region and a device type. It is possible to attach different
 * terminals to different regions; but in the simple case they are all
 * attached to the same region.
 *
 * These functions must return pointers to storage which will continue to
 * exist for the length of the run. A value returned from 'getenv'
 * (hence 'env_string') is OK, but if you want to build the region or device
 * name with 'sprintf', you must 'malloc' some store to build it in.
 */

const char * RegionName(int Terminal) {
   return env_string("REGION","DEFAULT") ;
}

const char * DeviceType(int Terminal) {
   return env_string("DEVICE_TYPE",NULL) ;
}
/*
 * (2) Sequencing the terminals
 */

/*
 * This structure defines the data which needs to be passed from the
 * transaction sequencer to the datastream generator. For this more complex
 * workload driver, several items (including which transaction needs to be
 * driven next) need to be passed.
 *
 * Note that all needed context information is within this structure, and there
 * are no 'static' variables. A 'static' variable would be shared between all
 * of the terminals; this would not be the desired effect.
 */
typedef struct
{
  TransactionType_t TransactionType ;
  TransactionType_t NextTransactionType ;
  unsigned int UserNumber ;
  unsigned int LookupKey ;
} TranKey_t ;

/*
 * This function is used only within this file.
 * The monitoring/statistics section makes recommendations about the
 * 'thinking time' for the typist at each simulated terminal; this
 * function maps the recommended time onto the actual thinking time, and
 * would typically be used to implement randomisation of times.
 *
 * For this more complex workload driver, we compute a pseudo-random
 * interval, distributed as negative exponential with the given mean.
 *
 */
static double ThinkTime(double recommended_time)
{
  return negexp(recommended_time) ;
}

/*
 * This function operates a simulated terminal. It decides on transactions to
 * run, delay time between transactions, and values to be used in datastream
 * generation; but it does not generate 3270 datastreams here.
 *
 * When a transaction completes, the underlying system reports the response
 * time to this function; and this function is expected to collect statistics
 * as may be required.
 *
 * Statistics assist is provided for calculating mean and standard deviation,
 * and for performing up to 32 tests on the acceptability of each
 * response time.
 *
 * This function will be called once for each terminal, concurrently. This
 * is done by allocating a separate program stack for each call. To find
 * out more about this, you should read the 'pthreads' (POSIX threads)
 * documentation; but it is not necessary to find out more at this stage if
 * you do not want to.
 */
void TerminalThread(Terminal_t * Terminal)
{
  int Transaction ;
  double Resp ;
  TranKey_t TranKey ;

  TranKey.UserNumber = terminalNumber(Terminal) ;
  /*
   * The first transaction to be driven will be the initial menu, i.e.
   * 'MENA'. Subsequent transactions are discovered from the
   * RETURN TRANSID values; look at 'HandleTransIdFromCICS' to see how
   * this is done.
   */
  TranKey.TransactionType = Menu_Transaction ;
  /*
   * Synchronise (wait) until all terminals are added
   */
  syncTerminal(Terminal) ;

  /*
   * Fire in transactions, exit if the monitor asks for it
   * Keep statistics for later summarisation
   */
  for (Transaction = 1 ; Transaction <= 1000000; Transaction += 1 )
  {
    TranKey.NextTransactionType = Menu_Transaction ;
    if (TranKey.TransactionType == Lookup_Transaction)
    {
       /*
        * 'drand48()' is a library function which returns a pseudorandom value
        * uniformly distributed between 0.0 and 1.0, guaranteed less than 1.0 .
        */
       TranKey.LookupKey = 1000*drand48() ;
    } /* endif */
    Resp = runTransaction(Terminal,&TranKey) ;
    TranKey.TransactionType = TranKey.NextTransactionType ;
    delaySeconds(ThinkTime(askInterval(Terminal))) ;
    if (finishRequest(Terminal))
    {
       break ;
    } /* endif */
    setCount(Terminal, Transaction) ;
    collectStatistic(Terminal, Menu_Transaction, Resp,
                       ATTRIBUTE(0, Resp < 0.005)
                     + ATTRIBUTE(1, Resp < 0.01)
                     + ATTRIBUTE(2, Resp < 0.02)
                     + ATTRIBUTE(3, Resp < 0.05)
                     + ATTRIBUTE(4, Resp < 0.1)
                     + ATTRIBUTE(5, Resp < 0.2)
                     + ATTRIBUTE(6, Resp < 0.5)
                    ) ;
  } /* endfor */

  /*
   * That's all
   */
}

/*
 * (3) Generation of inbound data
 *
 * It is convenient to have one function which builds the inbound
 * datastream for each screen type presented by the application; and then
 * a 'master' function (consisting mainly of a 'switch' statement) to
 * invoke the datastream builder for the actual screen required.
 */
/*
 * Build a datastream for the 'sign-on' transactionm
 */
static void BuildSignon(Datastream_t * Data, unsigned int UserNumber)
{
   char UserName[20] ;
   char Password[20] ;

   setTransaction(Data, "MENB") ;

   sprintf(UserName, "USER%04i", UserNumber) ;
   sprintf(Password, "PASS%04i", UserNumber) ;

   /*
    * 'sba' is set-buffer-address.
    *  GA23-0059  3270 Information Display System
    *                  Data Stream Programmer's Reference
    * tells you in detail what this means; for the moment, you need to know
    * that this matches up with the POS macro in the 'epimenu.bms' mapset
    * definition file; it identifies the location on the screen where the
    * 'text' will be presented.
    */
   sba_3270(Data, 11,22 ) ; text_3270(Data, UserName) ;
   sba_3270(Data, 12,22 ) ; text_3270(Data, Password) ;
}

/*
 * Build a datastream for the 'lookup' transactionm
 */
static void BuildLookup(Datastream_t * Data, unsigned int LookupKey)
{
   char LookupText[20] ;

   setTransaction(Data, "MENC") ;

   sprintf(LookupText, "CUST%04i", LookupKey) ;

   sba_3270(Data, 12,11 ) ; text_3270(Data, LookupText) ;
}

/*
 * Build a datastream for the 'menu' transactionm
 */
static void BuildMenu(Datastream_t * Data)
{
   double selection = drand48() ;
   setTransaction(Data,"MENA") ;
   if (selection < 0.2 )
   {
      setAID(Data, DFHPF1) ;
   }
   else if ( selection < 0.9 )
   {
      setAID(Data, DFHPF2) ;
   }
   else
   {
      setAID(Data, DFHPF3) ;
   } /* endif */
}

/*
 * 'master' datastream builder
 */
void BuildData(Datastream_t * Data, void * TranKey_v )
{
   TranKey_t * TranKey = TranKey_v ;
   switch (TranKey -> TransactionType)
   {
      case Menu_Transaction :
         BuildMenu(Data) ;
         break ;
      case Signon_Transaction :
         BuildSignon(Data, TranKey -> UserNumber) ;
         break ;
      case Lookup_Transaction :
         BuildLookup(Data, TranKey -> LookupKey) ;
         break;
      default :
         BuildMenu(Data) ;
   } /* endswitch */
}

/*
 * Provide a dummy datastream builder for any EXEC CICS CONVERSE requests
 * which arrive. This should never be driven, because these transactions
 * do not perform EXEC CICS CONVERSE
 */
void BuildDataConverse(Datastream_t * Data,
                       void * TranKey,
                       unsigned int ConverseCount)
{
}

/*
 * (4) Checking of responses.
 */
/*
 * A 'workload driver' frequently does not need to be concerned with the
 * detail of the response datastream from CICS; it can just stick to its
 * script. If you need to parse the returned datastream, you should consult
 *
 * GA23-0059  3270 Information Display System
 *                 Data Stream Programmer's Reference
 *
 * If you compile this workload driver with '-DDEBUG', this function will
 * expand to display the returned datastream.
 */
void HandleReplyFromCICS(Terminal_t * Terminal, void *TranKey, Reply_t *Reply)
{
#ifdef DEBUG
  fprintf(stderr, "\nReply from CICS for terminal %i\n%.*s\n",
                    terminalNumber(Terminal),
                    replyLength(Reply),
                    replyText(Reply) ) ;
#endif
}

/*
 * This workload driver needs to be sensitive to the values returned by the
 * RETURN TRANSID(....) calls in the transaction programs. Here is the
 * decoder for this application.
 */
void HandleTransIdFromCICS(Terminal_t * Terminal,
                           void * TranKey_v,
                           const char * TransID)
{
  TranKey_t * TranKey = TranKey_v ;
  if (0 == strcmp(TransID, "MENA") )
  {
     TranKey -> NextTransactionType = Menu_Transaction ;
  }
  else if ( 0 == strcmp(TransID, "MENB") )
  {
     TranKey -> NextTransactionType = Signon_Transaction ;
  }
  else if ( 0 == strcmp(TransID, "MENC") )
  {
     TranKey -> NextTransactionType = Lookup_Transaction ;
  }
  else if ( 0 == strcmp(TransID, "    ") )
  {
     TranKey -> NextTransactionType = Menu_Transaction ;
  }
  else
  {
    fprintf(stderr,
            "Warning : Terminal %i has received unexpected TRANSID (%s)\n",
             terminalNumber(Terminal), TransID)  ;
  } /* endif */
}

/*
 * This application should not cause any abends. If any should happen
 * (for CICS internal reasons), note them and carry on.
 */
void HandleAbendFromCICS(Terminal_t * Terminal, void * TranKey, const char * AbendCode)
{
  fprintf(stderr, "Warning : Terminal %i has received abend code %s\n",
                  terminalNumber(Terminal), AbendCode) ;
}

/*
 * (5) Monitoring
 */

/*
 * Monitor the terminals.
 */
void TerminalMonitor (Monitor_t * Monitor, int Terminals )
{
  int slot ;
  int prev_transactions = 0 ;
  int total_transactions ;
  int interval_transactions ;
  const double SlotTime = env_int("SLOTTIME", 5) ;
  const int Slots = env_int("SLOTS", 1) ;
         /* Initial typist think time */
  double ThinkTime = env_double("THINK_TIME", 1.0) ;
  /*
   * Request think time per terminal
   */
  setInterval(Monitor, ThinkTime) ;
  /*
   * Synchronise (wait) until all terminals are added
   */
  syncMonitor(Monitor) ;
  /*
   * Report on progress periodically
   */
  for (slot = 0 ; slot < Slots ; slot += 1)
  {
     delaySeconds(SlotTime) ;
     total_transactions = totalTransactions(Monitor) ;
     interval_transactions = total_transactions - prev_transactions ;
     prev_transactions = total_transactions ;
     fprintf(stderr,"%i transactions run in %f seconds.\n",
              interval_transactions, SlotTime) ;
     summariseStatistic(Monitor,
                        Menu_Transaction,
                        "MENU response time (seconds)") ;
     summariseStatistic(Monitor,
                        Signon_Transaction,
                        "SIGNON response time (seconds)") ;
     summariseStatistic(Monitor,
                        Lookup_Transaction,
                        "LOOKUP response time (seconds)") ;
  } /* endfor */
  /*
   * Ask for the run to end
   */
  requestFinish(Monitor) ;
}
