/*
 *  NAME: epiload.h
 *
 *
 *  VERSION: 1.1
 *
 *
 *  COPYRIGHT:                                                        
 *
 *  
 *    (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *    (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *    All Rights Reserved
 *    Licensed Materials - Property of IBM
 *   
 *    US Government Users Restricted Rights - Use, duplication or
 *    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *  
 *                                                                    
 *
 *
 *  Base level
 *
 *  SYNOPSIS:
 *
 *
 *
 *  DESCRIPTION:
 *
 * User-accessible parts of the workload drive simulator
 *
 *
 *  CAVEATS/WARNINGS:
 *
 *  RETURNED VALUES:
 *
 *
 *  RELATED INFORMATION:
 *
 *
 */

/*
 * Data structures used by the tools, but opaque to the workload driver
 */
typedef struct Terminal_s Terminal_t ;
typedef struct Monitor_s Monitor_t ;
typedef struct Datastream_s Datastream_t ;
typedef struct Reply_s Reply_t ;

/*
 * Configuration assistance
 */
             /* Text value from environment */
const char * env_string (const char * env_var, const char * default_value) ;

             /* Integer value from environment */
int          env_int ( const char * env_var, int default_value ) ;

             /* Floating-point value from environment */
double       env_double ( const char * env_var, double default_value) ;

             /* Random number seed, from RANDOM_SEED or clock nanoseconds */
int          random_seed(void) ;

             /* Random number distributed negative exponential with given mean*/
double       negexp(double mean) ;

/*
 *
 * Action functions supplied by the workload driver internals
 *
 */
       /* Indicate that a terminal is ready to go */
void   syncTerminal(Terminal_t * Terminal)                    ;

       /* Indicate that the monitor is ready to go */
void   syncMonitor(Monitor_t * Monitor)                       ;

       /* Indicate the number of transactions performed */
void   setCount(Terminal_t * Terminal, int Count) ;

       /* Delay, to simulate thinking time */
void   delaySeconds(double delaytime) ;

       /* For a terminal to ask whether the run should finish */
int    finishRequest(Terminal_t * Terminal) ;

       /* For the monitor to tell the terminals to finish */
void   requestFinish(Monitor_t *Monitor) ;

       /* For the monitor to set the mean inter-arrival time */
void   setInterval(Monitor_t *Monitor, double Interval) ;

       /* For a terminal to ask about the mean inter-arrival time */
double askInterval(Terminal_t *Terminal) ;

        /* For a terminal to ask which terminal number it is */
int     terminalNumber(Terminal_t *Terminal) ;

       /* For the monitor to ask about the total number of transactions */
int    totalTransactions(Monitor_t *Monitor) ;

       /* Run a transaction, and return the response time.
        * 'TranKey' is passed to the user exit which generates the
        * inbound datastream.
        */
double runTransaction(Terminal_t * Terminal, void * TranKey)                     ;
       /*
        * 'user' time-stamping support, for conversational transactions
        * where 'runTransaction' does not give a useful result
        *
        * This is a 'lap' timer; it reports the time in seconds since the
        * last call to lapTime for this terminal
        */
double lapTime(Terminal_t * Terminal) ;

/*
 * 3270 datastream generation assist
 */
     /* Set-buffer-address, 'Row' and 'Col' match the POS macro in the map */
void sba_3270  (Datastream_t *Data, int Row, int Col) ;

     /* Text (nul-terminated string) */
void text_3270 (Datastream_t *Data, const char * Text) ;

     /* Byte string (address and length) */
void bytes_3270(Datastream_t *Data, const void * Bytes, unsigned int Length) ;

     /* Place (4-character) transaction code in datastream */
void setTransaction(Datastream_t *Data, const char * TranCode) ;

     /* Place Attention Identifier (PF key) in datastream */
void setAID(Datastream_t *Data, int AID) ;

     /* Place cursor position in datastream. (0,0) is top left corner. */
void setCursor (Datastream_t *Data, int Row, int Col) ;

/*
 * Functions supplied by the workload driver to help in parsing
 * replies from the CICS region. These are available to be called by
 * 'HandleReplyFromCICS' (see below).
 */
     /* Command (see 3270 datastream reference) */
int replyCommand (Reply_t * Reply) ;

     /* Write-Control-Character (see 3270 datastream reference) */
int replyWCC (Reply_t * Reply) ;

     /* Address of start of reply text (not including order or write control char)
      * Note that the reply text is not (in general) NUL-terminated, and may
      * contain embedded NUL characters; it it necessary to call replyLength
      * to figure out the length of the datastream
      */
const char * replyText (Reply_t * Reply) ;

     /* Size (in bytes) of reply text (not including order or write control char)
      * Note : If 'replyLength' is <0, there is no replyText
      */

int replyLength (Reply_t *Reply) ;


/*
 * Statistics manipulation
 *
 * A 'statistic' is a measurement about a transaction, typically the
 * response time. If you have 5 transaction types to simulate, you will
 * have 5 statistics.
 *
 * With each statistic are associated a number of attributes, up to 32.
 * These are typically used to track whether the response time was within
 * each of a set of limit criteria.
 *
 * After running a transaction, you will presumably have a value for the
 * statistic for that transaction type, and you should call
 * 'collectStatistic' appropriately. For example
 *
 * collectStatistic(Terminal, 0, Response, ATTRIBUTE(0, Response < 0.1) +
 *                                         ATTRIBUTE(1, Response < 0.5) +
 *                                         ATTRIBUTE(2, Response < 1.0) ) ;
 *
 * The monitor should call 'summariseStatistic' appropriately at the
 * end of each sampling interval, to summarise the statistics across
 * all terminals being simulated.
 *
 */
#define ATTRIBUTE(P,A) ((A) << (P))
void collectStatistic(Terminal_t * Terminal,
                      unsigned int StatisticNumber,
                      double Observation ,
                      unsigned int AttributeSet
                      ) ;
void summariseStatistic(Monitor_t * Monitor,
                        unsigned int StatisticNumber,
                        const char * StatisticName) ;


/*
 * Workload driver function, supplied by the 'user'. Examples given.
 */
             /* This should be a compile-time constant, the number of
              * statistics that you need.
              */
extern int   StatisticCount ;

             /* This function will be called at the beginning, and must
              * return with the number of terminals to be simulated
              */
int          TerminalCount(void)                                          ;

             /*
              * These functions will be called for each terminal, and must
              * return with the relevant attributes for that terminal
              */
const char * RegionName(int terminal_number) ;
const char * DeviceType(int terminal_number) ;

             /* This function must simulate the behaviour of a typist
              * at a terminal. It chooses the 'think time' and which
              * transaction is to run, but it does not generate the
              * data for the transaction
              */
void         TerminalThread(Terminal_t * Terminal)                           ;

             /* This function must build the datastream to send to the
              * CICS region, in support of the requested transaction.
              */
void         BuildData(Datastream_t * Data, void * TranKey ) ;

             /* This function must build the datastream to send to the
              * CICS region, in support of an EXEC CICS CONVERSE request
	      * issued by a transaction. 
	      *
	      * ConverseCount will be 1 for the first 'CONVERSE' issued by a 
	      * transaction, and will be incremented thereafter.
	      *
	      * To simulate the effect of a user thinking, call 'delaySeconds';
	      * but note that this delay time will be included in the 
	      * transaction response time reported by 'runTransaction'.
              */
void         BuildDataConverse(Datastream_t * Data, void * TranKey , unsigned int ConverseCount ) ;

             /*
              * This function will be called whenever the CICS region replies
              * to a terminal. It should perform any necessary analysis on the
              * response datastream, placing the results of the analysis in the
              * 'TranKey' structure for use by the main-line code.
              *
              * 'Terminal' is provided so that this function can collect statistics
              * if required.
              *
              */
void         HandleReplyFromCICS(Terminal_t * Terminal,
				 void *TranKey,
				 Reply_t *Reply) ;

             /*
              * This function will be called whenever the CICS region sends a
              * 'TRANSID' to the terminal (as a result of E-C-RETURN TRANSID).
              * It should perform any necessary analysis; this will typically
              * be to 'memcpy' it to a place in the TranKey so it can be
              * supplied to a future BuildData call.
              *
              * In CICS, a TransID is up to 4 characters. This call guarantees
              * to provide a trailing NUL; so if you 'memcpy' 5 bytes, it will
              * be correctly framed for a 'setTransaction' call.
              */
void         HandleTransIdFromCICS(Terminal_t * Terminal,
				   void * TranKey,
				   const char * TransID) ;
             /*
	      * This function will be called whenever the CICS region reports
	      * that a transaction has abended. In CICS, an abend code is
	      * 4 characters; and this call guarantees to provide a trailing
	      * NUL. 
	      */
void         HandleAbendFromCICS(Terminal_t * Terminal,
				 void * TranKey,
				 const char * AbendCode) ;


             /* This function controls and reports on the progress of
              * all terminals being simulated
              */
void         TerminalMonitor(Monitor_t * Monitor, int Terminals ) ;
