/*
 *  NAME: epiload_int.h
 *
 *
 *  VERSION: 1.1
 *
 *
 *  COPYRIGHT:                                                        
 *
 *  
 *    (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *    (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *    All Rights Reserved
 *    Licensed Materials - Property of IBM
 *   
 *    US Government Users Restricted Rights - Use, duplication or
 *    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *  
 *                                                                    
 *
 *
 *  Base level
 *
 *  SYNOPSIS:
 *
 *
 *
 *  DESCRIPTION:
 *
 *    Internal headers for workload drive simulator EPI application
 *
 *  CAVEATS/WARNINGS:
 *
 *  RETURNED VALUES:
 *
 *
 *  RELATED INFORMATION:
 *
 *
 */
#include <memory.h>
#include <stdio.h>
#include <pthread.h>
#include <locale.h>
#include <math.h>

#include <cics_epi.h>           /* Include the EPI header file */
#include <dfhaid.h>

/*
 * Clock support
 */
void askClock(struct timestruc_t *Clock) ;
#define CLOCK(s, ns)              {                                     struct timestruc_t _now ;         askClock(&_now) ;                 (s) = _now . tv_sec ;             (ns) = _now . tv_nsec ;       }

/*
 * Define useful constants
 */
typedef enum {
   diagnose = 0 /* Whether to display diagnostic message */ ,
   diagnosex = 0 /* Whether to display datastreams in hex */ ,
   concurrentAdd = 10 /* How many terminals to sign on (or off) at a time */ ,
   max3270ds = 12288 /* The largest 3270 datastream to support */
   } constant_t ;

/*
 * 3270 6-bit addressing codes
 */

typedef enum {
   BG_0 = 0x20 /* Special one asked for explicitly ... */
} BinaryToGraphic_t ;

static const char binaryToGraphic[64] =
{
 BG_0, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47  ,
 0x48, 0x49, 0x5b, 0x2e, 0x3c, 0x28, 0x2b, 0x21  ,
 0x26, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f, 0x50  ,
 0x51, 0x52, 0x5d, 0x24, 0x2a, 0x29, 0x3b, 0x5e  ,
 0x2d, 0x2f, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58  ,
 0x59, 0x5a, 0x7c, 0x2c, 0x25, 0x5f, 0x3e, 0x3f  ,
 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37  ,
 0x38, 0x39, 0x3a, 0x23, 0x40, 0x27, 0x3d, 0x22
} ;

/*
 * Reporting functions supplied by the workload driver internals
 */
void report_failure(int result, const char * text , int expect , int terminal) ;
void report_warning(int event, const char * text , int terminal )              ;
void report_success(int result, const char * text , int terminal )             ;
void report_failureu(int result, const char * text , int expect )              ;
void report_warningu(int event, const char * text )                            ;
void report_successu(int result, const char * text )                           ;
void displayData(const void * Data, int Size)                                  ;

/*
 * Checking macros used internally
 */
#define CHECK(function, args, expect, terminal)                    {                                                                    int result = (function args) ;                                   if ( result != (expect))                                         {                                                                   report_failure(result, #function, expect, terminal ) ;        }                                                                else if ( diagnose )                                             {                                                                   report_success(result, #function, terminal ) ;                } /* endif */                                                }
#define CHECKU(function, args, expect)                   {                                                          int result = (function args) ;                         if ( result != (expect))                               {                                                         report_failureu(result, #function, expect ) ;       }                                                      else if ( diagnose )                                   {                                                         report_successu(result, #function ) ;               } /* endif */                                      }
/*
 * Internals of the 'monitor' structure.
 */
typedef struct Monitor_s
{
   pthread_mutex_t Mutex ;
   pthread_cond_t  AddCond ;
   pthread_cond_t  DeleteCond ;
   pthread_cond_t  AddSeq ;
   int             TerminalsAdding ;
   int             TerminalsCreated ;
   int             TerminalsAdded ;
   int             TerminalsDeleted ;
   int             RequestFinish ;
   Terminal_t *    FirstTerminal ;
   double          Interval ;
} ;

/*
 * Internals of 'statistics' support. These are set up to be able to
 * calculate the mean and standard deviation of the recorded values, and to
 * be able to test what fraction of the recorded values meet up to
 * 32 criteria. The calculations are normally done on the observations made in
 * the most recent 'time slot'; but could also be done for all observations
 * in the whole run of the workload driver.
 */
typedef struct
{
   double sx ;
   double sxx ;
   int s1 ;
   int sok[32] ;
} SampleStat_t ;

typedef struct
{
  SampleStat_t previous ;
  SampleStat_t current ;
} Statistic_t ;

/*
 * Internals for how time-stamps work
 */
typedef struct
{
  int seconds ;
  unsigned int nanoseconds ;
} Timestamp_t ;

/*
 * Internals of the 'per-terminal' structure
 */
typedef struct Terminal_s
{
   Monitor_t * Monitor ;
   const char * Region ;
   const char * Model ;
   int TransactionCount ;
   int TermIndex ;
   int Terminated ;
   int TermSequence ;
   Statistic_t * Statistic ;
   Timestamp_t Timestamp ; /* Timestamp for recent interaction with region */
}  ;

/*
 * Interface to the CICS EPI layer
 */
typedef struct Datastream_s
{
  int CurrentSize ;
  cics_char_t Transaction[CICS_EPI_TRANSID_MAX+1] ;
  cics_ubyte_t Stream[max3270ds] ;
} ;

typedef struct Reply_s
{
   const cics_ubyte_t * ReplyUbyte ;
   int ReplySize ;
} ;

/*
 * Encapsulation macros
 */
#define TerminalMon(X) (X)->Monitor
#define TerminalRegion(X) (X)->Region
#define TerminalModel(X) (X)->Model
#define TerminalTransactionCount(X) (X)->TransactionCount
#define TerminalTermIndex(X) (X)->TermIndex
#define TerminalRequestFinish(X) TerminalMon(X)->RequestFinish
#define TerminalTimestamp(X) (X)->Timestamp

/*
 * Functions which are part of the workload driver but
 * do not normally need to be exposed to the user
 */
void addTerminal(Terminal_t * Terminal)                        ;
void deleteTerminal(Terminal_t * Terminal)                     ;
void notifyMain(Terminal_t * Terminal)                         ;
void startTransaction(Terminal_t * Terminal, void * TranKey)   ;
void defaultDatastream(Datastream_t * Datastream)              ;
int processEventNoCallback(Terminal_t * Terminal)              ;
int processEvent(Terminal_t * Terminal, void * TranKey)        ;
void processEventToEnd(Terminal_t * Terminal, void * TranKey)  ;
void * terminalThreadWrap(void * Parm)                         ;
