/*
 *  NAME: processEvent.c
 *
 *
 *  VERSION: 1.1
 *
 *
 *  COPYRIGHT:                                                        
 *
 *  
 *    (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *    (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *    All Rights Reserved
 *    Licensed Materials - Property of IBM
 *   
 *    US Government Users Restricted Rights - Use, duplication or
 *    disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *  
 *                                                                    
 *
 *
 *  Base level
 *
 *  SYNOPSIS:
 *
 *    int processEventNoCallback(Terminal_t * Terminal)
 *    int processEvent(Terminal_t * Terminal, void * TranKey)
 *    void processEventToEnd(Terminal_t * Terminal, void * TranKey)
 *
 *  DESCRIPTION:
 *
 *      EPI workload driver function to process an EPI event
 *      received from a CICS region.
 *
 *      This would not normally be called by
 *      user code; it would normally be called internally within the
 *      workload driver only.
 *
 *  CAVEATS/WARNINGS:
 *
 *      This is set up only to take datastreams of up to 4 kbytes from
 *      the CICS region. This can be changed by changing the '4096'
 *      in the code.
 *
 *  RETURNED VALUES:
 *
 *
 *  RELATED INFORMATION:
 *
 *
 */

#include <epiload.h>
#include <epiload_int.h>
int processEventNoCallback(Terminal_t * Terminal)
{
   CICS_EpiEventData_t EventData;
   cics_ubyte_t Data[4096] ;
   int Rc ;
   int TermIndex=TerminalTermIndex(Terminal) ;

   EventData.Data = Data;
   EventData.Size = sizeof(Data);
   Rc = CICS_EpiGetEvent(TermIndex,
                         CICS_EPI_WAIT,   /* Wait for next event */
                         &EventData);
   if (Rc == CICS_EPI_NORMAL || Rc == CICS_EPI_ERR_MORE_EVENTS)
   {
      int Event = EventData.Event ;
      switch (Event)
      {
      case CICS_EPI_EVENT_SEND :

         if ( diagnose || diagnosex) {
           fprintf(stderr,"Message, size=%i, received from CICS\n",
                          EventData.Size) ;
           displayData(Data, EventData.Size) ;
         } /* endif */
         break;
      case CICS_EPI_EVENT_END_TRAN:
         break;
      case CICS_EPI_EVENT_END_TERM :
         Terminal -> Terminated = 1 ;
         break;
      case CICS_EPI_EVENT_CONVERSE :

         if ( diagnose || diagnosex) {
           fprintf(stderr,"Converse, size=%i, received from CICS\n",
                          EventData.Size) ;
           displayData(Data, EventData.Size) ;
         } /* endif */
         break;
      default :
         report_warning(Event, "processEvent",TermIndex) ;
      } /* endswitch */
      return Event ;
   }
   else
   {
       report_failure(Rc, "processEvent",CICS_EPI_NORMAL,TermIndex) ;
   } /* endif */

}

int processEvent(Terminal_t * Terminal, void * TranKey)
{
   CICS_EpiEventData_t EventData;
   cics_ubyte_t Data[max3270ds] ;
   int Rc ;
   int TermIndex=TerminalTermIndex(Terminal) ;
   Reply_t Reply ;
   int Event ;
   int FirstOfTransId ;
   int FirstOfAbendCode ;

   EventData.Data = Data;
   EventData.Size = sizeof(Data);
   Rc = CICS_EpiGetEvent(TermIndex,
                         CICS_EPI_WAIT,   /* Wait for next event */
                         &EventData);
   Event = EventData.Event ;
   FirstOfTransId = EventData.TransId[0] ;
   FirstOfAbendCode = EventData.AbendCode[0] ;
   Reply.ReplyUbyte = Data ;
   Reply.ReplySize = EventData.Size ;

   if (Rc == CICS_EPI_NORMAL || Rc == CICS_EPI_ERR_MORE_EVENTS)
   {
      switch (Event)
      {
      case CICS_EPI_EVENT_SEND :

         if ( diagnose || diagnosex) {
           fprintf(stderr,"Message, size=%i, received from CICS\n",
                          EventData.Size) ;
           displayData(Data, EventData.Size) ;
         } /* endif */
         HandleReplyFromCICS(Terminal, TranKey, &Reply) ;
         break;
      case CICS_EPI_EVENT_END_TRAN:
         if (FirstOfTransId != 0)
         {
            /*
             * The transaction ended with RETURN TRANSID(...). The workload
             * driver may want to know this.
             */
            HandleTransIdFromCICS(Terminal,
                                  TranKey,
                                  (const char *) EventData.TransId) ;
         } /* endif */
         if (FirstOfAbendCode != ' ' )
         {
            /*
             * The transaction abended. Inform the workload driver
             */
            HandleAbendFromCICS(Terminal,
                                TranKey,
                                (const char *) EventData.AbendCode
                               ) ;
         } /* endif */
         break;
      case CICS_EPI_EVENT_END_TERM :
         Terminal -> Terminated = 1 ;
         break;
      case CICS_EPI_EVENT_CONVERSE :

         if ( diagnose || diagnosex) {
           fprintf(stderr,"Converse, size=%i, received from CICS\n",
                          EventData.Size) ;
           displayData(Data, EventData.Size) ;
         } /* endif */
         HandleReplyFromCICS(Terminal, TranKey, &Reply) ;
         break;
      default :
         report_warning(Event, "processEvent",TermIndex) ;
      } /* endswitch */


      return Event ;
   }
   else
   {
       report_failure(Rc, "processEvent",CICS_EPI_NORMAL,TermIndex) ;
   } /* endif */

}

static void replyToConverse(
              Terminal_t * Terminal,
              void * TranKey,
              unsigned int ConverseCount
              )
{
   int TermIndex=TerminalTermIndex(Terminal) ;
   Datastream_t Datastream ;
   defaultDatastream(&Datastream) ;
   BuildDataConverse(&Datastream, TranKey, ConverseCount) ;

  /*
   * Diagnostic display of the datastream if required
   */
   if (diagnose || diagnosex)
   {
       fprintf(stderr,
         "Message, size=%i, sending to CICS for conversational reply\n"
         ,Datastream.CurrentSize) ;
       displayData(Datastream.Stream, Datastream.CurrentSize) ;
   } /* endif */

   CHECK(CICS_EpiReply,
         (TermIndex,
          Datastream.Stream,
          Datastream.CurrentSize
         ),
         CICS_EPI_NORMAL,TermIndex) ;
}

void processEventToEnd(Terminal_t * Terminal, void * TranKey)
{
   int Event ;
   unsigned int ConverseCount = 0 ;
   do
   {
      Event = processEvent(Terminal, TranKey) ;
      if (CICS_EPI_EVENT_CONVERSE == Event)
      {
         ConverseCount += 1 ;
         replyToConverse(Terminal, TranKey, ConverseCount ) ;
      } /* endif */
   } while ( Event == CICS_EPI_EVENT_SEND || Event == CICS_EPI_EVENT_CONVERSE ); /* enddo */
}
