/*
 *                                                                    
 * NAME:	cicsecic.c - CICS External Call Interface sample
 *                                                                  
 * VERSION:	1.1
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * DESCRIPION:
 *	This is a sample application which demonstrates how to         
 *	use the External Call Interface (ECI).                            
 *	It makes a single CICS_ExternalCall to determine the state of a
 * 	given region.  It then invokes a cics program which returns the
 *	formatted date and time first syncronously, then asyncronously.
 *	After the asyncronous call, it waits on a condition variable which 
 *	is signalled by the callback once a reply is available.
 *                                                                  
 *	You must fill in the name of your USERID, PASSWORD and SERVER
 *	in the GLOBALS section below.                                  
 *                                                               
 */

/*
 * HEADER FILES
 */
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <pthread.h>
#include <locale.h>

#include <cics_eci.h>
#include <cics_samples.h>

/*
 * Defines
 */
#define ECI_TIMESTAMP_SIZE  18

/*
 * Global Variables
 */
ECI_PARMS	EciParms;
char		Server[9] = "";		/* FILL IN YOUR SERVER HERE   */
char		UserID[9] = "";		/* FILL IN YOUR USER ID HERE  */
char		PassWd[9] = "";		/* FILL IN YOUR PASSWORD HERE */

pthread_cond_t	WaitCondVar;		/* Condition variable and     */
pthread_mutex_t	WaitMutex;		/* mutex for reply wait.      */
cics_bool_t	ReplyIn;		/* flag for pending reply     */

/*
 *  Function Prototypes
 */
int		main		(void);
void		EciStateSync	(void);
void		EciSync		(void);
void		EciAsync	(void);
void		EciGetReply	(void);
void		Response	(char *Call, short Rc, char *Abend);
void CICSEXIT	CallBackProc	(cics_ushort_t idx);

/*
 * main() - Sample Entry Point
 */
int main(void)
{
    puts("ECI Sample Program\n");
    puts("------------------\n\n");

    /*
     * Pick up the locale from the environment
     */
    (void)setlocale(LC_ALL,"");

    /*
     * Initialise pthread mutex and condvar
     */
    pthread_mutex_init(&WaitMutex, pthread_mutexattr_default);
    pthread_cond_init(&WaitCondVar, pthread_condattr_default);
    ReplyIn = FALSE;

    /*
     * Establish the system status
     */
    EciStateSync();

    /*
     * Make a syncronous link call
     */
    EciSync();

    /*
     * Make an asyncronous link call
     */
    EciAsync();

    /*
     * Now get the reply - note that the mutex wait & GET_REPLY 
     * combination could be replaced by a GET_REPLY_WAIT call
     */
    pthread_mutex_lock(&WaitMutex);
    while (!ReplyIn)
    {
	pthread_cond_wait(&WaitCondVar, &WaitMutex);
    }
    pthread_mutex_unlock(&WaitMutex);

    EciGetReply();

    exit(0);
} /* main */

/*
 * EciStateSync
 *
 * Issue a CICS_Externalcall for an ECI_STATE_SYNC
 */
void EciStateSync(void)
{
    char	*Name = "ECI_STATE_SYNC";
    short	 Rc;
    char	 CommArea [ECI_STATUS_LENGTH];
    ECI_STATUS	 EciStatus;
    char	*p, *q, *r;

    printf ("%s test\n", Name);

    memset (CommArea, '\0', ECI_STATUS_LENGTH);
    memset (&EciParms, 0, sizeof (ECI_PARMS));

    EciParms.eci_version                           = ECI_VERSION_1;
    EciParms.eci_call_type                         = ECI_STATE_SYNC;
    memcpy(&EciParms.eci_userid,       UserID,     8);
    memcpy(&EciParms.eci_password,     PassWd,     8);
    memcpy(&EciParms.eci_system_name,  Server,     8);
    EciParms.eci_commarea                          = CommArea;
    EciParms.eci_commarea_length                   = ECI_STATUS_LENGTH;
    EciParms.eci_extend_mode                       = ECI_STATE_IMMEDIATE;
    EciParms.eci_luw_token                         = ECI_LUW_NEW;
    EciParms.eci_timeout                           = 0;

    Rc = CICS_ExternalCall (&EciParms);
    Response(Name, Rc, EciParms.eci_abend_code);

    if (Rc == ECI_NO_ERROR)
    {
	memcpy (&EciStatus, CommArea, ECI_STATUS_LENGTH);
	switch (EciStatus.ConnectionType)
	{
	    case ECI_CONNECTED_NOWHERE:
	    p = "ECI_CONNECTED_NOWHERE";
	    break;
	    case ECI_CONNECTED_TO_SERVER:
	    p = "ECI_CONNECTED_TO_SERVER";
	    break;
	    case ECI_CONNECTED_TO_CLIENT:
	    p = "ECI_CONNECTED_TO_CLIENT";
	    break;
	} /* endswitch */
	printf ("Connection Type: %s\n", p);

	switch (EciStatus.CicsServerStatus)
	{
	    case ECI_SERVERSTATE_UNKNOWN:
	    q = "ECI_SERVERSTATE_UNKNOWN";
	    break;
	    case ECI_SERVERSTATE_UP:
	    q = "ECI_SERVERSTATE_UP";
	    break;
	    case ECI_SERVERSTATE_DOWN:
	    q = "ECI_SERVERSTATE_DOWN";
	    break;
	} /* endswitch */
	printf ("Server Status: %s\n", q);

	switch (EciStatus.CicsClientStatus)
	{
	    case ECI_CLIENTSTATE_UNKNOWN:
	    r = "ECI_CLIENTSTATE_UNKNOWN";
	    break;
	    case ECI_CLIENTSTATE_UP:
	    r = "ECI_CLIENTSTATE_UP";
	    break;
	    case ECI_CLIENTSTATE_INAPPLICABLE:
	    r = "ECI_CLIENTSTATE_INAPPLICABLE";
	    break;
	} /* endswitch */
	printf ("Client Status: %s\n", r);
    } /* endif */

    printf ("%s test complete\n\n", Name);

    return;
} /* EciStateSync */

/*
 * EciSync
 *
 * Issue a CICS_Externalcall for an ECI_SYNC link
 */
void EciSync(void)
{
    char	*Name = "ECI_SYNC";
    short	 Rc;
    char	 CommArea [ECI_TIMESTAMP_SIZE];

    printf ("%s test\n", Name);

    memset (CommArea, '\0', ECI_TIMESTAMP_SIZE);
    memset (&EciParms, 0, sizeof (ECI_PARMS));

    EciParms.eci_version                           = ECI_VERSION_1;
    EciParms.eci_call_type                         = ECI_SYNC;
    memcpy(&EciParms.eci_program_name, "ECIPROG",  7);
    memcpy(&EciParms.eci_userid,       UserID,     8);
    memcpy(&EciParms.eci_password,     PassWd,     8);
    memcpy(&EciParms.eci_system_name,  Server,     8);
    EciParms.eci_commarea                          = CommArea;
    EciParms.eci_commarea_length                   = ECI_TIMESTAMP_SIZE;
    EciParms.eci_extend_mode                       = ECI_NO_EXTEND;
    EciParms.eci_luw_token                         = ECI_LUW_NEW;
    EciParms.eci_timeout                           = 0;

    Rc = CICS_ExternalCall (&EciParms);
    Response(Name, Rc, EciParms.eci_abend_code);

    if (Rc == ECI_NO_ERROR)
    {
	CommArea[(ECI_TIMESTAMP_SIZE-1)] = '\0';
	printf ("CommArea Returned: %s\n", CommArea);
    } /* endif */

    printf ("%s test complete\n", Name);

    return;
} /* EciSync */

/*
 * EciAsync
 *
 * Issue a CICS_Externalcall for an ECI_ASYNC link
 */
void EciAsync(void)
{
    char	*Name = "ECI_ASYNC";
    short	 Rc;
    char	 CommArea [ECI_TIMESTAMP_SIZE];

    printf ("%s test\n", Name);

    memset (CommArea, '\0', ECI_TIMESTAMP_SIZE);
    memset (&EciParms, 0, sizeof (ECI_PARMS));

    EciParms.eci_version                           = ECI_VERSION_1;
    EciParms.eci_call_type                         = ECI_ASYNC;
    memcpy(&EciParms.eci_program_name, "ECIPROG",  7);
    memcpy(&EciParms.eci_userid,       UserID,     8);
    memcpy(&EciParms.eci_password,     PassWd,     8);
    memcpy(&EciParms.eci_system_name,  Server,     8);
    EciParms.eci_commarea                          = CommArea;
    EciParms.eci_commarea_length                   = ECI_TIMESTAMP_SIZE;
    EciParms.eci_extend_mode                       = ECI_NO_EXTEND;
    EciParms.eci_luw_token                         = ECI_LUW_NEW;
    EciParms.eci_timeout                           = 0;
    EciParms.eci_message_qualifier		   = 77;
    EciParms.eci_callback = (CICS_EciNotify_t)CallBackProc;

    Rc = CICS_ExternalCall (&EciParms);
    Response(Name, Rc, EciParms.eci_abend_code);

    printf ("%s test complete\n\n", Name);

    return;
} /* EciAsync */

/*
 * EciGetReply
 *
 * Issue a CICS_Externalcall for an ECI_GETREPLY query
 */
void EciGetReply(void)
{
    char	*Name = "ECI_GETREPLY";
    short	 Rc;
    char	 CommArea [ECI_TIMESTAMP_SIZE];

    printf ("%s test\n", Name);

    memset (CommArea, '\0', ECI_TIMESTAMP_SIZE);
    memset (&EciParms, 0, sizeof (ECI_PARMS));

    EciParms.eci_version                           = ECI_VERSION_1;
    EciParms.eci_call_type                         = ECI_GET_REPLY;
    memcpy(&EciParms.eci_userid,       UserID,     8);
    memcpy(&EciParms.eci_password,     PassWd,     8);
    memcpy(&EciParms.eci_system_name,  Server,     8);
    EciParms.eci_commarea                          = CommArea;
    EciParms.eci_commarea_length                   = ECI_TIMESTAMP_SIZE;
    EciParms.eci_extend_mode                       = ECI_NO_EXTEND;
    EciParms.eci_luw_token                         = ECI_LUW_NEW;
    EciParms.eci_timeout                           = 0;

    Rc = CICS_ExternalCall (&EciParms);
    Response(Name, Rc, EciParms.eci_abend_code);

    if (Rc == ECI_NO_ERROR)
    {
	CommArea[(ECI_TIMESTAMP_SIZE-1)] = '\0';
	printf ("CommArea Returned: %s\n", CommArea);
    } /* endif */

    printf ("%s complete\n\n", Name);

    return;
} /* EciGetReply */

/*
 * CallBackProc
 * This is the function that was registered on the ECI_ASYNC call.
 * It gets called when there is a reply ready.
 * It is passed the message qualifier of the ready message.
 */
void CICSEXIT CallBackProc (cics_ushort_t idx)
{
    printf("Callback received for message number %d\n", idx);

    /*
     * Set the flag and free any waiters
     */
    pthread_mutex_lock(&WaitMutex);
    ReplyIn = TRUE;
    pthread_cond_signal(&WaitCondVar);
    pthread_mutex_unlock(&WaitMutex);

    return;
} /* CallBackProc */

/*
 * Response
 *
 * Display the immediate response code from an ECI call
 *
 */
void Response(char *Call, short Rc, char *Abend)
{
    char             *p;

    switch (Rc)
    {
	case ECI_NO_ERROR:
	    p = "ECI_NO_ERROR               ";break;
	case ECI_ERR_INVALID_DATA_LENGTH:
	    p = "ECI_ERR_INVALID_DATA_LENGTH";break;
	case ECI_ERR_INVALID_EXTEND_MODE:
	    p = "ECI_ERR_INVALID_EXTEND_MODE";break;
	case ECI_ERR_NO_CICS:
	    p = "ECI_ERR_NO_CICS            ";break;
	case ECI_ERR_CICS_DIED:
	    p = "ECI_ERR_CICS_DIED          ";break;
	case ECI_ERR_REQUEST_TIMEOUT:
	    p = "ECI_ERR_REQUEST_TIMEOUT    ";break;
	case ECI_ERR_RESPONSE_TIMEOUT:
	    p = "ECI_ERR_RESPONSE_TIMEOUT   ";break;
	case ECI_ERR_TRANSACTION_ABEND:
	    p = "ECI_ERR_TRANSACTION_ABEND  ";break;
	case ECI_ERR_EXEC_NOT_RESIDENT:
	    p = "ECI_ERR_EXEC_NOT_RESIDENT  ";break;
	case ECI_ERR_SYSTEM_ERROR:
	    p = "ECI_ERR_SYSTEM_ERROR       ";break;
	case ECI_ERR_NULL_WIN_HANDLE:
	    p = "ECI_ERR_NULL_WIN_HANDLE    ";break;
	case ECI_ERR_NULL_MESSAGE_ID:
	    p = "ECI_ERR_NULL_MESSAGE_ID    ";break;
	case ECI_ERR_THREAD_CREATE_ERROR:
	    p = "ECI_ERR_THREAD_CREATE_ERROR";break;
	case ECI_ERR_INVALID_CALL_TYPE:
	    p = "ECI_ERR_INVALID_CALL_TYPE  ";break;
	case ECI_ERR_ALREADY_ACTIVE:
	    p = "ECI_ERR_ALREADY_ACTIVE     ";break;
	case ECI_ERR_RESOURCE_SHORTAGE:
	    p = "ECI_ERR_RESOURCE_SHORTAGE  ";break;
	case ECI_ERR_NO_SESSIONS:
	    p = "ECI_ERR_NO_SESSIONS        ";break;
	case ECI_ERR_NULL_SEM_HANDLE:
	    p = "ECI_ERR_NULL_SEM_HANDLE    ";break;
	case ECI_ERR_INVALID_DATA_AREA:
	    p = "ECI_ERR_INVALID_DATA_AREA  ";break;
	case ECI_ERR_INVALID_VERSION:
	    p = "ECI_ERR_INVALID_VERSION    ";break;
	case ECI_ERR_UNKNOWN_SERVER:
	    p = "ECI_ERR_UNKNOWN_SERVER     ";break;
	case ECI_ERR_CALL_FROM_CALLBACK:
	    p = "ECI_ERR_CALL_FROM_CALLBACK ";break;
	case ECI_ERR_INVALID_TRANSID:
	    p = "ECI_ERR_INVALID_TRANSID    ";break;
	case ECI_ERR_MORE_SYSTEMS:
	    p = "ECI_ERR_MORE_SYSTEMS       ";break;
	case ECI_ERR_NO_SYSTEMS:
	    p = "ECI_ERR_NO_SYSTEMS         ";break;
	case ECI_ERR_SECURITY_ERROR:
	    p = "ECI_ERR_SECURITY_ERROR     ";break;
	case ECI_ERR_MAX_SYSTEMS:
	    p = "ECI_ERR_MAX_SYSTEMS        ";break;
	case ECI_ERR_MAX_SESSIONS:
	    p = "ECI_ERR_MAX_SESSIONS       ";break;
	case ECI_ERR_ROLLEDBACK:
	    p = "ECI_ERR_ROLLEDBACK         ";break;
	default:
       {
          p = "!!!Unknown!!!";
          printf ("Unknown Return Code : %d\n", Rc);
       } /* endcase */
       break;
    } /* endswitch */

    printf ("%s returned: %s\n", Call, p);
    if (Rc != ECI_NO_ERROR)
    {
	printf("Abend code was \"%4.4s\"\n", Abend);
    } /* endif */

    if (Rc == ECI_ERR_SYSTEM_ERROR)
    {
        printf ("SysErr returns: %d(0x%04.4X)\n",
		EciParms.eci_sys_return_code, EciParms.eci_sys_return_code);
    } /* endif */

    return;
} /* Response */
