/*
 * NAME:        ECI Exit Library - Sample code
 *
 * VERSION:     1.1
 *
 * COPYRIGHT:                                                        
 * 
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * DESCRIPTION:
 *
 *      This is a sample ECI exits library.  It performs no useful function
 *      but does provide a skeleton containing all the exit functions 
 *      with details of valid return codes and their meanings.
 *
 *      When writing exits, the following rules must be obeyed:
 *      1. No EPI or ECI calls must be made from the exit
 *      2. Exits should contain the minimum possible executable code
 *         (to minimise impact on performance)
 *      3. All exit code must be re-entrant and thread safe.
 *      4. The primary entry point to this module must be named
 *         CICS_EciExitInit.  The names of the actual exits may
 *         be changed but the parameter lists must not be changed.

 *
 * NOTE:
 *
 *	If compiled without change, this file can be used to build a library
 *	which is functionally identical to the libcicseciexit shipped with 
 *	the CICS product.
 */

#include <cicstype.h>
#include <cics_eci.h>
#include <cicseciexit.h>

/*
 * Main entry point of dynamically loaded object.
 * This function returns a structure of addresses of exit functions.
 * Note that ALL exits must be defined if any are defined.
 * If any exit address is set to NULL no exits will be called.
 */
void CICS_EciExitInit(OUT CICS_EciExitList_t *ExitList)

{
    ExitList->InitialiseExit    = (void *)&CICS_EciInitialiseExit;
    ExitList->TerminateExit     = (void *)&CICS_EciTerminateExit;
    ExitList->ExternalCallExit1 = (void *)&CICS_EciExternalCallExit1;
    ExitList->ExternalCallExit2 = (void *)&CICS_EciExternalCallExit2;
    ExitList->SystemIdExit      = (void *)&CICS_EciSystemIdExit;
    ExitList->DataSendExit      = (void *)&CICS_EciDataSendExit;
    ExitList->DataReturnExit    = (void *)&CICS_EciDataReturnExit;
}

/*
 * InitialiseExit:
 * Called immediately after successful initialisation.
 *
 * Permitted return values:
 *   CICS_EXIT_OK		No problems - call exits where appropriate
 *   CICS_EXIT_NO_EXITS		No problems - do not call subsequent exits
 *				    (used by shipped dummy library)
 *   CICS_EXIT_CANT_INIT_EXITS  Exits cannot be used due to an 
 *				    (unspecified) error condition
 *   CICS_EXIT_USER_BASE + N	User specified value
 *
 * Notes:
 *   Any value except CICS_EXIT_OK will inhibit the calling of subsequent exits
 */
cics_sshort_t CICS_EciInitialiseExit( IN  cics_sshort_t     Version
				    , OUT CICS_EciAnchor_t *Anchor
				    )

{
        *Anchor = (void *)(-1);
        return CICS_EXIT_NO_EXITS;
}

/*
 * TerminateExit:
 * Called immediately prior to the destruction of the 
 * ECI data structures in the atexit processing.
 *
 * Permitted return values:
 *   CICS_EXIT_OK               No problems
 *   CICS_EXIT_BAD_ANCHOR	Invalid anchor value supplied
 *   CICS_EXIT_BAD_STORAGE	Exit cannot tidy up its storage
 *   CICS_EXIT_USER_BASE + N    User specified value           
 */
cics_sshort_t CICS_EciTerminateExit( IN CICS_EciAnchor_t Anchor )
{
	return CICS_EXIT_OK;
}

/*
 * SystemIdExit:
 * Called if the ECI would return ECI_ERR_NO_CICS, ECI_ERR_UNKNOWN_SERVER
 * or ECI_ERROR_SEC_ERROR.  This allows the choice of an alternative 
 * system id.  This is the only exit allowed to request a retry.
 * This exit will be called repeatedly until it returns a system which can
 * successfully fulfil the request or until it returns CICS_EXIT_OK.
 * CICS_EXIT_GIVE_UP can be used to force a non-error return from this loop.
 *
 *  Permitted return values:
 *   CICS_EXIT_OK               No problems
 *   CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
 *   CICS_EXIT_BAD_PARM         One or more parameters bad
 *   CICS_EXIT_GIVE_UP		There are no more alternative systems
 *   CICS_EXIT_USER_BASE + N    User specified value
 */
cics_sshort_t CICS_EciSystemIdExit
                        ( CICS_EciAnchor_t Anchor
                        , IN    CICS_EciToken_t  Token
                        , INOUT PECI_PARMS       ParmPtr
                        , IN    cics_sshort_t    Reason
                        )
{
        return CICS_EXIT_GIVE_UP;
}

/*
 * ExternalCallExit1:
 * Called after parameter verification for all LINK and STATUS calls.
 * This exit allows system selection.
 *
 *  Permitted return values:
 *   CICS_EXIT_OK               No problems
 *   CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
 *   CICS_EXIT_BAD_PARM         One or more parameters bad
 *   CICS_EXIT_USER_BASE + N    User specified value
 */
cics_sshort_t CICS_EciExternalCallExit1
                        ( IN    CICS_EciAnchor_t Anchor
                        , IN    CICS_EciToken_t  Token
                        , INOUT PECI_PARMS       ParmPtr
                        )
{
        return CICS_EXIT_OK;
}

/*
 * ExternalCallExit2:
 * Called prior to returning from all LINK and STATUS calls.
 *
 *  Permitted return values:
 *   CICS_EXIT_OK               No problems
 *   CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
 *   CICS_EXIT_BAD_PARM         One or more parameters bad
 *   CICS_EXIT_USER_BASE + N    User specified value
 */
cics_sshort_t CICS_EciExternalCallExit2
                        ( IN    CICS_EciAnchor_t Anchor
                        , IN    CICS_EciToken_t  Token
                        , INOUT PECI_PARMS       ParmPtr
                        )
{
        return CICS_EXIT_OK;
}

/*
 * DataSendExit:
 * Called immediately before data is sent to the host.
 *
 *  Permitted return values:
 *   CICS_EXIT_OK               No problems
 *   CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
 *   CICS_EXIT_BAD_PARM         One or more parameters bad
 *   CICS_EXIT_USER_BASE + N    User specified value
 */
cics_sshort_t CICS_EciDataSendExit
                        ( IN    CICS_EciAnchor_t Anchor
                        , IN    CICS_EciToken_t  Token
                        )
{
        return CICS_EXIT_OK;
}

/*
 * DataReturnExit:
 * Called immediately after data is received from the host.
 *
 *  Permitted return values:
 *   CICS_EXIT_OK               No problems
 *   CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
 *   CICS_EXIT_BAD_PARM         One or more parameters bad
 *   CICS_EXIT_USER_BASE + N    User specified value
 */
cics_sshort_t CICS_EciDataReturnExit
                        ( IN    CICS_EciAnchor_t Anchor
                        , IN    CICS_EciToken_t  Token
                        , INOUT PECI_PARMS       ParmPtr
                        )
{
        return CICS_EXIT_OK;
}
