/*
 * NAME:        EPI Sample 1 - cicsepi1a.c
 *
 * VERSION:     1.1
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * DESCRIPTION:
 *
 *      This is a sample EPI application program for CICS.  It takes
 *      one parameter, the name of a CICS region, and adds a single
 *      EPI terminal to that region.  It then runs the transaction EPI1
 *      against the terminal and displays the data sent by the transaction.
 *
 *      The EPI1 transaction simply does an EXEC CICS SEND of "Hello World"
 *      and terminates.
 *
 *      This sample demonstrates simple use of the following EPI functions:
 *
 *      - CICS_EpiInitialize()
 *      - CICS_EpiTerminate()
 *      - CICS_EpiAddTerminal()
 *      - CICS_EpiDelTerminal()
 *      - CICS_EpiStartTran()
 *      - CICS_EpiGetEvent()
 *      - CICS_EpiGetSysError()
 *
 *      It also illustrates processing of the following EPI events:
 *
 *      - CICS_EPI_EVENT_SEND
 *      - CICS_EPI_EVENT_END_TRAN
 *      - CICS_EPI_EVENT_END_TERM
 *
 *      For simplicity a secondary thread is started to handle all EPI
 *      event processing and condition variables are used to coordinate
 *      with the main thread.
 */

#include <cicstype.h>
#include <stdio.h>
#include <pthread.h>
#include <locale.h>

#include <cics_epi.h>           /* Include the EPI header file */
#include <cics_samples.h>

/*
 * This is a piece of dummy 3270 data (an ENTER AID key followed by a
 * cursor position of (0,0)).  It is used when starting transactions.
 */

#define DUMMY_DATA ((cics_ubyte_t *)"'  ")

/*
 * Declare the functions used in this program.
 */

void ErrorExit(cics_char_t *Function, cics_sshort_t Rc, cics_ushort_t TermIndex);
void *EventThread(void *Parm);
void DisplayData(cics_ubyte_t *Data, cics_ushort_t Size);


/*********************************************************************/
/* Condition Variables and Mutexes used for thread coordination      */
/*********************************************************************/

pthread_cond_t   TranEnded;
pthread_cond_t   TermEnded;
pthread_mutex_t  Mutex;
cics_bool_t	 TranEnd_Flag = FALSE;
cics_bool_t	 TermEnd_Flag = FALSE;


/*********************************************************************/
/* ErrorExit() - Report errors and exit                              */
/*********************************************************************/
/* This function is called to report serious and unexpected EPI      */
/* errors.  It will then terminate the program.                      */

void ErrorExit(cics_char_t *Function, cics_sshort_t Rc, cics_ushort_t TermIndex)
{
  CICS_EpiSysError_t SysErr;            /* Receives EPI error info.  */

   fprintf(stderr, "%s() call failed ", Function);
   if (Rc != CICS_EPI_ERR_FAILED)
   {
      fprintf(stderr, "with return code: %d\n", Rc);
   }
   else
   {
      CICS_EpiGetSysError(TermIndex, &SysErr);

      fprintf(stderr, "with a system error.\n");
      fprintf(stderr, "Cause: %u, Error: %u, Message: %s\n",
                      SysErr.Cause, SysErr.Value,
                      (SysErr.Msg != NULL) ? SysErr.Msg : "<none>");
   }

  exit(1);
}


/*********************************************************************/
/* EventThread() - Event processing thread                           */
/*********************************************************************/
/* This function is run in a secondary thread and handles all the    */
/* EPI event processing for the program.  It calls CICS_EpiGetEvent()*/
/* in blocking mode to wait for the arrival of EPI events.           */

void *EventThread(void *Parm)
{
  cics_sshort_t Rc;
  cics_ushort_t TermIndex;                  /* Installed TermIndex        */
  CICS_EpiEventData_t EventData;        /* Receives EPI event details */
  cics_ubyte_t Data[200];                   /* Receives event data        */

  TermIndex = *((cics_ushort_t *)Parm);

  /*
   * This thread runs in a loop receiving and processing events until
   * the final CICS_EPI_EVENT_END_TERM is received.
   */

   do
   {
      /*
       * Wait for next event.
       */

       EventData.Data = Data;           /* Point to space for any    */
       EventData.Size = sizeof(Data);   /* incoming datastream.      */

       Rc = CICS_EpiGetEvent(TermIndex,
                             CICS_EPI_WAIT,   /* Wait for next event */
                             &EventData);

      /*
       * Return codes of CICS_EPI_NORMAL and CICS_EPI_ERR_MORE_EVENTS both mean
       * the function completed successfully and obtained an EPI event.
       */

       if (Rc == CICS_EPI_NORMAL || Rc == CICS_EPI_ERR_MORE_EVENTS)
       {
          switch (EventData.Event)
          {
            /*
             * ENDTERMINAL: Nothing needs to be done here.  The event
             *              processing loop will end.
             */

             case CICS_EPI_EVENT_END_TERM:
                break;

            /*
             * SEND: Display the data received from a transaction.
             */

             case CICS_EPI_EVENT_SEND:
                DisplayData(EventData.Data, EventData.Size);
                break;

            /*
             * ENDTRAN: Signal the "TranEnded" condition variable to tell
             *          the main thread to proceed.
             */

             case CICS_EPI_EVENT_END_TRAN:
		pthread_mutex_lock(&Mutex);
		TranEnd_Flag = TRUE;
                pthread_cond_signal(&TranEnded);
	        pthread_mutex_unlock(&Mutex);
                break;

            /*
             * Display a warning for unexpected events.
             */

             default:
                printf("Unexpected event received (%d)\n", EventData.Event);
                break;
          }
       }
       else
       {
          ErrorExit("CICS_EpiGetEvent", Rc, TermIndex);
       }

   } while (EventData.Event != CICS_EPI_EVENT_END_TERM);

  /*
   * Drop through to here when the terminal has ended.  Display a message
   * if the terminal ended for an unexpected reason and signal the
   * "TermEnded" condition variable to tell the main thread to exit.
   */

   switch (EventData.EndReason)
   {
      case CICS_EPI_END_SIGNOFF:
         break;                         /* This is what we expect    */

      case CICS_EPI_END_FAILED:
         printf("Terminal Delete ABENDed (\"%s\")\n", EventData.AbendCode);
         break;

      default:
         printf("Unexpected terminal end (%d)\n", EventData.EndReason);
         break;
   }

   pthread_mutex_lock(&Mutex);
   TermEnd_Flag = TRUE;
   pthread_cond_signal(&TermEnded);
   pthread_mutex_unlock(&Mutex);

  return;
}


/*********************************************************************/
/* DisplayData() - Show data sent from a transaction                 */
/*********************************************************************/
/* This function will display the data sent to the EPI from a CICS   */
/* transaction.  The first two bytes of the data are a 3270 command  */
/* byte and the 3270 WCC, these are ignored.                         */

void DisplayData(cics_ubyte_t *Data, cics_ushort_t Size)
{
  cics_ushort_t Index;                      /* Loop counter              */

   if (Size > 0)
   {
      printf("The following message was received from CICS:\n");
      printf("\"");
      for (Index = 2; Index < Size; Index++)
         printf("%c", Data[Index]);
      printf("\"\n");
   }

  return;
}


/*********************************************************************/
/* main() - Sample Entry Point                                       */
/*********************************************************************/
/* This program should be called with the name of a CICS region.  If */
/* none is supplied it will exit with an error, otherwise it will    */
/* install and EPI terminal and run the EPI1 transaction.  When the  */
/* transaction ends the terminal is deleted.                         */

main(int argc, cics_char_t *argv[])
{
  cics_sshort_t Rc;                             /* EPI function return codes */
  cics_ushort_t TermIndex;                  /* EPI Terminal Index        */
  pthread_t Thread;                     /* Secondary thread id       */

  /*
   * Check the caller supplied the name of a CICS region to be used.
   * Give up if we haven't got one.
   */

   if (argc < 2)
   {
      fprintf(stderr, "A CICS region name must be specified.\n");
      exit(1);
   }

  /*
   * Pick up the locale from the environment
   */
   (void)setlocale(LC_ALL,"");

  /*
   * Initialize the condition variables and mutexes used for thread
   * coordination.
   */

   pthread_mutex_init(&Mutex, pthread_mutexattr_default);
   pthread_cond_init(&TermEnded, pthread_condattr_default);
   pthread_cond_init(&TranEnded, pthread_condattr_default);

  /*
   * Initialize the EPI.
   */

   Rc = CICS_EpiInitialize(CICS_EPI_VERSION_101);
   if (Rc != CICS_EPI_NORMAL)
   {
      ErrorExit("CICS_EpiInitialize", Rc, CICS_EPI_TERM_INDEX_NONE);
   }

  /*
   * Install a terminal against the specified region.
   * Specify the terminal model to avoid an unsuitable default.
   */

   Rc = CICS_EpiAddTerminal(NULL,       /* No Namespace needed       */
                            argv[1],    /* Use region specified      */
                            NULL,       /* Autoinstall the terminal  */
                            "hft",      /* High function terminal    */
                            NULL,       /* No event notification     */
                            NULL,       /* TermDetails not needed    */
                            &TermIndex); /* Receives new TermIndex   */
   if (Rc != CICS_EPI_NORMAL)
   {
      ErrorExit("CICS_EpiAddTerminal", Rc, CICS_EPI_TERM_INDEX_NONE);
   }

  /*
   * Once the terminal is installed start up the secondary event
   * processing thread.  If this fails just terminate the EPI and exit.
   */

   if (pthread_create(&Thread, pthread_attr_default,
                      EventThread, &TermIndex) == -1)
   {
      fprintf(stderr, "Cannot start event processing thread.\n");
      Rc = CICS_EpiTerminate();
      exit(1);
   }

  /*
   * Submit the EPI1 transaction and wait for it to complete.  This is
   * done by waiting on the "TranEnded" condition variable, signalled
   * by the event thread.  Some initial 3270 data must be provided so
   * use the DUMMY_DATA.
   */

   Rc = CICS_EpiStartTran(TermIndex,
                          "EPI1",
                          DUMMY_DATA,
                          sizeof(DUMMY_DATA)-1);
   if (Rc != CICS_EPI_NORMAL)
   {
      ErrorExit("CICS_EpiStartTran", Rc, TermIndex);
   }

   pthread_mutex_lock(&Mutex);
   while (!TranEnd_Flag)
   {
       pthread_cond_wait(&TranEnded, &Mutex);
   }
   pthread_mutex_unlock(&Mutex);

  /*
   * The transaction has finished.  The terminal is now deleted and the
   * "TermEnded" condition variable used to wait for deletion to complete.
   */

   Rc = CICS_EpiDelTerminal(TermIndex);
   if (Rc != CICS_EPI_NORMAL)
   {
      ErrorExit("CICS_EpiDelTerminal", Rc, TermIndex);
   }

   pthread_mutex_lock(&Mutex);
   while (!TermEnd_Flag)
   {
       pthread_cond_wait(&TermEnded, &Mutex);
   }
   pthread_mutex_unlock(&Mutex);

  /*
   * Finally terminate the EPI and the application.
   * Clean up mutexes and condition variables
   */

   Rc = CICS_EpiTerminate();
   if (Rc != CICS_EPI_NORMAL)
   {
      ErrorExit("CICS_EpiTerminate", Rc, CICS_EPI_TERM_INDEX_NONE);
   }

   pthread_mutex_destroy(&Mutex);
   pthread_cond_destroy(&TranEnded);
   pthread_cond_destroy(&TermEnded);

   exit(0);
 }

/* EOF */
