/*
 * NAME:        EPI Sample 3 - cicsepi3a.c
 *
 * VERSION:     1.1
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * DESCRIPTION:
 *
 *      This is a sample EPI application program for CICS.  It shows
 *      a list of available CICS regions.  It does not require any
 *      CICS transactions to run.
 *
 *      This sample demonstrates simple use of the following EPI functions:
 *
 *      - CICS_EpiInitialize()
 *      - CICS_EpiTerminate()
 *      - CICS_EpiListSystems()
 *      - CICS_EpiGetSysError()
 */

#include <cicstype.h>
#include <stdio.h>
#include <locale.h>

#include <cics_epi.h>           /* Include the EPI header file */

/*
 * Declare the functions used in this program.
 */

#define LIST_SIZE 20

void ErrorExit(cics_char_t *Function, cics_sshort_t Rc, cics_ushort_t TermIndex);


/*********************************************************************/
/* ErrorExit() - Report errors and exit                              */
/*********************************************************************/
/* This function is called to report serious and unexpected EPI      */
/* errors.  It will then terminate the program.                      */

void ErrorExit(cics_char_t *Function, cics_sshort_t Rc, cics_ushort_t TermIndex)
{
  CICS_EpiSysError_t SysErr;            /* Receives EPI error info.  */

   fprintf(stderr, "%s() call failed ", Function);
   if (Rc != CICS_EPI_ERR_FAILED)
   {
      fprintf(stderr, "with return code: %d\n", Rc);
   }
   else
   {
      CICS_EpiGetSysError(TermIndex, &SysErr);

      fprintf(stderr, "with a system error.\n");
      fprintf(stderr, "Cause: %u, Error: %u, Message: %s\n",
                      SysErr.Cause, SysErr.Value,
                      (SysErr.Msg != NULL) ? SysErr.Msg : "<none>");
   }

  exit(1);
}


/*********************************************************************/
/* main() - Sample Entry Point                                       */
/*********************************************************************/
/* This program should be called with no parameters.  It will show   */
/* a list of available CICS regions.                                 */

main(void)
{
  cics_sshort_t Rc;                             /* EPI function return codes */
  cics_ushort_t Count;                      /* Number of systems         */
  cics_ushort_t i;                          /* Loop counter              */
  CICS_EpiSystem_t List[LIST_SIZE];     /* Receives system list      */

  /*
   * Pick up the locale from the environment
   */
   (void)setlocale(LC_ALL,"");

  /*
   * Initialize the EPI.
   */

   Rc = CICS_EpiInitialize(CICS_EPI_VERSION_101);
   if (Rc != CICS_EPI_NORMAL)
   {
      ErrorExit("CICS_EpiInitialize", Rc, CICS_EPI_TERM_INDEX_NONE);
   }

  /*
   * Obtain a list of CICS regions.  The constant LIST_SIZE defines
   * the number of slots in our CICS_EpiSystem_t array, this is the
   * maximum number that the EPI will return.
   */

   printf("Requesting the list of available CICS regions...\n\n");

   Count = LIST_SIZE;                   /* Maximum number of results */

   Rc = CICS_EpiListSystems(NULL,       /* No Namespace required     */
                            &Count,     /* Updated with real count   */
                            List);      /* Resulting region list     */

   if (Rc == CICS_EPI_ERR_NO_SYSTEMS)
   {
      printf("No CICS regions are available.\n");
   }
   else if (Rc == CICS_EPI_ERR_MORE_SYSTEMS)
   {
      printf("There are a possible %u CICS regions available.\n", Count);
      printf("Only the first %u can be listed.\n\n", LIST_SIZE);
   }
   else if (Rc != CICS_EPI_NORMAL)
   {
      ErrorExit("CICS_EpiListSystems", Rc, CICS_EPI_TERM_INDEX_NONE);
   }

  /*
   * Print a listing of the regions returned.
   */

   if (Count > 0)
   {
      printf("Region Name   Description\n");
      printf("-----------   -----------\n");
      for (i = 0; i < Count; i++)
      {
         printf("%-8s      %s\n", List[i].SystemName, List[i].Description);
      }
   }

  /*
   * Terminate the EPI and exit.
   */

   Rc = CICS_EpiTerminate();
   if (Rc != CICS_EPI_NORMAL)
   {
      ErrorExit("CICS_EpiTerminate", Rc, CICS_EPI_TERM_INDEX_NONE);
   }

   exit(0);
 }

/* EOF */
