/*
 * NAME:	EPI multi-terminal sample - cicsepimt.c
 *
 *
 * VERSION:	1.1
 *
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * DESCRIPTION:
 *
 *      This is a sample EPI application program for CICS. It
 *      demonstrates how multiple terminals can be supported from an EPI
 *      application.
 *      This sample demonstrates, or can be used to demonstrate, use of the
 *      following EPI functions:
 *
 *      - CICS_EpiInitialize()
 *      - CICS_EpiTerminate()
 *      - CICS_EpiAddTerminal()
 *      - CICS_EpiDelTerminal()
 *      - CICS_EpiStartTran()
 *      - CICS_EpiGetEvent()
 *      - CICS_EpiGetSysError()
 *
 *      It also illustrates, or can be used to illustrate, processing of
 *      the following EPI events:
 *
 *      - CICS_EPI_EVENT_CONVERSE
 *      - CICS_EPI_EVENT_SEND
 *      - CICS_EPI_EVENT_START_ATI
 *      - CICS_EPI_EVENT_END_TRAN
 *      - CICS_EPI_EVENT_END_TERM
 *
 *      For simplicity, a secondary thread is started to handle all EPI
 *      event processing.
 *
 *      This sample can use one or more physical terminal devices, or one
 *      or more windows on an X-terminal. The instructions for using the
 *      sample application from an X-terminal are as follows:
 *
 *      (1)  Issue "dce_login" to authenticate as a DCE principal
 *      (2)  Issue "cicsepimt" to invoke the sample in the control window
 *      (3)  Open the required number of user windows. The sample supports
 *           a total of 10 terminals by default
 *      (4)  Issue "tty; sleep 999" in each user window. Make a note of each
 *           user window's device identifier
 *      (5)  In response to the "REGION?" prompt on the control window, enter
 *           either the name of the CICS region to connect to, or specify
 *           the device identifier of another window that you wish cicsepimt
 *           to control, e.g. /dev/pts/5
 *      (6)  Repeat step (5) for each additional user window required
 *      (7)  In response to the "REGION?" prompt on the user window, enter
 *           the name of the CICS region to connect to
 *      (8)  Repeat step (7) for all the user windows. Note that the sample
 *           program only allows one terminal to autoinstall at a time
 *      (9)  In response to the "TRAN?" prompt on the user window, enter
 *           the name of the CICS transaction to run
 *      (10) Interact with the transaction in response to the "CONVERSE?"
 *           prompt
 *           Restrictions:
 *           a.  only the top line of the screen (containing the prompt)
 *               can be used for input
 *           b.  PF keys are not supported directly, but are simulated via the
 *               ESCAPE key, e.g. <ESC>3 for PF3, <ESC>0 for PF10, <ESC>- for
 *               PF11, and <ESC>= for PF12
 *      (11) To exit from a user window, issue EOF (<CTRL>D) in response to
 *           the "TRAN?" prompt, and then interrupt the sleep command (<CTRL>C)
 *      (12) To exit from cicsepimt, exit from all the user windows and issue
 *           EOF in the control window in response to the "TRAN?" prompt
 */

#include <cicstype.h>
#include <stdio.h>
#include <stdarg.h>
#include <curses.h>		/* Prerequsite of term.h */
#include <term.h>		/* term structure definition */

#include <locale.h>
#include <pthread.h>
#include <dfhaid.h>		/* CICS 3270 AID values */
#include <cics_epi.h>		/* Include the EPI header file */


/*********************************************************************/
/* Define constants used within the program.                         */
/*********************************************************************/

#define COLS		80
#define ROWS		25
#define MAX_TERMS	10

#define	CMD_W		0x31	/* Write */
#define	CMD_RB		0x32	/* Read Buffer */
#define	CMD_EW		0x35	/* Erase/Write */
#define	CMD_RM		0x36	/* Read Modified */
#define	CMD_EWA		0x3D	/* Erase/Write Alternate */
#define	CMD_RMA		0x3E	/* Read Modified All */
#define	CMD_EAU		0x3F	/* Erase All Unprotected */

#define	ORDER_GE	0x08	/* Graphic Escape */
#define	ORDER_PT	0x09	/* Program Tab */
#define	ORDER_SFE	0x10	/* Start Field Extended */
#define	ORDER_SBA	0x11	/* Set Buffer Address */
#define	ORDER_EUA	0x12	/* Erase Unprotected to Address */
#define	ORDER_IC	0x13	/* Insert Cursor */
#define	ORDER_RA	0x14	/* Repeat to Address */
#define	ORDER_MF	0x1A	/* Modify Field */
#define ORDER_SF	0x1D	/* Start Field */
#define	ORDER_SA	0x1F	/* Set Attribute */

/*********************************************************************/
/* Declare the functions and globals used in this program.           */
/*********************************************************************/

typedef struct
{
    FILE *fp;
    int pos;
    cics_char_t display[COLS * ROWS];
} screen;

void scr_clear(screen *s);
void scr_clrtoeol(screen *s);
void scr_addstr(screen *s, cics_char_t *str);
void scr_refresh(screen *s);

void Init(cics_char_t *openmsg);
void ChangeTTY(cics_char_t *tty);
void SetInput(FILE *fp, cics_ushort_t TermIndex);
FILE *GetInput(cics_char_t *Buffer, cics_ushort_t *TermIndexP);

#define scr_move(s, r, c)		(s)->pos = ((r) * COLS + (c))
#define scr_addch(s, ch)		(s)->display[(s)->pos++] = (ch)
#define scr_mvaddch(s, r, c, ch)	scr_move(s, r, c), scr_addch(s, ch)

enum State
{
    STATE_IDLE,
    STATE_BUSY,
    STATE_CONVERSE
};

struct TheTermData
{
    enum State State;
    cics_char_t NextTran[CICS_EPI_TRANSID_MAX+1];
    FILE *fp;
};

static struct TheTermData TermData[MAX_TERMS];

static cics_ubyte_t GraphicToBinary[] =
{
/*  0,8   1,9   2,a   3,b   4,c   5,d   6,e   7,f  */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  /* 00 */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  /* 08 */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  /* 10 */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  /* 18 */
    0x00, 0x0f, 0x3f, 0x3b, 0x1b, 0x2c, 0x10, 0x3d,  /* 20 */
    0x0d, 0x1d, 0x1c, 0x0e, 0x2b, 0x20, 0x0b, 0x21,  /* 28 */
    0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37,  /* 30 */
    0x38, 0x39, 0x3a, 0x1e, 0x0c, 0x3e, 0x2e, 0x2f,  /* 38 */
    0x3c, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,  /* 40 */
    0x08, 0x09, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16,  /* 48 */
    0x17, 0x18, 0x19, 0x22, 0x23, 0x24, 0x25, 0x26,  /* 50 */
    0x27, 0x28, 0x29, 0x0a, 0x00, 0x1a, 0x1f, 0x2d,  /* 58 */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  /* 60 */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  /* 68 */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  /* 70 */
    0x00, 0x00, 0x00, 0x00, 0x2a, 0x00, 0x00, 0x00   /* 78 */
};

screen ScreenData[MAX_TERMS];

/*********************************************************************/
/* Screen handling functions.                                        */
/*********************************************************************/
/* The screen handling functions are called to update the control    */
/* and user terminals. They perform simple operations like clearing  */
/* the screen, clearing to the end of a screen line, adding a string */
/* to the screen, and refreshing the screen.                         */

FILE *Outfp;
int outc(cics_char_t c)
{
    fputc(c, Outfp);
}

void scr_clear(screen *s)
{
    memset(s->display, ' ', sizeof(s->display));
    s->pos = 0;
}

void scr_clrtoeol(screen *s)
{
    memset(&(s->display[s->pos]), ' ', COLS - s->pos % COLS);
}

void scr_addstr(screen *s, cics_char_t *str)
{
    while (*str)
    {
	scr_addch(s, *str++);
    }
}

void scr_refresh(screen *s)
{
    int row;
    int col;
    cics_char_t *line;


    Outfp = s->fp;
    tputs(tparm(cursor_home), 1, outc);		/* HOME */
    tputs(tparm(clear_screen), 1, outc);	/* CLEAR */
    fputc('\n', s->fp);				/* NEWLINE */

    for (row = 0; row < ROWS; ++row)
    {
	line = &s->display[row * COLS];

	for (col = COLS-2; col >= 0 && line[col] == ' '; --col)
		;

	if (row == ROWS-1)
	{
	    tputs(tparm(cursor_home), 1, outc);	/* HOME */
	    ++col;
	}

	if (col >= 0)
	{
	    line[col+1] = '\0';
	    fputs(line, s->fp);
	}

	if (row < ROWS-2)
	{
	    fputc('\n', s->fp);
	}
    }

    fflush(s->fp);
}


/*********************************************************************/
/* ConvertAddress() - Convert SBA to 3270 address.                   */
/*********************************************************************/
/* This function is called to convert a Set Buffer Address (SBA)     */
/* sequence to the equivalent 3270 buffer address.                   */

static cics_sshort_t ConvertAddress(cics_ubyte_t *Sba)
{
    cics_ubyte_t Hibits;
    cics_ubyte_t Lobits;
    cics_ushort_t Address;          /* 3270 buffer address */

    Hibits = GraphicToBinary[Sba[0]];
    Lobits = GraphicToBinary[Sba[1]];

    Address = ((Hibits & 0x3f) << 6) | (Lobits & 0x3f);

    return(Address);
}


/*********************************************************************/
/* Print() - Output a string on a screen.                            */
/*********************************************************************/
/* This function is called to output a string on the control or user */
/* terminal. It calls the relevant screen handling functions to do   */
/* so.                                                               */

static void Print(int Row, cics_ushort_t TermIndex, cics_char_t *format, va_list ap)
{
    cics_char_t Buffer[BUFSIZ];

    FILE *fp = TermData[TermIndex].fp;
    screen *scr = &ScreenData[TermIndex];

    vsprintf(Buffer, format, ap);
    scr_move(scr, Row, 0);
    scr_clrtoeol(scr);
    scr_addstr(scr, Buffer);
    scr_refresh(scr);
}


/*********************************************************************/
/* Message() - Output a message on row 22 of a screen.               */
/*********************************************************************/
/* This function is called to output a message on row 22 of the      */
/* control or user terminal. It calls the Print() function to do so. */

static void Message(cics_ushort_t TermIndex, cics_char_t *format, ...)
{
    va_list ap;

    va_start(ap, format);
    Print(22, TermIndex, format, ap);
    va_end(ap);
}


/*********************************************************************/
/* Prompt() - Output a prompt on row 24 of a screen.                 */
/*********************************************************************/
/* This function is called to output a prompt on row 24 of the       */
/* control or user terminal. It calls the Print() function to do so. */

void Prompt(cics_ushort_t TermIndex, cics_char_t *format, ...)
{
    va_list ap;

    va_start(ap, format);
    Print(24, TermIndex, format, ap);
    va_end(ap);
}


/*********************************************************************/
/* Error() - Report error.                                           */
/*********************************************************************/
/* This function is called to report an EPI error.                   */

void Error(cics_ushort_t TermIndex, cics_char_t *Function, cics_sshort_t Rc, FILE *fp)
{
    CICS_EpiSysError_t SysErr;

    if (fp == NULL)
    {
	fp = (TermIndex == CICS_EPI_TERM_INDEX_NONE ? stderr :
	      TermData[TermIndex].fp);
    }

    fprintf(fp, "error: %s() failed(%d)", Function, Rc);

    if (CICS_EpiGetSysError(TermIndex, &SysErr) == CICS_EPI_NORMAL)
    {
	fprintf(fp, " Cause(%u) Value(%u)\n\tMessage: %s",
	        SysErr.Cause, SysErr.Value, SysErr.Msg);
    }
    fprintf(fp, "\n");
    fflush(fp);
}


/*********************************************************************/
/* ErrorExit() - Report error and exit.                              */
/*********************************************************************/
/* This function is called to report serious and unexpected EPI      */
/* errors. It calls the Error() function to do so. It will then      */
/* terminate the program.                                            */

void ErrorExit(cics_char_t *Function, cics_sshort_t Rc)
{
    Error(CICS_EPI_TERM_INDEX_NONE, Function, Rc, NULL);
    exit(1);
}


/*********************************************************************/
/* Emulate() - Simple 3270 screen emulator.                          */
/*********************************************************************/
/* This function is called to emulate a 3720 screen when a SEND or   */
/* CONVERSE event is received. It calls the relevant screen handler  */
/* functions to do so.                                               */

void Emulate(cics_ushort_t TermIndex, cics_ubyte_t *Data, cics_sshort_t Size)
{
    static cics_sshort_t WritePosn;
    cics_sshort_t Addr;
    cics_sshort_t RA_Addr;
    cics_sshort_t RA_End_Addr;
    cics_char_t *cmd;
    cics_char_t Buffer[BUFSIZ];
    cics_char_t *BufP;
    int Ch;
    int i;
    int n;

    FILE *fp = TermData[TermIndex].fp;
    screen *scr = &ScreenData[TermIndex];

    switch (Data[0])
    {
        case CMD_W:
	    cmd = "W";
	    break;

        case CMD_EW:
	    WritePosn = 0;
	    scr_clear(scr);
	    cmd = "EW";
	    break;

        case CMD_RB:
	    cmd = "RB";
	    break;

        case CMD_RM:
	    cmd = "RM";
	    break;

        case CMD_RMA:
	    cmd = "RMA";
	    break;

        case CMD_EAU:
	    cmd = "EAU";
	    break;

        default:
	    cmd = "?";
	    break;
    }

    BufP = Buffer;
    Addr = WritePosn;
    for (i = 2; i < Size; ++i)
    {
	switch (Ch = Data[i])
	{
	    case ORDER_SF:
	        ++i;
	        break;

	    case ORDER_SFE:
	        n = Data[++i];
	        i += 2 * n;
	        break;

	    case ORDER_SA:
	    case ORDER_EUA:
	        i += 2;
	        break;

	    case ORDER_RA:
	        RA_Addr = Addr;
	        RA_End_Addr = ConvertAddress((cics_ubyte_t *)Data+i+1);
	        i += 3;
	        if (isprint(Data[i]))
	        {
		    while (RA_Addr < RA_End_Addr)
		    {
		        scr_mvaddch(scr, RA_Addr/80, RA_Addr%80, Data[i]);
		        ++RA_Addr;
		    }
	        }
	        break;

	    case ORDER_SBA:
	        Addr = WritePosn = ConvertAddress((cics_ubyte_t *)Data+i+1);
	        i += 2;
	        continue;

	    default:
	        if (isprint(Ch))
	        {
		    scr_mvaddch(scr, Addr/80, Addr%80, Ch);
		    ++Addr;
	        }
	        continue;
	}
    }
    scr_refresh(scr);
}


/*********************************************************************/
/* Start() - Start a transaction on a terminal.                      */
/*********************************************************************/
/* This function is called to start a transaction on the control or  */
/* user terminal. It calls the CICS_EpiStartTran() function to do    */
/* so.                                                               */

void Start(cics_ushort_t TermIndex, cics_char_t *Data, cics_sshort_t Size)
{
    CICS_EpiSysError_t SysErr;
    cics_ubyte_t Buffer[BUFSIZ];
    cics_char_t Tran[CICS_EPI_TRANSID_MAX+1];
    cics_sshort_t Rc;
    int i;

    if (TermData[TermIndex].NextTran[0] == '\0')
    {
	for (i = 0; i < CICS_EPI_TRANSID_MAX && i < Size; ++i)
	{
	    if (Data[i] == ' ')
		break;
	    Tran[i] = Data[i];
	}
	Tran[i] = '\0';
    }
    else
    {
	strcpy(Tran, TermData[TermIndex].NextTran);
	TermData[TermIndex].NextTran[0] = '\0';
    }

    Buffer[0] = (cics_ubyte_t) DFHENTER;	/* Simulated ENTER key */
    Buffer[1] = (cics_ubyte_t) 0x20;	/* Simulated cursor at 0,0 */
    Buffer[2] = (cics_ubyte_t) 0x20;	/* Simulated cursor at 0,0 */
    memcpy(Buffer+3, Data, Size);

    Rc = CICS_EpiStartTran(TermIndex, Tran, Buffer, Size+3);
    if (Rc == CICS_EPI_NORMAL)
    {
	TermData[TermIndex].State = STATE_BUSY;
    }
    else
    {
	CICS_EpiGetSysError(TermIndex, &SysErr);

        Message(TermIndex,
		"Can't start transaction '%s' -- %s (cause=%u, error=%u)\n",
		Tran, SysErr.Msg, SysErr.Cause, SysErr.Value);
	Prompt(TermIndex, "TRAN? ");
    }
}


/*********************************************************************/
/* Reply() - Reply to a converse.                                    */
/*********************************************************************/
/* This function is called to reply to a CONVERSE event. It calls    */
/* the CICS_EpiReply() function to do so.                            */

void Reply(cics_ushort_t TermIndex, cics_sshort_t AID, cics_char_t *Data, cics_sshort_t Size)
{
    cics_sshort_t Rc;
    cics_ubyte_t Buffer[BUFSIZ];
    cics_sshort_t i;

    /* Cursor address */
    i = 0;
    Buffer[i++] = (cics_ubyte_t) AID;
    Buffer[i++] = (cics_ubyte_t) 0x20;
    Buffer[i++] = (cics_ubyte_t) 0x41;

    if (AID != DFHENTER || Size == 0)
    {
	Rc = CICS_EpiReply(TermIndex, Buffer, i);
    }
    else
    {
	/* Modified field address */
	Buffer[i++] = (cics_ubyte_t) ORDER_SBA;
	Buffer[i++] = (cics_ubyte_t) 0x20;
	Buffer[i++] = (cics_ubyte_t) 0x41;

	/* Data */
	memcpy(Buffer+i, Data, Size);

	Rc = CICS_EpiReply(TermIndex, Buffer, Size+i);
    }

    if (Rc != CICS_EPI_NORMAL)
    {
       Error(TermIndex, "CICS_EpiReply", Rc, NULL);
    }
    TermData[TermIndex].State = STATE_BUSY;

}


/*********************************************************************/
/* EventThread() - Event processing thread.                          */
/*********************************************************************/
/* This function is run in a secondary thread and handles all the    */
/* EPI event processing for the program. It calls CICS_EpiGetEvent() */
/* in blocking mode to wait for the arrival of EPI events.           */

void *EventThread(void *Parm)
{
    cics_sshort_t Rc;
    CICS_EpiEventData_t EventData;	/* Receives EPI event details */
    cics_ubyte_t Data[BUFSIZ];		/* Receives event data        */
    cics_ushort_t TermIndex;
    struct TheTermData *Tp;

    for (;;)
    {
	EventData.Data = Data;
	EventData.Size = sizeof(Data);
	Rc = CICS_EpiGetEvent(CICS_EPI_TERM_INDEX_NONE, CICS_EPI_WAIT,
			      &EventData);

	TermIndex = EventData.TermIndex;

	if (Rc != CICS_EPI_NORMAL && Rc != CICS_EPI_ERR_MORE_EVENTS)
	{
	    Error(CICS_EPI_TERM_INDEX_NONE, "CICS_EpiGetEvent", Rc, NULL);
	    continue;
	}

	Tp = &TermData[TermIndex];

	switch (EventData.Event)
	{
	    case CICS_EPI_EVENT_SEND:
		Emulate(TermIndex, EventData.Data, EventData.Size);
		break;

	    case CICS_EPI_EVENT_CONVERSE:
		Tp->State = STATE_CONVERSE;
		Emulate(TermIndex, EventData.Data, EventData.Size);
		Prompt(TermIndex, "CONVERSE? ");
		break;

	    case CICS_EPI_EVENT_END_TRAN:
		Tp->State = STATE_IDLE;

	        if (EventData.AbendCode[0] != ' ')
	        {
		    Message(TermIndex, "ABEND: '%s'\n", EventData.AbendCode);
	        }

	        if (EventData.TransId[0] != ' ')
	        {
		    strcpy(Tp->NextTran, EventData.TransId);
	        }
	        Prompt(TermIndex,
		       (Tp->NextTran[0] != '\0' ? "NEXTTRAN(%s)? " : "TRAN? "),
		       Tp->NextTran);
	        break;

	    case CICS_EPI_EVENT_START_ATI:
	        Tp->State = STATE_BUSY;
	        Message(TermIndex, "ATI('%s') has been started\n",
		        EventData.TransId);
	        break;

	    case CICS_EPI_EVENT_HELP:
	        Message(TermIndex, "Help event subject: '%s'\n",
			EventData.Data);
	        break;

	    case CICS_EPI_EVENT_END_TERM:
	        switch (EventData.EndReason)
	        {
		    case CICS_EPI_END_SIGNOFF:
		        Message(TermIndex, "cicsepimt: terminal signed off\n");
		        break;

		    case CICS_EPI_END_FAILED:
		        Message(TermIndex, "Terminal Delete ABENDed (\"%s\")\n",
				EventData.AbendCode);
		        break;

	    	    default:
	       		Message(TermIndex, "Unexpected terminal end (%d)\n",
				EventData.EndReason);
	       		break;
	        }

	        SetInput(Tp->fp, CICS_EPI_TERM_INDEX_NONE);
		Tp->State = STATE_IDLE;
	        break;

	    default:
	        Message(TermIndex, "cicsepimt: bad event(%d)\n",
			EventData.Event);
	        break;
	}
    }
}


/*********************************************************************/
/* GetAID() - Obtain AID value from user input.                      */
/*********************************************************************/
/* This function is called to obtain the Attention Identification    */
/* (AID) from the user input. It maps ESCAPE key sequences to        */
/* Program Function (PF) keys.                                       */

int GetAID(cics_char_t *Buffer)
{
    int AID;

    if (Buffer[0] == '\033')
    {
	switch (Buffer[1])
	{
	    case '1': AID = DFHPF1; break;
	    case '2': AID = DFHPF2; break;
	    case '3': AID = DFHPF3; break;
	    case '4': AID = DFHPF4; break;
	    case '5': AID = DFHPF5; break;
	    case '6': AID = DFHPF6; break;
	    case '7': AID = DFHPF7; break;
	    case '8': AID = DFHPF8; break;
	    case '9': AID = DFHPF9; break;
	    case '0': AID = DFHPF10; break;
	    case '-': AID = DFHPF11; break;
	    case '=': AID = DFHPF12; break;
	    default: AID = DFHCLEAR; break;
	}
	Buffer[0] = '\0';	/* No data returned */
    }
    else
    {
	AID = DFHENTER;
    }
    return (AID);
}


/*********************************************************************/
/* AddTerm() - Install a new terminal.                               */
/*********************************************************************/
/* This function is called to install the control or user terminal.  */
/* It calls the CICS_EpiAddTerminal() function to do so.             */

cics_ushort_t AddTerm(cics_char_t *RegionName, FILE *fp)
{
    CICS_EpiDetails_t TermDetails;
    cics_ushort_t TermIndex = CICS_EPI_TERM_INDEX_NONE;
    cics_sshort_t Rc;
    screen *scr;

    if ((RegionName == NULL) || (RegionName[0] == '\0'))
    {
	fprintf(fp, "REGION? ");
	fflush(fp);
	return(CICS_EPI_TERM_INDEX_NONE);
    }

    fprintf(fp, "Connecting to region %s...\n", RegionName);
    fflush(fp);

    /*
     * Install a terminal against the specified region.
     * Specify the terminal model to avoid an unsuitable default.
     */
    Rc = CICS_EpiAddTerminal(NULL,		/* No namespace needed        */
			     RegionName,	/* Use region specified       */
			     NULL,		/* Autoinstall the terminal   */
                            "hft",      /* High function terminal    */
			     NULL,		/* No event notification      */
			     &TermDetails,	/* Receives new TermDetails   */
			     &TermIndex);	/* Receives new TermIndex     */

    if (Rc != CICS_EPI_NORMAL)
    {
	Error(TermIndex, "CICS_EpiAddTerminal", Rc, fp);
	fprintf(fp, "REGION? ");
    }
    else
    {
	SetInput(fp, TermIndex);
	TermData[TermIndex].fp = fp;
	TermData[TermIndex].NextTran[0] = '\0';

	scr = &ScreenData[TermIndex];

	scr->fp = fp;
	scr_clear(scr);

	Message(TermIndex,
		"Connected NetName(%s)\n", TermDetails.NetName);
	Prompt(TermIndex, "TRAN? ");
    }
    fflush(fp);

    return(TermIndex);
}


/*********************************************************************/
/* Control() - Main control loop.                                    */
/*********************************************************************/
/* This function is the main control loop for the sample. It loops   */
/* around processing input from the terminals, until EOF is received */
/* on all the terminal.                                              */

void Control(void)
{
    struct TheTermData *Tp;
    cics_char_t Buffer[BUFSIZ];
    cics_sshort_t BufLen;
    cics_ushort_t TermIndex;
    FILE *fp;
    cics_sshort_t Rc;
    cics_sshort_t AID;

    while ((fp = GetInput(Buffer, &TermIndex)) != NULL)
    {
	BufLen = strlen(Buffer);
	Buffer[BufLen-1] = '\0';	/* Take out the NEWLINE */

	if (BufLen == 0)
	{
	    fprintf(fp, "EOF\n");
	    fflush(fp);

	    if (TermIndex != CICS_EPI_TERM_INDEX_NONE)
	    {
		TermData[TermIndex].State = STATE_BUSY;
		Rc = CICS_EpiDelTerminal(TermIndex);
		if (Rc != CICS_EPI_NORMAL)
		{
		    Error(TermIndex, "CICS_EpiDelTerminal", Rc, NULL);
		}
	    }

	    /*
	     * Close terminal.
	     */
	    SetInput(fp, CICS_EPI_TERM_INDEX_NONE);
	    continue;
	}

	if (strncmp(Buffer, "/dev", 4) == 0)
	{
	    /*
	     * Open terminal.
	     */
	    ChangeTTY(Buffer);
	}
	else if (TermIndex == CICS_EPI_TERM_INDEX_NONE)
	{
	    TermIndex = AddTerm(Buffer, fp);
	}
	else
	{
	    switch (TermData[TermIndex].State)
	    {
	        case STATE_BUSY:
		    /*
		     * Reject input in busy state, although the EPI will
		     * not allow new transactions to be started anyway.
		     */
		    Message(TermIndex, "X SYSTEM - input rejected\n");
		    break;

	        case STATE_IDLE:
		    Start(TermIndex, Buffer, strlen(Buffer));
		    break;

	        case STATE_CONVERSE:
		    AID = GetAID(Buffer);
		    Reply(TermIndex, AID, Buffer, strlen(Buffer));
		    break;
	    }
	}
    }
    /* All terminals closed at this point */
}

/*********************************************************************/
/* main() - Sample Entry Point                                       */
/*********************************************************************/

void main(int ArgC, cics_char_t **ArgV)
{
    cics_sshort_t Rc;                             /* EPI function return codes */
    pthread_t Thread;                     /* Secondary thread id       */

    (void)setlocale(LC_ALL, "");

    setupterm(0, 1, 0);

    /*
     * Initialize the EPI.
     */
    Rc = CICS_EpiInitialize(CICS_EPI_VERSION_101);
    if (Rc != CICS_EPI_NORMAL)
    {
	ErrorExit("CICS_EpiInitialize", Rc);
    }

    /*
     * Start up the secondary event processing thread.
     * If this fails just terminate the EPI and exit.
     */
    if (pthread_create(&Thread, pthread_attr_default, EventThread, NULL) == -1)
    {
	fprintf(stderr, "cicsepimt: cannot start event processing thread!\n");
	Rc = CICS_EpiTerminate();
	exit(1);
    }

    Init("REGION? ");	/* Initialize the control tty (stdin) */

    Control();	/* Returns at EOF on all terminals */

    /*
     * All the terminals have now finished, so terminate the EPI and
     * the application.
     */
    fprintf(stderr, "cicsepimt: goodbye!\n");
    CICS_EpiTerminate();

    exit(0);
}

/* EOF */
