/*
 * NAME:	Additional EPI multi-terminal functions - cicsepimta.c
 *
 *
 * VERSION	1.1
 *
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * DESCRIPTION:
 *
 *      This file defines most of the input functions used by the EPI
 *      multi-terminal sample (cicsepimt). The rest of the functions used
 *      by cicsepimt are defined in the file cicsepimt.c.
 */

#include <cicstype.h>
#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>

#if defined(SELECT)
#include <sys/select.h>
#endif

#include <sys/signal.h>

#include <cics_epi.h>		/* Include the EPI header file */


/*********************************************************************/
/* Declare the globals used in these functions.                      */
/*********************************************************************/

fd_set	ReadMask;
int	Nfds;
cics_char_t	OpenMsg[80];

struct node
{
    struct node *next;
    FILE	*fp;
    cics_sshort_t	TermIndex;
    cics_char_t	name[1];
};

static struct node *head = NULL;

static void dummy(int sig)	/* Dummy function to catch alarm signals */
{
}


/*********************************************************************/
/* nodep_from_name() - Return information for a named terminal.      */
/*********************************************************************/
/* This function returns the information for the terminal device     */
/* corresponding to the supplied name.                               */

static struct node **nodep_from_name(cics_char_t *name)
{
    struct node **npp;

    for (npp = &head; (*npp) != NULL; npp = &((*npp)->next))
    {
	if (strcmp((*npp)->name, name) == 0)
	{
	    return (npp);
	}
    }
    return (NULL);
}


/*********************************************************************/
/* node_from_fd() - Return information for a terminal.               */
/*********************************************************************/
/* This function returns the information for the terminal device     */
/* corresponding to the supplied file descriptor.                    */

static struct node *node_from_fd(int fd)
{
    struct node *np;

    for (np = head; np != NULL; np = np->next)
    {
	if (fileno(np->fp) == fd)
	{
	    return (np);
	}
    }
    return (NULL);
}


/*********************************************************************/
/* change_tty() - Open or close a terminal.                          */
/*********************************************************************/
/* This function closes a terminal if it already exists, or opens a  */
/* new terminal. Information about the open is saved for use later.  */

static int change_tty(cics_char_t *tty, fd_set *ReadMaskP)
{
    struct node **npp;
    struct node *np;
    FILE *fp;
    int maxfd;

    if ((npp = nodep_from_name(tty)) != NULL)
    {
	/*
	 * Close terminal.
	 */
	np = *npp;
	*npp = np->next;
	printf("cicsepimt: close %s %d\n", tty, fileno(np->fp));
	fclose(np->fp);
	free(np);
    }
    else
    {
	/*
	 * Open terminal.
	 */
	if (strcmp(tty, "/dev/stdin") == 0)
	{
	   fp = stdin;
	}
	else
	{
	    (void) signal(SIGALRM, dummy);
	    alarm(1);
	    fp = fopen(tty, "r+");
	}

	if (fp == NULL)
	{
	    alarm(0);
	    perror(tty);
	}
	else
	{
	    alarm(0);
	    np = malloc(sizeof(*np) + strlen(tty));
	    np->fp = fp;
	    strcpy(np->name, tty);
	    np->TermIndex = CICS_EPI_TERM_INDEX_NONE;
	    np->next = head;
	    head = np;
	    printf("cicsepimt: open %s %d\n", tty, fileno(np->fp));
	    if (fp == stdin)
	    {
		fp = stdout;
	    }
	    fprintf(fp, "%s", OpenMsg);
	    fflush(fp);
	}
    }

    FD_ZERO(ReadMaskP);
    maxfd = -1;
    for (np = head; np != NULL; np = np->next)
    {
	FD_SET(fileno(np->fp), ReadMaskP);
	if (maxfd < (int) fileno(np->fp))
	{
	    maxfd = (int) fileno(np->fp);	/* Set max fd */
	}
    }
    return (maxfd+1);
}


/*********************************************************************/
/* ChangeTTY() - Open or close a terminal.                           */
/*********************************************************************/
/* This function is basically a wrapper around the change_tty()      */
/* function, and is used to provide a simpler interface.             */

void ChangeTTY(cics_char_t *tty)
{
    Nfds = change_tty(tty, &ReadMask);
}


/*********************************************************************/
/* fp_select() - Check for input on the open file descriptors.       */
/*********************************************************************/
/* This function uses the select() system call to check if any of    */
/* the file descriptors associated with terminals have input ready,  */
/* and returns the terminal details associated with the first one.   */

static struct node *fp_select(void)
{
    fd_set readmask;
    int fd;
    int i;

    for (i = 0; Nfds > 0 && i < 10; ++i)
    {
	readmask = ReadMask;	/* structure copy */
	if (select(Nfds, &readmask, (fd_set *)0, (fd_set *)0,
	    (struct timeval *)0) > 0)
	{
	    for (fd = 0; fd <= Nfds; ++fd)
	    {
		if (FD_ISSET(fd, &readmask))
		{
		    return (node_from_fd(fd));
		}
	    }
	}
    }
    return (NULL);
}


/*********************************************************************/
/* Init() - Initialize the control terminal.                         */
/*********************************************************************/
/* This function initializes and then opens stdin as the control     */
/* terminal. It calls ChangeTTY() to do so.                          */

void Init(cics_char_t *openmsg)
{
    cics_char_t *tty;

    strncpy(OpenMsg, openmsg, sizeof(OpenMsg));

    FD_ZERO(&ReadMask);

    /*
     * Select from stdin as the control terminal.
     */
    ChangeTTY("/dev/stdin");
}


/*********************************************************************/
/* SetInput() - Change terminal input.                               */
/*********************************************************************/
/* This function returns the TermIndex for the terminal associated   */
/* with the supplied file pointer, unless the supplied TermIndex     */
/* value is CICS_EPI_TERM_INDEX_NONE when the terminal is closed.    */

void SetInput(FILE *fp, cics_ushort_t TermIndex)
{
    struct node *np;
    FILE *fp2;

    fp2 = (fp == stdout ? stdin : fp);

    if ((np = node_from_fd(fileno(fp2))) != NULL)
    {
	if (TermIndex == CICS_EPI_TERM_INDEX_NONE)
	{
	    /*
	     * Close terminal.
	     */
	    ChangeTTY(np->name);
	}
	np->TermIndex = TermIndex;
    }
}


/*********************************************************************/
/* GetInput() - Get input from the next available terminal.          */
/*********************************************************************/
/* This function gets a line of input from the next available        */
/* terminal. If input is available, GetInput() returns the file      */
/* pointer for the appropriate terminal. GetInput() returns NULL     */
/* when EOF is received on the control terminal.                     */

FILE *GetInput(cics_char_t *Buffer, cics_ushort_t *TermIndexP)
{
    struct node *np;
    FILE *fp;
    int n;

    while ((np = fp_select()) != NULL)
    {
	if ((fgets(Buffer, BUFSIZ, np->fp) == NULL) ||
	    ((n = strlen(Buffer)) == 0))
	{
	    *Buffer = '\0';	/* EOF on the terminal */
	}

	*TermIndexP = (np->TermIndex);
	fp = (np->fp == stdin ? stdout : np->fp);

	return(fp);
    }
    return(NULL);
}

/* EOF */
