/*
 * NAME:        EPI Motif Sample - cicsepixa.c
 *
 * VERSION:     1.1
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * DESCRIPTION:
 *
 *      This is a sample Motif EPI application program for CICS.
 *      It takes one parameter, the name of a CICS region, and displays
 *      a window on the screen.  When a button on the window is pressed it
 *      submits the EPIX transaction.
 *
 *      The EPIX transaction uses CICS to query and format the time of day.
 *      It sends the formatted time back to this application which displays
 *      it in its window.
 *
 *      It demonstrates the use of the EPI event notification function to
 *      cause asynchronous processing of EPI events.
 */

#include <cicstype.h>
#include <stdio.h>
#include <string.h>
#include <memory.h>
#include <locale.h>

#include <cics_epi.h>      /* Include the EPI header file */

#include <Xm/Xm.h>          /* Include Motif standard header files   */
#include <Xm/Form.h>        /* Include Form widget definition        */
#include <Xm/Frame.h>       /* Include Frame widget definition        */
#include <Xm/PushB.h>       /* Include Push Button Widget definition */
#include <Xm/Label.h>       /* Include Label Widget definition       */

#define SA(resource, value) {XtSetArg(args[n], (resource), (value)); n++;}
static Arg args[20];
static int n;

/*
 * This is a piece of dummy 3270 data (an ENTER AID key followed by a
 * cursor position of (0,0)).  It is used when starting transactions.
 */

#define DUMMY_DATA  ((cics_ubyte_t *)"'  ")

#define CICS_EVENT  (1)

/*
 * Declare the functions and globals used in this program.
 */

Widget TimeButton;              /* Time button Widget */
Widget QuitButton;              /* Quit button Widget */
Widget TimeLabel;               /* Time label Widget  */
Widget Form;                    /* Form  Widget */

int CicsToMotifPipe[2];		/* Comms pipe for Cics to Motif comms */

static cics_ushort_t TermIndex;     /* Index number of installed terminal */

void ErrorExit(cics_char_t *Function, cics_sshort_t Rc, cics_ushort_t TermIndex);
void EventFunc(cics_ushort_t Index);
void TimeCB(Widget Widget, XtPointer Closure, XtPointer CallData);
void QuitCB(Widget Widget, XtPointer Closure, XtPointer CallData);
void CicsCB(XtPointer Client_data, int *source, XtInputId *Id);

/*********************************************************************/
/* ErrorExit() - Report errors and exit                              */
/*********************************************************************/
/* This function is called to report serious and unexpected EPI      */
/* errors.  It will then terminate the program.                      */

void ErrorExit(cics_char_t *Function, cics_sshort_t Rc, cics_ushort_t TermIndex)
{
  CICS_EpiSysError_t SysErr;            /* Receives EPI error info.  */

   fprintf(stderr, "%s() call failed ", Function);
   if (Rc != CICS_EPI_ERR_FAILED)
   {
      fprintf(stderr, "with return code: %d\n", Rc);
   }
   else
   {
      CICS_EpiGetSysError(TermIndex, &SysErr);

      fprintf(stderr, "with a system error.\n");
      fprintf(stderr, "Cause: %u, Error: %u, Message: %s\n",
                      SysErr.Cause, SysErr.Value,
                      (SysErr.Msg != NULL) ? SysErr.Msg : "<none>");
   }

  exit(1);
}


/*********************************************************************/
/* EventFunc() - EPI Event Callback.                                 */
/*********************************************************************/
/*
 * This function is invoked by the EPI when an event occurs.  It writes to
 * a pipe which causes Motif to invoke another callback in the correct Motif
 * context.  Motif calls cannot be made here since this function is 
 * invoked in the epi context.
 */

void EventFunc(cics_ushort_t Index)
{
  cics_ubyte_t type;

  type = CICS_EVENT;
  write(CicsToMotifPipe[1], &type,1);
  return;
}


/*********************************************************************/
/* CicsCB() - Event Handler.                                         */
/*********************************************************************/
/*
 * This callback is invoked by Motif in the correct context for performing
 * Motif functions.  It will be invoked when data has been written to the   
 * CicsToMotifPipe pipe.  It reads the pipe data and acts accordingly.
 */

void CicsCB(XtPointer Client_data, int *source, XtInputId *Id)
{
  cics_sshort_t Rc;
  cics_char_t Dummy;
  cics_char_t Result[9];                       /* Time received from CICS    */
  XmString String;
  CICS_EpiEventData_t EventData;        /* Receives EPI event details */
  cics_ubyte_t Data[200];                   /* Receives event data        */
  cics_ubyte_t type;			/* type of event	      */

  /*
   * Find out what is in the pipe (for this sample, must be CICS_EVENT)
   */
  read(CicsToMotifPipe[0], &type, 1);

  switch (type)
  {
    case CICS_EVENT:
      /*
       * Get the EPI event that must now be outstanding.
       */

      EventData.Data = Data;               /* Point to space for any */
      EventData.Size = sizeof(Data);       /* incoming 3270 data.    */

      Rc = CICS_EpiGetEvent(TermIndex, CICS_EPI_NOWAIT, &EventData);
      if (Rc != CICS_EPI_NORMAL && Rc != CICS_EPI_ERR_MORE_EVENTS)
      {
        ErrorExit("CICS_EpiGetEvent", Rc, TermIndex);
      }

      /*
       * Process the event received.
       */

      switch (EventData.Event)
      {
        /*
         * SEND: Display time sent in the window.
         */
        case CICS_EPI_EVENT_SEND:
          /*
           * Check that we have the expected amount of data - 8 bytes plus 2
           * datastream bytes.
           */
          if (EventData.Size == 10)
          {
            memset(Result, '\0', sizeof(Result));
            memcpy(Result, Data+2, sizeof(Result)-1);
            String = XmStringCreate(Result, XmSTRING_DEFAULT_CHARSET);
            n = 0;
            SA(XmNlabelString, String);
            XtSetValues(TimeLabel, args, n);
            XmStringFree(String);
          }
          break;

        /*
         * END_TRAN: Re-activate the buttons.
         */
        case CICS_EPI_EVENT_END_TRAN:
          XtSetSensitive(TimeButton, True);
          XtSetSensitive(QuitButton, True);
          break;

        /*
         * Ignore other events.
         */
        default:
           break;
      }
    /*
     * Ignore other values on this pipe (there should not be any)
     */
    default:
      break;
  }
  return;
}


/*********************************************************************/
/* TimeCB() - Callback function for the "Time" button                */
/*********************************************************************/
/* This function is the Motif callback issued when the "Time" button */
/* is pressed.  It starts the EPIX transaction.                      */

void TimeCB(Widget Widget, XtPointer Closure, XtPointer CallData)
{
  cics_sshort_t Rc;

  /*
   * Disable the buttons until the transaction completes.
   */

   XtSetSensitive(TimeButton, False);
   XtSetSensitive(QuitButton, False);

  /*
   * Submit the EPIX transaction.  Some initial 3270 data must be
   * provided so use DUMMY_DATA.
   */

   Rc = CICS_EpiStartTran(TermIndex,
                          "EPIX",
                          DUMMY_DATA,
                          sizeof(DUMMY_DATA)-1);
   if (Rc != CICS_EPI_NORMAL)
   {
      ErrorExit("CICS_EpiStartTran", Rc, TermIndex);
   }

  return;
}


/*********************************************************************/
/* QuitCB() - Callback function for the "Quit" button                */
/*********************************************************************/
/* This function is the Motif callback issued when the "Quit" button */
/* is pressed.  It deletes the installed terminal.                   */

void QuitCB(Widget Widget, XtPointer Closure, XtPointer CallData)
{
  cics_sshort_t Rc;

  /*
   * Disable the buttons when quitting.
   */

   XtSetSensitive(TimeButton, False);
   XtSetSensitive(QuitButton, False);

  /*
   * Delete the terminal.
   */

   Rc = CICS_EpiDelTerminal(TermIndex);
   if (Rc != CICS_EPI_NORMAL)
   {
      ErrorExit("CICS_EpiDelTerminal", Rc, TermIndex);
   }

  exit(0);
}


/*********************************************************************/
/* main() - Sample Entry Point                                       */
/*********************************************************************/
/* This program should be called with the name of a CICS region.  If */
/* none is supplied it will exit with an error, otherwise it will    */
/* install an EPI terminal and display a window on the screen.  When */
/* a button in the window is pressed a transaction is run and its    */
/* results displayed.                                                */

main(int argc, cics_char_t *argv[])
{
  cics_sshort_t Rc;                             /* EPI function return codes */
  Widget   TopLevel;
  Widget   Frame;
  XmString String;

  /*
   * Check the caller supplied the name of a CICS region to be used.
   * Give up if we haven't got one.
   */

   if (argc < 2)
   {
      fprintf(stderr, "A CICS region name must be specified.\n");
      exit(1);
   }

  /*
   * Pick up the locale from the environment
   */
   (void)setlocale(LC_ALL,"");

  /*
   * Create the Motif window with its various buttons and labels.
   */

    TopLevel = XtInitialize(argv[0], "cicsepixa", NULL, 0, &argc, argv);
    n = 0;
    Form = XmCreateForm(TopLevel, "form", args, n);

    /* Main window */
    n = 0;
    SA(XmNrightAttachment, XmATTACH_FORM);
    SA(XmNtopAttachment, XmATTACH_FORM);
    SA(XmNleftAttachment, XmATTACH_POSITION);
    SA(XmNbottomAttachment, XmATTACH_POSITION);
    SA(XmNleftPosition, 50);
    SA(XmNbottomPosition, 50);
    Frame = XmCreateFrame(Form, "frame", args, n);
    XtManageChild(Frame);

    /* The "GetTime" button */
    n = 0;
    SA(XmNtopAttachment, XmATTACH_FORM);
    SA(XmNleftAttachment, XmATTACH_FORM);
    SA(XmNrightAttachment, XmATTACH_POSITION);
    SA(XmNbottomAttachment, XmATTACH_POSITION);
    SA(XmNrightPosition, 50);
    SA(XmNbottomPosition, 50);
    TimeButton = XmCreatePushButton(Form, "Get Time", args, n);
    XtManageChild(TimeButton);
    XtAddCallback(TimeButton, XmNactivateCallback, TimeCB, NULL);

    /* The "Time" value label */
    String = XmStringCreate("00:00:00", XmSTRING_DEFAULT_CHARSET);
    n = 0;
    SA(XmNlabelString, String);
    TimeLabel = XmCreateLabel(Frame, "time", args, n);
    XtManageChild(TimeLabel);
    XmStringFree(String);

    /* The "Quit" button */
    n = 0;
    SA(XmNtopAttachment, XmATTACH_POSITION);
    SA(XmNrightAttachment, XmATTACH_POSITION);
    SA(XmNbottomAttachment, XmATTACH_FORM);
    SA(XmNleftAttachment, XmATTACH_FORM);
    SA(XmNtopPosition, 50);
    SA(XmNrightPosition, 50);
    QuitButton = XmCreatePushButton(Form, "Quit", args, n);
    XtManageChild(QuitButton);
    XtAddCallback(QuitButton, XmNactivateCallback, QuitCB, NULL);

    /*
     * Create a pipe for communication between the motif and
     * epi contexts.  This will be used to trigger a motif 
     * callback from an epi callback.
     */
    pipe(CicsToMotifPipe);

    XtAppAddInput(XtWidgetToApplicationContext(TopLevel), /* app context*/
		  CicsToMotifPipe[0],	/* file descriptor of ip source */
		 (XtPointer)XtInputReadMask,	/* condition mask       */
		 (XtInputCallbackProc)CicsCB,	/* callback procedure	*/
		  NULL);		/* no client data		*/

    /*
    * Display the window.
    */

    XtManageChild(Form);
    XtRealizeWidget(TopLevel);

  /*
   * Initialize the EPI.
   */

   Rc = CICS_EpiInitialize(CICS_EPI_VERSION_101);
   if (Rc != CICS_EPI_NORMAL)
   {
      ErrorExit("CICS_EpiInitialize", Rc, CICS_EPI_TERM_INDEX_NONE);
   }

  /*
   * Install a terminal against the specified region.
   * Specify the terminal model to avoid an unsuitable default.
   */

   Rc = CICS_EpiAddTerminal(NULL,       /* No Namespace needed       */
                            argv[1],    /* Use region specified      */
                            NULL,       /* Autoinstall the terminal  */
                            "hft",      /* High function terminal    */
                            EventFunc,  /* Event notification        */
                            NULL,       /* TermDetails not needed    */
                            &TermIndex); /* Receives new TermIndex   */
   if (Rc != CICS_EPI_NORMAL)
   {
      ErrorExit("CICS_EpiAddTerminal", Rc, CICS_EPI_TERM_INDEX_NONE);
   }

   /*
    * Display and process the window.
    */

    XtMainLoop();
}

/* EOF */
