/*
 * NAME:        EPI Exit Object - Sample code
 *
 * VERSION:     1.1
 *
 * COPYRIGHT:                                                        
 * 
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * DESCRIPTION:
 *
 *      This is a sample EPI exits object.  It performs no useful function
 *      but does provide a skeleton containing all the exit functions 
 *      with details of valid return codes and their meanings.
 *
 *      When writing exits, the following rules must be obeyed:
 *      1. No EPI or ECI calls must be made from the exit
 *      2. Exits should contain the minimum possible executable code
 *         (to minimise impact on performance)
 *      3. All exit code must be re-entrant and thread safe.
 *	4. The primary entry point to this module must be named
 *	   CICS_EpiExitInit.  The names of the actual exits may
 *	   be changed but the parameter lists must not be changed.
 *
 */

#include <cicstype.h>
#include <cics_epi.h>
#include <cicsepiexit.h>

/*
 * Main entry point of dynamically loaded object.
 * This function returns a structure of addresses of exit functions.
 * Note that ALL exits must be defined if any are defined.
 * If any exit address is set to NULL no exits will be called.
 */
void CICS_EpiExitInit(OUT CICS_EpiExitList_t *ExitList)

{
    ExitList->InitialiseExit	= (void *)&CICS_EpiInitialiseExit;
    ExitList->TerminateExit	= (void *)&CICS_EpiTerminateExit;
    ExitList->AddTerminalExit	= (void *)&CICS_EpiAddTerminalExit;
    ExitList->StartTranExit	= (void *)&CICS_EpiStartTranExit;
    ExitList->ReplyExit		= (void *)&CICS_EpiReplyExit;
    ExitList->DelTerminalExit	= (void *)&CICS_EpiDelTerminalExit;
    ExitList->GetEventExit	= (void *)&CICS_EpiGetEventExit;
    ExitList->TranFailedExit	= (void *)&CICS_EpiTranFailedExit;
    ExitList->SystemIdExit	= (void *)&CICS_EpiSystemIdExit;
    ExitList->TermIdExit	= (void *)&CICS_EpiTermIdExit;
}

/*
 * InitialiseExit:
 * Called from CICS_EpiInitialize after successful initialisation, but
 * prior to returning to the caller
 *
 * Permitted return values:
 *   CICS_EXIT_OK		No problems - call exits where appropriate
 *   CICS_EXIT_NO_EXITS		No problems - do not call subsequent exits
 *				    (used by shipped dummy library)
 *   CICS_EXIT_CANT_INIT_EXITS  Exits cannot be used due to an 
 *				    (unspecified) error condition
 *   CICS_EXIT_USER_BASE + N	User specified value
 *
 * Notes:
 *   Any value except CICS_EXIT_OK will inhibit the calling of subsequent exits
 */
cics_sshort_t CICS_EpiInitialiseExit( IN  cics_ulong_t      Version
				    , OUT CICS_EpiAnchor_t *Anchor
				    )

{
        *Anchor = (void *)(-1);

        return  CICS_EXIT_OK;
}

/*
 * TerminateExit:
 * Called from CICS_EpiTerminate immediately prior to the destruction of the 
 * EPI data structures.
 *
 * Permitted return values:
 *   CICS_EXIT_OK               No problems
 *   CICS_EXIT_BAD_ANCHOR	Invalid anchor value supplied
 *   CICS_EXIT_BAD_STORAGE	Exit cannot tidy up its storage
 *   CICS_EXIT_USER_BASE + N    User specified value           
 */
cics_sshort_t CICS_EpiTerminateExit( IN CICS_EpiAnchor_t Anchor )
{
	return CICS_EXIT_OK;
}

/*
 * AddTerminalExit:
 * Called from CICS_EpiAddTerminal after validation of the input parameters
 * but before the add of the terminal.  This exit may change any of the input 
 * parameters NameSpace, System, NetName, DevType.
 *
 * Permitted return values:
 *   CICS_EXIT_OK               No problems 
 *   CICS_EXIT_DONT_ADD_TERMINAL Do not add this terminal
 *   CICS_EXIT_BAD_ANCHOR	Invalid anchor value supplied
 *   CICS_EXIT_BAD_PARM		One or more parameters bad
 *   CICS_EXIT_USER_BASE + N    User specified value           
 *
 * Notes:
 *   After a return of CICS_EXIT_DONT_ADD_TERMINAL, CICS_EpiAddTerminal will
 *   return EPI_ERROR_FAILED, and a subsequent call to CICS_EpiGetSysError
 *   will return a cause of 5.
 *   This exit must take great care when changing the system specified if
 *   the system is not blank, since in this case the calling application
 *   will not expect a change and may behave badly as a result.
 */
cics_sshort_t CICS_EpiAddTerminalExit
                        ( IN    CICS_EpiAnchor_t Anchor
                        , INOUT cics_char_t      NameSpace[1025]
                        , INOUT cics_char_t      System[CICS_EPI_SYSTEM_MAX+1]
                        , INOUT cics_char_t      NetName[CICS_EPI_NETNAME_MAX+1]
                        , INOUT cics_char_t      DevType[CICS_EPI_DEVTYPE_MAX+1]
                        )
{
	return CICS_EXIT_OK;
}

/*
 * StartTranExit:
 * Called from CICS_EpiStartTran after validation of input parameters. 
 * This exit must not change any of the input parameters
 *
 * Permitted return values:
 *   CICS_EXIT_OK               No problems 
 *   CICS_EXIT_BAD_ANCHOR	Invalid anchor value supplied
 *   CICS_EXIT_BAD_PARM		One or more parameters bad
 *   CICS_EXIT_USER_BASE + N    User specified value           
 */
cics_sshort_t CICS_EpiStartTranExit
                        ( IN    CICS_EpiAnchor_t Anchor
                        , IN    cics_char_t      TransId[CICS_EPI_TRANSID_MAX+1]
                        , IN    cics_ubyte_t    *Data
                        , IN    cics_ushort_t    Size
                        )
{
	return CICS_EXIT_OK;
}

/*
 * ReplyExit:
 * Called from CICS_EpiReply after validation of input parameters. 
 * This exit must not change the input parameters.
 *
 * Permitted return values:
 *   CICS_EXIT_OK               No problems
 *   CICS_EXIT_BAD_ANCHOR	Invalid anchor value supplied
 *   CICS_EXIT_BAD_PARM		One or more parameters bad
 *   CICS_EXIT_USER_BASE + N    User specified value           
 */
cics_sshort_t CICS_EpiReplyExit
                        ( IN    CICS_EpiAnchor_t Anchor
                        , IN    cics_ushort_t    TermIndex
                        , IN    cics_ubyte_t    *Data
                        , IN    cics_ushort_t    Size
                        )
{
	return CICS_EXIT_OK;
}

/*
 * DelTerminalExit:
 * Called from CICS_EpiDelTerminal after validation of input parameters. 
 * This exit must not change the input parameters.
 *
 * Permitted return values:
 *   CICS_EXIT_OK               No problems
 *   CICS_EXIT_BAD_ANCHOR	Invalid anchor value supplied
 *   CICS_EXIT_BAD_PARM		One or more parameters bad
 *   CICS_EXIT_USER_BASE + N    User specified value           
 */
cics_sshort_t CICS_EpiDelTerminalExit
                        ( IN    CICS_EpiAnchor_t Anchor
                        , IN    cics_ushort_t    TermIndex
                        )
{
	return CICS_EXIT_OK;
}

/*
 * GetEventExit:
 * Called from CICS_EpiGetEvent after validation of input parameters. 
 * This exit must not change the input parameters.
 *
 * Permitted return values:
 *   CICS_EXIT_OK               No problems
 *   CICS_EXIT_BAD_ANCHOR	Invalid anchor value supplied
 *   CICS_EXIT_BAD_PARM		One or more parameters bad
 *   CICS_EXIT_USER_BASE + N    User specified value           
 */
cics_sshort_t CICS_EpiGetEventExit
                        ( IN    CICS_EpiAnchor_t       Anchor
                        , IN    cics_ushort_t          TermIndex
                        , IN    CICS_EpiWait_t         Wait
                        , IN    CICS_EpiEventData_t   *Event
                        )
{
	return CICS_EXIT_OK;
}

/*
 * TranFailedExit:
 * Called on receipt of an END_TRAN or END_TERM event with an associated abend.
 * This exit must not change the input parameters.
 *
 * Permitted return values:
 *   CICS_EXIT_OK               No problems
 *   CICS_EXIT_BAD_ANCHOR	Invalid anchor value supplied
 *   CICS_EXIT_BAD_PARM		One or more parameters bad
 *   CICS_EXIT_USER_BASE + N    User specified value           
 */
cics_sshort_t CICS_EpiTranFailedExit
                        ( IN    CICS_EpiAnchor_t       Anchor
                        , IN    cics_ushort_t          TermIndex
                        , IN    CICS_EpiWait_t         Wait
                        , IN    CICS_EpiEventData_t   *Event
                        )
{
	return CICS_EXIT_OK;
}

/*
 * SystemIdExit:
 * Called from CICS_EpiAddTerminal after a failure to add a terminal.
 * This exit may change the input parameters 
 * NameSpace, System, NetName, DevType
 * This exit will be called repeatedly until it returns a system which can
 * successfully fulfil the request or until it returns CICS_EXIT_OK.
 * CICS_EXIT_GIVE_UP can be used to force a non-error return from this loop.
 *
 * Permitted return values:
 *   CICS_EXIT_OK               No problems
 *   CICS_EXIT_BAD_ANCHOR	Invalid anchor value supplied
 *   CICS_EXIT_BAD_PARM		One or more parameters bad
 *   CICS_EXIT_GIVE_UP		Do not make any further retry attempts
 *   CICS_EXIT_USER_BASE + N    User specified value           
 *
 * NOTE:
 *   UserID and PassWord are included for compatibility with other
 *   platforms but are ignored on CICS/UNIX platforms
 */
cics_sshort_t CICS_EpiSystemIdExit
		    ( IN    CICS_EpiAnchor_t Anchor
		    , INOUT cics_char_t      NameSpace[1025]
		    , INOUT cics_char_t      System[CICS_EPI_SYSTEM_MAX+1]
		    , INOUT cics_char_t      NetName[CICS_EPI_NETNAME_MAX+1]
		    , INOUT cics_char_t      DevType[CICS_EPI_DEVTYPE_MAX+1]
		    , IN    cics_char_t      FailedSystem[CICS_EPI_SYSTEM_MAX+1]
		    , IN    cics_sshort_t    Reason
		    , IN    cics_ulong_t     SubReason
		    , OUT   cics_char_t      UserId[64]
		    , OUT   cics_char_t      PassWord[64]
		    )
{
	return CICS_EXIT_DONT_ADD_TERMINAL;
}

/*
 * TermIdExit:
 * Called from CICS_EpiAddTerminal after successful installation of terminal
 * This exit must not change the input parameters.
 *
 * Permitted return values:
 *   CICS_EXIT_OK               No problems
 *   CICS_EXIT_BAD_ANCHOR	Invalid anchor value supplied
 *   CICS_EXIT_BAD_PARM		One or more parameters bad
 *   CICS_EXIT_USER_BASE + N    User specified value           
 */
cics_sshort_t CICS_EpiTermIdExit
                        ( IN    CICS_EpiAnchor_t	 Anchor
			, IN cics_ushort_t		 TermIndex
			, IN cics_char_t		*System
                        )
{
	return CICS_EXIT_OK;
}
