/*
 * NAME:	cics_api.h - Definition of the cics_api structure.
 *
 * VERSION:	1.3
 * 
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * DESCRIPTION:
 *	This file contains the definition for the EXEC CICS command argument
 *	structure used to pass arguments between user code and CICS.
 *	It is automatically included in user programs by cicstran.
 */

#if !defined cics_api_h			/* multi-inclusion lock [ */

#define	cics_api_h



#include <cicstype.h>
#include <cics_eib.h>
#include <stddef.h>

/*
 * OBJECT: cics_args (args) - EXEC CICS Command Argument Structure
 */
#define CICS_API_NARGS		32
#define CICS_API_SHORT_STRMAX	8
#define CICS_API_LONG_STRMAX	256
#define CICS_API_NUM_LONG_STR	3
#define CICS_API_DATA_VAL_MAX	8

struct cics_args
{
    cics_sshort_t  FnCode;		/* identifies EXEC CICS command */
    cics_sshort_t  DebugLine;		/* CEDF line number */
    cics_slong_t   StackLen;		/* CEDF working storage length */
    void           *StackPtr;		/* CEDF working storage */
    cics_slong_t   ArgMask;		/* identifies arguments supplied */
    cics_sshort_t  ArgCount;		/* number of aggregate arguments */
    cics_sshort_t  ArgCode[CICS_API_NARGS]; /* aggregate argument codes */
    cics_char_t    Filler[2];
    int		   ProgLang;		/* programming language */
    DFHEIBLK       *EIBPtr;		/* pointer to Exec Interface Block */
    union cics_argdata
    {
	cics_ubyte_t	ByteValue;
	cics_ubyte_t   *ByteArea;
	cics_sshort_t	ShortValue;
	cics_sshort_t  *ShortArea;
	cics_slong_t	LongValue;
	cics_slong_t   *LongArea;
	void	       *DataArea;
	void	       *PointerValue;
	void	       *PointerRef; /* really (void **,) but ansi compiler
				     * then requires a pointer to a (void *)
				     * rather than just a pointer to any
				     * type of pointer.
				     */
	cics_char_t     StringValue[CICS_API_SHORT_STRMAX]; /* not null
							     * terminated
							     */
	cics_char_t    *LongStringValue; /* points at LongStringTab element */
	cics_char_t    *StringArea;
	cics_ubyte_t	Bcd4Value[4];
	cics_ubyte_t	Bcd8Value[8];
	cics_ubyte_t	DataValue[CICS_API_DATA_VAL_MAX];
    }		ArgData[CICS_API_NARGS];
    cics_char_t	LongStringTab[CICS_API_NUM_LONG_STR][CICS_API_LONG_STRMAX];
};

void cics_api_exec_c(struct cics_args *CicsArgs);
void cics_api_strncpy(cics_char_t *Str1, const cics_char_t *Str2,
                      cics_sshort_t Len);
void         cics_api_memcpy(void *Str1, const void *Str2, cics_sshort_t Len);
cics_slong_t cics_api_stack(void **StackPtr);
int          cics_api_edf_init_c(int EDFOn, struct cics_args *Args);
extern void cics_api_exec_cpp(struct cics_args *CicsArgs);
/*
 * DESCRIPTION:
 * This structure contains all the information needed to run an EXEC CICS
 * command. It is populated from code generated using the CICS command
 * translator (cicstran) in the user's application program.
 *
 * All supported languages must define an identical data structure, so it is
 * kept as simple as possible.
 *
 * The members of this structures are as follows:
 *
 * :dl.
 * :dt.FnCode
 * :dd.The function code of the EXEC CICS command.
 * :dt.DebugLine
 * :dd.The line number for debugging with CEDF.
 * :dt.StackLen
 * :dd.This field is used for debugging with CEDF. When debugging C
 * a transaction program, StackLen is the length of the stack frame
 * for the current function. In cobol it is the length of the working
 * storage section for the currently executing procedure division.
 * :dt.StackPtr
 * :dd.This field is used for debugging with CEDF. When debugging C
 * a transaction program, StackPtr is a pointer to the stack frame
 * for the current function. In cobol it is a pointer to the start of
 * the working storage section for the currently executing procedure division.
 * :dt.ArgMask
 * :dd.A bitmask specifying which options are supplied.
 * :dt.ArgCount
 * :dd.The number of aggregate options.
 * :dt.ArgCode[CICS_API_NARGS]
 * :dd.:hp2.ArgCount:ehp2. option codes of aggregate options
 * :dt.Filler
 * :dd.A "filler" used to explicitly align ProgLang on a 4-byte boundary.
 * This is not needed in C, because ProgLang will automatically be aligned
 * on a 4-byte boundary; however, this is not necessarily true in other
 * languages, eg Cobol, so this "filler" field is used to document the gap
 * in the cics_args structure which must be duplicated when the structure
 * is translated into other languages.
 * :dt.ProgLang
 * :dd.The programming language for debugging with CEDF.
 * :dt.ArgData[CICS_API_NARGS]
 * :dd.An array of option argument data. For each non-aggregate option with
 * an argument, data is placed in the element indexed by the number of the
 * option bit set in ArgMask. For each aggregate option with an argument
 * specified in the ArgCode array, data is placed in the corresponding
 * element of ArgData. Each array element is a union consisting of a member
 * for each possible argument type; option argument data must be assigned
 * to the appropriate member of the union.
 * :dt.LongStringTab
 * :dd.An overflow table for long string values. A string value which is
 * potentially longer than CICS_API_SHORT_STRMAX is placed in this table,
 * and the LongStringValue pointer in the ArgData union is set to point at
 * it. LongStringTab has CICS_API_NUM_LONG_STR elements, each of which is
 * CICS_API_LONG_STRMAX characters long.
 * :edl.
 */


static struct cics_args CicsArgs;

/*
 * definition of EIB (used in generated code in transaction programs)
 */
static DFHEIBLK *dfheiptr;


#endif			/* multi-inclusion lock ] */

