/* 
 * NAME:     cics_aut.h - COMMAREA structure definition for AutoInstall
 *			  User-Exit program.
 * 
 * VERSION:  1.1
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 *
 * DESCRIPTION:
 *	This file contains all the data definitions required for the
 *	AutoInstall User-Exit program in order to perform selection
 *	of an appropriate model to use to create the new terminal definition
 *	of a terminal. A data structure containing a list of the names of
 *	all candidate models is passed into the program via the COMMAREA.
 *	This structure is different depending on the nature of the 
 *	AutoInstall request - Installation/UnInstallation - dictated by the 
 *	first byte of the COMMAREA.
 *
 *	(See CICS/MVS 2.1 Customisation Guide SC33-0507 for more 
 *	details about these structures.)
 *
 */

#if !defined(_CICS_AUT_H)

#define _CICS_AUT_H


#include <cicstype.h>				/* CICS-supplied datatypes */

/*
 * OBJECT: Definitions for the AutoInstall operation.
 *
 */

#define CICS_AUT_INSTALLOPERATION	'0'	/* Install operation required.*/
#define CICS_AUT_UNINSTALLOPERATION	'1'	/* Uninstall operation reqd.  */
#define CICS_AUT_SUCCESS_RETURN_STATUS	0x00	/* AutoInstall accepted.      */
#define CICS_AUT_ERROR_RETURN_STATUS	0x01	/* AutoInstall vetoed.        */

/*
 * DESCRIPTION:
 *	These macros define contants or limits within the
 *	structures defined in this file. 
 *
 *	CICS_AUT_INSTALLOPERATION
 *		Signals, in the operation field of the COMMAREA data
 *		structure, that an Install operation is to be performed
 *		as apposed to an Uninstall operation.
 *
 *	CICS_AUT_UNINSTALLOPERATION
 *		Signals, in the operation fiels of the COMMAREA data
 *		structure, that an UnInstall operation is required.
 *
 *	CICS_AUT_SUCCESS_RETURN_STATUS
 *		Value returned in 'Status' field of COMMAREA to signify
 *		that AutoInstall is permissible and is NOT vetoed.
 *
 *	CICS_AUT_ERROR_RETURN_STATUS
 *		Value returned in 'Status' field of COMMAREA to signify
 *		that AutoInstall should not go ahead because of some error
 *		or boundary condition.
 *
 */

/*
 * OBJECT: Structure of User Exit program parameter data for Installation.
 */

struct CICS_AUT_NetName 			/* Data structure 2.          */
{
    cics_ushort_t NetLength;			/* Network name length.       */
    cics_char_t FullName[8];			/* Network name.              */
};

struct CICS_AUT_AutInstNames			/* Data structure 3.          */
{
    cics_ushort_t NumAutInstNames;		/* Number of models in list.  */
    cics_char_t AutInstNames[1][8];		/* Dynamic array of Names.    */
};

struct CICS_AUT_ReturnData			/* Data structure 4.          */
{
    cics_char_t AutInstName[8];			/* Selected model name.       */
    cics_char_t Terminal[4];			/* Created terminal name.     */
    cics_char_t Printer[4];			/* Created printer name.      */
    cics_char_t AltPrinter[4];			/* Created alt. printer name. */
    cics_ubyte_t Status;			/* Status byte, 0 = Success.  */
};

struct CICS_AUT_VTAMLogon 			/* Data structure 5.          */
{
    cics_ushort_t LogLength;			/* CINIT request data length. */
};

struct CICS_AUT_InstUserExitData
{
    cics_ubyte_t Operation;			/* 0 = Install.               */
    cics_ubyte_t Dummy1[3];			/* Blank space.               */
    struct CICS_AUT_NetName *FullWord2;		/* Netname section pointer.   */
    struct CICS_AUT_AutInstNames *FullWord3;	/* AutInstNames section ptr.  */
    struct CICS_AUT_ReturnData *FullWord4;	/* Return section pointer.    */
    struct CICS_AUT_VTAMLogon *FullWord5;	/* VTAM section pointer.      */

    struct CICS_AUT_NetName NetName;		/* NetName section.           */
    struct CICS_AUT_ReturnData ReturnData;	/* Return Data section.       */
    struct CICS_AUT_VTAMLogon VTAMLogon;	/* VTAM section.              */
};

/*
 * DESCRIPTION:
 *	This data structure represents the User Exit parameter structure
 *	used for passing of terminal data into in and out for 
 *	AutoInstallation. It is based on the current definitions of 
 *	CICS types in terms of AIX types. All data except the 
 *	Model name list is included, as it is a dynamic structure, 
 *	and will be tagged onto the end following this structure in 
 *	a contiguous block of memory allocated for this purpose.
 *	(See CICS/MVS 2.1 Customisation Guide SC33-0507 for more 
 *	details about this structure.)
 *
 *	VTAM Login CINIT request data is not included, as it is not
 *	relevent, and the data length (VTAMLogon.LogLength) should be 
 *	set to zero.
 *	
 */

/*
 * OBJECT: Structure of User Exit program parameter data for UnInstallation.
 */

struct CICS_AUT_UnInstUserExitData
{
    cics_ubyte_t Operation;			/* 1 = UnInstall.             */
    cics_ubyte_t Dummy1[3];			/* Space.                     */
    cics_char_t Terminal[4];			/* Name of the terminal.      */
};

/*
 * DESCRIPTION:
 *	This structure is passed to the AutoInstall User-Exit
 *	program when an UnInstall operation is being performed.
 *	The only data contained is a 1 in the first byte, to signify
 *	that an UnInstall operation is underway, and the name of
 *	the terminal that is UnInstalling. The User-Exit program
 *	cannot veto the UnInstallation, therefore, there is no return
 *	data area.
 */

#endif /* _CICS_AUT_H */

