/*
 * NAME:	cics_esm.h - typedefs etc for ESM interface
 * VERSION:	1.1
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 *
 * DESCRIPTION:
 *
 *	This file contains definitions for all the types used in the ESM
 *	interface. It is intended for use by the security module and the ESM
 *	module.
 */

#if !defined CICS_ESM_H

#define CICS_ESM_H



#include <cicstype.h>			/* CICS-supplied datatypes */

/*
 * OBJECT:  CICS_ESM_INIT_OK - Return code for successful ESM initialisation.
 */

#define CICS_ESM_INIT_OK 0

/*
 * DESCRIPTION:
 *	The entry point for the ESM load module is used as an
 *	initialisation function for the ESM. It returns an integer value
 *	to indicate success or failure of the intialisation call. A
 *	zero value indicates success, any other value indicates failure.
 *	The returned value is logged if an error arises. Implementors of
 *	an ESM may use non-zero return codes to identify different
 *	initialisation errors.
 */

/*
 * OBJECT:  CICS_ESM_Return_t - The error codes returned by the ESM functions.
 */

typedef enum CICS_ESM_Return
{
       CICS_ESM_RETURN_OK,
       CICS_ESM_RETURN_INVALID,
       CICS_ESM_RETURN_FAIL
} CICS_ESM_Return_t;

/*
 * DESCRIPTION:
 *	These error codes are returned by the ESM interface functions.
 *	They are not defined using the product macros as they are
 *	made publicly available for users wishing to write their own
 *	ESM module.
 *	:p.CICS_ESM_RETURN_OK is returned by
 *	CICS_ESM_TSLCheck and CICS_ESM_RSLCheck to indicate that an
 *	access is valid.
 *	:p.CICS_ESM_RETURN_INVALID is
 *	returned by CICS_ESM_TSLCheck and CICS_ESM_RSLCheck to indicate
 *	that an access is invalid.
 *	:p.CICS_ESM_RETURN_FAIL is
 *	returned by CICS_ESM_TSLCheck and CICS_ESM_RSLCheck to indicate
 *	that an access could not be
 *	validated and that an internal check should be performed.
 */

/*
 * OBJECT:  CICS_ESM_ResourceClass - Enum of resource types passed to ESM.
 */

enum CICS_ESM_ResourceClass
{
  CICS_ESM_RESOURCE_CLASS_FD,
  CICS_ESM_RESOURCE_CLASS_TSD,
  CICS_ESM_RESOURCE_CLASS_TDD,
  CICS_ESM_RESOURCE_CLASS_JD,
  CICS_ESM_RESOURCE_CLASS_PD,
  CICS_ESM_RESOURCE_CLASS_TD
};

/*
 * DESCRIPTION:
 *	The resource class argument is passed to the ESM RSL validation
 *	function to resolve the type of the resource id to be validated
 *	and the resource specific data.
 */

/*
 * OBJECT: CICS_ESM_*_LEN - Length of various resource ids strings
 */

#define CICS_ESM_FILEID_LEN  8 
#define CICS_ESM_APPLID_LEN  8 
#define CICS_ESM_SYSID_LEN  4 
#define CICS_ESM_TDQID_LEN  4 
#define CICS_ESM_EXTRAPARTITION_FILE_LEN  127 
#define CICS_ESM_TSQID_LEN  9 
#define CICS_ESM_JRNLID_LEN  2 
#define CICS_ESM_PROGID_LEN  8 
#define CICS_ESM_TRANID_LEN  4 

#define CICS_ESM_TERMID_LEN  4 
#define CICS_ESM_USERID_LEN  8 
#define CICS_ESM_PRINCIPAL_LEN  256 
#define CICS_ESM_RELEASE_NO_LEN  4 
#define CICS_ESM_SERVER_NAME_SIZE  32 
#define CICS_ESM_FILE_NAME_SIZE  32 
#define CICS_ESM_INDEX_NAME_SIZE  32 

/*
 * DESCRIPTION:
 * These symbols define the lengths of various id fields not including
 * the null byte at the end.
 * These have to be supplied here as the ESM cannot get them from
 * product headers. They are used to identify the length of various
 * char[] id and name fields in the resource descriptors passed to the
 * ESM.
 */

/*
 * OBJECT: CICS_ESM_*Data - Resource-specific data passed to ESM.
 */

struct CICS_ESM_FileData
{
  cics_char_t Id[CICS_ESM_FILEID_LEN + 1];
  enum {
	CICS_ESM_FILE_LOCAL,
	CICS_ESM_FILE_REMOTE
       } Type;
  cics_char_t RemoteName[CICS_ESM_FILEID_LEN + 1];
  cics_char_t RemoteSysId[CICS_ESM_SYSID_LEN + 1];
  cics_char_t FileServer[CICS_ESM_SERVER_NAME_SIZE  + 1];
  cics_char_t FileName[CICS_ESM_FILE_NAME_SIZE  + 1];
  cics_char_t FileIndex[CICS_ESM_INDEX_NAME_SIZE  + 1];
};

struct CICS_ESM_TDQData
{
  cics_char_t Id[CICS_ESM_TDQID_LEN + 1];
  enum {
	CICS_ESM_TDQ_INTRAPARTITION,
	CICS_ESM_TDQ_EXTRAPARTITION,
	CICS_ESM_TDQ_REMOTE,
	CICS_ESM_TDQ_INDIRECT
       } Type;
  cics_char_t RemoteName[CICS_ESM_TDQID_LEN + 1];
  cics_char_t RemoteSysId[CICS_ESM_SYSID_LEN + 1];
  cics_char_t ExtrapartitionFile[CICS_ESM_EXTRAPARTITION_FILE_LEN + 1];
  cics_char_t IndirectName[CICS_ESM_TDQID_LEN + 1];
};

struct CICS_ESM_TSQData
{
  cics_char_t Id[CICS_ESM_TSQID_LEN + 1];
  enum {
	CICS_ESM_TSQ_MAIN,
	CICS_ESM_TSQ_AUXILIARY,
	CICS_ESM_TSQ_TEMPLATE,
	CICS_ESM_TSQ_REMOTE
       } Type;
  cics_char_t RemoteName[CICS_ESM_TSQID_LEN + 1];
  cics_char_t RemoteSysId[CICS_ESM_SYSID_LEN + 1];
};

struct CICS_ESM_JournalData
{
  cics_char_t Id[CICS_ESM_JRNLID_LEN + 1];
};

struct CICS_ESM_ProgramData
{
  cics_char_t Id[CICS_ESM_PROGID_LEN + 1];
};

struct CICS_ESM_TranData
{
  cics_char_t Id[CICS_ESM_TRANID_LEN + 1];
  enum {
	CICS_ESM_TRAN_LOCAL,
	CICS_ESM_TRAN_REMOTE
       } Type;
  cics_char_t RemoteName[CICS_ESM_TRANID_LEN + 1];
  cics_char_t RemoteSysId[CICS_ESM_SYSID_LEN + 1];
};

union CICS_ESM_ResourceData
{
  struct CICS_ESM_FileData		File;
  struct CICS_ESM_TDQData		TDQ;
  struct CICS_ESM_TSQData		TSQ;
  struct CICS_ESM_JournalData		Jrnl;
  struct CICS_ESM_ProgramData		Prog;
  struct CICS_ESM_TranData		Tran;
};

/*
 * DESCRIPTION:
 *	The ESM is passed a variety of different data from a resource
 *	definition to enable it to validate access to the
 *	resource. This data varies depending upon the
 *	resource class. A generic ResourceData value is passed in the ESM
 *	call. The ESM uses the ResourceClass argument to resolve it to a
 *	particular type.
 *	:p.The current interface supplies the resource id, the
 *	remote resource id  and remote sysid(if appropriate) and
 *	identifiers for AIX and sfs data which are
 *	used to implement the resource.
 *	:p.The field :hp2.Type:ehp2. in a CICS_ESM_FileData struct is
 *	used to indicate whether the file is local or remote. It is only
 *	in the latter case that RemoteName and RemoteSysId are valid.
 *	:p.The field :hp2.Type:ehp2. in a CICS_ESM_TDQData struct is
 *	used to indicate the type of queue and hence which of the
 *	fields RemoteName, RemoteSysId, ExtrapartitionFile,
 *	and IndirectName are valid.
 *	:p.The field :hp2.Type:ehp2. in a CICS_ESM_TSQData struct is
 *	used to indicate the type of queue and hence whether the
 *	fields RemoteName and RemoteSysId are valid.
 *	:p.The field :hp2.Type:ehp2. in a CICS_ESM_TranData struct is
 *	used to indicate whether the transaction is remote and hence whether
 *	RemoteName and RemoteSysId are valid.
 *	:p.Note that all string fields contain an extra element to allow
 *	for a C null byte as a terminator.
 */

/*
 * OBJECT: CICS_ESM_RegionData - Data supplied to the ESM defining the region
 */

struct CICS_ESM_RegionData
{
  cics_char_t Id[CICS_ESM_APPLID_LEN + 1];
  cics_char_t ReleaseNo[CICS_ESM_RELEASE_NO_LEN + 1];
};

/*
 * DESCRIPTION:
 * ESM_Init and the ESM callback functions are passed data describing
 * the region.
 */

/*
 * OBJECT: CICS_ESM_ContextData - Data supplied to the ESM defining the
 * transaction, user, term etc.
 */

struct CICS_ESM_UserData
{
  cics_char_t Id[CICS_ESM_USERID_LEN + 1];
  cics_char_t Principal[CICS_ESM_PRINCIPAL_LEN + 1];
};

struct CICS_ESM_TermData
{
  cics_char_t Id[CICS_ESM_TERMID_LEN + 1];
};

struct CICS_ESM_CommData
{
  cics_char_t Id[CICS_ESM_SYSID_LEN + 1];
};

struct CICS_ESM_ContextData
{
  struct CICS_ESM_TranData *TranData;
  struct CICS_ESM_UserData *UserData;
  struct CICS_ESM_TermData *TermData;
  struct CICS_ESM_ProgramData *ProgData;
  struct CICS_ESM_CommData *CommData;
};

/*
 * DESCRIPTION:
 *	The ESM is passed a variety of task context data describing the user,
 *	terminal etc. for the current transaction. Each resource is
 *	described by a struct. These structs are wrapped up in another
 *	struct. This is to make it easier to extend the interface at a
 *	later date.
 *	:p.For each resource descriptor the interface supplies the resource
 *	id and the remote resource id  and remote sysid(if appropriate).
 *	:p.The field ReleaseNo in a CICS_ESM_RegionData struct is used
 *	to identify the product release version.
 *	:p.Note that ProgData may be null when performing a TSL check.
 *	:p.Note that CommData will be null unless the transaction
 *	has been scheduled by a remote system.
 *	:p.Note also that some of the structs included in the generic
 *	secure resource union above are also used to pass context data.
 *	:p.Note also that all string fields contain an extra element to allow
 *	for a C null byte as a terminator.
 */

/*
 * OBJECT: CICS_ESM_TaskData - User data area addresses passed to ESM
 */

struct CICS_ESM_TaskData
{
  cics_ulong_t  CICS_ESM_CWA_Len;
  void           *CICS_ESM_CWA;
  cics_ushort_t CICS_ESM_TCTUA_Len;
  void           *CICS_ESM_TCTUA;
};

/*
 * DESCRIPTION:
 *	The ESM is supplied with the addresses of two user data areas,
 *	the CWA and the TCTUA. The addresses are passed as pointers
 *	to void in a structure. Note that these pointers address the data
 *	area following the structure signature.
 */

/*
 * OBJECT: CICS_ESM_FnBuf - Buffer for passing ESM check function
 * addresses.
 */

struct CICS_ESM_FnBuf
{
  CICS_ESM_Return_t (*TSLCheckFn)(struct CICS_ESM_RegionData *,
					struct CICS_ESM_ContextData *);
  CICS_ESM_Return_t (*RSLCheckFn)(struct CICS_ESM_RegionData *,
					struct CICS_ESM_ContextData *,
					union CICS_ESM_ResourceData *,
					enum CICS_ESM_ResourceClass,
					struct CICS_ESM_TaskData *,
					cics_char_t *);
};

/*
 * DESCRIPTION:
 *	The ESM_Init function is the entry point for an AIX
 *	:hp2.load:ehp2.ed module. It is passed a structure in which
 *	to return the addresses of the ESM check functions, thus
 *	avoiding the need to resolve references to the ESM at load
 *	time.
 */



int CICS_ESM_Init(struct CICS_ESM_RegionData *RegionData,
		struct CICS_ESM_FnBuf *FnBuf);

CICS_ESM_Return_t CICS_ESM_TSLCheck(struct CICS_ESM_RegionData *RegionData,
				struct CICS_ESM_ContextData *ContextData);

CICS_ESM_Return_t CICS_ESM_RSLCheck(struct CICS_ESM_RegionData *RegionData,
				struct CICS_ESM_ContextData *ContextData,
				union CICS_ESM_ResourceData *ResourceData,
				enum CICS_ESM_ResourceClass ResourceClass,
	   			struct CICS_ESM_TaskData *TaskData,
				cics_char_t *EIBFn);

#endif /* CICS_ESM_H */


/* End of Module */
