/*
 * NAME:	09 - CICS user conversion program data declarations.
 * VERSION:	1.1
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 *
 * DESCRIPTION:
 *     This file contains definitions for all the types required by the
 *     Function Shipping User Exit interface.
 */


#if !defined(CICS_FSCV_H)

#define CICS_FSCV_H


#include <cicstype.h>		/* CICS-supplied datatypes */

/*
 * OBJECT: CICS_FSCV_Dir - Conversion direction.
 */

enum CICS_FSCV_Dir
{
    CICS_FSCV_DIR_LOCAL_TO_REMOTE,
    CICS_FSCV_DIR_REMOTE_TO_LOCAL
};


/*
 * DESCRIPTION:
 * This enum is passed to the user conversion routines to indicate which
 * direction conversion should be performed in.
 * :dl tsize=32.
 * :dt.CICS_FSCV_DIR_LOCAL_TO_REMOTE
 * :dd.Convert from local to remote
 * :dt.CICS_FSCV_DIR_REMOTE_TO_LOCAL
 * :dd.Convert from remote to local
 * :edl.
 */


/*
 * OBJECT: CICS_FSCV_ResourceT - Resource type definitions.
 */

enum CICS_FSCV_ResourceT
{
    CICS_FSCV_RESOURCET_FC,
    CICS_FSCV_RESOURCET_TD,
    CICS_FSCV_RESOURCET_TS,
    CICS_FSCV_RESOURCET_IC,
    CICS_FSCV_RESOURCET_IC_NOCHK,
    CICS_FSCV_RESOURCET_DPL
};


/*
 * DESCRIPTION:
 *
 * Defines the types of resource that the user data conversion function may use
 * as a key to obtaining a conversion template by.
 * The types are as follows:
 * :dl tsize=30.  
 * :dt.CICS_FSCV_RESOURCET_FC
 * :dd.Resource type File Control
 * :dt.CICS_FSCV_RESOURCET_TD
 * :dd.Resource type Transient Data
 * :dt.CICS_FSCV_RESOURCET_TS
 * :dd.Resource type Temporary Storage
 * :dt.CICS_FSCV_RESOURCET_IC
 * :dd.Resource type Interval Control
 * :dt.CICS_FSCV_RESOURCET_IC_NOCHK
 * :dd.Resource type NOCHECK Interval Control 
 * :dt.CICS_FSCV_RESOURCET_DPL
 * :dd.Resource type Distributed Program Link
 * :edl.
 */


/*
 * OBJECT: CICS_FSCV_ConvType - Types of conversion that may be applied to
 *                              user data
 */

enum CICS_FSCV_ConvType
{
    CICS_FSCV_CONVTYPE_CHAR,
    CICS_FSCV_CONVTYPE_GRAPHIC,
    CICS_FSCV_CONVTYPE_NONE,
    CICS_FSCV_CONVTYPE_USERDATA,
    CICS_FSCV_CONVTYPE_HW_NUMERIC,
    CICS_FSCV_CONVTYPE_FW_NUMERIC
};

/*
 * DESCRIPTION:
 * This describes the type of the data to be converted. A type may or may not
 * result in a conversion, depending on the local system byte order.
 * :dl tsize=31.
 * :dt.CICS_FSCV_CONVTYPE_CHAR
 * :dd.Data is character. Convert as a character, using an operating system
 * conversion routine.
 * :dt.CICS_FSCV_CONVTYPE_GRAPHIC
 * :dd.Data is pure DBCS data. Convert as per CHAR, after surrounding the data 
 * with SOSI bytes.
 * :dt.CICS_FSCV_CONVTYPE_NONE
 * :dd.No conversion required.
 * :dt.CICS_FSCV_CONVTYPE_USERDATA
 * :dd.User Data. Call UCNV to convert.
 * :dt.CICS_FSCV_CONVTYPE_HW_NUMERIC
 * :dd.2 byte numeric data.
 * :dt.CICS_FSCV_CONVTYPE_FW_NUMERIC
 * :dd.4 byte numeric data.
 * :edl.
 */

/*
 * OBJECT: CICS_FSCV_ByteOrder - Byte order of remote system
 */

enum CICS_FSCV_ByteOrder
{
    CICS_FSCV_BYTEORDER_370,
    CICS_FSCV_BYTEORDER_INTEL
};

/*
 * DESCRIPTION:
 * Defines the byte ordering of the remote system.
 * :dl tsize=30.
 * :dt.CICS_FSCV_BYTEORDER_370
 * :dd.The remote system uses 370 integer format byte ordering.
 * :dt.CICS_FSCV_BYTEORDER_INTEL
 * :dd.The remote system uses INTEL integer format byte ordering.
 * :edl.
 */


/*
 * OBJECT: CICS_FSCV_ConvField - Field part of a conversion template
 */

#define CICS_FSCV_SIGNATURE_LEN		8

struct CICS_FSCV_ConvField
{
    cics_char_t 		Signature[CICS_FSCV_SIGNATURE_LEN];
    struct CICS_FSCV_ConvField 	*Next;
    cics_ushort_t 		ConversionOffset;
    enum CICS_FSCV_ConvType 	ConvertType;
    cics_ushort_t 		ConvertLength;
    cics_ushort_t 		UserEscapeType;
};

#define CICS_FSCV_FIELD_SIGNATURE	"ERZ27CVF"

/*
 * DESCRIPTION:
 * Field data for a conversion template. This describes what to convert if
 * a match was found. This is part of a linked list, such that several
 * conversions may be made.
 * :dl tsize=19.
 * :dt.Signature
 * :dd.Structure signature (CICS_FSCV_FIELD_SIGNATURE)
 * :dt.Next
 * :dd.Pointer to next conversion template element. NULL for end of list.
 * :dt.ConversionOffset
 * :dd.Offset in data to be to begin conversion at
 * :dt.ConversionType
 * :dd.Type of data to convert
 * :dt.ConvertLength
 * :dd.Length of conversion data
 * :dt.UserEscapeType
 * :dd.Arbitrary value passed to DFHUCNV
 * :edl.
 */


/*
 * OBJECT: CICS_FSCV_DataType - Types of user data that may need conversion
 */
enum CICS_FSCV_SelectType
{
    CICS_FSCV_SELECT_TYPE_DEFAULT,
    CICS_FSCV_SELECT_TYPE_DATA,
    CICS_FSCV_SELECT_TYPE_XDATA,
    CICS_FSCV_SELECT_TYPE_KEY
};

/*
 * DESCRIPTION:
 * :dl tsize=31.
 * :dt.CICS_FSCV_SELECT_TYPE_DEFAULT
 * :dd.Indicates that option type DEFAULT has been specifed.
 * :dt.CICS_FSCV_SELECT_TYPE_DATA
 * :dd.Indicates that the data is in character format.
 * :dt.CICS_FSCV_SELECT_TYPE_XDATA
 * :dd.Indicates that the data is in hexadecimal format.
 * :dt.CICS_FSCV_SELECT_TYPE_KEY
 * :dd.Indicates the start of conversions to be applied to keys.
 * :edl.
 */


/*
 * OBJECT: CICS_FSCV_ConvSelect - Select part of Conversion Template
 */

struct CICS_FSCV_ConvSelect
{
    cics_char_t 		Signature[CICS_FSCV_SIGNATURE_LEN];
    struct CICS_FSCV_ConvSelect *Next;
    enum CICS_FSCV_SelectType	SelectType;
    cics_ushort_t 		CompareOffset;
    cics_ushort_t 		CompareLength;
    void 			*CompareData;
    struct CICS_FSCV_ConvField 	*FieldRoot;
};

#define CICS_FSCV_SELECT_SIGNATURE	"ERZ27CVS"

/*
 * DESCRIPTION:
 * Select data for a conversion template. This is part of a linked list 
 * describing the parts of a resource that may be compared against. Each
 * element in the list contains the root of a linked list of Field entries.
 * Compare data is ignored if this is the default select, or this is a FC Key
 * :dl tsize=16.
 * :dt.Signature
 * :dd.Structure signature (CICS_FSCV_SELECT_SIGNATURE)
 * :dt.Next
 * :dd.Next Select element in linked list. NULL if end of list.
 * :dt.SelectType
 * :dd.An enum depending on which type of option is specified and whether the
 *     key has been specified.
 * :dt.IsKey
 * :dd.TRUE if this select is for a FC Key.
 * :dt.CompareOffset
 * :dd.Offset from start of data area to begin comparison at
 * :dt.CompareLength
 * :dd.Length of comparison data
 * :dt.CompareData
 * :dd.Data to compare against
 * :dt.FieldRoot
 * :dd.Root of a linked list of field data. 
 * :edl.
 */


/*
 * OBJECT: CICS_FSCV_ConvInitial - Initial part of Conversion template 
 */

struct CICS_FSCV_ConvInitial
{
    cics_char_t			Signature[CICS_FSCV_SIGNATURE_LEN];
    cics_bool_t			UserExitReq;
    cics_ubyte_t 		*RemoteCodePage;
    cics_ubyte_t 		*LocalCodePage;
    struct CICS_FSCV_ConvSelect *SelectRoot;
};

#define CICS_FSCV_INITIAL_SIGNATURE	"ERZ27CVI"

/*
 * DESCRIPTION:
 * Initial part of a conversion template for a single resource name/type.
 * There is one instance of this per resource type/name. It holds data 
 * common to all parts of a conversion template, and the root of a linked list 
 * of Select data for the resource. This data (and all its list contents) are
 * defined statically per resource name/type.
 * :dl tsize=14.
 * :dt.Signature
 * :dd.Structure signature (CICS_FSCV_INITIAL_SIGNATURE)
 * :dt.RemoteCodePage
 * :dd.Code page of requesting system (specified by CLINTCP in DFHCNV macros) 
 * :dt.LocalCodePage
 * :dd.Code page of local system (specified by SRVERCP in DFHCNV macros) 
 * :dt.SelectRoot
 * :dd.Root of a linked list of select data. 
 * :edl.
 */


#include <iconv.h>

/*
 * OBJECT: CICS_FSCV_UserConv - Structure passed in the COMMAREA to the
 *                              DFHUCNV user conversion program.
 */

#define CICS_FSCV_RSC_MAXLEN		 64 
#define CICS_FSCV_MAX_CP_LEN		80

struct CICS_FSCV_UserConv
{
    cics_char_t 			Signature[CICS_FSCV_SIGNATURE_LEN];
    enum CICS_FSCV_Dir 			ConvDir;
    cics_char_t 			ResourceName[CICS_FSCV_RSC_MAXLEN + 1];
    cics_ushort_t			ResourceNameLength;
    enum CICS_FSCV_ResourceT 		ResourceT;
    void 				*UserData;
    cics_ushort_t 			UserDataLength;
    void 				*KeyData;
    cics_ushort_t 			KeyDataLength;
     iconv_t  			LocalToRemote;
     iconv_t  			RemoteToLocal;
    cics_ubyte_t 			LocalCodePage[CICS_FSCV_MAX_CP_LEN];
    cics_ubyte_t 			RemoteCodePage[CICS_FSCV_MAX_CP_LEN];
    enum CICS_FSCV_ByteOrder		ByteOrder;
    struct CICS_FSCV_ConvSelect 	*ConvTable;
};

#define CICS_FSCV_USER_SIGNATURE	"ERZ27CVU"

/*
 * DESCRIPTION:
 * This structure describes the parameter list passed in the COMMAREA 
 * to the user supplied conversion program, DFHUCNV. This allows DFHUCNV 
 * to access the conversion table and the data to be converted. All pointers
 * refer to part of the COMMAREA
 * The members are:
 * :dl tsize=20.
 * :dt.Signature
 * :dd.Structure signature (CICS_FSCV_USER_SIGNATURE)
 * :dt.ConvDir
 * :dd.Conversion direction
 * :dt.ResourceName
 * :dd.Name of resource.
 * If the conversion template has not been initialised it is set to NULL.
 * :dt.ResourceNameLength
 * :dd.Length of ResourceName. 
 * If the conversion template has not been initialised it's value is 0.
 * :dt.ResourceT
 * :dd.Resource type
 * :dt.UserData
 * :dd.Pointer to the user data for conversion. This may be NULL if there is 
 * no user data for conversion.
 * :dt.UserDataLength
 * :dd.Length of UserData. If there is no user data this is set to 0.
 * :dt.KeyData
 * :dd.Pointer to File Control Key data for conversion. This may be NULL if 
 * there is no key data for conversion.
 * :dt.KeyDataLength
 * :dd.Length of KeyData. If there is no key data this is set to 0.
 * :dt.LocalToRemote
 * :dd.iconv code page conversion descriptor for conversion from
 * local code page to remote.
 * :dt.RemoteToLocal
 * :dd.iconv code page conversion descriptor for conversion from
 * remote code page to local.
 * :dt.LocalCodePage
 * :dd.Code page of local system (specified by SRVERCP in DFHCNV macros) 
 * :dt.RemoteCodePage
 * :dd.Code page of requesting system (specified by CLINTCP in DFHCNV macros) 
 * :dt.ByteOrder
 * :dd.Defines the byte ordering of the remote system.
 * :dt.ConvTable
 * :dd.Pointer to start of conversion template data. 
 * :edl.
 */


#endif /* cics_fscv.h */

