/*
 * NAME:	cicssfmt.h - Declarations for Sample cicssfmt code
 * VERSION:	1.1
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * DESCRIPTION:	This header provides the declarations required by
 *		the sample cicssfmt code.
 */

#ifndef CICSSFMT_H

#define CICSSFMT_H			/* prevent recursive inclusion	*/


#include <cicstype.h>                  /* CICS-supplied external datatypes */
#include <cics_samples.h>	       /* CICS-supplied macro definitions  */
#include <time.h>		       /* Time management functions */
#include <limits.h>                    /* POSIX defines */
#include <cics_stat.h>		       /* Statistics record definition */
#include <nl_types.h>		       /* NLS catalogue types */


/*
 * OBJECT: CICS_SFMT_StatsRecord (statrec)- Holds a statistics record.
 */

struct CICS_SFMT_StatsRecord
{
    CICSSTAT_STATSRECORD(CICSSTAT_MAX_STAT_DETS) Record;
};

typedef struct CICS_SFMT_StatsRecord CICS_SFMT_StatsRecord_t;

/*
 * DESCRIPTION:
 * This structure is used to store a statistics record as defined in the
 * statistics file. 
 * :ul.
 * :li.:hp2.Record:ehp2. is the  statistics record with CICSSTAT_MAX_STAT_DETS
 * detail sub-records.
 * :eul.
 *
 */

/*
 * OBJECT: CICSSFMT_File (record) - Statistics File Structure
 */

struct CICS_SFMT_File
{
    int FileHandle;
    CICS_SFMT_StatsRecord_t Record;
};

typedef struct CICS_SFMT_File CICS_SFMT_File_t;

/*
 * DESCRIPTION:
 * This structure is used by the SFMT functions to access the statistics
 * record/file.
 * :ul.
 * :li.
 * :hp2.FileHandle:ehp2. - The File Descriptor returned by 
 * :hp2.CICS_SFMT_OpenStatsFile:ehp2. when the statistics file was opened.
 * :li.
 * :hp2.Record:ehp2. - The current record. This record structure is defined
 * in cics_stat.h. Its value is updated by the :hp2.CICS_SFMT_ReadRecord:ehp2.
 * :eul.
 */

/*
 * OBJECT: CICS_SFMT_Status (status) - Return codes from the SFMT functions.
 */

enum CICS_SFMT_Status
{
    CICS_SFMT_STATUS_OK,
    CICS_SFMT_FILE_ERR,
    CICS_SFMT_FILE_SHORT_READ,
    CICS_SFMT_FILE_ZERO_READ,
    CICS_SFMT_REC_NOT_FOUND,
    CICS_SFMT_MEM_ERROR
};

/*
 * DESCRIPTION:
 * This enum defines the return status from :hp2.CICS_SFMT_OpenStatsFile:ehp2.,
 * :hp2.CICS_SFMT_ReadRecord:ehp2., :hp2.CICS_SFMT_FindRecord:ehp2., and
 * :hp2.CICS_SFMT_CloseStatsFile:ehp2. as follows:
 * :dl tsize=26.
 * :dt.CICS_SFMT_STATUS_OK
 * :dd.Function completed successfully.
 * :dt.CICS_SFMT_FILE_ERR
 * :dd.The function detected an I/O error. The global variable,
 * :hp2.errno:ehp2. contains the &os. error code.
 * :dt.CICS_SFMT_FILE_SHORT_READ
 * :dd.The function encountered EOF before a complete record could be read.
 * This is indicative of an inconsistent statistics file. 
 * :dt.CICS_SFMT_FILE_ZERO_READ
 * :dd.The function did not read any data, because EOF was detected.
 * :dt.CICS_SFMT_REC_NOT_FOUND
 * :dd.The find function was unable to find the specified record in the file.
 * :dt.CICS_SFMT_MEM_ERROR
 * :dd.Unable to allocate memory.
 * :edl.
 */

/*
 * OBJECT: Limits - cicssfmt wide limits
 */

#define CICS_SFMT_NUM_CATEGORIES            15

#define CICS_SFMT_MAX_HEADERS		    3
#define CICS_SFMT_COL_PAGENO		    20

/* 
 * DESCRIPTION:
 * These defines provide :hp2.cicssfmt:ehp2. with a set of fixed limits.
 * All code should use them where appropriate in order to ensure uniform
 * restrictions are in place on the relevant objects.
 * 
 * :ul.
 * :li.
 * :hp2.CICS_SFMT_NUM_CATEGORIES:ehp2. The maximum number of statistics 
 * categories supported by :hp2.cicssfmt:ehp2. plus 1.
 * :li.
 * :hp2.CICS_SFMT_MAX_HEADERS:ehp2. is the number of lines in the page header.
 * :li.
 * :hp2.CICS_SFMT_COL_PAGENO:ehp2. is the number of columns away from the
 * right hand side of the page at which to print the :hp4.Page No:ehp4. string.
 * :eul.
 *
 */

/*
 * OBJECT: Bit Masking Macros (macros) - cicssfmt bit masking macros
 */

#define CICS_SFMT_CATSET_NUM(Cat_Id)	 (((cics_ulong_t)(Cat_Id)) >> 16)
#define CICS_SFMT_CATDESC_NUM(Cat_Id) (((cics_ulong_t)(Cat_Id)) & (0xFFFF))

/*
 * DESCRIPTION:
 * These defines provide :hp2.cicssfmt:ehp2. with a set of bit manipulations
 * in order to obtain the message set numbers and the message numbers from the
 * category identifiers defined in the cics_stat.h
 *
 * :ul.
 * :li.
 * :hp2.CICS_SFMT_CATSET_NUM:ehp2. gives the message set number for the 
 * specified category id.
 * :li.
 * :hp2.CICS_SFMT_CATDESC_NUM:ehp2. gives the message number for the
 * specified description id.
 * :eul.
 *
 */

/*
 * OBJECT: CICS_SFMT_Input (input)- Holds all the information specified by
 *                              the caller.
 */

struct CICS_SFMT_Input
{
    cics_char_t *Category[CICS_SFMT_NUM_CATEGORIES];
    time_t Starttime;
    time_t Endtime;
    cics_sshort_t Pagelength;
    cics_sshort_t Pagewidth;
    int Header;
    int Detailed;
};

/* 
 * DESCRIPTION:
 * This structure is used to store the input parameters as specified on the
 * command line. The input values are stored in this structure after they
 * have been validated and declared as correct.
 * This structure is initialised to the default values where applicable.
 * 
 * :ul.
 * :li.
 * :hp2.Category:ehp2. is an array that holds the statistics category 
 * to be reported. The default value is all categories.
 * :li.
 * :hp2.Starttime:ehp2. is the start time for the report in 'yymmddhhmmss'
 *  format. The default value is 0.
 * :li.
 * :hp2.Endtime:ehp2. is the end time for the report in 'yymmddhhmmss'
 *  format.The default value is 0.
 * :li.
 * :hp2.Pagelength:ehp2. is the page length in number of lines. The 
 * default value is 24.
 * :li.
 * :hp2.Pagewidth:ehp2. is the page width in number of columns. The 
 * default value is 80.
 * :li.
 * :hp2.Header:ehp2. indicates whether to print the header or not. The
 * default value is TRUE.
 * :li.
 * :hp2.Detailed:ehp2. indicates whether to print the 
 * Interval Detailed Report or not. The default value is FALSE.
 * :eul.
 *
 */

/*
 * OBJECT: CICS_SFMT_SummaryList (sumlist)- Holds all the summary statistics 
 *                             records read by CICS_SFMT_ReadRecord.
 */

struct CICS_SFMT_SummaryList
{
    CICS_SFMT_StatsRecord_t Sum_Record;
    struct CICS_SFMT_SummaryList *Next;
};

typedef struct CICS_SFMT_SummaryList CICS_SFMT_SummaryList_t;

/*
 * DESCRIPTION:
 * This structure is used to store the individual summary statistics record
 * read by CICS_SFMT_ReadRecord. It is used to calculate the Average Statistics
 * in case of the Interval Summary Report generation. In case of the Interval 
 * Detailed Report, this structure is used to store the summary statistics 
 * record, as the summary record precedes the detail records in the statistics 
 * file.
 * :ul.
 * :li.:hp2.Sum_Record:ehp2. is a statistics record with CICSSTAT_MAX_STAT_DETS
 * detail sub-records.
 * :li.:hp2.Next:ehp2. is the next element of the Summary list.
 * :eul.
 */

#if !defined(TRUE)
#define TRUE (1==1)
#endif

#if !defined(FALSE)
#define FALSE (!TRUE)
#endif

#if defined(SFMT_DEBUG_ON)
#define SFMT_DEBUG(x) printf("TRACE :"); printf x; fflush(stdout);
#else
#define SFMT_DEBUG(x)
#endif

/*
 * OBJECT: cicssfmt static data - Define to resolve multiple static data 
 *                                declaration
 */
#ifdef CICSSFMT_STATIC_DATA
    nl_catd CICS_SFMT_CatDescriptor;
    cics_char_t *CICS_SFMT_CatSet[CICS_SFMT_NUM_CATEGORIES];
    CICS_SFMT_File_t CICS_SFMT_StatsFile;
    struct CICS_SFMT_Input CICS_SFMT_InputInfo;
    CICSSTAT_Detail_t CICS_SFMT_ReportDetails[CICSSTAT_MAX_STAT_DETS];
    cics_ulong_t CICS_SFMT_LineNo = 0;
    cics_ulong_t CICS_SFMT_PageNo = 1;
    cics_char_t *CICS_SFMT_StatsFileName;
#else
    extern nl_catd CICS_SFMT_CatDescriptor;
    extern cics_char_t *CICS_SFMT_CatSet[CICS_SFMT_NUM_CATEGORIES];
    extern CICS_SFMT_File_t CICS_SFMT_StatsFile;
    extern struct CICS_SFMT_Input CICS_SFMT_InputInfo;
    extern CICSSTAT_Detail_t CICS_SFMT_ReportDetails[CICSSTAT_MAX_STAT_DETS];
    extern cics_ulong_t CICS_SFMT_LineNo;
    extern cics_ulong_t CICS_SFMT_PageNo;
    extern cics_char_t *CICS_SFMT_StatsFileName;
#endif

/*
 * DESCRIPTION:
 * :dl tsize=20.
 * :dt.CatDescriptor
 * :dd.cicssfmt message catalogue descriptor
 * :dt.CICS_SFMT_CatSet
 * :dd.array of strings holding the permissible statistic categories.
 * :dt.StatsFile
 * :dd.holds the statsfile handle and the current statistics record.
 * :dt.InputInfo
 * :dd.holds all the command line parameters after validation.
 * :dt.StatsRecord
 * :dd.holds a statistics record.
 * :dt.ReportDetails
 * :dd.array that holds the report details when printing the statistics.
 * :dt.LineNo
 * :dd.current line number. Reset at page start.
 * :dt.PageNo
 * :dd.current page number.
 * :dt.PageHeader
 * :dd.holds the fully formed header for each page.
 * :edl.
 */

/*
 * Function Prototypes:
 */

/*
 * Statistics File Operation Functions
 */
enum CICS_SFMT_Status CICS_SFMT_OpenStatsFile(IN cics_char_t *FileName,
				     INOUT CICS_SFMT_File_t *StatsFileRecord);
enum CICS_SFMT_Status CICS_SFMT_CloseStatsFile(
				    IN CICS_SFMT_File_t *StatsFileRecord);
enum CICS_SFMT_Status CICS_SFMT_ReadRecord(
				    INOUT CICS_SFMT_File_t *StatsFileRecord);
enum CICS_SFMT_Status CICS_SFMT_FindRecord(IN time_t StartTime,
					  IN time_t EndTime,
				      INOUT CICS_SFMT_File_t *StatsFileRecord);

/*
 * Catalogue Operation Functions
 */
void CICS_SFMT_Msg2Int(IN CICSSTAT_Mesg_t MsgIndex,
			   OUT int *MsgNumber,
                   	   OUT int *MsgSet);

/*
 * Main program support functions prototypes
 */
enum CICS_SFMT_Status  CICS_SFMT_GenDetail(void);

enum CICS_SFMT_Status  CICS_SFMT_GenSummary(void);

int CICS_SFMT_CatIsThere(IN cics_ulong_t Id);

void CICS_SFMT_AverageCalc(IN cics_bool_t IsInterval,
			   INOUT CICS_SFMT_SummaryList_t *List);

void CICS_SFMT_OutputStats(IN CICS_SFMT_StatsRecord_t *StatRecord);

void CICS_SFMT_InitCatSet(void);

void CICS_SFMT_String2Timet(IN cics_char_t * TmString,
			    OUT time_t *Time);

void CICS_SFMT_IncrLine(IN int Num);

enum CICS_SFMT_Status CICS_SFMT_StoreRec(IN CICS_SFMT_StatsRecord_t *StatRec,
				    INOUT CICS_SFMT_SummaryList_t **List);

int CICS_SFMT_ValidateTime(IN cics_char_t *TmString);

void CICS_SFMT_OutputMessage(IN int SetNo,
                             IN int MessageNo,
                             ...);

int CICS_SFMT_PageHeaderSetUp(cics_char_t *FileName);

#endif

