/*
 * NAME:        cicsue.h - CICS/6000 User Exit Interface header
 *
 * VERSION:     1.2 
 *
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * DESCRIPTION:
 *  cicsue.h
 *
 *  This file contains all the User Exit (UE) Interface 
 *  public definitions.
 *  Refer to the Administration Guide and Administration Reference
 *  for further information on User Exits and guidelines on 
 *  writing programs
 */

#ifndef _CICSUE_H            /* Prevent multiple #includes of this file */

#define _CICSUE_H



/* 
 * **************************************************************************
 * **************************************************************************
 *   Generic User Exit definitions
 * **************************************************************************
 * **************************************************************************
 */

/*
 * Constants.
 */

#define  cics_UE_NAME_MAX  8         /* max length of UE name  */
#define  cics_UE_TRAN_MAX  4         /* transaction name max length */
#define  cics_UE_USER_MAX  8         /* userid max length */
#define  cics_UE_TERM_MAX  4         /* termid max length */
#define  cics_UE_PROG_MAX  8         /* application program name max length */
#define  cics_UE_SYSID_MAX 4         /* Max length of a sysid */
#define  cics_UE_APPL_MAX  8         /* Max length of a region name */
#define  cics_UE_HEADER_VERSION 1    /* all version number start at 1 */
#define  cics_UE_WORKAREA_SIZE 128   /* fixed size for this version */


/*
 * Data Types.
 */

typedef cics_ulong_t  cics_UE_Return_t;          /* return code from UE */

typedef enum UE_YesNo
{
    UE_Yes,
    UE_No
} cics_UE_YesNo_t;

/*
 * DESCRIPTION:
 *  standard header - cics_UE_Header_t which is passed to all User Exits
 *
 * :dl.
 * :dt. UE_Version 
 * :dd. This is the version number for this structure and takes the value 
 * defined in cics_UE_HEADER_VERSION. It is advisable to check this in
 * UE programs to ensure that the version number passed when the UE
 * is called is the same as that contained in the version of cicsue.h used
 *  when the program was generated. If a mismatch is detected then contents
 * of fields are unpredictable and the UE program should be regenerated
 * against the correct version of cicsue.h
 * :dt.  UE_ProcessId 
 * :dd. The process id
 * :dt.  UE_Thread   
 * :dd. The thread in which the User Exit is called
 * :dt.  UE_Number 
 * :dd. The external number of the User Exit. 
 * This is the number specified for the UserExitNumber parameter in the PD stanza 
 * :dt.  UE_Name
 * :dd. The name of the User Exit.
 * :dt.  UE_TransId
 * :dd. The transaction which was active when the User Exit was called 
 * :dt.  UE_TaskNo 
 * :dd. The task number 
 * :dt.  UE_Workarea 
 * :dd. Pointer to the workarea associated with User Exits for this task. It can 
 * be used to pass information between User Exits for the same task and is also
 * useful when debugging User Exits programs. The workarea is located in storage 
 * immediately after the cics_UE_header storage; both areas are traced
 * when an A15Z abend is issued, if trace is active for the component which
 * called the User Exit at which the A15Z was caught. It can also be traced based
 * on the setting of CICS_TraceOnExit.
 * :dt.  UE_WorkareaSize 
 * :dd. The size of the UE_Workarea, as defined in cics_UE_WORKAREA_SIZE 
 * Note that this is a fixed value. 
 * :dt.  UE_TraceOnExit 
 * :dd. This is always set to UE_No when a User Exit is called but can be set
 * to UE_Yes by the User Exit program to cause the User Exit parameter lists and 
 * UE_Workarea to be traced on return from the User Exit, if tracing is active
 * for the component which called the User Exit.
 * :dt.  UE_RegionName
 * :dd.  The name of this CICS region.
 * :edl.
 */

typedef struct
{
    cics_ushort_t     UE_Version;
    cics_ulong_t      UE_ProcessId;
    cics_ulong_t      UE_Thread;
    cics_ushort_t     UE_Number;
    cics_char_t       UE_Name[cics_UE_NAME_MAX+1];
    cics_char_t       UE_TransId[cics_UE_TRAN_MAX+1];
    cics_ulong_t      UE_TaskNo;
    cics_char_t      *UE_Workarea;
    cics_ushort_t     UE_WorkareaSize;
    cics_UE_YesNo_t   UE_TraceOnExit;
    cics_char_t       UE_RegionName[cics_UE_APPL_MAX+1];
}  cics_UE_Header_t;


/*
 * DESCRIPTION:
 * Following are provided for cics_UE_Return_t values
 *
 * These define the required action on return from a User Exit and must
 * be set by the User Exit program.
 * This mechanism enables the User Exit program to participate in the subsequent
 * actions taken by CICS. The meanings of the return codes are User Exit 
 * dependent and are as detailed in the CICS Administration Reference
 * for each User Exit. 
 *
 * These return codes are visible in trace on exit from a User Exit.
 * 
 * UE_Shortonstorage is set by CICS code when it is unable to obtain 
 * storage in which to build the User Exit parameter lists. In this situation
 * the User Exit program is not invoked and the action taken by CICS is 
 * when this is returned varies by User Exit, but generally will be the default 
 * action.
 */


#define  UE_Normal           (cics_UE_Return_t) 0
#define  UE_Bypass           (cics_UE_Return_t) 1
#define  UE_Abend            (cics_UE_Return_t) 2
#define  UE_Shortonstorage   (cics_UE_Return_t) 3
#define  UE_Term_Abend       (cics_UE_Return_t) 4
#define  UE_Terminate        (cics_UE_Return_t) 5
#define  UE_ProgramNotKnown  (cics_UE_Return_t) 6
#define  UE_HeaderVersion    (cics_UE_Return_t) 7
#define  UE_SpecificVersion  (cics_UE_Return_t) 8

/*
 * **************************************************************************
 * **************************************************************************
 *   Specific definitions for UE014015 
 * **************************************************************************
 * **************************************************************************
 */

/*
 * Constants
 */
#define  cics_UE014015_VERSION 1    /* all version number start at 1 */

/*
 * Data Types.
 */

typedef enum
{
    UE_Normaltermination,
    UE_Abnormaltermination
} cics_UE_Terminationtype_t;

/*
 * DESCRIPTION:
 *  User Exit specific structure for UE014015 which is always
 *  passed when a User Exit program is called for that User Exit 
 *
 * :dl.
 * :dt. UE_Version 
 * :dd. This is the version number for this structure and takes the value 
 * defined in cics_UE014015_VERSION. It is advisable to check this 
 * in User Exit programs to ensure that the version number passed when the User Exit
 * is called is the same as that contained in the version of cicsue.h used
 *  when the program was generated. If a mismatch is detected then contents
 * of fields are unpredictable and the User Exit program should be regenerated
 * against the correct version of cicsue.h
 * :dt. UE_Terminationtype
 * :dd. This takes 2 values as defined in cics_UE_Terminationtype_t.
 * In this context a task ending with an abend is considered to be 
 * UE_Normaltermination, taking as it does the CICS internal definition of
 * normal which means terminating in a controlled manner. 
 * :edl.
 */


typedef struct
{

    cics_ushort_t             UE_Version;
    cics_UE_Terminationtype_t UE_Terminationtype;

} cics_UE014015_t;


/*
 * Functions.
 */

cics_UE_Return_t cics_UE014015(cics_UE_Header_t       *UE_Header,
	  	 	       cics_UE014015_t        *UE_Specific);


/* 
 * **************************************************************************
 * **************************************************************************
 *   Specific definitions for UE052017 
 * **************************************************************************
 * **************************************************************************
 */

/*
 * Constants
 */

#define cics_UE_DUMP_MAX  4                /* max length of dump code name  */
#define cics_UE052017_VERSION 1            /* all version number start at 1 */

/*
 * Data Types.
 */

typedef enum UE_DumpReason     /* reason for dump */
{
    UE_DUMP_USER,
    UE_DUMP_ABEN,
    UE_DUMP_ASRA,
    UE_DUMP_ASRB,
    UE_DUMP_SNAP,
    UE_DUMP_SHUT,
    UE_DUMP_SYSA
} cics_UE_DumpReason_t;

/*
 * DESCRIPTION:
 *  Note that UE052017 is similar in function to the CICS/ESA
 *  Global User Exit XDUREQ.
 *
 *  User Exit specific structure for UE052017 which is always
 *  passed when a User Exit program is called for that User Exit 
 *
 * :dl.
 * :dt. UE_Version 
 * :dd. This is the version number for this structure and takes the value 
 * defined in cics_UE052017_VERSION. It is advisable to check this 
 * in User Exit programs to ensure that the version number passed when the User Exit
 * is called is the same as that contained in the version of cicsue.h used
 *  when the program was generated. If a mismatch is detected then contents
 * of fields are unpredictable and the User Exit program should be regenerated
 * against the correct version of cicsue.h
 * :dt. UE_User 
 * :dd. This is the userid in operation for the task issuing the dump request. 
 * It may not be meaningful for system dump requests. 
 * :dt. UE_Term
 * :dd. This is the terminal for the task issuing the dump request. 
 * It may not be meaningful for system dump requests. 
 * :dt. UE_Prog
 * :dd. This is the program for the task issuing the dump request. 
 * It may not be meaningful for system dump requests. 
 * :dt. UE_Dumpcode
 * :dd. This is the dump code which will be used for the dump, if the 
 * cics_UE_Return_t requests that dumping continues. 
 * It cannot be altered by a User Exit program.
 * :dt. UE_DumpReason
 * :dd. This is the type of dump which has been requested; the meanings are
 * as defined in cics_UE_DumpReason_t.
 * :dt. UE_CWA 
 * :dd. This is a pointer to the CWA. The onus is on the customer to ensure
 * that a User Exit program does not compromise the integrity of this area,
 * which is shared by all tasks in the CICS region.
 * :dt. UE_CWALEN 
 * :dd. This is the length of the CWA as definined in the RD stanza
 * :dt. UE_TCATWA 
 * :dd. This is a pointer to the TWA for the task
 * It may not be meaningful for system dump requests. 
 * :dt. UE_TCATWA_LEN 
 * :dd. This is the length of the TWA as definined in the TD stanza for the
 * transaction (UE_TransId in the cics_UE_Header parameter list) 
 * It may not be meaningful for system dump requests. 
 * :dt. UE_TransDump 
 * :dd. This is the value of the TransDump parameter as defined in the TD
 * stanza for the transaction. It cannot be altered by a User Exit program.
 * It may not be meaningful for system dump requests. 
 *
 * :edl.
 */
typedef struct
{

    cics_ushort_t          UE_Version;
    cics_char_t            UE_User[cics_UE_USER_MAX+1];
    cics_char_t            UE_Term[cics_UE_TERM_MAX+1];
    cics_char_t            UE_Prog[cics_UE_PROG_MAX+1];
    cics_char_t            UE_Dumpcode[cics_UE_DUMP_MAX+1];
    cics_UE_DumpReason_t   UE_DumpReason;
    cics_char_t           *UE_CWA;
    cics_ushort_t          UE_CWALEN;
    cics_char_t           *UE_TCATWA;
    cics_ushort_t          UE_TCATWA_LEN;
    cics_UE_YesNo_t        UE_TransDump;

} cics_UE052017_t;


/*
 * Functions.
 */


cics_UE_Return_t cics_UE052017(cics_UE_Header_t       *UE_Header,
			       cics_UE052017_t        *UE_Specific);



/*
 * **************************************************************************
 * **************************************************************************
 *   Specific definitions for UE014025
 * **************************************************************************
 * **************************************************************************
 */

/*
 * Constants
 */

#define cics_UE_ABND_MAX      4            /* Max length of an abend code   */
#define cics_UE_USERAREA_SIZE 128          /* User Exit user area size      */
#define cics_UE014025_VERSION 2            /* All version number start at 1 */

/*
 * Data Types.
 */

typedef enum cics_UE_Func                  /* Reason for UE invocation      */
{
    UE_ROUTESEL,
    UE_RTESELERR,
    UE_ROUTETERM,
    UE_RTERESVD,
    UE_RTEABEND
} cics_UE_Func_t;

typedef enum cics_UE_Error                 /* Reason for route sel error   */
{
    UE_SYSID_UNKNOWN,
    UE_SYSID_OUT_SRV,
    UE_CON_FAILURE,
    UE_NO_ERROR
} cics_UE_Error_t;


/*
 * DESCRIPTION:
 *  User Exit specific structure for UE014025 which is always
 *  passed when a User Exit program is called for that User Exit
 *
 * :dl.
 * :dt. UE_Version
 * :dd. This is the version number for this structure and takes the value
 * defined in cics_UE014025_VERSION. It is advisable to check this
 * in User Exit programs to ensure that the version number passed when the User Exit 
 * is called is the same as that contained in the version of cicsue.h used
 * when the program was generated. If a mismatch is detected then contents
 * of fields are unpredictable and the User Exit program should be regenerated
 * against the correct version of cicsue.h
 * 
 * The Version Number was altered to 2 when *UE_Dyrcomptr and UE_Dyrcomlgth
 * were added to the specific structure.
 *
 * :dt. UE_Dyrfunc
 * :dd. contains the reason why the User Exit has been invoked for a one
 * particular attempt to route a transaction. Possible reasons are
 * defined in cics_UE_Func_t.
 * :dt. UE_Dyrcount
 * :dd. contains the number of times the User Exit has been invoked for a one
 * attempt to route a transaction.
 * :dt. UE_Dyrsysid
 * :dd. contains the system Id to which the transaction will be routed.
 * This can be a remote system Id or even the local system Id.
 * For the initial route selection this will be taken from the TD stanza,
 * but for any subsequent invocations of the User Exit for a particular attempt
 * to route a transaction, it will contain the last system Id you specified.
 * Changing this parameter will change to system Id to be routed to.
 * :dt. UE_Dyrtptr
 * :dd. is the pointer to the TCTUA.
 * :dt. UE_Dyrtlgth
 * :dd. is the length of the TCTUA.
 * :dt. UE_Dyrcwaptr
 * :dd. This is a pointer to the CWA. The onus is on the customer to ensure
 * that a User Exit program does not compromise the integrity of this area,
 * which is shared by all tasks in the CICS region.
 * :dt. UE_Dyrcwalgth
 * :dd. This is the length of the CWA as definined in the RD stanza
 * :dt. UE_Dyrlclsys
 * :dd. contains the name of the local system Id.
 * :dt. UE_Dyrlclapl
 * :dd. contains the applid of the local region.
 * :dt. UE_Dyropter
 * :dd. specifies whether the User Exit should be invoked at termination time.
 * UE_Yes means invoke at termination time, UE_No means do not invoke
 * at termination time.
 * :dt. UE_Dyrretry
 * :dd. specifies whether the User Exit should be invoked if a connection
 * failure occurs. UE_Yes means invoke at failure time, UE_No means do
 * not invoke if a failure occurs.
 * :dt. UE_Dyrerror
 * :dd. contains a value to indicate what type of route selection failure
 * has occurred. This is only relevant if UE_dyrfunc is UE_RTESELERR.
 * Possible values are described by cics_UE_Error_t.
 * :dt. .UE_Dyrtran
 * :dd. is the name of the remote transaction to be run on the remote
 * system. This, like UE_dyrsysid, can be changed and again contains
 * the TD stanza RemoteName value or the modified transaction Id depending on
 * the invocation of the User Exit. This will be ignored if the transaction
 * is run locally.
 * :dt. UE_Dyrprog
 * :dd. is the name of the program to be run if the transaction is to be run
 * locally. This can be changed.
 * :dt. UE_Dyrabcde
 * :dd. contains the transaction abend code, when the User Exit is invoked with a
 * reason of UE_RTEABEND.
 * :dt. UE_Dyruser
 * :dd. is a pointer to a user area. This area is initialised. when the User Exit
 * is first invoked to route a transaction, and is reset once the
 * transaction routing request has been completed.
 * :dt. UE_Dyrusersize
 * :dd. is the length of the user area. This will initially be 128 bytes.
 * :dt. UE_Dyrcomptr
 * :dd. This is a pointer to the commarea.
 * :dt. UE_Dyrcomlgth
 * :dd. This is the length of the commarea.
 *
 * :edl.
 */
typedef struct
{

    cics_ushort_t       UE_Version;
    cics_UE_Func_t      UE_Dyrfunc;
    cics_ushort_t       UE_Dyrcount;
    cics_char_t         UE_Dyrsysid[cics_UE_SYSID_MAX+1];
    cics_ubyte_t       *UE_Dyrtptr;
    cics_ushort_t       UE_Dyrtlgth;
    cics_char_t        *UE_Dyrcwaptr;
    cics_ushort_t       UE_Dyrcwalgth;
    cics_char_t         UE_Dyrlclsys[cics_UE_SYSID_MAX+1];
    cics_char_t         UE_Dyrlclapl[cics_UE_APPL_MAX+1];
    cics_UE_YesNo_t     UE_Dyropter;
    cics_UE_YesNo_t     UE_Dyrretry;
    cics_UE_Error_t     UE_Dyrerror;
    cics_char_t         UE_Dyrtran[cics_UE_TRAN_MAX+1];
    cics_char_t         UE_Dyrprog[cics_UE_PROG_MAX+1];
    cics_char_t         UE_Dyrabcde[cics_UE_ABND_MAX+1];
    cics_char_t        *UE_Dyruser;
    cics_ushort_t       UE_Dyrusersize;
    cics_char_t        *UE_Dyrcomptr;
    cics_ushort_t       UE_Dyrcomlgth;

} cics_UE014025_t;


/*
 * Functions.
 */

cics_UE_Return_t cics_UE014025(cics_UE_Header_t        *UE_Header,
                               cics_UE014025_t         *UE_Specific);


/*
 * **************************************************************************
 * **************************************************************************
 *   Specific definitions for UE015050
 * **************************************************************************
 * **************************************************************************
 */

/*
 * Constants
 */

#define cics_UE_DPLAREA_SIZE 128           /* User Exit user area size      */
#define cics_UE015050_VERSION 1            /* All version number start at 1 */

/*
 * Data Types.
 */

typedef enum cics_UE_LinkFunc              /* Reason for UE invocation      */
{
    UE_LINKSEL,
    UE_LINKUNKNOWN,
    UE_LINKTERM,
    UE_LINKABEND
} cics_UE_LinkFunc_t;

/*
 * DESCRIPTION:
 *  User Exit specific structure for UE015050 which is always
 *  passed when a User Exit program is called for that User Exit
 *
 * :dl.
 * :dt. UE_version
 * :dd. This is the version number for this structure and takes the value
 * defined in cics_UE015050_VERSION. It is advisable to check this
 * in User Exit programs to ensure that the version number passed when the User Exit
 * is called is the same as that contained in the version of cicsue.h used
 * when the program was generated. If a mismatch is detected then contents
 * of fields are unpredictable and the User Exit program should be regenerated
 * against the correct version of cicsue.h
 * :dt. UE_Dplfunc
 * :dd. contains the reason why the User Exit has been invoked for a one
 * particular attempt to route a program. Possible reasons are
 * defined in cics_UE_LinkFunc_t.
 * :dt. UE_Dplsysid
 * :dd. contains the system Id to which the DPL request will be routed.
 * This can be a remote system Id or even the local system Id.
 * For the initial route selection this will be taken from either the PD stanza
 * or from the SYSID attribute specified on the EXEC CICS LINK command,
 * but for any subsequent invocations of the User Exit for a particular attempt
 * to route a program, it will contain the last system Id you specified.
 * Changing this parameter will change to system Id to be routed to.
 * :dt. UE_Dpltptr
 * :dd. is the pointer to the TCTUA.
 * :dt. UE_Dpltlgth
 * :dd. is the length of the TCTUA.
 * :dt. UE_Dplcwaptr
 * :dd. This is a pointer to the CWA. The onus is on the customer to ensure
 * that a User Exit program does not compromise the integrity of this area,
 * which is shared by all tasks in the CICS region.
 * :dt. UE_Dplcwalgth
 * :dd. This is the length of the CWA as definined in the RD stanza
 * :dt. UE_Dpllclsys
 * :dd. contains the name of the local system Id.
 * :dt. UE_Dpllclapl
 * :dd. contains the applid of the local region. This will depend upon
 * the type of connection that UE_Dplsysid points to.
 * :dt. UE_Dplmirtran
 * :dd. is the name of the remote mirror transaction on the remote system to
 * be routed to.
 * :dt. UE_Dplprog
 * :dd. is the name of the program on the remote system.
 * :dt. UE_Dplcomptr
 * :dd. This is a pointer to the commarea.
 * :dt. UE_Dplcomlgth
 * :dd. This is the length of the commarea.
 * :dt. UE_Dpldatlgth
 * :dd. This is the length of the commarea that is relevant for this DPL request. This
 * can be equal to or less than the length of the commarea.
 * :dt. UE_Dpluser
 * :dd. is a pointer to a user area. This area is initialised. when the User Exit
 * is first invoked to route a program, and is reset once the
 * DPL routing request has been completed.
 * :dt. UE_Dplusersize
 * :dd. is the length of the user area. This will initially be 128 bytes.
 * :dt. UE_Dplinvokingpgm
 * :dd. The name of the program executing the DPL command. On a remote
 * region this will invariably be "DFHMIRS".
 * :dt. UE_Dpluserid
 * :dd. The userid executing the DPL command.  This field may be updated to a
 * different userid for issuing a DPL request on a remote system.  The userid
 * will be sent independently of the OutboundUserids attribute setting 
 * for the CD for UE_Dplsysid; the request will be sent already-verified.
 * If this field is unchanged, then normal behaviour of the OutBoundUserids
 * attribute for the CD will prevail, and any userid or client password
 * will be used as defined by the CD, etc.
 * Note: if this DPL is part of an existing logical unit of work to the
 * remote system, then the userid specified by UE_Dpluserid will be ignored.
 *
 * :edl.
 */
typedef struct
{

    cics_ushort_t       UE_Version;
    cics_UE_LinkFunc_t  UE_Dplfunc;
    cics_char_t         UE_Dplsysid[cics_UE_SYSID_MAX+1];
    cics_ubyte_t       *UE_Dpltptr;
    cics_ushort_t       UE_Dpltlgth;
    cics_char_t        *UE_Dplcwaptr;
    cics_ushort_t       UE_Dplcwalgth;
    cics_char_t         UE_Dpllclsys[cics_UE_SYSID_MAX+1];
    cics_char_t         UE_Dpllclapl[cics_UE_APPL_MAX+1];
    cics_UE_YesNo_t     UE_Dplopter;
    cics_char_t         UE_Dplmirtran[cics_UE_TRAN_MAX+1];
    cics_char_t         UE_Dplprog[cics_UE_PROG_MAX+1];
    cics_char_t        *UE_Dplcomptr;
    cics_ushort_t       UE_Dplcomlgth;
    cics_ushort_t       UE_Dpldatlgth;
    cics_char_t        *UE_Dpluser;
    cics_ushort_t       UE_Dplusersize;
    cics_char_t         UE_Dplinvokingpgm[cics_UE_PROG_MAX+1];
    cics_char_t         UE_Dpluserid[cics_UE_USER_MAX+1];

} cics_UE015050_t;

/*
 * Functions.
 */

cics_UE_Return_t cics_UE015050(cics_UE_Header_t        *UE_Header,
                               cics_UE015050_t         *UE_Specific);


/*
 * **************************************************************************
 * **************************************************************************
 *   Specific definitions for UE016051
 * **************************************************************************
 * **************************************************************************
 */

/*
 * Constants
 */

#define cics_UE016051_VERSION 1            /* All version number start at 1 */

/*
 * Data Types.
 */

typedef enum UE_SyncType                   /* Reason for UE invocation      */
{
    UE_USER_SYNCPOINT
} cics_UE_SyncType_t;

typedef enum UE_SyncResp
{
    UE_COMMITTED,
    UE_ROLLEDBACK,
    UE_ABEND
} cics_UE_SyncResp_t;

typedef enum UE_SyncTRPrompt
{
    UE_PROMPT_FINISH,
    UE_NO_PROMPT_FINISH
} cics_UE_SyncTRPrompt_t;


/*
 * DESCRIPTION:
 *  User Exit specific structure for UE016051 which is always
 *  passed when a User Exit program is called for that User Exit
 *
 * :dl.
 * :dt. UE_version
 * :dd. This is the version number for this structure and takes the value
 * defined in cics_UE016051_VERSION. It is advisable to check this
 * in User Exit programs to ensure that the version number passed when the User Exit
 * is called is the same as that contained in the version of cicsue.h used
 * when the program was generated. If a mismatch is detected then contents
 * of fields are unpredictable and the User Exit program should be regenerated
 * against the correct version of cicsue.h
 * :dt. UE_SyncType
 * :dd. contains the reason why the User Exit has been invoked.
 *
 * :dt. UE_SyncResp
 * :dd. contains the result of the last syncpoint/rollback API command
 *
 * :dt. UE_SyncTRPrompt
 * :dd. contains the 'Syncpoint' attribute in the transactions TD.stanza.
 * See the Administration Reference for more infomation.
 *
 * :edl.
 */
typedef struct
{

    cics_ushort_t          UE_Version;
    cics_UE_SyncType_t     UE_SyncType;
    cics_UE_SyncResp_t     UE_SyncResp;
    cics_UE_SyncTRPrompt_t UE_SyncTRPrompt;

} cics_UE016051_t;

/*
 * Functions.
 */

cics_UE_Return_t cics_UE016051(cics_UE_Header_t        *UE_Header,
                               cics_UE016051_t         *UE_Specific);



/*
 * Generic definition of specific header
 *
 */
typedef union
{
    cics_UE014015_t   cics_UE014015;
    cics_UE052017_t   cics_UE052017;
    cics_UE014025_t   cics_UE014025;
    cics_UE015050_t   cics_UE015050;
    cics_UE016051_t   cics_UE016051;
}  cics_UE_Specific_t;

/*
 *
 * DESCRIPTION:
 * Generic Function definition for ANY User Exits 
 *
 * All User Exits are called in a generic manner passing 2 parameters 
 *   Pointer to the cics_UE_Header parameter list
 *   Pointer to the cics_UE_* parameter list which is specific to
 *     the User Exit
 * All User Exits require that a return code of cics_Return_t be returned to CICS
 *   by the User Exit program
 *
 */

cics_UE_Return_t cics_UE_Entry(cics_UE_Header_t   *UE_Header,
		  	       cics_UE_Specific_t *UE_Specific);


#endif /* _CICSUE_H */


/* EOF */
