#!/bin/ksh
#
# NAME: backup_log
#
#
# VERSION: 1.1
#
#
# COPYRIGHT:                                                        
# 
#   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
#   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
#   All Rights Reserved
#   Licensed Materials - Property of IBM
#  
#   US Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# 
# 
#            NOTICE TO USERS OF THE SOURCE CODE EXAMPLES
# 
# INTERNATIONAL BUSINESS MACHINES CORPORATION PROVIDES THE SOURCE CODE
# EXAMPLES, BOTH INDIVIDUALLY AND AS ONE OR MORE GROUPS, "AS IS"
# WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,
# INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE
# RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE CODE EXAMPLES,
# BOTH INDIVIDUALLY AND AS ONE OR MORE GROUPS, IS WITH YOU.  SHOULD 
# ANY PART OF THE SOURCE CODE EXAMPLES PROVE DEFECTIVE, YOU (AND NOT 
# IBM ) ASSUME 
# THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
#                                                                   
#
# SYNOPSIS:
#
# DESCRIPTION:
#
# The following script contains the following examples of backup and restore:
#
#     <1> ..... Backup Region directory
#     <2> ..... Restore Region directory
#
#
# This script will back up the CICS region, and then if required at a later
# date restore it.
#
# These individual options perform the following basic operations:
#
#   <1> ..... Backup Region directory
#
#       - Make the backup directory if it is required.
#       - Copy everything in the region directory, including all subdirectories,
#         to the backup directory. This includes the region log, the permanent
#         and runtime region databases and user code placed within the 
#         region directory.
#       - Create the script that will be used to restore the region.
#
#       The following variables must be set
#
#       RegionBackupPath: The directory to which the backup will be written.
#       CICSRegion      : The name of the CICS region to be backed up.
#       RegRestScript   : The name of the script to be created (in the
#                         directory defined by RegionBackupPath).
#
#   <2> ..... Restore Region directory
#
#       - Run the script created by the backup process, which will replace the
#         backed up region directory.
#
#       The following variables must be set
#
#       RegionBackupPath: The directory from which the restore will be made.
#       CICSRegion      : The name of the CICS region to be restored.
#       RegRestScript   : The name of the script to be used in the restore
#                         (in the directory defined by RegionBackupPath).
#
#
# CAVEATS/WARNINGS:
# None
#
# RETURNED VALUES:
#
# 0 : Backup completed normally
# 1 : Backup did not complete
#
# RELATED INFORMATION:
# None.
#
#

unset RegionPID

typeset BackupRoot=$HOME
typeset RegionBackupPath=$BackupRoot"/backup"
typeset RegRestScript="restore.reg"
typeset CICSRegion="akrause1"
typeset RegionPID
typeset MenuOption=1
typeset ExitOption=3



#
# Display error message and then exit the sample.
#
function ErrorExit
{
    echo "Error: "$1
    echo "Stopping program execution."
    echo "\n\n"
    exit 1
}


#
# Print the screen header, after clearing the screen.
#
function PrintScreenHeader
{
    clear
    echo "Region Name: "$CICSRegion
    echo "---------------------------------------\c"
    echo "----------------------------------------"
}


#
# This just prompts the user to press <Enter>. It is used when a command
# has successfully completed.
#
function OutputFinalMessage
{
    echo "\n\n------------------------------------\c"
    echo "-------------------------------------------"
    echo "\n Press <Enter> to return to main menu.\c"
    read AnyKey
}


#
# This function creates a suitable directory to hold the incremental backup
# (if it does not already exist), and sets permissions so that we are able
# to read and write the backup to the directory.
#
function MakeBackupDirectory
{
    Dir=$1

    if [[ ! -d ${Dir} ]]
    then
        echo "...Backup directory "${Dir}" does not exist. It will be created."
        mkdir -p ${Dir}
        chmod g+rwx ${Dir}
    else
        echo "...Backup directory "${Dir}" already exists, \c"
        echo "so do not create it."
    fi
}


#
# Check that the region is not running.
#
function CheckRegionIsRunning
{
    echo "...Checking Region is not running."
    RegionPID=$(ps -ef | grep cics | grep ${CICSRegion}) 
    
    if [[ ! -z ${RegionPID} ]]
    then
        ErrorExit "Region is running."
    fi
}

#
# This function backs up all of the CICS Region directory structure. This is
# simply done by copying everyhing under the region directory.
#
# Note: The region cannot be active while performing the backup.
#
function PerformRegionBackup
{
    #
    # Clear the screen ready for the messages concerning the backup.
    #
    PrintScreenHeader
    echo "\n\nBacking up Region..."
    echo "...Region name is ................ [" $CICSRegion "]"
    echo "...Backup directory is ........... [" $RegionBackupPath "]"

    #
    # Check that the region is not running.
    #
    CheckRegionIsRunning

    #
    # Make the backup directory, which will be used to store the backup.
    #
    MakeBackupDirectory $RegionBackupPath

    #
    # Backup the CICS region. If this does not work correctly then produce
    # an error message and exit the sample.
    #
    echo "...Backing up region. >>>> Please wait. <<<<"
    cp -r /var/cics_regions/${CICSRegion}/* $RegionBackupPath
    if [[ $? -ne 0 ]]
    then
        ErrorExit "Could not back up Region."
    fi

    #
    # Now build a script which will be used to restore the CICS region if it
    # is required. The script will contain a copy command which will do the
    # reverse of the above step.
    #
    echo "...Creating recovery script ...... [ \c"
    echo $RegionBackupPath/$RegRestScript "]"
    echo "# Script to retore Region."         >$RegionBackupPath/$RegRestScript
    echo "# Backed up on $(date)."            >>$RegionBackupPath/$RegRestScript
    echo "cp -r "$RegionBackupPath"/* /var/cics_regions/"${CICSRegion}        \
                                              >>$RegionBackupPath/$RegRestScript

    echo "Back up of region database "$CICSRegion" complete.\n\n\n\n\n\n"

    #
    # Wait for the user to press <Enter> before returning.
    #
    OutputFinalMessage
}


#
# This function restores the previously backed up CICS region. The routine
# will restore ALL backed up region files from the backup directory.
#
function PerformRegionRestore
{
    #
    # Clear the screen ready for the messages concerning the backup.
    #
    PrintScreenHeader
    echo "\n\nBacking up Region..."
    echo "...Region name is ................ [" $CICSRegion "]"
    echo "...Restoring from directory ...... [" $RegionBackupPath "]"

    #
    # Check that the region is not running.
    #
    CheckRegionIsRunning

    #
    # Use the script created by the backup process to restore the region.
    # Check the return code from the script to ensure that the restire went
    # correctly.
    #
    echo "...Using recovery script ......... [ \c"
    echo $RegionBackupPath/$RegRestScript "]"
    echo "...Restoring the region database. >>>> Please wait. <<<<"

    ksh $RegionBackupPath/$RegRestScript >/dev/null 2>&1
    if [[ $? -ne 0 ]]
    then
        ErrorExit "Failed to restore region database."
    fi

    echo "Restore of region database "$CICSRegion" complete.\n\n\n\n\n\n\n"

    #
    # Wait for the user to press <Enter> before returning.
    #
    OutputFinalMessage
}


#
# This function displays the main menu options. It then calls the correct
# menu option routines depending on the users input. When the exit option
# is selected then the problem halts.
#
function DisplayMainMenu
{
    ErrorMessage=""

    while [[ $MenuOption != $ExitOption ]]
    do
        PrintScreenHeader
        echo "\n                               *****************"
        echo "                               *** MAIN MENU ***"
        echo "                               *****************"
        echo "\n   <1> ..... Backup Region directory"
        echo "   <2> ..... Restore Region directory"
        echo "\n\n   <"$ExitOption"> ..... Exit the utility\n\n\n\n\n\n"
        echo "\n------------------------------------\c"
        echo "-------------------------------------------"
        echo $ErrorMessage
        echo " Please enter your choice > \c"
        read MenuOption

        case $MenuOption in
            1) PerformRegionBackup ;;
            2) PerformRegionRestore ;;
            *) ErrorMessage=">>>> Invalid Menu selection, please retry <<<<" ;;
        esac

    done
}

# -----------------------------------------------------------------------------
#                        Start of the main routine
# -----------------------------------------------------------------------------

clear
PrintScreenHeader
echo "\n\nPlease wait setting up environment ...."

# Show the main menu.
DisplayMainMenu

clear
echo "Region Backup/Restore Utility has been exited."
exit 0

#
# Module what string
#
#@(#)supos,22:42:58,Apr 08 1998,s211-L980223
#
