#!/bin/ksh
#
# NAME: backup_log
#
#
# VERSION: 1.1
#
#
# COPYRIGHT:                                                        
# 
#   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
#   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
#   All Rights Reserved
#   Licensed Materials - Property of IBM
#  
#   US Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# 
# 
#            NOTICE TO USERS OF THE SOURCE CODE EXAMPLES
# 
# INTERNATIONAL BUSINESS MACHINES CORPORATION PROVIDES THE SOURCE CODE
# EXAMPLES, BOTH INDIVIDUALLY AND AS ONE OR MORE GROUPS, "AS IS"
# WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,
# INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE
# RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE CODE EXAMPLES,
# BOTH INDIVIDUALLY AND AS ONE OR MORE GROUPS, IS WITH YOU.  SHOULD 
# ANY PART OF THE SOURCE CODE EXAMPLES PROVE DEFECTIVE, YOU (AND NOT 
# IBM ) ASSUME 
# THE ENTIRE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
#                                                                   
#
# SYNOPSIS:
#
# DESCRIPTION:
#
# This program contains the following examples of backup and restore:
#
#     <1> ..... MRA: Backup of SFS Data 
#     <2> ..... MRA: Backup SFS Log Archive Files
#     <3> ..... MRA: Restore SFS log and data using archive files
#     <4> ..... dd copy SFS logical volumes
#     <5> ..... Restore dd copy of SFS logical volumes
#
###############################################################################
#
# <1>,<2> and <3> are an example of backup and restore using Media
# Recovery Archiving (MRA). <1> enables MRA and takes a backup of the 
# the SFS data. <2> takes backup of the log archive files that are
# written by the SFS as a result of MRA being enabled. <3> restores
# the SFS data from the level it was at when the backup was taken to 
# the level it was at when the latest log archive file was written.
#
# After taking a backup of the SFS data the log archives files should be
# backed up and deleted from the archive directory at intervals that
# will prevent the archive directory from becoming full.
#
# The SFS should not be restored from backup unless necessary as it will
# inevitably result in some loss of data. 
#
# These individual options perform the following basic operations:
#
#   <1> ..... Backup of SFS Data
#
#       - Enable Media Recovery Archiving 
#       - Backup of each of the SFS data volumes owned by the SFS.
#       - Backup the SFS restart files
#       - Backup the SSD definitions
#       - Create a script to be used by the restore routine <3> 
#         to recreate the data logical volumes if necessary (AIX only).
#
#       The following variables must be set:
#
#       EncinaSFSServer : The SFS server to be backed up. 
#       SFSBackupPath   : The directory to which the backup will be written.
#       SFSRestScript   : The name of the script to be created (in the
#                         directory defined by SFSBackupPath).
#
#       Note: The SFS server should be running when this option is selected.
#
#   <2> ..... Backup SFS Log Archive Files
#
#       - Backup log archive files  
#       - Delete backed up archive files from the archive directory if OK'd
#         by user.
#       - Create a script containing the name of the newest archive file backed
#         up and a command to restore all backed up archives to the archive
#         directory. This is used by the restore routine <3>. 
#
#       The following variables must be set:
#
#       EncinaSFSServer : The SFS server to be backed up.
#       MRABackupPath   : The directory to which the backup will be written.
#       MRARestScript   : The name of the file to be created (in the
#                         directory defined by MRABackupPath). 
#
#   <3> ..... MRA: Restore SFS log and data using archive files
#       
#       - Shutdown the SFS if is running
#       - Run the scripts created by backup to restore any backed up archive
#         files and recreate the logical volumes (if necessary).
#       - Cold start and shutdown SFS server so it is reconfigured for its
#         logical volumes. 
#       - Start the SFS in administration mode.
#       - Restore the SFS Log Volume using the log archive files written
#         to backup.
#       - Restore the SFS Data Volumes.
#       - Shut the server down. 
#
#       The following variables must be set:
#
#       EncinaSFSServer : The SFS server to be restored.
#       SFSBackupPath   : The directory to which the data volume backups were
#                         written.
#       SFSRestScript   : The name of the script (in the directory defined
#                         by SFSBackupPath) created during SFS data backup.
#       MRABackupPath   : The directory to which the log archive backups were 
#                         writen.
#       MRARestScript   : The name of the file created (in the directory 
#                         defined by MRABackupPath) during MRA backup.
#       LogFilename     : The value of the LogFile attribute for the SFS in
#                         the SSD stanza.
#       Protection      : The value of the Protection attribute for the SFS in
#                         the SSD stanza.
#       ServerDir       : Path of the SFS server database directory.
#       
#
###############################################################################
#
# <4> and <5> are an example of using an operating system command to
# take a backup of an SFS server, for this example "dd". These scripts
# take a copy of the log and data logical volumes owned the SFS. This
# type of backup has an advantage over MRA in that the level to which
# the server can be restored is known, but has the disadvanatge that the
# server must be stopped, and backups should be taken more regularly.      
#
# The individual options perform the following basic operations:
#
#   <4> ..... dd copy SFS logical volumes
#       
#       - Obtain a list of logical volumes owned by the server.
#       - Shutdown the SFS server
#       - use "dd" to copy the logical volumes to backup.
#       - Create a script to be used by the restore routine <5>
#         to recreate the logical volumes if necessary.
#       - Create a script to restore the backups using "dd".
#
#       The following variables must be set:
#
#       EncinaSFSServer : The SFS server to be backed up.
#       DDBackupPath    : The directory to which the backup will be written.
#       DDRestScript    : The name of the file created (in the directory
#                         defined by DDBackupPath) to recreate the logical
#                         voulumes. .
#       DDCopyScript    : The name of the file created (in the directory
#                         defined by DDBackupPath) containing the dd command
#                         needed to restore. 
#                         
#  
#
#       Note: The server should be running when this option is selected.
#
#   <5> ..... Restore dd copy of SFS logical volumes
#
#       - Shutdown the SFS if it is running
#       - Run the script to recreate the logical volumes (if necessary).
#       - Cold start and shutdown SFS server so it is reconfigured for its
#         logical volumes.
#       - Run the script to restore the backups using "dd".
#
#       The following variables must be set:
#
#       EncinaSFSServer : The SFS server to be backed up.
#       DDBackupPath    : The directory to which the backup will be written.
#       DDRestScript    : The name of the file created (in the directory
#                         defined by DDBackupPath) to recreate the logical
#                         voulumes, created during backup.
#       DDCopyScript    : The name of the file created (in the directory
#                         defined by DDBackupPath) containing the dd command
#                         needed to restore, created during backup.
#                         
##############################################################################
# 
# CAVEATS/WARNINGS:
# None
#
# RETURNED VALUES:
#
# 0 : Backup/Restore completed normally
# 1 : Backup/Restore did not complete
#
# RELATED INFORMATION:
# None.
#
#


#         
# Variables to be modified as required
#
typeset EncinaSFSServer="/.:/cics/sfs/Martin"
typeset BackupRoot="/var/cics_servers/backups"
typeset SFSBackupPath=$BackupRoot"/SFS"
typeset DDBackupPath=$BackupRoot"/DD"
typeset MRABackupPath=$BackupRoot"/MRA"
typeset SFSRestScript="restore.sfs"
typeset MRARestScript="restore.mra"
typeset DDRestScript="restore.dd"
typeset DDCopyScript="restore.ddcopy"
typeset LogFile="logfile"
typeset LogVolume="log_SFS_M"
typeset Protection="none"
typeset ServerDir="/var/cics_servers/SSD"
 

#
# Variable that is operating system unique
#
#    AIX  : /dev
#    HPUX : /dev/<volume group>
#    DEC  : /dev/vol/<volume group>
#
typeset VolDir="/dev"

#
# Internal variables
#
typeset SystemName
typeset SFSProcess
typeset SFSVolumeName
typeset Authorization
typeset MenuOption=1
typeset ExitOption=6
typeset NumFiles
typeset AskUser=1
typeset DoNotAskUser=0
typeset Warm=1
typeset Cold=0


#
# Display error message and then exit the sample.
#
function ErrorExit
{
    echo "Error: "$1
    echo "Stopping program execution."
    echo "\n\n" 
    exit 1
}


#
# Print the screen header, after clearing the screen.
#
function PrintScreenHeader
{
    clear
    echo  "SFS Server Name: "$EncinaSFSServer 
    echo "---------------------------------------\c"
    echo "----------------------------------------"
}


#
# This just prompts the user to press <Enter>. It is used when a command
# has successfully completed.
#
function OutputFinalMessage
{
    echo "\n\n------------------------------------\c"
    echo "-------------------------------------------"
    echo  "\n Press <Enter> to return to main menu.\c"
    read AnyKey
}


# 
# Check that we are correctly DCE logged in, as cell_admin.
#
function CheckDCELogin
{
    klist | grep "Global Principal.*/cell_admin[^/]*$" >/dev/null ||
      ErrorExit "Must be dce_logged in as cell_admin to run this script"
}


#
# This function creates a suitable directory to hold the incremental backup
# (if it does not already exist), and sets permissions so that the backup and
# restore routines can access the directory.
#
function MakeBackupDirectory
{
    Dir=$1

    if [[ ! -d ${Dir} ]] 
    then
        echo "...Backup directory "${Dir}" does not exist. It will be created."
        mkdir -p ${Dir}
        chmod 777 ${Dir}
    else
        echo "...Backup directory "${Dir}" already exists, \c"
        echo "so do not create it."
    fi 
}


#
# Shut down the SFS server using the CICS provided script (cicssfsshut).
#
# Depending on the parameter passed to this routine we will either prompt the
# user on whether they want to go ahead with shut down or just go straight 
# ahead with the shut down.
#
function CICSShutDownSFSServer
{
    PromptUser=$1

    #
    # Check to see if we need to prompt the user about shutting down the SFS
    # Server.
    #
    if [[ $PromptUser -eq $AskUser ]]   
    then
        echo "...The SFS Server is running, can it be shut down [Y/N]? \c"
        read ShutSFSServer
    else
        ShutSFSServer="Y"
    fi

    if [[ $ShutSFSServer = Y || $ShutSFSServer = y ]]
    then
        cicssfsshut $EncinaSFSServer >/dev/null 2>&1  
        if [[ $? -ne 0 ]]
        then 
            ErrorExit "Could not shut down the SFS server."
        fi
        echo "...Shutdown of SFS server now complete."
    else
        ErrorExit "SFS Server running, cannot continue."
    fi
}


#
# This starts the SFS Server using the CICS scripts.
#
# This function takes a parameter which gives details on the start type 
# required. This can either be 'Warm' or 'Cold'
#
function CICSStartSFSServer
{
    StartType=$1

    if [[ $StartType -eq $Cold ]]
    then
        echo "...Cold starting SFS Server."
        cicssfs $EncinaSFSServer StartType=cold
        RetCode=$?
    else 
        echo "...Warm starting SFS Server."
        cicssfs $EncinaSFSServer
        RetCode=$?
    fi

    if [[ $RetCode -ne 0 ]]
    then
        ErrorExit "Could not start the SFS Server."
    fi
}


#
# Convert the protection level from an string to an integer. This is understood
# by the SFS start command.
#
GeneralProtStringToNumber()
 {
   case $1 in
     default)     echo 0 ;;
     none)        echo 1 ;;
     connect)     echo 2 ;;
     call)        echo 3 ;;
     pkt)         echo 4 ;;
     pkt_integ)   echo 5 ;;
     pkt_privacy) echo 6 ;; 
   esac
 }   


#
# This function warm starts the SFS server in administration mode, using the
# raw 'sfs' command.
#
function StartSFSServerAdminMode
{
    echo "...Starting SFS server in admimistration mode."

    ProtectLevel=$(GeneralProtStringToNumber $Protection)
    if [ $ProtectLevel -eq 1 ]
    then
         Authorization=0
    else
         Authorization=1
    fi

    ServerPrin=${EncinaSFSServer#/.:/}

    cicssfs -a $EncinaSFSServer

    echo "...Waiting for SFS Server to initialise."
    sfs_wait $EncinaSFSServer -a

    echo "...SFS server has been successfully started."

}


#
# This function shuts down the SFS Server, using the tkadmin shut down command.
#
function tkadminShutDownSFSServer
{
    echo "...Shutting down the SFS Server."
    tkadmin stop server -server $EncinaSFSServer

    if [[ $? -ne 0 ]]
    then
        ErrorExit "Could not shut down the SFS Server."
    fi
}


#
# Determine the UNIX process number of the SFS server. This routine takes
# one parameter which is the name of the SFS Server. We find the process 
# number by querying all processes which contain both "sfs$" and the SFS
# server name.
#
function FindServerPid
{
    ps xaww | grep "sfs" | grep $EncinaSFSServer | grep -v "grep" |          \
              awk '{ print 1 }'
}


#
# Find the SFS Servers volume name. This is done using the sfsadmin lvols
# command.
#
function FindSFSServerDetails
{
    echo "...Finding SFS Server details."

    #
    # If the SFS server is not running then we have a problem.
    #
    SFSProcess=$(FindServerPid)

    if [[ -z ${SFSProcess} ]]
    then
        ErrorExit "SFS Server not running."
    fi

    #
    # Find just the SFS Server logical volume.
    #
    SFSVolumeName=$(sfsadmin list lvols -server $EncinaSFSServer | sed '1d'    \
                                     | sed 's/.*	//') 

    #
    # Find a list of all the logical volumes associated with the SFS Server.
    #
    VolumeNames=$(tkadmin list lvols -server $EncinaSFSServer) 

    echo "...SFS Server Name ............... [" $EncinaSFSServer "]"
    echo "...SFS Server Volume name(s)...... [" $SFSVolumeName "]"
}


#
# Backs up the SFS Servers restart and stanza files.
#
# This takes as a parameter the name of the restore script which any restore 
# commands should be written to. This script is always in the backup directory,
# so by removing the directory name from the front of the script name we
# can find the name of the backup directory.
#
function BackupRestartFiles
{
    BackupDir=$1

    echo "...Backing up restart and stanza files."
    #
    # The restart files are held in the home directory of the
    # SFS server userid. 
    #
    HomeDir=${EncinaSFSServer#/.:/}
    HomeDir=${ServerDir}/${HomeDir}

    #
    # Find the name of the backup directory.
    #
    OutDir=$(dirname ${BackupDir})

    #
    # Back the restart files and the stanza files up.
    #
    for File in $HomeDir"/restart" $HomeDir"/restart.bak"                     \
          $ServerDir"/SSD.stanza" $ServerDir"/AUTO.stanza"
    do
        OutFile=$(basename $File)
        OutFile=$OutDir/${OutFile}

        #
        # If the backup directory does not exist then build it.
        #

        # Now copy the file into the backup directory.
        cp -hp ${File} ${OutFile} >/dev/null 2>&1 
        if [[ $? -ne 0 ]]
        then 
            echo "...No "$File" to backup."
        fi

    done
}


#
# Check on space for the SFS backup. This is done using the tkadmin query lvol
# command, which places information about a logical volume storing log data
# into a file called $TempPath/tkvol. 
#
function CheckSpaceForSFSBackup
{
    echo "...Determining the space required for a full backup."

    # 
    # Query the logical volume using the tkadmin command. The syntax of the
    # command is:
    #  - tkadmin query lvol -server [SFS Server Name] [SFS Logical Volume Name]
    #   
    # We return the results of this query into a file $TempPath/tkvol. We then
    # use the awk command to extract the information that we need.
    #
    tkadmin query lvol -server $EncinaSFSServer $SFSVolumeName >/tmp/tkvol

    #
    # Now use 'awk' to stirp the relevant information out of the file produced
    # by the tkadmin query lvol command.
    #
    awk    '$1 == "size:"   {
                               if ($3 == "chunkSize:")
                                 numpages = $2
                               next
                             }
           $12 == "Kbytes"   {
                               pagesize = $11
                               next
                             }
           END               {
                               numkbytes = numpages*pagesize
                               numfiles = int(0.99999+(numkbytes/4096))
                               numspace = numfiles*4097
                               printf( "%s %s %s %s %s\n", 
                              numpages, pagesize, numkbytes, numfiles, numspace)
                              }' /tmp/tkvol | \
                   read NumPages PageSize NumKBytes NumFiles NumSpace Rest

    echo "...Number of backup files required [" $NumFiles "]" 
    echo "...A full backup will require .... [" $NumSpace "] Kbytes of space"
}


#
# Create a file of commands to recreate the SFS volume and to chown the
# the SFS Servers logical volume. The script is called:
#     ${SFSBackupPath}/${SFSRestScript}
#
# This script is used by the recovery routine.
#
function CreateSFSRecoveryScript
{
    #
    # This function uses AIX specific syntax, so return if we are not AIX.
    #
    if [[ uname != "AIX" ]]
    then
        return
    fi

    #
    # This script will contain the command which will create the SFS logical
    # volumes.
    #
    for Volume in $VolumeNames
    do
        lsvg rootvg -l | grep ${Volume}" " |     \
        awk '{print "mklv -y "$1" rootvg "$3}'   \
                    >>${SFSBackupPath}/${SFSRestScript}

        #
        # This script will contain the command which will change the ownership
        # details of the SFS logical volume. 
        #
        ls -l ${VolDir}/${Volume} |                                           \
          awk '{NF=10;print "chown "$3":"$4" "$10" /dev/r"substr($10,6)}'     \
                                             >>${SFSBackupPath}/${SFSRestScript}
    done
} 
   

#
# This performs all required actions to backup the SFS data.
#
# The basic process for backing up the SFS data is:
#  - Work out how much space is required to perform the backup.
#  - Enable the media archiving for our server.
#  - Perform the backup request.
#  - Build recovery scripts.
#
# note: The SFS Server MUST be running for this function to correctly
# take a backup. 
#
function PerformBackupSFSData
{
    #
    # Set up the screen ready to display messages.
    #
    PrintScreenHeader
    echo "\n\nSFS Server about to be backed up..."

    #
    # Find out the required details about the SFS Server. The server must be
    # running for this function to work correctly.
    #
    FindSFSServerDetails

    #
    # Check to see if the back up directory exists. If it doesn't then 
    # create it.
    #
    echo "...Backup directory is ........... [" $SFSBackupPath "]"
    MakeBackupDirectory $SFSBackupPath

    #
    # Always enable media archiving. If it is already enabled then this
    # call just returns without doing anything. The format of the command
    # is:
    #  - tkadmin enable mediaarchiving -server [SFS Server Name]
    #
    tkadmin enable mediaarchiving -server ${EncinaSFSServer}

    #
    # Display the status of the media archiving. This should always tell
    # you media archiving is on. The format of the command is:
    #  - tkadmin query mediaarchiving -server [SFS Server Name]
    #
    Message=$(tkadmin query mediaarchiving -server ${EncinaSFSServer})
    echo "..."$Message

    # 
    # Create the stubs for the restore data volumes scripts. These are updated
    # each time a data volume is backed up, so we only want to clear the file
    # once 
    #
    echo "\c...Creating recovery script ...... [ "
    echo $SFSBackupPath/$SFSRestScript" ]"
    echo "# Create the SFS data volumes."    >${SFSBackupPath}/${SFSRestScript}
    echo "# File created on $(date)"         >>${SFSBackupPath}/${SFSRestScript}

    #
    # Create the SFS recovery scripts. These are used to make the logical
    # volume if a recovery is required and to also change the ownership
    # details of this logical volume. They should allow the recovery process
    # to recreate the exact logical volume that was backed up.
    #
    CreateSFSRecoveryScript

    #
    # Check to see how much space is required to perform the backup of the
    # SFS Server.
    #
    CheckSpaceForSFSBackup

    #
    # Now perform the actual back up of the SFS data. This is done using the
    # tkadmin backup lvol command. We supply the command the name of the SFS
    # server to be backed up, the name of the SFS logical volume, the path 
    # of where we want to back up to, the number of files that will make up
    # the back up and the size of each back up file (4k). The num of files
    # in the back up has been worked out by 'CheckSpaceForSFSBackup'.
    # The format of the command is:
    #  - tkadmin backup lvol -server [SFS Server Name] [SFS Volume Name]
    #                       [Backup Directory] [Number of Files] [File Size]
    #
    echo "...Backing up volume ............. [" $SFSVolumeName "\c"
    echo " ]. >>>> Please wait <<<<"
    BackupMessages=$(tkadmin backup lvol -server ${EncinaSFSServer}           \
                   ${SFSVolumeName}                                           \
                   -fileprefix ${SFSBackupPath}/${SFSVolumeName}              \
                   -filesize 4097k                                            \
                   -nfiles ${NumFiles}                                        \
                   2>&1 )

    #
    # Check the messages produced by the backup to see if we were successful
    # or not.
    #
    if [[ -z ${BackupMessages} ]]
    then
        echo "...Completed backup of volume .... [" $SFSVolumeName "]"
    else
        echo $BackupMessages 
        ErrorExit "The backup has failed."
    fi

    #
    # Back up the SFS servers restart and stanza files.
    #
    BackupRestartFiles ${SFSBackupPath}/${SFSRestScript}
 
    echo "Backup of "$EncinaSFSServer" is now complete." 

    #
    # Wait for the user to press <Enter> before returning.
    #
    OutputFinalMessage
}


#
# We need to backup all of the MRA files from the servers archive directory.
#
# As part of the recovery scripts will be built which will create all of the
# logical volumes and copy everything back to the correct place. 
#
function PerformBackupMRAFiles
{
    #
    # Clear the screen ready for the messages concerning the backup.
    #
    PrintScreenHeader
    echo "\n\nArchive (MRA) files are about to be backed up..."

    #
    # Check to see if the backup directory exists. If it does not then we want
    # to create it.
    #
    echo "...Backup directory is ........... [" $MRABackupPath "]"
    MakeBackupDirectory $MRABackupPath

    #
    # Copy all of the MRA files into the backup directory. We need to check that
    # this has worked correctly, as this is the key stage of the backup.
    #
    echo "...Backing up all MRA files. >>>> Please wait <<<<"
    cp /var/cics_servers/archives/${SystemName}/*.LA.* ${MRABackupPath}
    if [[ $? -ne 0 ]]
    then
        ErrorExit "Could not backup MRA files."
    fi

    #
    # We now need to build the script which replaces the backed up MRA files.
    #
    echo "...Creating recovery script ...... [ \c"
    echo $MRABackupPath/$MRARestScript "]"
    echo "# Script to restore backed up MRAs."   >$MRABackupPath/$MRARestScript
    echo "# File created on $(date)."            >>$MRABackupPath/$MRARestScript

    #
    # List the files in age order. We want to find the latest MRA archive file
    # as this is used when performing the restore.
    #
    Latest=$(ls -rt /var/cics_servers/archives/${SystemName}/*.LA.* | tail -1)
    Latest=$(basename $Latest)
    echo "# Latest MRA file: "${Latest} >>$MRABackupPath/$MRARestScript
    echo "cp ${MRABackupPath}/*.LA.* /var/cics_servers/archives/${SystemName}"\
                                        >>$MRABackupPath/$MRARestScript

    #
    # Now we want to delete the backed up MRA files. However, the user will be 
    # prompted before this is done.
    #
    echo "...MRA files backed up. Do you want to delete the originals [Y/N]? \c"
    read DeleteMRAFiles
    if [[ $DeleteMRAFiles = Y || $DeleteMRAFiles = y ]] 
    then
        echo "...Deleting MRA files."
        rm -f /var/cics_servers/archives/${SystemName}/*.LA.*
    else
        echo "...Keeping original MRA files."
    fi

    echo "Backup of MRA files for SFS Server "$EncinaSFSServer" complete."

    #
    # Wait for the user to press <Enter> before returning.
    #
    OutputFinalMessage
}    


#
# Restore the backed up SFS Server data. This is done using the 
# tkadmin restore lvols command.
#
# If the SFS Server is currently running then the script will shut it down.
# The SFS server will then be restarted in admin mode, so that the restore
# can take place.
#
function PerformRestoreSFSData
{
    #
    # Clear the screen ready for the messages concerning the backup. 
    #
    PrintScreenHeader
    echo "\n\nSFS Server about to be restored..."

    echo "...SFS Server Name ............... [" $EncinaSFSServer "]"
    echo "...Restore from directory ........ [" $SFSBackupPath "]"

    #
    # Confirm that the user really wants to restore the SFS server.
    #
    echo "...This will destroy the existing SFS Server data."
    echo "...Do you want to continue with the restore [Y/N]? \c"
    read Confirm
    if [[ $Confirm = N || $Confirm = n ]]
    then
        echo "Restore will be cancelled.\n\n\n\n\n\n"
        OutputFinalMessage
        return
    fi


    #
    # We need to have the SFS Server in a shut down state so that we can 
    # begin the restore. We will later restart it in admin mode.
    #
    SFSProcess=$(FindServerPid)

    if [[ -z ${SFSProcess} ]]
    then
        echo "...SFS Server is not currently running. It will be started."
        cicssfslock -u $EncinaSFSServer
    else
        echo "...SFS Server must be shut down, before restore can begin."
        CICSShutDownSFSServer $AskUser
    fi

    #
    # Check that there are no more MRA files in the archive directory. If
    # there are then give the user a chance to use these in the restore  
    # process.
    #
    List=$(ls /var/cics_servers/archives/${SystemName}/*.LA.* 2>/dev/null)
    if [[ $? -eq 0 ]]
    then
        echo "...Found MRA files in your archive directory.\c"
        echo " Use these in the restore [Y/N]? \c"
        read BackupMRA
    else
        echo "...No new MRA files in your archive directory since backup."
        BackupMRA="N"
    fi

    #
    # If there were new MRA files in the archive directory and the user 
    # wants to use them in the restore then we need to find the latest MRA
    # file in the directory before copying back the backed up MRA files.
    # Otherwise we will just use the backed up files. In this case we 
    # need to look up the latest backed up file name.
    if [[ $BackupMRA = Y || $BackupMRA = y ]]
    then
        LatestMRA=$(ls -rt /var/cics_servers/archives/${SystemName}/*.LA.*    \
                                                                   | tail -1)
        LatestMRA=$(basename $Latest)
    else
        LatestMRA=$(cat ${MRABackupPath}/${MRARestScript} | sed -n '3p' |     \
                                            sed 's/# Latest MRA file: //')
    fi
    echo "...Latest MRA file to be used ... [" $LatestMRA "]"

    #
    # Run the script to copy back the MRA archive files.
    #
    echo "...Recovering MRA archives ...... [" $MRABackupPath/$MRARestScript "]"
    ksh ${MRABackupPath}/${MRARestScript}

    #
    # Run the restore SFS Server log volume script.
    #
    echo "...Using recovery script ........ [" $SFSBackupPath/$SFSRestScript "]"
    echo "...Recreating logical volumes if necessary."
    ksh ${SFSBackupPath}/${SFSRestScript} 2>/dev/null

    #
    # Cold start the server to make sure it knows where its volumes are.
    #
    CICSStartSFSServer $Cold
 
    #
    # Now get some details from the server.
    #
    FindSFSServerDetails $EncinaSFSServer
    HomeDir=${EncinaSFSServer#/.:/}
    HomeDir=${ServerDir}/${HomeDir}
    
    #
    # Now shut the server down and start it in admin mode. We MUST be in admin
    # mode so that we can perform the restore.
    #
    echo "...SFS Server must be shut down, so it can be started in admin mode."
    CICSShutDownSFSServer $DoNotAskUser
    StartSFSServerAdminMode

    #
    # Restore the data of a logical volume storing log data. This is done 
    # using the tkadmin restore logvol command. The syntax of this command
    # is:
    #  - tkadmin restore logvol -server [SFS Server Name] [Log Volume Name]
    #                                   [MRA backup directory] [Latest MRA file]
    #
    echo "...Restoring logical volume .... [" $LogVolume "].\c"
    echo " >>>> Please wait <<<<"
    tkadmin restore logvol -server $EncinaSFSServer                           \
                           $LogVolume                                         \
                           /var/cics_servers/archives/${SystemName}           \
                           /var/cics_servers/archives/${SystemName}/$LatestMRA
    
    if [[ $? -ne 0 ]]
    then
        tkadminShutDownSFSServer
        ErrorExit "Could not restore the log volume."
    fi

    #
    # Enable the restored log file for use by the server.
    #
    echo "...Enabling LogFile ............ [" $LogFile "]"
    tkadmin enable logfile -server $EncinaSFSServer $LogVolume/$LogFile

    if [[ $? -ne 0 ]]
    then
        tkadminShutDownSFSServer
        ErrorExit "Could not enable LogFile."
    fi
 
    #
    # The SFS Server restore is done using the tkadmin restore lvols
    # command. This takes the format:
    #  - tkadmin restore lvols -server [SFS Server Name] [Number of Volumes]
    #                          [SFS Volume Name]
    #
    echo "...Restoring SFS Volume Name ..... [" $SFSVolumeName "]. \c"
    echo " >>>> Please wait <<<<"
    tkadmin restore lvols -server $EncinaSFSServer 1 $SFSVolumeName

    if [[ $? -ne 0 ]]
    then
        tkadminShutDownSFSServer
        ErrorExit "Could not restore the logical volume."
    fi
   
    # 
    # Now recover the logical volumes for the SFS Server.
    #
    echo "...Recovering SFS Volume."
    tkadmin recover lvols -server $EncinaSFSServer

    if [[ $? -ne 0 ]]
    then
        CICSShutDownSFSServer $AskUser
        ErrorExit "Could not recover the logical volume."
    fi

    #
    # Finally shut down the SFS Server so that it is ready for the user to
    # warm start it when reequired.
    #
    tkadminShutDownSFSServer
 
    echo "The restore of SFS Server "$EncinaSFSServer" has completed."
    echo "Warm start SFS Server when required."

    #
    # Wait for the user to press <Enter> before returning.
    #
    OutputFinalMessage
}


#
# This function takes a copy of the logical volumes owned
# by an SFS server. This includes all data and log volumes.
# In addition, the restart files are copied.
# This procedure requires that the SFS server is stopped.
#
function ddCopyVolumes
{
    #
    # Clear the screen ready for the messages concerning the backup.
    #
    PrintScreenHeader
    echo "\n\nAbout to perform a ddcopy of the SFS Servers logical volumes..."

    #
    # We need to ensure that the SFS Server is running so that we can get
    # the logical volume information. The SFS Server will be shut down before
    # the logical volumes are backed up.
    #
    SFSProcess=$(FindServerPid)
    if [[ -z ${SFSProcess} ]]
    then
        ErrorExit "SFS Server not running."
    fi

    #
    # Get a list of the volumes owned by the server
    #
    echo "...Finding SFS Server details."
    ServerVolumes=$(tkadmin list lvols -server $EncinaSFSServer)
    echo "...SFS Server Name ............... [" $EncinaSFSServer "]"
    echo "...SFS Server Volume Name(s) ..... [" $ServerVolumes "]"

    #
    # Create the backup directory if one does not already exist.
    #
    echo "...Backup directory is ........... [" $DDBackupPath "]"
    MakeBackupDirectory $DDBackupPath 

    #
    # Shut down the SFS server if it is running.
    #
    CICSShutDownSFSServer $AskUser

    #
    # Initialise the DD copy restore script.
    #
    echo "...Creating recovery script ...... [" $DDBackupPath/$DDRestScript "]"
    echo "# DD copy restore script."          >$DDBackupPath/$DDRestScript
    echo "# Backed up on $(date)."            >>$DDBackupPath/$DDRestScript

    #
    # Initialise dd copy script, which will actually perform the dd copy.
    #
    echo "...Creating ddcopy script ........ [" $DDBackupPath/$DDCopyScript "]"
    echo "# DD copy restore script."          >$DDBackupPath/$DDCopyScript
    echo "# Backed up on $(date)."            >>$DDBackupPath/$DDCopyScript

    #
    # Now copy them to the backup directory
    #
    echo "...Backing up list of logical volumes."
    for Volume in $ServerVolumes
    do
        echo "...Backing up volume ............. [" $Volume "].\c"
        echo " >>>> Please wait <<<<"
        OutFile=$DDBackupPath/${Volume}
        OutDir=$DDBackupPath

        dd if=${VolDir}/${Volume} of=${OutFile} obs=4096k
        if [[ $? -ne 0 ]]
        then
            ErrorExit "Could not copy logical volumes using dd copy."
        fi

        lsvg rootvg -l | grep ${Volume##*/r}" " |                             \
              awk '{print "mklv -y "$1" rootvg "$3}'                          \
                                              >>$DDBackupPath/$DDRestScript
        ls -l ${VolDir}/${Volume} |                                           \
           awk '{print "chown "$3":"$4" "$10" ${VolDir}"substr($10,7)}'       \
                                              >>$DDBackupPath/$DDRestScript
        echo "dd if=${OutFile} of=${VolDir}/${Volume##*/} ibs=4096k"         \
                                              >>$DDBackupPath/$DDCopyScript
    done

    #
    # Now copy the restart files
    #
    BackupRestartFiles ${DDBackupPath}/${DDRestScript}

    echo "Backup of SFS Server "$EncinaSFSServer" is complete.\n\n"

    #
    # Wait for the user to press <Enter> before returning.
    #
    OutputFinalMessage
}


#
# Restore the logical volumes owned by the SFS Server.
# This includes all data and log volumes. Also the restart files
# are restored as well.
#
function ddRestoreVolumes
{
    #
    # Clear the screen ready for the messages concerning the backup.
    #
    PrintScreenHeader

    echo "\n\nAbout to restore the logical volumes using ddcopy..."
    echo "...SFS Server Name ............... [" $EncinaSFSServer "]"
    echo "...Restore from directory ........ [" $DDBackupPath "]"

    #
    # Confirm that the user really wants to restore the SFS server.
    #
    echo "...This will destroy the existing SFS Server data."
    echo "...Do you want to continue with the restore [Y/N]? \c"
    read Confirm
    if [[ $Confirm = N || $Confirm = n ]]
    then
        echo "Restore will be cancelled.\n\n\n\n\n\n"
        OutputFinalMessage
        return
    fi

    #
    # We need the SFS Server running in admin mode in order to restore the
    # data from the MRA files. So we first need to check if the server is
    # running. If it is we shut it down and restart it in admin mode. If it
    # is not then we just start it in admin mode.
    #
    SFSProcess=$(FindServerPid)

    if [[ -z ${SFSProcess} ]]
    then
        echo "...SFS Server is not currently running."
        cicssfslock -u $EncinaSFSServer
    else
        echo "...SFS Server must be shut down, before restore can begin."
        CICSShutDownSFSServer $AskUser
    fi

    #
    # Run the restore, using the script which was built when the backup was
    # performed.
    #
    echo "...Using recovery script ........ [" $DDBackupPath/$DDRestScript "]"
    echo "...Restoring logical volumes. >>>> Please wait <<<<"
    ksh $DDBackupPath/$DDRestScript 2>/dev/null

    #
    # Cold start the server to make sure it knows where its volumes are.
    #
    CICSStartSFSServer $Cold

    echo "...Shutting down SFS Server to allow ddcopy to take place."
    CICSShutDownSFSServer $DoNotAskUser

    #
    # Run the restore, using the script which was built when the backup was
    # performed.
    #
    echo "...Restoring the SFS data ....... [" $DDBackupPath/$DDCopyScript "]"
    echo "...Restoring logical volumes. >>>> Please wait <<<<"
    ksh $DDBackupPath/$DDCopyScript
      
    echo "The restore of SFS Server "$EncinaSFSServer" has completed." 
    echo "Warm start SFS Server when ready."

    #
    # Wait for the user to press <Enter> before returning.
    #
    OutputFinalMessage
}


#
# This function displays the main menu options. It then calls the correct
# menu option routines depending on the users input. When the exit option
# is selected then the problem halts.
#
function DisplayMainMenu
{
    ErrorMessage=""
 
    while [[ $MenuOption != $ExitOption ]]
    do
        PrintScreenHeader
        echo "\n                               *****************"
        echo "                               *** MAIN MENU ***"
        echo "                               *****************"
        echo "\n   <1> ..... MRA: Backup of SFS Data"
        echo "   <2> ..... MRA: Backup SFS Log Archive Files"
        echo "   <3> ..... MRA: Restore SFS log and data using archive files\n"
        echo "   <4> ..... dd copy SFS logical volumes"
        echo "   <5> ..... Restore dd copy of SFS logical volumes\n"
        echo "\n   <"$ExitOption"> ..... Exit the utility\n\n"
        echo "\n------------------------------------\c"
        echo "-------------------------------------------"
        echo $ErrorMessage
        echo " Please enter your choice > \c"
        read MenuOption
        
        case $MenuOption in
            1) PerformBackupSFSData ;;
            2) PerformBackupMRAFiles ;;
            3) PerformRestoreSFSData ;; 
            4) ddCopyVolumes ;;
            5) ddRestoreVolumes;;
            *) ErrorMessage=">>>> Invalid Menu selection, please retry <<<<" ;;
        esac 
        
    done
}

# -----------------------------------------------------------------------------
#                        Start of the main routine
# -----------------------------------------------------------------------------

clear
PrintScreenHeader
echo "\n\nPlease wait setting up environment ...."
SystemName=$(basename $EncinaSFSServer)

NameService=$(cicsgetattribute -c ssd -k $EncinaSFSServer -a NameService)
if [ "$NameService" = "NONE" ]
then
    # Help Encina to find the binding information.
    if [ "$ENCINA_BINDING_FILE" = "" ]
    then 
	export ENCINA_BINDING_FILE="/var/cics_servers/server_bindings"
    fi
else
    # Check that we are logged in as cell_admin.
    CheckDCELogin
fi

# Show the main menu.
DisplayMainMenu

clear
echo "SFS And Log Data Backup/Restore Utility has been exited."
exit 0
