/*
 * NAME:    ESM Module - Sample implementation of ESM interface functions
 * FILE:    cics_esm.c
 * VERSION: 1.1
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 *
 * SYNOPSIS:
 *
 * #include <cicstype.h>
 * #include <cics_esm.h>
 *
 * int CICS_ESM_Init(struct CICS_ESM_RegionData *RegionData,
 *		     struct CICS_ESM_FnBuf *FnBuf)
 *
 * CICS_ESM_Return_t CICS_ESM_TSLCheck(struct CICS_ESM_RegionData *RegionData,
 *			       struct CICS_ESM_ContextData *ContextData)
 *
 * CICS_ESM_Return_t CICS_ESM_RSLCheck(struct CICS_ESM_RegionData *RegionData,
 *			       struct CICS_ESM_ContextData *ContextData,
 *			       union CICS_ESM_ResourceData *ResourceData,
 *			       enum CICS_ESM_ResourceClass ResourceClass,
 *			       struct CICS_ESM_TaskData *TaskData,
 *			       cics_char_t *EIBFn)
 *    
 * DESCRIPTION:
 *
 * These functions define the interface to the External Security Manager
 * (ESM). They are used by CICS to initialise the ESM and invoke ESM
 * validation. Resource data relevant to the ESM call is supplied as a
 * set of structure reference arguments, one for each resource. All the
 * structures contain at least one common 'Id' field, the CICS resource
 * name. Various other fields are included in each type of structure as
 * appropriate to the type of resource in question. This use of
 * resource structures is intended to allow the data interface to be
 * extended conservatively as requirements for extra data are
 * identified.
 *
 * The use of the names ESM_Init, ESM_TSLCheck and ESM_RSLCheck is
 * purely for heuristic purposes. The functions are only accessed from
 * this module indirectly by address, rather than via a call naming the
 * function. Thus the actual names used in the ESM module can be
 * defined by the implementor of an alternative ESM.
 *
 * :hp2.ESM_Init:ehp2. is called during CICS startup to
 * enable the ESM to perform any system specific initialisation and to
 * establish the addresses of the ESM callback functions
 * :sl.
 * :li.:hp2.RegionData:ehp2. describes the CICS system.
 * :li.:hp2.ESM_FnBuf:ehp2. is a structure which is used to return the addresses
 * of the callback functions which perform ESM validation, ESM_TSLCheck and
 * ESM_RSLCheck. The use of a structure will enable new callbacks to be
 * added if the requirement should arise while still extending the
 * interface conservatively.
 * :esl.
 *
 * :hp2.ESM_TSLCheck:ehp2. and :hp2.ESM_RSLCheck:ehp2. are callback
 * functions called to perform TSL and RSL validation, respectively.
 * :sl.
 * :li.:hp2.RegionData:ehp2. describes the
 * CICS region.
 * :li.:hp2.ContextData:ehp2. contains five sub-structures
 * describing the current transaction, user, terminal, program and link
 * region. The program sub-structure is null for a TSL check as no
 * program is running when this is made.
 * The link region sub-structure will be null unless the transaction was
 * scheduled from a remote system.
 * :li.:hp2.ResourceClass:ehp2. is used
 * to identify the type of the resource being checked.
 * :li.:hp2.ResourceData:ehp2. provides
 * a generic resource description whose type can be determined using
 * ResourceClass. ResourceData is a union of structures holding data
 * from the resource database relevant to the resource being checked.
 * :li.:hp2.TaskData:ehp2. is a structure holding the addresses of user
 * storage areas, initially the CWA and the TCTUA. In circumstances
 * where the task context is not available it may be supplied as a NULL
 * pointer.
 * :li.:hp2.EIBFn:ehp2. is the 2 byte code for the EXEC CICS operation which is
 * performing a resource access obtained from the FN field of the EIB.
 * :esl.
 *
 * CAVEATS/WARNINGS:
 *
 * None.
 *
 *
 * RETURNED VALUES:
 * :dl tsize=25.
 * :dt.CICS_ESM_RETURN_OK
 * :dd.ESM_TSLCheck and ESM_RSLCheck return this to indicate
 * that access to the transaction/resource is valid.
 * :dt.CICS_ESM_RETURN_INVALID
 * :dd.ESM_TSLCheck and ESM_RSLCheck return this to indicate that access
 * to the transaction/resource is invalid.
 * :dt.CICS_ESM_RETURN_FAIL
 * :dd.ESM_TSLCheck and ESM_RSLCheck return this to indicate
 * that access to the transaction/resource cannot be validated
 * externally.
 * :edl.
 *
 * RELATED INFORMATION:
 *
 *   None.
 *
 */

#include <stdio.h>
#include <cicstype.h>			/* CICS-supplied datatypes */
#include <cics_esm.h>

int CICS_ESM_Init(struct CICS_ESM_RegionData *RegionData,
			struct CICS_ESM_FnBuf *FnBuf)
{
    if (FnBuf != NULL)
    {
       FnBuf->TSLCheckFn = CICS_ESM_TSLCheck;
       FnBuf->RSLCheckFn = CICS_ESM_RSLCheck;
    }

    return 0;
}

CICS_ESM_Return_t CICS_ESM_TSLCheck(struct CICS_ESM_RegionData *RegionData,
					struct CICS_ESM_ContextData *ContextData)
{
    return (CICS_ESM_RETURN_OK);
}

CICS_ESM_Return_t CICS_ESM_RSLCheck(struct CICS_ESM_RegionData *RegionData,
					struct CICS_ESM_ContextData *ContextData,
					union CICS_ESM_ResourceData *ResourceData,
					enum CICS_ESM_ResourceClass ResourceClass,
					struct CICS_ESM_TaskData *TaskData,
					cics_char_t *EIBFn)
{
    return (CICS_ESM_RETURN_OK);
}
