/*
 * NAME:        CICS_SFMT_GenDetail
 * VERSION:     1.1
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * SYNOPSIS:
 *
 * #include <cicstype.h>
 * #include <cicssfmt.h>
 *
 * PRIVATE enum CICS_SFMT_Status CICS_SFMT_GenDetail(void);
 *
 * DESCRIPTION:
 *
 * :hp2.CICS_SFMT_GenDetail:ehp2. generates the Interval Detailed Report
 * using the file handle from the global :hp4.CICS_SFMT_StatsFile:ehp4.
 * structure and the input information from the global
 * :hp4.CICS_SFMT_Input:ehp4. structure.
 *
 * CAVEATS/WARNINGS:
 * None.
 *
 * RETURNED VALUES:
 * :hp2.CICS_SFMT_GenDetail:ehp2 functions returns :
 * :ul. compact.
 * :li.CICS_SFMT_STATUS_OK - Function completed successfully.
 * :li.CICS_SFMT_FILE_ERR  - CICS_SFMT_ReadRecord returns file error.
 * :li.CICS_SFMT_MEM_ERROR - Unable to allocate memory returned by call
 *                           to CICS_SFMT_StoreRec.
 * :eul.
 *
 * RELATED INFORMATION:
 * See the main :hp2.cicssfmt:ehp2. program for the usage of this function.
 *
 */

#include <cicstype.h>	       /* CICS-supplied datatypes */
#include <stdio.h>
#include <cicssfmt.h>
#include <cicssfmt_msg.h>

enum CICS_SFMT_Status CICS_SFMT_GenDetail(void)
{
    CICS_SFMT_SummaryList_t *Sum_list = NULL; /* holds the summary stats */

    enum CICS_SFMT_Status RetStatus;        /* return code */
    enum CICS_SFMT_Status Ret;              /* return code */
    cics_ulong_t PrevCatId = 0;             /* previous category Id */
    cics_ulong_t CurrCatId = 0;             /* Current category Id */
    cics_ulong_t DetId = 0;                 /* Current details Id */
    cics_ulong_t PrevCollectType = 0;       /* Previous Collection type */
    cics_ulong_t CollectType = 0;           /* Collection type */
    int IsThere;                            /* category is specified */

    cics_char_t *WorkBuffer;                /* space to manipulate strings */
    int WorkBufferLen;                      /* size of WorkBuffer */
    cics_char_t *BufferPtr;                 /* pointer to current write pos. */
    int numbytes;                           /* num bytes to allocate mem. to */

    int MsgNo;                              /* Message Number */
    int SetNo;                              /* Message Set Number */
    cics_char_t *Msg;                       /* Message Buffer */
    cics_char_t *OutputMsg;                 /* message buffer */
    cics_char_t *DefaultMsg;                /* Def msg for catalogue failure */
    struct tm *TimeData;                    /* manipulate time */
    cics_char_t *TimeString;                /* manipulate time */
    int End = FALSE;                        /* endtime reached ? */
    int i;                                  /* loop variable */
    cics_slong_t TimeValue;                 /* Current supplied record time */

    SFMT_DEBUG(("IN CICS_SFMT_GenDetail\n"));

    numbytes = CICS_SFMT_InputInfo.Pagewidth + 1;

    /* Allocate memory to the workbuffer & initialise it to spaces*/
    WorkBuffer = (cics_char_t *)malloc(numbytes);
    memset(WorkBuffer, ' ', numbytes);
    WorkBuffer[numbytes - 1] = '\0';     /* terminate buffer */

    /* we have one record in the StatFile global variable
     * hence process it first
     */

    do
    {
        CurrCatId = CICS_SFMT_StatsFile.Record.Record.Id;
        switch (CurrCatId)
        {
            case CICSSTAT_CATID_INTERVAL :
            case CICSSTAT_CATID_ENDOFDAY :
            case CICSSTAT_CATID_REQUESTED :
            case CICSSTAT_CATID_ENDOFREGION :
            case CICSSTAT_CATID_REQUESTED_RESET :
            case CICSSTAT_CATID_UNSOLICITED :
                /*
                 * If all goes well, we will display collection TYPE, and the
                 * collection timestamp.
                 */

                CICS_SFMT_IncrLine(4);

                /* Set default message in case of catalogue errors */
                DefaultMsg = "Failed to access catalogue for Type";

                /* get the "Type: " message string */
                Msg = catgets(CICS_SFMT_CatDescriptor,
                              CICS_SFMT_REPORT_SET,
                              CICS_SFMT_SRH_TYPE,
                              DefaultMsg);

                /* Check the result of the catalogue access */
                if (strcmp(Msg, DefaultMsg) == 0)
                {
                    perror(Msg);
                }
                else
                {
                    /* get the message set number for this stats category */
                    SetNo = CICS_SFMT_CATSET_NUM(CurrCatId);

                    /* get the text for the category */
                    /* first message in this sets gives the category text */
                    MsgNo = 1;

                    /* Set default message in case of catalogue errors */
                    DefaultMsg =
                              "Failed to access catalogue for Collection Type";

                    /* get the "Type: " message string */
                    OutputMsg = catgets(CICS_SFMT_CatDescriptor,
                                         SetNo,
                                         MsgNo,
                                         DefaultMsg);

                    /* Check the result of the catalogue access */
                    if (strcmp(OutputMsg, DefaultMsg) == 0)
                    {
                        perror(OutputMsg);
                    }
                    else
                    {
                        sprintf(WorkBuffer, Msg, (numbytes - 1), OutputMsg);
#if defined(SFMT_DEBUG_ON)
                        fprintf(stderr, "\n");
                        fprintf(stderr, WorkBuffer);
                        fprintf(stderr, "\n");
#else
                        fprintf(stdout, "\n");
                        fprintf(stdout, WorkBuffer);
                        fprintf(stdout, "\n");
#endif
                        memset(WorkBuffer, ' ', numbytes);
                        WorkBuffer[numbytes - 1] = '\0';
                    }
                }

                /* get the time info. from it's details sub-record */

                DetId = CICS_SFMT_StatsFile.Record.Record.Details[0].Id;
                switch (DetId)
                {
                    case CICSSTAT_CATID_TIME_START :
                    case CICSSTAT_CATID_TIME_NA :
                        SFMT_DEBUG(("TIME START\n"));

                        TimeValue =
                            CICS_SFMT_StatsFile.Record.Record.Details[0].Value;

                        if ((CICS_SFMT_InputInfo.Endtime != -1) &&
                            (TimeValue >= CICS_SFMT_InputInfo.Endtime))
                        {
                            SFMT_DEBUG(("Reached time = [%ld], End = [%ld]\n",
                                TimeValue, CICS_SFMT_InputInfo.Endtime));

                            End = TRUE;
                            break;      /* exit switch early */
                        }


                        /* get the message from the catalogue */
                        SetNo = CICS_SFMT_CATSET_NUM(DetId);

                        /* Set default message in case of catalogue errors */
                        DefaultMsg = "Failed to access catalogue for Type";

                        /* get the "Type: " message string */
                        Msg = catgets(CICS_SFMT_CatDescriptor,
                                      SetNo,
                                      1,
                                      DefaultMsg);

                        /* Check the result of the catalogue access */
                        if (strcmp(Msg, DefaultMsg) == 0)
                        {
                            perror(Msg);
                        }
                        else
                        {
                            /* convert the 'Value' into string */
                            TimeData   = localtime(&TimeValue);
                            TimeString = asctime(TimeData);
                            *(TimeString + strlen(TimeString) - 1) = '\0';

                            sprintf(WorkBuffer, Msg, (numbytes - 1),
                                    TimeString);





                            fprintf(stdout, WorkBuffer);
                            fprintf(stdout, "\n\n");


                            memset(WorkBuffer, ' ', numbytes);
                            WorkBuffer[numbytes - 1] = '\0';
                        }
                        break;
                    default :
                        SFMT_DEBUG(("Default Id : [%#x]\n", DetId));
                        break;
                }
                /*
                 * Reset the global CICS_SFMT_ReportDetails array to zero
                 */
                for (i = 0; i < CICSSTAT_MAX_STAT_DETS; i++)
                {
                    memset(&(CICS_SFMT_ReportDetails[i]),
                           0,
                           sizeof(CICSSTAT_Detail_t));
                }

                CollectType = CurrCatId;
                break;
            case CICSSTAT_CATID_SUMMARY :
                SFMT_DEBUG(("SUMMARY\n"));
                CollectType = CurrCatId;
                break;
            case CICSSTAT_CATID_ALL_DETAILS :
            case CICSSTAT_CATID_ONE_DETAIL :
                SFMT_DEBUG(("ALL,ONE DETAIL\n"));
                CollectType = CurrCatId;
                break;
            case CICSSTAT_CATID_DUMP :
            case CICSSTAT_CATID_FILE :
            case CICSSTAT_CATID_ISCM :
            case CICSSTAT_CATID_JOUR :
            case CICSSTAT_CATID_LUWM :
            case CICSSTAT_CATID_PROG :
            case CICSSTAT_CATID_RTDB :
            case CICSSTAT_CATID_STOR :
            case CICSSTAT_CATID_STOT :
            case CICSSTAT_CATID_TASK :
            case CICSSTAT_CATID_TERM :
            case CICSSTAT_CATID_TDQU :
            case CICSSTAT_CATID_TRAN :
            case CICSSTAT_CATID_TSQU :
                SFMT_DEBUG(("CATEGORY : \n"));
                if ((IsThere = CICS_SFMT_CatIsThere(CurrCatId)) == 0)
                {
                    SFMT_DEBUG(("VALID CATEGORY\n"));
                    switch (CollectType)
                    {
                        case CICSSTAT_CATID_SUMMARY :
                            SFMT_DEBUG(("COLLECT TYPE SUMMARY\n"));
                            if (Sum_list != NULL)
                            {
                                if (Sum_list->Sum_Record.Record.ItemName[0]
                                           == '\0')
                                {
                                    SetNo = CICS_SFMT_CATSET_NUM(
                                                        CICSSTAT_CATID_SUMMARY);
                                    MsgNo = 1;
                                    /* Set default message in case of
                                                            catalogue errors */
                                    DefaultMsg = "Failed to access catalogue";

                                    /* get the message string */
                                    OutputMsg = catgets(CICS_SFMT_CatDescriptor,
                                                        SetNo,
                                                        MsgNo,
                                                        DefaultMsg);

                                    /* Check the result of the
                                                            catalogue access */
                                    if (strcmp(OutputMsg, DefaultMsg) == 0)
                                    {
                                        perror(OutputMsg);
                                    }
                                    else
                                    {
                                        strcpy(
                                           Sum_list->Sum_Record.Record.ItemName,
                                               OutputMsg);
                                    }
                                }

                                CICS_SFMT_OutputStats(&(Sum_list->Sum_Record));

                                free((void *)Sum_list);
                                SFMT_DEBUG(("FREE Sum_list in GenDetail\n"));
                                Sum_list = NULL;
                            }
                            /*
                             * store the record in summary list
                             */
                            if ( (Ret =
                                     CICS_SFMT_StoreRec(
                                                  &(CICS_SFMT_StatsFile.Record),
                                                           &Sum_list))
                                      != CICS_SFMT_STATUS_OK )
                            {
                                SFMT_DEBUG(("Unable to allocate memory\n"));
                                free(WorkBuffer);
                                return(Ret);
                            }
                            break;

                        case CICSSTAT_CATID_ALL_DETAILS :
                        case CICSSTAT_CATID_ONE_DETAIL :
                            SFMT_DEBUG(("COLLECT TYPE DETAIL\n"));
                            CICS_SFMT_OutputStats(
                                                 &(CICS_SFMT_StatsFile.Record));
                            break;
                    }
                    PrevCollectType = CollectType;
                }
                PrevCatId = CurrCatId;
                break;

            case CICSSTAT_CATID_END :
                SFMT_DEBUG(("CATID_END : [%#x]\n"));
                if ( Sum_list != NULL )
                {
                    if (Sum_list->Sum_Record.Record.ItemName[0] == '\0')
                    {
                        SetNo = CICS_SFMT_CATSET_NUM(CICSSTAT_CATID_SUMMARY);
                        MsgNo = 1;

                        /* Set default message in case of catalogue errors */
                        DefaultMsg = "Failed to access catalogue";

                        /* get the message string */
                        OutputMsg = catgets(CICS_SFMT_CatDescriptor,
                                      SetNo,
                                      MsgNo,
                                      DefaultMsg);

                        /* Check the result of the catalogue access */
                        if (strcmp(OutputMsg, DefaultMsg) == 0)
                        {
                            perror(OutputMsg);
                        }
                        else
                        {
                            strcpy(Sum_list->Sum_Record.Record.ItemName,
                                   OutputMsg);
                        }
                    }
                    CICS_SFMT_OutputStats(&(Sum_list->Sum_Record));

                    free((void *)Sum_list);
                    SFMT_DEBUG(("FREE Sum_list in CATID_END GenDetail\n"));
                    Sum_list = NULL;
                    SFMT_DEBUG(("Emptied Sum_list\n"));
                }

                /* Check if Endtime reached */

                DetId = CICS_SFMT_StatsFile.Record.Record.Details[0].Id;
                switch (DetId)
                {
                    case CICSSTAT_CATID_TIME_STOP :
                    case CICSSTAT_CATID_TIME_NA :
                        SFMT_DEBUG(("TIME END\n"));

                        TimeValue =
                            CICS_SFMT_StatsFile.Record.Record.Details[0].Value;

                        if ((CICS_SFMT_InputInfo.Endtime != -1) &&
                            (TimeValue >= CICS_SFMT_InputInfo.Endtime))
                        {
                            SFMT_DEBUG(("REACHED ENDTIME\n"));
                            End = TRUE;
                        }


                        /* Set default message in case of catalogue errors */
                        DefaultMsg = "Failed to access catalogue for Type";

                        SetNo = CICS_SFMT_CATSET_NUM(DetId);

                        /* get the message from the catalogue */
                        Msg = catgets(CICS_SFMT_CatDescriptor,
                                      SetNo,
                                      1,
                                      DefaultMsg);

                        /* Check the result of the catalogue access */
                        if (strcmp(Msg, DefaultMsg) == 0)
                        {
                            perror(Msg);
                        }
                        else
                        {
                            /* convert the 'Value' into string */
                            TimeData   = localtime(&TimeValue);
                            TimeString = asctime(TimeData);
                            *(TimeString + strlen(TimeString) - 1) = '\0';

                            sprintf(WorkBuffer, Msg, (numbytes - 1),
                                    TimeString);





                            fprintf(stdout, WorkBuffer);
                            fprintf(stdout, "\n\n");


                            memset(WorkBuffer, ' ', numbytes);
                            WorkBuffer[numbytes - 1] = '\0';
                        }
                        break;
                    default :
                        SFMT_DEBUG(("Default Id : [%#x]\n", DetId));
                        break;
                }
                CollectType = CurrCatId;
                break;
        } /* end switch */

        if (End)  /* reached endtime */
        {
            SFMT_DEBUG(("Break On ENDTIME \n"));
            break;
        }
    } while ( (RetStatus = CICS_SFMT_ReadRecord(&CICS_SFMT_StatsFile))
                         == CICS_SFMT_STATUS_OK );

    switch (RetStatus)
    {
        case CICS_SFMT_FILE_ERR :
            SFMT_DEBUG(("FREE WorkBuffer GenDetail with FILE_ERR\n"));
            free(WorkBuffer);
            return (RetStatus);

        case CICS_SFMT_FILE_SHORT_READ :
            SFMT_DEBUG(("FREE WorkBuffer GenDetail with FILE_SHORT_READ\n"));
            free(WorkBuffer);
            return (RetStatus);

        case CICS_SFMT_FILE_ZERO_READ :
            SFMT_DEBUG(("FREE WorkBuffer GenDetail with FILE_ZERO_READ\n"));
            free(WorkBuffer);
            return CICS_SFMT_STATUS_OK;

        case CICS_SFMT_STATUS_OK :
            SFMT_DEBUG(("FREE WorkBuffer GenDetail with STATUS_OK \n"));
            free(WorkBuffer);
            return CICS_SFMT_STATUS_OK;
    }
}
