/*
 * NAME:        CICS_SFMT_GenSummary
 * VERSION:     1.2
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * SYNOPSIS:
 *
 * #include <cicstype.h>
 * #include <cicssfmt.h>
 *
 * PRIVATE enum CICS_SFMT_Status CICS_SFMT_GenSummary(void);
 *
 * PRIVATE int CICS_SFMT_CatIsThere(IN cics_ulong_t Id);
 *
 * PRIVATE void CICS_SFMT_AverageCalc(IN cics_bool_t IsInterval,
 *                                    INOUT CICS_SFMT_SummaryList_t *List);
 *
 * DESCRIPTION:
 * :hp2.CICS_SFMT_GenSummary:ehp2. generates the Interval Summary Report
 * using the file handle from the global :hp4.CICS_SFMT_StatsFile:ehp4.
 * structure and the input information from the global
 * :hp4.CICS_SFMT_Input:ehp4. structure.
 *
 * :hp2.CICS_SFMT_CatIsThere:ehp2. simply checks is the given category is
 * specified on the command line input.
 *
 * :hp2.CICS_SFMT_AverageCalc:ehp2. calculates the average statistics for the
 * Interval/EndofDay summary collection types. :hp4.IsInterval:ehp4. is used
 * to output the collection type message. Average calculation is done by
 * totaling  and averaging the summary statistics values obtained for each
 * category in both the collections types.
 *
 * CAVEATS/WARNINGS:
 * None.
 *
 * RETURNED VALUES:
 * :hp2.CICS_SFMT_GenSummary:ehp2 functions returns :
 * :ul. compact.
 * :li.CICS_SFMT_STATUS_OK - Function completed successfully.
 * :li.CICS_SFMT_FILE_ERR  - CICS_SFMT_ReadRecord returns file error.
 * :li.CICS_SFMT_MEM_ERROR - Unable to allocate memory returned by call
 *                           to CICS_SFMT_StoreRec.
 * :eul.
 * :hp2.CICS_SFMT_CatIsThere:ehp2 functions returns :
 * :ul. compact.
 * :li. 0 - Category is specified.
 * :li. 1 - Category is not specified.
 * :eul.
 *
 * RELATED INFORMATION:
 * See the main :hp2.cicssfmt:ehp2. program for the usage of the function.
 *
 */

#include <cicstype.h>		/* CICS-supplied datatypes */
#include <stdio.h>
#include <cicssfmt.h>
#include <cicssfmt_msg.h>

enum CICS_SFMT_Status CICS_SFMT_GenSummary(void)
{
    CICS_SFMT_SummaryList_t *INT_List = NULL; /* Interval summary records */
    CICS_SFMT_SummaryList_t *EOD_List = NULL; /* End of Day summary records */

    enum CICS_SFMT_Status RetStatus;        /* return code */
    enum CICS_SFMT_Status Ret;              /* return code */
    cics_ulong_t PrevCatId;                 /* previous category Id */
    cics_ulong_t CurrCatId;                 /* Current category Id */
    cics_ulong_t DetId = 0;                 /* Current details Id */
    cics_ulong_t CollectType;               /* Collection type */
    cics_ulong_t PrevCollectType;           /* Previous Collection type */
    cics_char_t *CurrCatText;               /* current category text */
    int IsThere;                            /* category is specified */
    cics_bool_t Title;                      /* print title ? */

    cics_char_t *WorkBuffer;                /* space to manipulate strings */
    int WorkBufferLen;                      /* size of WorkBuffer */
    cics_char_t *BufferPtr;                 /* pointer to current write pos. */
    int numbytes;                           /* num bytes to allocate mem. to */

    int MsgNo;                              /* Message Number */
    int SetNo;                              /* Message Set Number */
    cics_char_t *Msg;                       /* Message Buffer */
    cics_char_t *Msg1;                      /* message buffer */
    cics_char_t *DefaultMsg;                /* Def msg for catalogue failure */
    struct tm *TimeData;                    /* manipulate time */
    cics_char_t *TimeString;                /* manipulate time */
    cics_slong_t TimeValue;                 /* Retrieved Time value */
    int End = FALSE;                        /* endtime reached ? */
    int i;                                  /* loop variable */

    SFMT_DEBUG(("IN CICS_SFMT_GenSummary\n"));

    numbytes = CICS_SFMT_InputInfo.Pagewidth + 1;

    /*
     * Allocate memory to the workbuffer & initialise it to spaces
     */
    WorkBuffer = (cics_char_t *)malloc(numbytes);
    memset(WorkBuffer, ' ', numbytes);
    WorkBuffer[numbytes - 1] = '\0';     /* terminate buffer */

    /*
     * we have one record in the StatFile global variable
     * hence process it first
     */

    do
    {
        CurrCatId = CICS_SFMT_StatsFile.Record.Record.Id;
        switch (CurrCatId)
        {
            case CICSSTAT_CATID_INTERVAL :
            case CICSSTAT_CATID_ENDOFDAY :
            case CICSSTAT_CATID_REQUESTED :
            case CICSSTAT_CATID_ENDOFREGION :
            case CICSSTAT_CATID_REQUESTED_RESET :
                SFMT_DEBUG(("Request type is: %s\n",
                  ((CurrCatId == CICSSTAT_CATID_INTERVAL)    ? "INTERVAL"    :
                   (CurrCatId == CICSSTAT_CATID_ENDOFDAY)    ? "ENDOFDAY"    :
                   (CurrCatId == CICSSTAT_CATID_REQUESTED)   ? "REQUESTED"   :
                   (CurrCatId == CICSSTAT_CATID_ENDOFREGION) ? "ENDOFREGION" :
                   "REQUESTED_RESET")));

                CollectType = CurrCatId;
                PrevCollectType = CollectType;

                /*
                 * If all goes well, we will display collection TYPE, and the
                 * collection timestamp.
                 */

                CICS_SFMT_IncrLine(4);

                /* Set default message in case of catalogue errors */
                DefaultMsg = "Failed to access catalogue for Type";

                /* get the "Type: " message string */
                Msg = catgets(CICS_SFMT_CatDescriptor,
                              CICS_SFMT_REPORT_SET,
                              CICS_SFMT_SRH_TYPE,
                              DefaultMsg);

                /* Check the result of the catalogue access */
                if (strcmp(Msg, DefaultMsg) == 0)
                {
                    perror(Msg);
                }
                else
                {
                    /* get the message set number for this stats category */
                    SetNo = CICS_SFMT_CATSET_NUM(CurrCatId);

                    /* get the text for the category */
                    /* first message in this sets gives the category text */
                    MsgNo = 1;

                    /* Set default message in case of catalogue errors */
                    DefaultMsg =
                              "Failed to access catalogue for Collection Type";

                    Msg1 = catgets(CICS_SFMT_CatDescriptor,
                                   SetNo,
                                   MsgNo ,
                                   DefaultMsg);

                    if (strcmp(Msg1, DefaultMsg) == 0)
                    {
                        perror(Msg1);
                    }
                    else
                    {
                        sprintf(WorkBuffer, Msg, (numbytes - 1), Msg1);

#if defined(SFMT_DEBUG_ON)
                        fprintf(stderr, "\n");
                        fprintf(stderr, WorkBuffer);
                        fprintf(stderr, "\n");
#else
                        fprintf(stdout, "\n");
                        fprintf(stdout, WorkBuffer);
                        fprintf(stdout, "\n");
#endif
                        memset(WorkBuffer, ' ', numbytes);
                        WorkBuffer[numbytes - 1] = '\0';
                    }
                }

                /* get the time info. from it's details sub-record */

                DetId = CICS_SFMT_StatsFile.Record.Record.Details[0].Id;
                switch (DetId)
                {
                    case CICSSTAT_CATID_TIME_START :
                    case CICSSTAT_CATID_TIME_NA :
                        SFMT_DEBUG(("TIME START\n"));

                        TimeValue =
                            CICS_SFMT_StatsFile.Record.Record.Details[0].Value;

                        if ((CICS_SFMT_InputInfo.Endtime != -1) &&
                            (TimeValue >= CICS_SFMT_InputInfo.Endtime))
                        {
                            SFMT_DEBUG(("Reached time = [%ld], End = [%ld]\n",
                                TimeValue, CICS_SFMT_InputInfo.Endtime));

                            End = TRUE;
                            break;      /* exit switch early */
                        }

                        /* Set default message in case of catalogue errors */
                        DefaultMsg = "Failed to access catalogue for Type";

                        SetNo = CICS_SFMT_CATSET_NUM(DetId);

                        /* get the message from the catalogue */
                        /* first message in this sets gives the category text */
                        MsgNo = 1;

                        Msg = catgets(CICS_SFMT_CatDescriptor,
                                      SetNo,
                                      MsgNo,
                                      DefaultMsg);

                        /* Check the result of the catalogue access */
                        if (strcmp(Msg, DefaultMsg) == 0)
                        {
                            perror(Msg);
                        }
                        else
                        {
                            /* convert the 'Value' into string */
                            TimeData   = localtime(&TimeValue);
                            TimeString = asctime(TimeData);
                            *(TimeString + strlen(TimeString) - 1) = '\0';

                            SFMT_DEBUG(("Msg : [%s] TimeString : [%s]\n",
                                         Msg, TimeString));
                            sprintf(WorkBuffer, Msg, (numbytes - 1),
                                    TimeString);
                            SFMT_DEBUG(("WorkBuffer : [%s]\n", WorkBuffer));





                            fprintf(stdout, WorkBuffer);
                            fprintf(stdout, "\n\n");


                            memset(WorkBuffer, ' ', numbytes);
                            WorkBuffer[numbytes - 1] = '\0';
                        }
                        break;
                    default :
                        SFMT_DEBUG(("Default Id : [%#x]\n", DetId));
                        break;
                }

                /*
                 * Reset the global CICS_SFMT_ReportDetails array to zero
                 */
                for (i = 0; i < CICSSTAT_MAX_STAT_DETS; i++)
                {
                    memset(&(CICS_SFMT_ReportDetails[i]),
                           0,
                           sizeof(CICSSTAT_Detail_t));
                }
                break;

            case CICSSTAT_CATID_SUMMARY :
                SFMT_DEBUG(("SUMMARY\n"));
                CollectType = CurrCatId;
                break;

            case CICSSTAT_CATID_DUMP :
            case CICSSTAT_CATID_FILE :
            case CICSSTAT_CATID_ISCM :
            case CICSSTAT_CATID_JOUR :
            case CICSSTAT_CATID_LUWM :
            case CICSSTAT_CATID_PROG :
            case CICSSTAT_CATID_RTDB :
            case CICSSTAT_CATID_STOR :
            case CICSSTAT_CATID_STOT :
            case CICSSTAT_CATID_TASK :
            case CICSSTAT_CATID_TERM :
            case CICSSTAT_CATID_TDQU :
            case CICSSTAT_CATID_TSQU :
                SFMT_DEBUG(("CATEGORY [%#x]\n", CurrCatId));

                if ( (CollectType == CICSSTAT_CATID_SUMMARY) &&
                     ((IsThere = CICS_SFMT_CatIsThere(CurrCatId)) == 0) )
                {
                    SFMT_DEBUG(("VALID CATEGORY - Print it\n"));

                    CICS_SFMT_OutputStats(&(CICS_SFMT_StatsFile.Record));

                    if (PrevCollectType == CICSSTAT_CATID_INTERVAL)
                    {
                        if ( (Ret = CICS_SFMT_StoreRec(
                                                 &(CICS_SFMT_StatsFile.Record),
                                                       &INT_List))
                                  != CICS_SFMT_STATUS_OK )
                        {
                            SFMT_DEBUG(("Unable to allocate memory\n"));
                            return(Ret);
                        }
                    }
                    if (PrevCollectType == CICSSTAT_CATID_ENDOFDAY)
                    {
                        if ( (Ret = CICS_SFMT_StoreRec(
                                                 &(CICS_SFMT_StatsFile.Record),
                                                       &EOD_List))
                                  != CICS_SFMT_STATUS_OK )
                        {
                            SFMT_DEBUG(("Unable to allocate memory\n"));
                            free(WorkBuffer);
                            return(Ret);
                        }
                    }
                }
                PrevCatId = CurrCatId;
                break;

            case CICSSTAT_CATID_END :
                SFMT_DEBUG(("CATID_END : [%#x]\n", CurrCatId));

                TimeValue = CICS_SFMT_StatsFile.Record.Record.Details[0].Value;

                /* Check if Endtime reached */

                DetId = CICS_SFMT_StatsFile.Record.Record.Details[0].Id;
                switch (DetId)
                {
                    case CICSSTAT_CATID_TIME_STOP :
                    case CICSSTAT_CATID_TIME_NA :
                        SFMT_DEBUG(("TIME END : [%#x]\n", DetId));
                        if ((CICS_SFMT_InputInfo.Endtime != -1) &&
                            (TimeValue >= CICS_SFMT_InputInfo.Endtime))
                        {
                            SFMT_DEBUG(("REACHED ENDTIME\n"));
                            End = TRUE;
                        }

			/* unsolicited stats are details only */
                	if (PrevCollectType == CICSSTAT_CATID_UNSOLICITED)
			    break;

                        /* Set default message in case of catalogue errors */
                        DefaultMsg = "Failed to access catalogue for Type";

                        SetNo = CICS_SFMT_CATSET_NUM(DetId);

                        /* get the "Type: " message string */
                        /* first message in this sets gives the category text */
                        MsgNo = 1;

                        Msg = catgets(CICS_SFMT_CatDescriptor,
                                      SetNo,
                                      MsgNo,
                                      DefaultMsg);

                        /* Check the result of the catalogue access */
                        if (strcmp(Msg, DefaultMsg) == 0)
                        {
                            perror(Msg);
                        }
                        else
                        {
                            /* convert the 'Value' into string */
                            TimeData   = localtime(&TimeValue);
                            TimeString = asctime(TimeData);
                            *(TimeString + strlen(TimeString) - 1) = '\0';

                            sprintf(WorkBuffer, Msg, (numbytes - 1),
                                    TimeString);





                            fprintf(stdout, WorkBuffer);
                            fprintf(stdout, "\n\n");

                            memset(WorkBuffer, ' ', numbytes);
                            WorkBuffer[numbytes - 1] = '\0';
                        }
                        break;

                    default :
                        SFMT_DEBUG(("Default Id : [%#x]\n", DetId));
                        break;
                }
                break;
            case CICSSTAT_CATID_UNSOLICITED :
                SFMT_DEBUG(("UNSOLICITED\n"));
                CollectType = CurrCatId;
                PrevCollectType = CollectType;
                break;
            default :
                SFMT_DEBUG(("OTHER category : [%#x]\n", CurrCatId));
                CollectType = CurrCatId;
                break;
        } /* end switch */

        if (End)  /* reached endtime */
        {
            SFMT_DEBUG(("Break On ENDTIME \n"));
            break;
        }

    } while ( (RetStatus = CICS_SFMT_ReadRecord(&CICS_SFMT_StatsFile))
                         == CICS_SFMT_STATUS_OK );

    switch (RetStatus)
    {
        case CICS_SFMT_FILE_SHORT_READ :
        case CICS_SFMT_FILE_ERR :
            free(WorkBuffer);
            return (RetStatus);

        case CICS_SFMT_FILE_ZERO_READ :
        case CICS_SFMT_STATUS_OK :
            /* calculate the average */
            if (EOD_List != NULL) CICS_SFMT_AverageCalc(FALSE, EOD_List);
            if (INT_List != NULL) CICS_SFMT_AverageCalc(TRUE,  INT_List);
            free(WorkBuffer);
            return CICS_SFMT_STATUS_OK;
    }
}

int CICS_SFMT_CatIsThere(IN cics_ulong_t Id)
{
    int i;                  /* loop variable */
    int found = FALSE;      /* category found ? */
    cics_char_t *Msg;              /* message pointer */
    int SetNo;              /* message set number */
    int MsgNo;              /* Message Number */
    cics_char_t *DefaultMsg;       /* Def msg for catalogue failure */
    cics_char_t *CatAll = "ALL";   /* default for 'all' categories */

    SFMT_DEBUG(("IN CICS_SFMT_CatIsThere : [%#x] \n", Id));

    if (strcmp(CICS_SFMT_InputInfo.Category[0],CatAll) == 0)
    {
        SFMT_DEBUG(("OUT Format ALL Categories\n"));
        return(0);
    }




    /* Set default message in case of catalogue errors */
    DefaultMsg = "Failed to access message catalogue";

    SetNo = CICS_SFMT_CATSET_NUM(Id);

    /* get the message string */
    /* first message in this sets gives the category text */
    MsgNo = 1;

    Msg = catgets(CICS_SFMT_CatDescriptor,
                  SetNo,
                  MsgNo,
                  DefaultMsg);

    /* Check the result of the catalogue access */
    if (strcmp(Msg, DefaultMsg) == 0)
    {
        perror(Msg);
    }
    else
    {
        for(i = 0; i < CICS_SFMT_NUM_CATEGORIES ; i++)
        {
            if ( CICS_SFMT_InputInfo.Category[i] == NULL )
            {
                break;
            }
            if (strcmp(CICS_SFMT_InputInfo.Category[i], Msg) == 0)
            {
                found = TRUE;
            }
        }
        if (found)
        {
            SFMT_DEBUG(("OUT Category required\n"));
            return(0);
        }
        else
        {
            SFMT_DEBUG(("OUT Category NOT required\n"));
            return(1);
        }
    }
}

void CICS_SFMT_AverageCalc(IN cics_bool_t IsInterval,
                           INOUT CICS_SFMT_SummaryList_t *List)
{
    cics_char_t *WorkBuffer;            /* buffer to manipulate strings */
    int numbytes;                       /* no. of bytes to allocate memory */
    cics_char_t *Msg;                   /* ptr to message string */
    cics_char_t *Msg1;                  /* ptr to message string */
    int MsgNo;                          /* Message Number */
    cics_char_t *DefaultMsg;            /* Def msg for catalogue failure */
    int SetNo;                          /* message set number */
    int i;                              /* loop variable */

    CICS_SFMT_SummaryList_t     **Prev;     /* Ptr to previous record in list */
    CICS_SFMT_SummaryList_t     *SumPtr;    /* Ptr to record in list */
    CICS_SFMT_SummaryList_t     *Detach;    /* Ptr to the record to be free'd */
    CICS_SFMT_StatsRecord_t     Average;    /* Ptr to the Average record */
    int                         AveCount;   /* total no. of occurrences */
    cics_char_t                 *ItemName;  /* holds the ItemName string */
    cics_ulong_t                MatchId;    /* Cat Id to match against */
    cics_ulong_t                DetailId;   /* Details Id to match against */
    cics_bool_t                 MatchFound; /* match found ? */


    SFMT_DEBUG(("IN CICS_SFMT_AverageCalc\n"));

    /*
     * Reset the global CICS_SFMT_ReportDetails array to zero
     */
    for (i = 0; i < CICSSTAT_MAX_STAT_DETS; i++)
    {
        memset(&(CICS_SFMT_ReportDetails[i]),
               0,
               sizeof(CICSSTAT_Detail_t));
    }

    numbytes = CICS_SFMT_InputInfo.Pagewidth + 1;

    /*
     * Allocate memory to the workbuffer
     */

    WorkBuffer = (cics_char_t *)malloc(numbytes);
    memset(WorkBuffer, ' ', numbytes);
    WorkBuffer[numbytes - 1] = '\0';

    /*
     * Display the Title
     */

    /* Set default message in case of catalogue errors */
    DefaultMsg = "Failed to access Message catalogue for CICS_SFMT_SRH_AVERAGE";

    /* get the Average title message string */
    Msg = catgets(CICS_SFMT_CatDescriptor,
                  CICS_SFMT_REPORT_SET,
                  CICS_SFMT_SRH_AVERAGE,
                  DefaultMsg);

    /* Check the result of the catalogue access */
    if (strcmp(Msg, DefaultMsg) == 0)
    {
        perror(Msg);
    }
    else
    {
        sprintf(WorkBuffer, "%.*s", (numbytes - 1), Msg);

        CICS_SFMT_IncrLine(3);

#if defined(SFMT_DEBUG_ON)
        fprintf(stderr, "\n");
        fprintf(stderr, WorkBuffer);
        fprintf(stderr, "\n\n");
#else
        fprintf(stdout, "\n");
        fprintf(stdout, WorkBuffer);
        fprintf(stdout, "\n\n");
#endif
        memset(WorkBuffer, ' ', numbytes);
        WorkBuffer[numbytes - 1] = '\0';
    }

    /* Set default message in case of catalogue errors */
    DefaultMsg = "Failed to access catalogue for Type";

    /* get the "Type: " message string */
    Msg = catgets(CICS_SFMT_CatDescriptor,
                  CICS_SFMT_REPORT_SET,
                  CICS_SFMT_SRH_TYPE,
                  DefaultMsg);

    /* Check the result of the catalogue access */
    if (strcmp(Msg, DefaultMsg) == 0)
    {
        perror(Msg);
    }
    else
    {
        /*
         * get the collection type string from the catalogue
         */
        if (IsInterval)
        {
            SetNo = CICS_SFMT_CATSET_NUM(CICSSTAT_CATID_INTERVAL);
        }
        else
        {
            SetNo = CICS_SFMT_CATSET_NUM(CICSSTAT_CATID_ENDOFDAY);
        }

        /* Set default message in case of catalogue errors */
        DefaultMsg = "Failed to access catalogue for Collection Type";

        /* get the message string */
        /* first message in this sets gives the category text */
        MsgNo = 1;

        Msg1 = catgets(CICS_SFMT_CatDescriptor,
                       SetNo,
                       MsgNo,
                       DefaultMsg);

        /* Check the result of the catalogue access */
        if (strcmp(Msg1, DefaultMsg) == 0)
        {
            perror(Msg1);
        }
        else
        {
            SFMT_DEBUG(("Msg : [%s] Msg1 : [%s]\n", Msg, Msg1));
            sprintf(WorkBuffer, Msg, (numbytes - 1), Msg1);
            SFMT_DEBUG(("WorkBuffer : [%s]\n", WorkBuffer));

            CICS_SFMT_IncrLine(3);

#if defined(SFMT_DEBUG_ON)
            fprintf(stderr, "\n");
            fprintf(stderr, WorkBuffer);
            fprintf(stderr, "\n\n");
#else
            fprintf(stdout, "\n");
            fprintf(stdout, WorkBuffer);
            fprintf(stdout, "\n\n");
#endif
            memset(WorkBuffer, ' ', numbytes);
            WorkBuffer[numbytes - 1] = '\0';
        }
    }


    /*
     * Repeatedly work on the list, until it is empty.
     */

    while (List != NULL)
    {
        SFMT_DEBUG(("Loop Head = %#x\n", List));

        SumPtr = List;

        Average = SumPtr->Sum_Record;   /* copy whole structure */
        AveCount = 1;
        ItemName = Average.Record.ItemName;
        MatchId = Average.Record.Id;
        DetailId = Average.Record.Details[0].Id;

        /*
         * Release the head of the list, as its information is now stored in
         * Average.
         */

        Detach = SumPtr;
        SumPtr = SumPtr->Next;

        SFMT_DEBUG(("Release element one = %#x\n", Detach));
        free(Detach);

        /*
         * Setup the List traversal
         */

        List = SumPtr;
        Prev = &List;

        SFMT_DEBUG(("Search on ID = %d, item = <%s>\n", MatchId, ItemName));
        SFMT_DEBUG(("Internal Loop starting at %#x\n", SumPtr));

        /*
         * Traverse stored list.
         */

        while ( SumPtr != (CICS_SFMT_SummaryList_t *)NULL )
        {
            SFMT_DEBUG(("Traverse List; elem = %#x\n", SumPtr));

            MatchFound = FALSE;

            /*
             * if category is 'STOR' then match the current head element,
             * in ID and first 'Details' Id
             */
            if (SumPtr->Sum_Record.Record.Id == CICSSTAT_CATID_STOR)
            {
                if ((SumPtr->Sum_Record.Record.Id == MatchId) &&
                    (SumPtr->Sum_Record.Record.Details[0].Id == DetailId))
                {
                    MatchFound = TRUE;
                }
            }
            else
            {
                /*
                 * Only add on the values of records that match the current head
                 * element, in both ID and itemname.
                 */

                if ((SumPtr->Sum_Record.Record.Id == MatchId) &&
                    (strcmp(SumPtr->Sum_Record.Record.ItemName, ItemName) == 0))
                {
                    MatchFound = TRUE;
                }
            }

            if ( MatchFound )
            {
                SFMT_DEBUG(("Match Found\n"));

                /*
                 * Add in the values to the Averaging record.
                 */

                for(i = 0; i < SumPtr->Sum_Record.Record.NumStats; i++)
                {
                    Average.Record.Details[i].Value +=
                        SumPtr->Sum_Record.Record.Details[i].Value;
                }

                AveCount ++;

                /*
                 * Detach this record from the list.
                 */

                Detach = SumPtr;
                *Prev  = SumPtr->Next;          /* unlink the element */
                SumPtr = SumPtr->Next;
                free(Detach);
            }
            else
            {
                Prev   = &(SumPtr->Next);
                SumPtr = SumPtr->Next;
            }
        }


        /*
         * Perform the averaging. And output the generated statistics.
         */

        SFMT_DEBUG(("Averaging: %d elements\n", AveCount));

        for(i = 0; i < Average.Record.NumStats; i++)
        {
            Average.Record.Details[i].Value /= AveCount;
        }

        CICS_SFMT_OutputStats(&Average);
    }

    free(WorkBuffer);
    SFMT_DEBUG(("OUT CICS_SFMT_AverageCalc\n"));
}
