/*
 * NAME:        CICSSFMT Support Functions
 * VERSION:     1.1
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * SYNOPSIS:
 *
 * #include <cicstype.h>
 * #include <cicssfmt.h>
 *
 * PRIVATE void CICS_SFMT_InitCatSet(void);
 *
 * PRIVATE void CICS_SFMT_String2Timet(IN cics_char_t * TmString,
 *                          OUT time_t *Time);
 *
 * PRIVATE void CICS_SFMT_IncrLine(IN int Num);
 *
 * PRIVATE enum CICS_SFMT_Status CICS_SFMT_StoreRec(
 *                                     IN CICS_SFMT_StatsRecord_t *StatRec,
 *                                     INOUT CICS_SFMT_SummaryList_t *List);
 *
 * PRIVATE int CICS_SFMT_ValidateTime(IN cics_char_t *TMString);
 *
 * PRIVATE void CICS_SFMT_OutputMessage(IN int SetNo,
 *                              IN int MessageNo,
 *                              ...);
 *
 * PRIVATE int CICS_SFMT_PageHeaderSetUp(cics_char_t *FileName);
 *
 * PRIVATE void CICS_SFMT_OutputStats(IN CICS_SFMT_StatsRecord_t *StatRecord);
 *
 * DESCRIPTION:
 * :hp2.CICS_SFMT_InitCatSet:ehp2. initialises the global array
 * :hp4.CICS_SFMT_CatSet:ehp4. to the statistics categories supported by
 * cicssfmt.
 *
 * :hp2.CICS_SFMT_String2Timet:ehp2. converts the time :hp4.TmString:ehp4.
 * string into :hp4.Time:ehp4. time_t form.
 *
 * :hp2.CICS_SFMT_IncrLine:ehp2. increments the current line number
 * :hp4.CICS_SFMT_LineNo:ehp4. by :hp4.Num:ehp4.. If the line number
 * exceeds the Pagelength field of the :hp4.CICS_SFMT_Input:ehp4.
 * structure then the :hp4.CICS_SFMT_PageNo:ehp4. is incremented and
 * the the page header is output, depending on the '-h' option passed via
 * :hp4.CICS_SFMT_InputInfo:ehp4..
 *
 * :hp2.CICS_SFMT_StoRec:ehp2. stores the statistics record :hp4.StatRec:ehp4.
 * in memory by adding the record to the list :hp4.List:ehp4. supplied.
 *
 * :hp2.CICS_SFMT_ValidateTime:ehp2. does a simple validation of the input
 * time string. It is called from the main program to validate the
 * :hp4.starttime:ehp4. and :hp4.endtime:ehp4. specified.
 *
 * :hp2.CICS_SFMT_OutputMessage:ehp2. obtains the message :hp4.MessageNo:ehp4.
 * from the message set :hp4.SetNo:ehp4. from the message catalogue and
 * prints it to user's stderr.The :hp4.VariableArgList:ehp4. is used to
 * substitute the variable arguments in the message text.
 *
 * :hp2.CICS_SFMT_PageHeaderSetUp:ehp2. sets up the PageHeader using the
 * message catalogue for the various header strings and the statistics
 * filename to be output :hp4.FileName:ehp4.. It then outputs the header to
 * user's stdout.
 *
 * :hp2.CICS_SFMT_OutputStats:ehp2. formats and prints the statistics
 * information passed in the :hp4.StatRecord:ehp4. to user's stdout.
 *
 * CAVEATS/WARNINGS:
 * None.
 *
 * RETURNED VALUES:
 * :hp2.CICS_SFMT_StoreRec:ehp2. function returns :
 * :ul. compact.
 * :li.CICS_SFMT_STATUS_OK - Function returned successfully.
 * :li.CICS_SFMT_MEM_ERROR - Unable to allocate memory.
 * :eul.
 * :hp2.CICS_SFMT_ValidateTime:ehp2. function returns :
 * :ul. compact.
 * :li. 0 - Valid time string
 * :li. 1 - Invalid month value
 * :li. 2 - Invalid day value
 * :li. 3 - Invalid hours value
 * :li. 4 - Invalid minutes value
 * :li. 5 - Invalid seconds value
 * :eul.
 * :hp2.CICS_SFMT_PageHeaderSetUp:ehp2. function returns :
 * :ul. compact.
 * :li. 0 - Function returned successfully.
 * :li. 1 - Function failed.
 * :eul.
 * :hp2.CICS_SFMT_OutputStats:ehp2. may exit with the following exit code:
 * :ul. compact.
 * :li.1 - Unable to generate Detail/Summary Report due to insufficient
 * pagewidth.
 * :eul.
 *
 *
 * RELATED INFORMATION:
 * See the functions :hp2.CICS_SFMT_GenDetail:ehp2 and
 * :hp2.CICS_SFMT_GenSummary:ehp2. and the main program for the usage of
 * these support functions.
 *
 */

#include <cicstype.h>		/* CICS-supplied datatypes */
#include <stdio.h>
#include <stdarg.h>

#include <cicssfmt.h>
#include <cicssfmt_msg.h>

void CICS_SFMT_InitCatSet(void)
{
    cics_ulong_t i;                           /* loop variable */
    cics_char_t *DefaultMsg;                  /* default message string */
    cics_char_t *Msg;                         /* ptr to category message */
    int setno;                                 /* message set number */
    int msgno;                                 /* message number */

    SFMT_DEBUG(("IN CICS_SFMT_InitCatSet\n"));

    for (i=1; i<=CICS_SFMT_NUM_CATEGORIES; i++)
    {
        setno = i;
        msgno = 1;      /* 1st message in the set is the Category message */

        /* Set default message in case of catalogue errors */
        DefaultMsg = "Unknown Category";

        /* get the message string */
        Msg = catgets(CICS_SFMT_CatDescriptor,
                      setno,
                      msgno,
                      DefaultMsg);

        /* Check the result of the catalogue access */
        if (strcmp(Msg, DefaultMsg) != 0)    /* found a message */
        {
            CICS_SFMT_CatSet[i-1] = (cics_char_t *)strdup(Msg); /* strdup mallocs storage */
        }
    }

    SFMT_DEBUG(("OUT CICS_SFMT_InitCatSet\n"));
}


void CICS_SFMT_String2Timet(IN cics_char_t * TmString, OUT time_t *Time)
{
    cics_char_t subtime[3];
    struct tm  theTime;

    SFMT_DEBUG(("IN CICS_SFMT_String2Timet Time  : [%s]\n",TmString));

    /*
     * Convert Input time string into 'struct tm'
     */
    sprintf(subtime,"%c%c",TmString[0],TmString[1]);
    theTime.tm_year=atoi(subtime);
    sprintf(subtime,"%c%c",TmString[2],TmString[3]);
    theTime.tm_mon=atoi(subtime) - 1;
    sprintf(subtime,"%c%c",TmString[4],TmString[5]);
    theTime.tm_mday=atoi(subtime);
    sprintf(subtime,"%c%c",TmString[6],TmString[7]);
    theTime.tm_hour=atoi(subtime);
    sprintf(subtime,"%c%c",TmString[8],TmString[9]);
    theTime.tm_min=atoi(subtime);
    sprintf(subtime,"%c%c",TmString[10],TmString[11]);
    theTime.tm_sec=atoi(subtime);
    theTime.tm_isdst = -1;
    /*
     *     Call mktime to convert the above structure into time_t
     */

    *Time =  mktime(&theTime);

    SFMT_DEBUG(("OUT CICS_SFMT_String2Timet Time : [%d]\n",*Time));
}


void CICS_SFMT_IncrLine(IN int Num)
{
    SFMT_DEBUG(("IN CICS_SFMT_IncrLine with Num : [%d]\n",Num));

    /*
     * Always leave two blank lines at the bottom of the page.
     */

    if ((CICS_SFMT_LineNo + Num) >= (CICS_SFMT_InputInfo.Pagelength - 2))
    {
        /*
         * Pad out the remainder of the page with blank lines.
         */

        for (; CICS_SFMT_LineNo < CICS_SFMT_InputInfo.Pagelength; CICS_SFMT_LineNo++)
        {
            SFMT_DEBUG(("Adding padding line. (pad to page end)\n"));
#if defined(SFMT_DEBUG_ON)
            fprintf(stderr, "\n");
#else
            fprintf(stdout, "\n");
#endif
        }

#if defined(SFMT_DEBUG_ON)
        fprintf(stderr, "\f");          /* Cause a printer page throw */
#else
        fprintf(stdout, "\f");          /* Cause a printer page throw */
#endif

        CICS_SFMT_PageNo++;
        CICS_SFMT_LineNo = 0;
        if (CICS_SFMT_InputInfo.Header)
        {
            CICS_SFMT_PageHeaderSetUp(CICS_SFMT_StatsFileName);
        }
    }

    CICS_SFMT_LineNo += Num;

    SFMT_DEBUG(("OUT CICS_SFMT_IncrLine\n"));
}

enum CICS_SFMT_Status CICS_SFMT_StoreRec(IN CICS_SFMT_StatsRecord_t *StatRec,
                                         INOUT CICS_SFMT_SummaryList_t **List)
{
    CICS_SFMT_SummaryList_t *SumPtr;
    CICS_SFMT_SummaryList_t *TempPtr;
    int i;

    SFMT_DEBUG(("IN CICS_SFMT_StoreRec \n"));

    /*
     * Allocate memory to SumPtr
     */
    SumPtr = (CICS_SFMT_SummaryList_t *)
                  malloc(sizeof(CICS_SFMT_SummaryList_t));

    if ( SumPtr == (CICS_SFMT_SummaryList_t *)NULL )
    {
        /* failed to malloc memory */
        SFMT_DEBUG(("Failed to malloc memory in CICS_SFMT_StoreRec\n"));
        return CICS_SFMT_MEM_ERROR;
    }

    /* copy the contents of StatRec into SumPtr */
    memcpy((void *)&(SumPtr->Sum_Record),
           (void *)StatRec,
           sizeof(CICS_SFMT_StatsRecord_t));

    SumPtr->Next = NULL;

    if (*List == (CICS_SFMT_SummaryList_t *)NULL )
    {
        /* Create the list */

        *List = SumPtr;
        SFMT_DEBUG(("CREATED List\n"));
        return CICS_SFMT_STATUS_OK;
    }
    else
    {
        TempPtr = *List;
        /* add this record at the last */
        while ( TempPtr->Next != (CICS_SFMT_SummaryList_t *)NULL )
        {
            TempPtr = TempPtr->Next;
        }
        TempPtr->Next = SumPtr;
        SFMT_DEBUG(("ADDED to List\n"));
        return CICS_SFMT_STATUS_OK;
    }
}

int CICS_SFMT_ValidateTime(IN cics_char_t *TMString)
{
    cics_char_t subtime[3];
    cics_char_t buf[13];
    struct tm Outtm;

    SFMT_DEBUG(("IN CICS_SFMT_ValidateTime : [%s]\n",TMString));
    strcpy(buf,TMString);

    /*
     * Extract details of the date below,
     * Note that we take 1 from the month value as the tm
     * structure has months from 0 to 11 but the user inputs
     * months from 1 to 12.
     */

    sprintf(subtime,"%c%c",buf[0],buf[1]);
    Outtm.tm_year=atoi(subtime);
    sprintf(subtime,"%c%c",buf[2],buf[3]);
    Outtm.tm_mon=atoi(subtime) -1;
    sprintf(subtime,"%c%c",buf[4],buf[5]);
    Outtm.tm_mday=atoi(subtime);
    sprintf(subtime,"%c%c",buf[6],buf[7]);
    Outtm.tm_hour=atoi(subtime);
    sprintf(subtime,"%c%c",buf[8],buf[9]);
    Outtm.tm_min=atoi(subtime);
    sprintf(subtime,"%c%c",buf[10],buf[11]);
    Outtm.tm_sec=atoi(subtime);

    if ( Outtm.tm_mon <  0 || Outtm.tm_mon > 11 )
    {
        return(1);
    }
    if ( Outtm.tm_mday < 1 || Outtm.tm_mday > 31 )
    {
        return(2);
    }
    if ( Outtm.tm_hour < 0 || Outtm.tm_hour > 23 )
    {
        return(3);
    }
    if ( Outtm.tm_min < 0 || Outtm.tm_min > 59 )
    {
        return(4);
    }
    if ( Outtm.tm_sec < 0 || Outtm.tm_sec > 59 )
    {
        return(5);
    }
    return(0);
}

void CICS_SFMT_OutputMessage(IN int SetNo,
                             IN int MessageNo,
                             ...)
{
    va_list ArgPtr;             /* var args */
    cics_char_t *Msg;           /* message string pointer */
    cics_char_t *DefaultMsg;    /* default message string */
    cics_char_t MsgBuf[1024];   /* buffer to hold the formatted message */

    SFMT_DEBUG(("IN CICS_SFMT_OutputMessage \n"));
    SFMT_DEBUG(("SetNo : [%d] MessageNo : [%d] \n",SetNo,MessageNo));

    va_start(ArgPtr, MessageNo);

    /* Set default message in case of catalogue errors */
    DefaultMsg = "Failed to access catalogue for Type";

    /* get the "Type: " message string */
    Msg = catgets(CICS_SFMT_CatDescriptor,
                  SetNo,
                  MessageNo,
                  DefaultMsg);

    /* Check the result of the catalogue access */
    if (strcmp(Msg, DefaultMsg) == 0)
    {
        fprintf(stderr, "%s\n",Msg);
        return;
    }
    else
    {
        vsprintf(MsgBuf, Msg, ArgPtr);
        fprintf(stderr, "%s\n", MsgBuf);
    }
    va_end(ArgPtr);

    SFMT_DEBUG(("OUT CICS_SFMT_OutputMessage \n"));
}

int CICS_SFMT_PageHeaderSetUp(cics_char_t *FileName)
{
    cics_char_t *WorkBuffer;    /* space to manipulate strings */
    int WorkBuffLen;            /* size of WorkBuffer */
    int Line,Col;               /* current line column position */
    cics_char_t *TimeString;    /* for manipulating time */
    struct tm *TimeData;        /* for manipulating time */
    time_t Time;                /* hold the current time */
    cics_char_t *BufferPtr;     /* ptr to current write pos */
    cics_char_t *OutputMsg;     /* ptr to current output msg */
    cics_char_t *DefaultMsg;    /* default message string */
    int numbytes;               /* number of bytes to allocate memory */
    int MsgNo;                  /* message number for detailed/summary title */
    cics_char_t *PageHeader[CICS_SFMT_MAX_HEADERS];
                                /* strings to hold the formatted page header */

    SFMT_DEBUG(("IN CICS_SFMT_PageHeaderSetUp for file [%s]\n", FileName));

    numbytes = CICS_SFMT_InputInfo.Pagewidth + 1;

    /* Allocate memory to the workbuffer */
    WorkBuffer = (cics_char_t *)malloc(numbytes);
    memset(WorkBuffer, ' ', numbytes);
    WorkBuffer[numbytes - 1] = '\0';

    /*
     * Allocate memory to each PageHeader element,
     * the size of Pagewidth+1 and 'zero' it
     */

    for(Line=0; Line < CICS_SFMT_MAX_HEADERS; Line++)
    {
        PageHeader[Line] = (cics_char_t *)malloc(numbytes);
        memset(PageHeader[Line], ' ', numbytes);
        PageHeader[Line][numbytes - 1] = '\0';  /* terminate the buffer */
    }

    /*
     * Create the first header line i.e
     * Statistics Filename : <filename>
     *
     */

    BufferPtr = PageHeader[0];

    /* Set default message in case of catalogue errors */
    DefaultMsg = "Failed to access Message catalogue for CICS_SFMT_SRH_FILE";

    /* get the message string */
    OutputMsg = catgets(CICS_SFMT_CatDescriptor,
                  CICS_SFMT_REPORT_SET,
                  CICS_SFMT_SRH_FILE,
                  DefaultMsg);

    /* Check the result of the catalogue access */
    if (strcmp(OutputMsg, DefaultMsg) == 0)
    {
        perror(OutputMsg);
        return 1;
    }
    SFMT_DEBUG(("OutputMessage : [%s] FileName : [%s]\n",
                 OutputMsg, FileName));
    SFMT_DEBUG(("numbytes : [%d]\n", numbytes));

    sprintf(WorkBuffer, OutputMsg, (numbytes - 1), FileName);
    SFMT_DEBUG(("Workbuffer : [%s]\n", WorkBuffer));
    strcpy(BufferPtr, WorkBuffer);

    SFMT_DEBUG(("First header line : [%s]\n",PageHeader[0]));

    /*
     * Create the second header line i.e
     * Date : <date>                                         PageNo : <no>
     *
     */

    memset(WorkBuffer, ' ', numbytes);
    BufferPtr = PageHeader[1];

    /* Set default message in case of catalogue errors */
    DefaultMsg = "Failed to access Message catalogue for CICS_SFMT_SRH_DATE";

    /* get the message string */
    OutputMsg = catgets(CICS_SFMT_CatDescriptor,
                        CICS_SFMT_REPORT_SET,
                        CICS_SFMT_SRH_DATE,
                        DefaultMsg);

    /* Check the result of the catalogue access */
    if (strcmp(OutputMsg, DefaultMsg) == 0)
    {
        perror(OutputMsg);
        return 1;
    }

    Time = time(NULL);
    TimeData = localtime(&Time);
    TimeString = asctime(TimeData);
    *(TimeString + strlen(TimeString) - 1) = '\0';

    SFMT_DEBUG(("OutputMessage : [%s] TimeString : [%s]\n",
                 OutputMsg, TimeString));
    sprintf(WorkBuffer, OutputMsg, (numbytes - 1), TimeString);
    SFMT_DEBUG(("Workbuffer : [%s]\n", WorkBuffer));
    WorkBuffLen = strlen(WorkBuffer);
    memcpy(BufferPtr, WorkBuffer, WorkBuffLen);

    /* Set default message in case of catalogue errors */
    DefaultMsg = "Failed to access Message catalogue for CICS_SFMT_SRH_PAGE";

    /* get the message string */
    OutputMsg = catgets(CICS_SFMT_CatDescriptor,
                        CICS_SFMT_REPORT_SET,
                        CICS_SFMT_SRH_PAGE,
                        DefaultMsg);

    /* Check the result of the catalogue access */
    if (strcmp(OutputMsg, DefaultMsg) == 0)
    {
        perror(OutputMsg);
        return 1;
    }

    memset(WorkBuffer, ' ', numbytes);
    WorkBuffer[numbytes -  1] = '\0';
    sprintf(WorkBuffer, OutputMsg, CICS_SFMT_PageNo);
    WorkBuffLen = strlen(WorkBuffer);

    Col = numbytes - ( CICS_SFMT_COL_PAGENO > WorkBuffLen
                              ? CICS_SFMT_COL_PAGENO : WorkBuffLen);
    BufferPtr += Col;

    memcpy(BufferPtr, WorkBuffer, WorkBuffLen);
    BufferPtr += WorkBuffLen;
    SFMT_DEBUG(("Second header line : [%s]\n",PageHeader[1]));

    /*
     * Create the third header line i.e
     *           CICS/xxxx STATISTICS INTERVAL SUMMARY/DETAILED REPORT
     *
     */

    memset(WorkBuffer, ' ', numbytes);
    WorkBuffer[numbytes -  1] = '\0';
    BufferPtr = PageHeader[2];
    if ( CICS_SFMT_InputInfo.Detailed )
    {
        MsgNo = CICS_SFMT_SRH_DETAILED;
    }
    else
    {
        MsgNo = CICS_SFMT_SRH_SUMMARY;
    }

    /* Set default message in case of catalogue errors */
    DefaultMsg = "Failed to access Message catalogue for Report Title";

    /* get the message string */
    OutputMsg = catgets(CICS_SFMT_CatDescriptor,
                        CICS_SFMT_REPORT_SET,
                        MsgNo,
                        DefaultMsg);

    /* Check the result of the catalogue access */
    if (strcmp(OutputMsg, DefaultMsg) == 0)
    {
        perror(OutputMsg);
        return 1;
    }

    sprintf(WorkBuffer, "%.*s", (numbytes - 1), OutputMsg);
    WorkBuffLen = strlen(WorkBuffer);

    /* this should be in the centre of the pagewidth */
    Col = (CICS_SFMT_InputInfo.Pagewidth - WorkBuffLen)/2;
    BufferPtr += Col;

    memcpy(BufferPtr, WorkBuffer, WorkBuffLen);
    BufferPtr += WorkBuffLen;
    SFMT_DEBUG(("Third header line : [%s]\n",PageHeader[2]));

#if defined(SFMT_DEBUG_ON)
    fprintf(stderr, PageHeader[0]);
    fprintf(stderr, "\n");
    fprintf(stderr, PageHeader[1]);
    fprintf(stderr, "\n");
    fprintf(stderr, "\n");
    fprintf(stderr, PageHeader[2]);
    fprintf(stderr, "\n");
    fprintf(stderr, "\n");
#else
    fprintf(stdout, PageHeader[0]);
    fprintf(stdout, "\n");
    fprintf(stdout, PageHeader[1]);
    fprintf(stdout, "\n");
    fprintf(stdout, "\n");
    fprintf(stdout, PageHeader[2]);
    fprintf(stdout, "\n");
    fprintf(stdout, "\n");
#endif

    CICS_SFMT_LineNo += 5;

    for(Line=0; Line < CICS_SFMT_MAX_HEADERS; Line++)
    {
        free(PageHeader[Line]);     /* free the PageHeader memory allocation */
    }
    free(WorkBuffer);   /* Free the WorkBuffer memory allocation */

    return 0;
}

void CICS_SFMT_OutputStats(IN CICS_SFMT_StatsRecord_t *StatRecord)
{
    cics_char_t *DescTab[CICSSTAT_MAX_STAT_DETS];  /* holds the description texts
                                               for each Details element */
    cics_sshort_t FieldSpace = 2;           /* constant field spacing */
    cics_ulong_t i;                         /* loop control variable */
    int MaxLength = 0;                      /* max. length of the desc. text */
    int MsgNo;                              /* message number */
    int SetNo;                              /* message set number */
    cics_char_t *Msg;                       /* ptr to message string */
    cics_char_t *DefaultMsg;                /* default message string */
    cics_char_t *Category;                  /* stores the Category text */
    int ColOnewidth;                        /* width of 1st col. */
    int ColDescwidth;                       /* width of Desc. cols */
    int WidthRemain;                        /* remaining width */
    int NumCols;                            /* number of cols. */
    int NumTitleLines;                      /* number of title lines */
    int line,col;
    int index;
    cics_char_t *TitleLine;                 /* formatted Title line */
    cics_char_t *WorkBuffer;                /* space to manipulate strings */
    int WorkBufferLen;                      /* size of WorkBuffer */
    cics_char_t *BufferPtr;                 /* pointer to current write pos. */
    int numbytes;                           /* num bytes to allocate mem. to */
    cics_bool_t Title;                      /* print Title ? */
    enum CICS_SFMT_Status RetStatus;        /* return from CICS_SFMT_ funcs */

    SFMT_DEBUG(("IN CICS_SFMT_OutputStats\n"));


    /*
     * Compare the global CICS_SFMT_ReportDetails array with the
     * incoming Details array.
     * If they are identical, do not print the title
     * else print the title.
     */
    if (CICS_SFMT_ReportDetails[0].Id != StatRecord->Record.Details[0].Id)
    {
        for(i=0; i < StatRecord->Record.NumStats; i++)
        {
            memcpy(&(CICS_SFMT_ReportDetails[i]),
                   &(StatRecord->Record.Details[i]),
                   sizeof(CICSSTAT_Detail_t));
        }
        Title = TRUE;
        SFMT_DEBUG(("PRINT TITLE\n"));
    }
    else
    {
        Title = FALSE;
        SFMT_DEBUG(("DONT PRINT TITLE\n"));
    }


    /*
     * get the message set number for this stats category
     */
    SetNo = CICS_SFMT_CATSET_NUM(StatRecord->Record.Id);

    SFMT_DEBUG(("Check title len. %d stats\n", StatRecord->Record.NumStats));

    MaxLength = 0;
    for(i = 0; i < StatRecord->Record.NumStats; i++)
    {
        MsgNo = CICS_SFMT_CATDESC_NUM(StatRecord->Record.Details[i].Id) + 2 ;

        /* Set default message in case of catalogue errors */
        DefaultMsg = "Failed to access Message catalogue for Description";

        /* get the message string */
        Msg = catgets(CICS_SFMT_CatDescriptor,
                            SetNo,
                            MsgNo,
                            DefaultMsg);

        /* Check the result of the catalogue access */
        if (strcmp(Msg, DefaultMsg) == 0)
        {
            perror(Msg);
            return;
        }
        else
        {
            SFMT_DEBUG(("Field title = [%s]\n", Msg));

            numbytes   = strlen(Msg);
            DescTab[i] = (cics_char_t *)malloc(numbytes + 1);
            SFMT_DEBUG(("MALLOC ptr = [%#x]\n", DescTab[i]));

            strcpy(DescTab[i], Msg);

            if (MaxLength < numbytes)
            {
                MaxLength = numbytes;
                SFMT_DEBUG(("MaxLength now set to %d\n", MaxLength));
            }
        }
    }

    SFMT_DEBUG(("MaxLength : [%d]\n", MaxLength));

    /*
     * get the Category text for the stats category
     * first message in this sets gives the category text
     */
    MsgNo = 1;

    /* Set default message in case of catalogue errors */
    DefaultMsg = "Failed to access Message catalogue for Category";

    /* get the message string */
    Msg = catgets(CICS_SFMT_CatDescriptor,
                  SetNo,
                  MsgNo,
                  DefaultMsg);

    /* Check the result of the catalogue access */
    if (strcmp(Msg, DefaultMsg) == 0)
    {
        perror(Msg);
        return;
    }
    else
    {
        numbytes = strlen(Msg);
        SFMT_DEBUG(("Got category [%s] OK, Alloc len[%d]\n", Msg, numbytes));
        Category = (cics_char_t *)malloc(numbytes + 1);
        SFMT_DEBUG(("MALLOC ptr = [%#x]\n", Category));
        strcpy(Category, Msg);
    }

    SFMT_DEBUG(("Category : [%s], length %d\n", Category, strlen(Category)));

    ColOnewidth   = (CICSSTAT_ITEM_NAME_LEN > strlen(Category)
                    ? CICSSTAT_ITEM_NAME_LEN
                    : strlen(Category));

    SFMT_DEBUG(("ColOnewidth = %d\n", ColOnewidth));

    ColOnewidth  += FieldSpace;
    WidthRemain   = CICS_SFMT_InputInfo.Pagewidth - ColOnewidth;
    ColDescwidth  = MaxLength + FieldSpace;

    SFMT_DEBUG(("ITEM col width %d; DESC col %d\n", ColOnewidth, ColDescwidth));
    SFMT_DEBUG(("WidthRemain = %d\n", WidthRemain));

    NumCols       = (WidthRemain / ColDescwidth);


    /*
     * If there isn't sufficient page width to display the data, display a
     * message, and exit.
     */

    if (NumCols == 0)
    {
        SFMT_DEBUG(("EXIT - Insufficient page width to display category\n"));

        CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET, ERZ5514E);


        /*
         * close the file previously opened
         */

        RetStatus = CICS_SFMT_CloseStatsFile(&CICS_SFMT_StatsFile);

        if (RetStatus != CICS_SFMT_STATUS_OK)
        {
            CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
                                    ERZ5502E,
                                    CICS_SFMT_StatsFileName);

        }


        /*
         * Close the message catalogue
         */

        (void)catclose(CICS_SFMT_CatDescriptor);

        exit(1);
    }

    NumTitleLines = StatRecord->Record.NumStats / NumCols;

    if (NumTitleLines == 0)
    {
        NumTitleLines = 1;
    }
    else
    {
        if ( (StatRecord->Record.NumStats % NumCols) != 0)
        {
            SFMT_DEBUG(("Include the remaining titles\n"));
            NumTitleLines++;
        }
    }

    SFMT_DEBUG(("NumCols = %d, NumTitleLines = %d\n", NumCols, NumTitleLines));


    numbytes   = CICS_SFMT_InputInfo.Pagewidth + 1;
    WorkBuffer = (cics_char_t *)malloc(numbytes);
    SFMT_DEBUG(("MALLOC ptr = [%#x]\n", WorkBuffer));

    TitleLine  = (cics_char_t *)malloc(numbytes);
    SFMT_DEBUG(("MALLOC ptr = [%#x]\n", TitleLine));

    if ( Title )
    {
        /*
         * To display the title, NumTitleLines is required, pre-allocate
         * to page space with IncrLine. This figure should also include the
         * stats data.
         */

        SFMT_DEBUG(("TITLE REQUIRED: Display it\n"));
        CICS_SFMT_IncrLine(NumTitleLines * 2);

        col = 0;            /* indicate how many desc. have been delt with */
        for(line = 0; line < NumTitleLines; line++)
        {
            memset(WorkBuffer, ' ', numbytes);
            WorkBuffer[numbytes - 1] = '\0';

            memset(TitleLine, ' ', numbytes);
            TitleLine[numbytes - 1] = '\0';  /* terminate the buffer */

            BufferPtr = TitleLine;

            if (line == 0)    /* first title line, hence print Category text */
            {
                sprintf(WorkBuffer, "%.*s", (numbytes - 1), Category);
                WorkBufferLen = strlen(WorkBuffer);
                memcpy(BufferPtr, WorkBuffer, WorkBufferLen);
            }
            BufferPtr += ColOnewidth;

            if ( col >= CICSSTAT_MAX_STAT_DETS)
            {
                SFMT_DEBUG(("cols exceeded max. number of statistics desc\n"));
                break;
            }

            for (i = 0; i < NumCols ; i++)
            {
                index = i + col;
                if (index >= StatRecord->Record.NumStats)
                {
                    SFMT_DEBUG(("index : [%d] exceeds NumStats : [%d]\n",
                               index,
                               StatRecord->Record.NumStats));
                    break;
                }
                sprintf(WorkBuffer, "%.*s", (numbytes - 1), DescTab[index]);
                WorkBufferLen = strlen(WorkBuffer);
                memcpy(BufferPtr, WorkBuffer, WorkBufferLen);
                BufferPtr += ColDescwidth;
            }
            col += NumCols;

#if defined(SFMT_DEBUG_ON)
            fprintf(stderr, TitleLine);
            fprintf(stderr, "\n");
#else
            fprintf(stdout, TitleLine);
            fprintf(stdout, "\n");
#endif

        } /* end for line */
    }
    else
    {
        /*
         * If there is no title, then we only need to allocate page space
         * for the stats data. One line of data for each line of title.
         */

        CICS_SFMT_IncrLine(NumTitleLines);
    }

    /*
     * simply output the stats values in the corresponding cols.
     */


    col = 0;
    for(line = 0; line < NumTitleLines; line++)
    {
        memset(WorkBuffer, ' ', numbytes);
        WorkBuffer[numbytes - 1] = '\0';

        memset(TitleLine, ' ', numbytes);
        TitleLine[numbytes - 1] = '\0';  /* terminate the buffer */

        BufferPtr = TitleLine;

        if (line == 0)      /* first line, hence print ItemName text */
        {
            sprintf(WorkBuffer, "%.*s", (numbytes - 1),
                    StatRecord->Record.ItemName);
            WorkBufferLen = strlen(WorkBuffer);
            memcpy(BufferPtr, WorkBuffer, WorkBufferLen);
        }

        BufferPtr += ColOnewidth;

        SFMT_DEBUG(("Number of stats. handled so far are : [%d]\n", col));
        if ( col >= CICSSTAT_MAX_STAT_DETS)
        {
            SFMT_DEBUG(("cols exceeded max. number of statistics \n"));
            break;
        }

        for (i = 0; i < NumCols ; i++)
        {
            index = i + col;
            if (index >= StatRecord->Record.NumStats)
            {
                SFMT_DEBUG(("index : [%d] exceeds NumStats : [%d]\n",
                           index,
                           StatRecord->Record.NumStats));
                break;
            }
            sprintf(WorkBuffer, "%d", StatRecord->Record.Details[index].Value);
            WorkBufferLen = strlen(WorkBuffer);
            memcpy(BufferPtr, WorkBuffer, WorkBufferLen);
            SFMT_DEBUG(("index : [%d] WorkBuffer : [%s]\n", index, WorkBuffer));
            BufferPtr += ColDescwidth;
        }
        col += NumCols;

#if defined(SFMT_DEBUG_ON)
        fprintf(stderr, TitleLine);
        fprintf(stderr, "\n");
#else
        fprintf(stdout, TitleLine);
        fprintf(stdout, "\n");
#endif

    } /* end for line */

    /* insert a blank line after each category print */

    CICS_SFMT_IncrLine(1);
#if defined(SFMT_DEBUG_ON)
    fprintf(stderr, "\n");
#else
    fprintf(stdout, "\n");
#endif

    for(i = 0; i < StatRecord->Record.NumStats; i++)
    {
        SFMT_DEBUG(("FREE ptr = [%#x] - DescTab[%d]\n", DescTab[i], i));
        free(DescTab[i]);
    }

    SFMT_DEBUG(("FREE ptr = [%#x] - Category\n", Category));
    free(Category);
    SFMT_DEBUG(("FREE ptr = [%#x] - WorkBuffer\n", WorkBuffer));
    free(WorkBuffer);
    SFMT_DEBUG(("FREE ptr = [%#x] - TitleLine\n", TitleLine));
    free(TitleLine);
    SFMT_DEBUG(("OUT CICS_SFMT_OutputStats\n"));
}
