/*
 * NAME:        cicssfmt (main) - Format the Statistics File.
 * VERSION:     1.1
 *
 * COPYRIGHT:                                                        
 * 
 *   (C) COPYRIGHT Hewlett Packard Company. 1993, 1997
 *   (C) COPYRIGHT International Business Machines Corp. 1993, 1997
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *  
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * 
 *                                                                   
 *
 * SYNOPSIS:
 *
 * cicssfmt [-c category[,...]] [-s starttime] [-e endtime]  [-l pagelength]
 *          [-w pagewidth] [-h] [-i] file1 file2 ....
 *
 * DESCRIPTION:
 * The cicssfmt utility is invoked from the &os. shell to format data from
 * &canna. Statistics file, using any of the following options specified
 * by the user :
 * :ul compact.
 * :li.
 * -c :hp4.category:ehp4. is the category to report which can be from the
 *    following set:
 * :sl.
 * :li.Dump Statistics (DUMP).
 * :li.File Statistics (FILE).
 * :li.Inter System Communications Management Statistics (ISCM).
 * :li.Journal Statistics (JOUR).
 * :li.Logical Units of Work Management Statistics (LUWM).
 * :li.Program Statistics (PROG).
 * :li.Runtime Database Statistics(RTDB).
 * :li.Region Storage Statistics (STOR).
 * :li.Task Storage Statistics (STOT).
 * :li.Task Statistics (TASK).
 * :li.Terminal Statistics (TERM).
 * :li.Transient Data Queue Statistics (TDQU).
 * :li.Transaction Statistics (TRAN).
 * :li.Temporary Storage Queue Statistics (TSQU).
 * :esl.
 * The default is all categories. Multiple categories are specified
 * using a comma separated list.
 * :li.
 * -s :hp4.starttime:ehp4. is the start time for the report to be produced in
 *    'yymmddhhmmss' format.
 * :li.
 * -e :hp4.endtime:ehp4. is the end time for the report to be produced in
 *    'yymmddhhmmss' format.
 * :li.
 * -l :hp4.pagelength:ehp4. is the length of the page in number of lines. The
 *    default value is 24 lines per page.
 * :li.
 * -w :hp4.pagewidth:ehp4. is the width of the page in number of columns. The
 *    default value is 80 columns.
 * :li.
 * -h Do not print page headers. The default is to print page headers.
 * :li.
 * -i Produce CICS Statistics Interval Detailed Report otherwise only the
 *    CICS Statistics Interval Summary Report will be produced.
 * :li.
 * :hp4.file1 file2 ...:ehp4. are the statistics files to be formatted
 * specified with the complete path.
 * :eul.
 *
 * For every statistics file specified by :hp4.file1 file2 ..:ehp4.,
 * :hp2.cicssfmt:ehp2. reads each record from it and generates either
 * of the following two reports :
 * :ul compact.
 * :li.
 * CICS Statistics Interval Detailed Report.
 * :li.
 * CICS Statistics Interval Summary Report.
 * :eul.
 *
 * If the :hp2.starttime:ehp2. value is not specified, the default start
 * point for statistics formatting is the start of the file.
 *
 * If the :hp2.endtime:ehp2. value is not specified, the default end point
 * for statistics formatting is the end of the file.
 *
 * If neither the :hp2.starttime:ehp2. nor the :hp2.endtime:ehp2. values
 * are specified, the default is to format the entire statistics file.
 *
 * Help information is printed if the -? parameter is supplied or
 * when cicssfmt is entered with the wrong syntax.
 *
 * CAVEATS/WARNINGS:
 * None.
 *
 * RETURNED VALUES:
 * :ul. compact.
 * :li.0 - Program completed successfully
 * :li.1 - Unable to generate Detail/Summary Report
 * :li.2 - Invalid usage.
 * :li.3 - Unable to open message catalogue.
 * :eul.
 *
 * RELATED INFORMATION:
 * None.
 *
 */

#include <cicstype.h>            /* CICS-supplied datatypes */
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <errno.h>
#include <locale.h>


#define CICSSFMT_STATIC_DATA            /* Cause declaration of global data */
#include <cicssfmt.h>
#include <cicssfmt_msg.h>               /* cicssfmt message numbers */

int main(int ArgC, cics_char_t ** ArgV)
{
    int c;                             /* value returned by getopt */
    extern int optind;
    extern cics_char_t *optarg;

    int InvalidUsage = FALSE;          /* Invocation OK ? */
    cics_char_t *MsgBuf = "";          /* message buffer */
    enum CICS_SFMT_Status RetStatus;   /* return from CICS_SFMT_* functions */
    int ret;                           /* return from ValidateTime */
    int i;                             /* loop control */
    int index;                         /* index into Category array */
    int found;                         /* category found ? */

    cics_char_t *category = NULL;      /* command line category */
    cics_char_t *tok;                  /* parse input categories */
    cics_char_t *starttime = NULL;     /* command line starttime */
    cics_char_t *endtime = NULL;       /* command line endtime */
    cics_char_t *pagelength = NULL;    /* command line pagelength */
    cics_char_t *pagewidth = NULL;     /* command line pagewidth */
    cics_char_t *tmpptr = NULL;        /* temp pointer for syntax checks */
    time_t Starttime = -1;             /* converted starttime */
    time_t Endtime = -1;               /* converted endtime */

    int FatalError = FALSE;	       /* A fatal error was encountered */


    setlocale(LC_ALL, "");

    /*
     * Open the message catalogue
     */

    if ( (CICS_SFMT_CatDescriptor = catopen(SF_CICSSFMT, 0)) == (nl_catd)-1 )
    {
        SFMT_DEBUG(("Catalog not found or Too many catalogs open\n"));
        perror("Failed to open message catalogue\n");
        exit(3);
    }
    SFMT_DEBUG(("Message Catalogue Open Successfully\n"));

    /*
     * Initialise CICS_SFMT_InputInfo structure
     */
    for(i = 0; i < CICS_SFMT_NUM_CATEGORIES; i++)
    {
        CICS_SFMT_InputInfo.Category[i] = NULL;
    }
    CICS_SFMT_InputInfo.Starttime  = -1;
    CICS_SFMT_InputInfo.Endtime    = -1;
    CICS_SFMT_InputInfo.Pagelength = 24;
    CICS_SFMT_InputInfo.Pagewidth  = 80;
    CICS_SFMT_InputInfo.Header     = TRUE;
    CICS_SFMT_InputInfo.Detailed   = FALSE;

    while ((c= getopt(ArgC, ArgV, ":c:s:e:l:w:hi")) != EOF)
    {
        switch (c)
        {
            case 'c' :
                      category = optarg;
                      break;
            case 's' :
                      starttime = optarg;
                      break;
            case 'e' :
                      endtime = optarg;
                      break;
            case 'l' :
                      pagelength = optarg;
                      if (atoi(pagelength) < 0)
                      {
                        CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
                                                ERZ5507E);
                        InvalidUsage = TRUE;
                      }
                      /*
                       * Check that pagelength only contains digits.
                       */
                      for (tmpptr = pagelength; tmpptr[0] !='\0'; tmpptr++)
                      {
                         if (!isdigit(tmpptr[0]))
                         {
                             CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
                                                     ERZ5507E);
                             InvalidUsage = TRUE;
                             break;
                         }
                      }
                      CICS_SFMT_InputInfo.Pagelength = atoi(pagelength);
                      break;
            case 'w' :
                      pagewidth = optarg;
                      if (atoi(pagewidth) < 0)
                      {
                        CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
                                                ERZ5508E);
                        InvalidUsage = TRUE;
                      }
                      /*
                       * Check that pagewidth only contains digits.
                       */
                      for (tmpptr = pagewidth; tmpptr[0] !='\0'; tmpptr++)
                      {
                         if (!isdigit(tmpptr[0]))
                         {
                             CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
                                                     ERZ5508E);
                             InvalidUsage = TRUE;
                             break;
                         }
                      }

                      CICS_SFMT_InputInfo.Pagewidth = atoi(pagewidth);
                      break;
            case 'h' :
                      CICS_SFMT_InputInfo.Header = FALSE;
                      break;
            case 'i' :
                      CICS_SFMT_InputInfo.Detailed = TRUE;
                      break;
            case '?' :
                      InvalidUsage = TRUE;
        }
    }

    if ( InvalidUsage )
    {
        CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,ERZ5501E,ArgV[0]);

        /*
         * Close the message catalogue
         */
        (void)catclose(CICS_SFMT_CatDescriptor);

        exit (2);
    }

    /*
     * Initialise the global CICS_SFMT_CatSet array
     */
    CICS_SFMT_InitCatSet();

    if (category == NULL)
    {
        SFMT_DEBUG(("Format ALL Categories\n"));
        category = "ALL";

        /*
         * Set global CICS_SFMT_InputInfo.Category[0] to ALL
         * by mallocing and copying using strdup
         */
        CICS_SFMT_InputInfo.Category[0] = strdup(category);
    }
    else
    {
        SFMT_DEBUG(("Validate categories\n"));
        if ((tok = strtok(category,",")) != NULL)
        {
            index = 0;
            while ( tok != NULL)
            {
              /*
               * Check if duplicate
               */
                found = FALSE;
                for( i=0 ; i<index ; i++)
                {   if( strcmp( tok , CICS_SFMT_InputInfo.Category[i])==0)
                    {
                        found = TRUE ;
                        break;
                    }
                }
                if( found )
                {
                    tok = strtok(NULL,",");
                    continue;
                }

               /*
                * Check if category specified is in CICS_SFMT_CatSet
                */
                for(i = 0; i < CICS_SFMT_NUM_CATEGORIES; i++)
                {
                    if ( strcmp(tok,CICS_SFMT_CatSet[i]) == 0)
                    {
                        found = TRUE;
                        /*
                         * copy those token into CICS_SFMT_InputInfo.Category
                         */
                         CICS_SFMT_InputInfo.Category[index] = strdup(tok);
                         index++;
                    }
                }
                if (!found)
                {
                    CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
                                            ERZ5506E,
                                            tok);

                    /*
                     * Close the message catalogue
                     */
                    (void)catclose(CICS_SFMT_CatDescriptor);

                    exit(2);
                }

                /*
                 * if generating  Summary report and category is TRAN
                 * then exit
                 */

                if ( !(CICS_SFMT_InputInfo.Detailed) )
                {
                    /* CICS_SFMT_CatSet[12] corresponds to the
                       TRAN category */

                    if (strcmp(tok,CICS_SFMT_CatSet[12]) == 0)
                    {
                        CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
                                                ERZ5511E,
                                                CICS_SFMT_CatSet[12]);

                        /*
                         * Close the message catalogue
                         */
                        (void)catclose(CICS_SFMT_CatDescriptor);

                        exit(2);
                    }
                }
                tok = strtok(NULL,",");
            }
        }
    }

    /*
     * Validate Starttime format
     */
    SFMT_DEBUG(("Validate starttime\n"));
    if (starttime != NULL)
    {
        if ( (ret = CICS_SFMT_ValidateTime(starttime)) != 0 )
        {
            CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET, ERZ5509E);

            /*
             * Close the message catalogue
             */
            (void)catclose(CICS_SFMT_CatDescriptor);

            exit(2);
        }
        /*
         * Convert time string into time_t form
         */
         CICS_SFMT_String2Timet(starttime,&Starttime);
    }

    /*
     * Validate Endtime format
     */
    SFMT_DEBUG(("Validate endtime\n"));
    if (endtime != NULL)
    {
        if ( (ret = CICS_SFMT_ValidateTime(endtime)) != 0 )
        {
            CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET, ERZ5509E);

            /*
             * Close the message catalogue
             */
            (void)catclose(CICS_SFMT_CatDescriptor);

            exit(2);
        }
        /*
         * Convert time string into time_t form
         */
         CICS_SFMT_String2Timet(endtime,&Endtime);
    }

    if ((starttime != NULL) && (endtime != NULL))
    {
        /*
         * Check that Starttime is not greater than Endtime
         */
        if ( Starttime > Endtime )
        {
            CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET, ERZ5505E);

            /*
             * Close the message catalogue
             */
            (void)catclose(CICS_SFMT_CatDescriptor);

            exit(2);
        }
    }

    /*
     * We have now set up the CICS_SFMT_InputInfo structure
     */

    if (optind == ArgC)
    {
        CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET, ERZ5513E);

        /*
         * Close the message catalogue
         */
        (void)catclose(CICS_SFMT_CatDescriptor);

        exit(2);
    }

    for (; optind < ArgC; optind++)
    {
        CICS_SFMT_StatsFileName = ArgV[optind];

	CICS_SFMT_InputInfo.Starttime = Starttime;
	CICS_SFMT_InputInfo.Endtime   = Endtime;

	SFMT_DEBUG(("CICS_SFMT_InputInfo.Starttime : [%ld]\n",
		     CICS_SFMT_InputInfo.Starttime));
	SFMT_DEBUG(("CICS_SFMT_InputInfo.Endtime : [%ld]\n",
		     CICS_SFMT_InputInfo.Endtime));


        if ( CICS_SFMT_InputInfo.Starttime != -1)     /* starttime specified */
        {
	    /*
	     * Open the stats file
	     */

	    RetStatus = CICS_SFMT_OpenStatsFile(CICS_SFMT_StatsFileName,
						&CICS_SFMT_StatsFile);
	    if ( RetStatus != CICS_SFMT_STATUS_OK)
	    {
		CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
					ERZ5502E,
					CICS_SFMT_StatsFileName);

		/*
		 * Close the message catalogue
		 */
		(void)catclose(CICS_SFMT_CatDescriptor);
		exit(1);
	    }

            /*
             * Find the record that indicates the start of collection
             */

	    RetStatus =  CICS_SFMT_FindRecord(CICS_SFMT_InputInfo.Starttime,
                                              CICS_SFMT_InputInfo.Endtime,
                                              &CICS_SFMT_StatsFile);

            if (RetStatus != CICS_SFMT_STATUS_OK )
            {
                switch (RetStatus)
                {
                    case CICS_SFMT_FILE_ERR :
			CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
					       ERZ5503E,
					       CICS_SFMT_StatsFileName);
			FatalError = TRUE;
			break;

                    case CICS_SFMT_FILE_SHORT_READ :
			CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
					       ERZ5504E,
					       CICS_SFMT_StatsFileName);
			FatalError = TRUE;
			break;

                    case CICS_SFMT_REC_NOT_FOUND :
			CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
					       ERZ5512W,
					       CICS_SFMT_StatsFileName);

			/*
			 * Deactivate START and END time locacation
			 */

			CICS_SFMT_InputInfo.Starttime = -1;
			CICS_SFMT_InputInfo.Endtime   = -1;

			break;
                }

                /*
                 * close the file previously opened
                 */

		RetStatus = CICS_SFMT_CloseStatsFile(&CICS_SFMT_StatsFile);

                if ( RetStatus != CICS_SFMT_STATUS_OK )
                {
                    CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
                                            ERZ5502E,
                                            CICS_SFMT_StatsFileName);

                }

		if (FatalError)
		{
		    /*
		     * Close the message catalogue
		     */

		    (void)catclose(CICS_SFMT_CatDescriptor);
		    exit(1);
		}
            }
        }


	/*
	 * Two possibilities for STARTTIME not being set now,
	 *     i)  the -s option not given on cmd line
	 *     ii) the specified -s value was not in the statistics file,
	 *         so all the statistics records are formatted.
	 */

        if (CICS_SFMT_InputInfo.Starttime == -1)  	/* starttime not given*/
        {
	    /*
	     * Open the stats file
	     */

	    RetStatus = CICS_SFMT_OpenStatsFile(CICS_SFMT_StatsFileName,
						&CICS_SFMT_StatsFile);

	    if ( RetStatus != CICS_SFMT_STATUS_OK)
	    {
		CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
					ERZ5502E,
					CICS_SFMT_StatsFileName);

		/*
		 * Close the message catalogue
		 */
		(void)catclose(CICS_SFMT_CatDescriptor);
		exit(1);
	    }

            /*
             * Read the first record from the statistics files
             */

	    RetStatus = CICS_SFMT_ReadRecord(&CICS_SFMT_StatsFile);

            /*
             * Check that the record time is not after the end time
             * specified.
             */

            if (( Endtime > 0 ) &&
            (Endtime < (&CICS_SFMT_StatsFile)->Record.Record.Details[0].Value))
            {
                switch ((&CICS_SFMT_StatsFile)->Record.Record.Id)
                {
                    case CICSSTAT_CATID_INTERVAL :
                    case CICSSTAT_CATID_ENDOFDAY :
                    case CICSSTAT_CATID_UNSOLICITED :
                    case CICSSTAT_CATID_REQUESTED_RESET :
                    case CICSSTAT_CATID_REQUESTED :
                    case CICSSTAT_CATID_ENDOFREGION :
                        CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
                                               ERZ5512W,
                                               CICS_SFMT_StatsFileName);

                        /*
                         * Deactivate END time 
                         */

                        CICS_SFMT_InputInfo.Endtime   = -1;
                        break;
                    default:
                        break;
                }
            }

            if ( RetStatus != CICS_SFMT_STATUS_OK )
            {
                switch (RetStatus)
                {
                    case CICS_SFMT_FILE_ERR :
			SFMT_DEBUG(("FILE ERROR received\n"));
			CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
                                                ERZ5503E,
                                                CICS_SFMT_StatsFileName);
			FatalError = TRUE;
			break;

                    case CICS_SFMT_FILE_SHORT_READ :
			SFMT_DEBUG(("Partial READ error\n"));
			CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
					        ERZ5504E,
						CICS_SFMT_StatsFileName);
			FatalError = TRUE;
			break;

                    case CICS_SFMT_FILE_ZERO_READ :
			SFMT_DEBUG(("FILE empty\n"));
			CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
					        ERZ5515W,
						CICS_SFMT_StatsFileName);
			break;
                }

		if (FatalError)
		{
		    SFMT_DEBUG(("A fatal error occurred - exit\n"));

		    /*
		     * close the file previously opened
		     */

		    RetStatus = CICS_SFMT_CloseStatsFile(&CICS_SFMT_StatsFile);

		    if (RetStatus != CICS_SFMT_STATUS_OK)
		    {
			CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
						ERZ5502E,
						CICS_SFMT_StatsFileName);

		    }

		    /*
		     * Close the message catalogue
		     */
		    (void)catclose(CICS_SFMT_CatDescriptor);
		    exit(1);
		}
		else
		{
		    SFMT_DEBUG(("STATs file is unusable - next\n"));
		    continue; 			/* Move onto next file */
		}
            }
        }

        /*
         * We have atleast one statistics record in the file,
         * so begin processing the file.
         */

        /* Initialise  Page number and Line number */

        CICS_SFMT_PageNo = 1;
        CICS_SFMT_LineNo = 0;

        /*
         * Initialise the global CICS_SFMT_ReportDetails array to zero
         */
        for (i = 0; i < CICSSTAT_MAX_STAT_DETS; i++)
        {
            memset(&(CICS_SFMT_ReportDetails[i]), 0, sizeof(CICSSTAT_Detail_t));
        }

        if ( CICS_SFMT_InputInfo.Header )      /* '-h' option not specified */
        {
            CICS_SFMT_PageHeaderSetUp(CICS_SFMT_StatsFileName);
        }

        /*
         * Do the Real Processing  to generate the report
         */

        if (CICS_SFMT_InputInfo.Detailed)       /* generate Detailed Report */
        {
            if ( (RetStatus = CICS_SFMT_GenDetail()) != CICS_SFMT_STATUS_OK )
            {
                /*
                 * output message unable to allocate memory
                 */


                switch (RetStatus)
                {
                    case CICS_SFMT_FILE_ERR :
                                CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
                                                       ERZ5503E,
                                                       CICS_SFMT_StatsFileName);
                                break;
                    case CICS_SFMT_FILE_SHORT_READ :
                                CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
                                                       ERZ5504E,
                                                       CICS_SFMT_StatsFileName);
                                break;
                    case CICS_SFMT_MEM_ERROR :
                                CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
                                                       ERZ5510E,
                                                       CICS_SFMT_StatsFileName);
                                break;
                }


                if ( (RetStatus = CICS_SFMT_CloseStatsFile(
                                                          &CICS_SFMT_StatsFile))
                             != CICS_SFMT_STATUS_OK )
                {
                    CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
                                            ERZ5502E,
                                            CICS_SFMT_StatsFileName);

                }

                /*
                 * Close the message catalogue
                 */
                (void)catclose(CICS_SFMT_CatDescriptor);



                  exit(1);

            }
        }
        else                        /* generate the Summary Report */
        {
            if ( (RetStatus = CICS_SFMT_GenSummary()) != CICS_SFMT_STATUS_OK )
            {
                /*
                 * output message unable to allocate memory
                 */

                switch (RetStatus)
                {
                    case CICS_SFMT_FILE_ERR :
                                CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
                                                       ERZ5503E,
                                                       CICS_SFMT_StatsFileName);
                                break;
                    case CICS_SFMT_FILE_SHORT_READ :
                                CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
                                                       ERZ5504E,
                                                       CICS_SFMT_StatsFileName);
                                break;
                    case CICS_SFMT_MEM_ERROR :
                                CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
                                                       ERZ5510E,
                                                       CICS_SFMT_StatsFileName);
                                break;
                }


                if ( (RetStatus = CICS_SFMT_CloseStatsFile(
                                                         &CICS_SFMT_StatsFile))
                             != CICS_SFMT_STATUS_OK )
                {
                    CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
                                            ERZ5502E,
                                            CICS_SFMT_StatsFileName);

                }

                /*
                 * Close the message catalogue
                 */
                (void)catclose(CICS_SFMT_CatDescriptor);

                exit(1);


            }
        }

        if ( (RetStatus = CICS_SFMT_CloseStatsFile(&CICS_SFMT_StatsFile))
                     != CICS_SFMT_STATUS_OK )
        {
            CICS_SFMT_OutputMessage(CICS_SFMT_ERROR_SET,
                                    ERZ5502E,
                                    CICS_SFMT_StatsFileName);

        /*
         * Close the message catalogue
         */
        (void)catclose(CICS_SFMT_CatDescriptor);

        exit(1);

        }

    } /* for end for each file */

    /*
     * Close the message catalogue
     */
    (void)catclose(CICS_SFMT_CatDescriptor);

    exit(0);
}
